#include "parser.h"

#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#define LANGUAGE_VERSION 13
#define STATE_COUNT 2400
#define LARGE_STATE_COUNT 25
#define SYMBOL_COUNT 170
#define ALIAS_COUNT 1
#define TOKEN_COUNT 65
#define EXTERNAL_TOKEN_COUNT 9
#define FIELD_COUNT 28
#define MAX_ALIAS_SEQUENCE_LENGTH 10
#define PRODUCTION_ID_COUNT 206

enum {
  sym_identifier = 1,
  anon_sym_package = 2,
  anon_sym_DOT = 3,
  anon_sym_object = 4,
  anon_sym_import = 5,
  anon_sym_COMMA = 6,
  anon_sym_LBRACE = 7,
  anon_sym_RBRACE = 8,
  anon_sym_EQ_GT = 9,
  anon_sym_case = 10,
  anon_sym_class = 11,
  anon_sym_trait = 12,
  anon_sym_LBRACK = 13,
  anon_sym_RBRACK = 14,
  anon_sym_PLUS = 15,
  anon_sym_DASH = 16,
  anon_sym_LT_COLON = 17,
  anon_sym_GT_COLON = 18,
  anon_sym_LT_PERCENT = 19,
  anon_sym_COLON = 20,
  anon_sym_AT = 21,
  anon_sym_val = 22,
  anon_sym_EQ = 23,
  anon_sym_var = 24,
  anon_sym_type = 25,
  anon_sym_def = 26,
  anon_sym_abstract = 27,
  anon_sym_final = 28,
  anon_sym_sealed = 29,
  anon_sym_implicit = 30,
  anon_sym_lazy = 31,
  anon_sym_override = 32,
  anon_sym_private = 33,
  anon_sym_protected = 34,
  anon_sym_extends = 35,
  anon_sym_LPAREN = 36,
  anon_sym_RPAREN = 37,
  anon_sym_with = 38,
  anon_sym_POUND = 39,
  anon_sym_STAR = 40,
  anon_sym_PIPE = 41,
  anon_sym_if = 42,
  anon_sym_else = 43,
  anon_sym_match = 44,
  anon_sym_try = 45,
  anon_sym_catch = 46,
  anon_sym_finally = 47,
  anon_sym_new = 48,
  anon_sym_BANG = 49,
  anon_sym_TILDE = 50,
  sym_wildcard = 51,
  sym_operator_identifier = 52,
  sym_number = 53,
  anon_sym_DOLLAR = 54,
  anon_sym_SEMI = 55,
  sym_comment = 56,
  sym__automatic_semicolon = 57,
  sym__simple_string = 58,
  sym__string_start = 59,
  sym__string_middle = 60,
  sym__string_end = 61,
  sym__multiline_string_start = 62,
  sym__multiline_string_middle = 63,
  sym__multiline_string_end = 64,
  sym_compilation_unit = 65,
  sym_package_clause = 66,
  sym_package_identifier = 67,
  sym_package_object = 68,
  sym_import_declaration = 69,
  sym__import_expression = 70,
  sym_import_selectors = 71,
  sym_renamed_identifier = 72,
  sym_object_definition = 73,
  sym__object_definition = 74,
  sym_class_definition = 75,
  sym_trait_definition = 76,
  sym_type_parameters = 77,
  sym__variant_type_parameter = 78,
  sym_covariant_type_parameter = 79,
  sym_contravariant_type_parameter = 80,
  sym__type_parameter = 81,
  sym_upper_bound = 82,
  sym_lower_bound = 83,
  sym_view_bound = 84,
  sym_context_bound = 85,
  sym_template_body = 86,
  sym_annotation = 87,
  sym_val_definition = 88,
  sym_val_declaration = 89,
  sym_var_declaration = 90,
  sym_var_definition = 91,
  sym_type_definition = 92,
  sym_function_definition = 93,
  sym_function_declaration = 94,
  sym_modifiers = 95,
  sym_extends_clause = 96,
  sym_class_parameters = 97,
  sym_parameters = 98,
  sym_class_parameter = 99,
  sym_parameter = 100,
  sym__block = 101,
  sym_block = 102,
  sym__type = 103,
  sym__annotated_type = 104,
  sym__simple_type = 105,
  sym_compound_type = 106,
  sym_infix_type = 107,
  sym_tuple_type = 108,
  sym_stable_type_identifier = 109,
  sym_stable_identifier = 110,
  sym_generic_type = 111,
  sym_projected_type = 112,
  sym_function_type = 113,
  sym_parameter_types = 114,
  sym_lazy_parameter_type = 115,
  sym_repeated_parameter_type = 116,
  sym_case_class_pattern = 117,
  sym_infix_pattern = 118,
  sym_capture_pattern = 119,
  sym_typed_pattern = 120,
  sym_alternative_pattern = 121,
  sym_tuple_pattern = 122,
  sym__expression = 123,
  sym_if_expression = 124,
  sym_match_expression = 125,
  sym_try_expression = 126,
  sym_catch_clause = 127,
  sym_finally_clause = 128,
  sym_case_block = 129,
  sym_case_clause = 130,
  sym_guard = 131,
  sym_assignment_expression = 132,
  sym_generic_function = 133,
  sym_call_expression = 134,
  sym_field_expression = 135,
  sym_instance_expression = 136,
  sym_infix_expression = 137,
  sym_prefix_expression = 138,
  sym_tuple_expression = 139,
  sym_parenthesized_expression = 140,
  sym_type_arguments = 141,
  sym_arguments = 142,
  sym_string_transform_expression = 143,
  sym_string = 144,
  sym_interpolation = 145,
  aux_sym_compilation_unit_repeat1 = 146,
  aux_sym_package_identifier_repeat1 = 147,
  aux_sym_import_declaration_repeat1 = 148,
  aux_sym_import_selectors_repeat1 = 149,
  aux_sym_class_definition_repeat1 = 150,
  aux_sym_class_definition_repeat2 = 151,
  aux_sym_type_parameters_repeat1 = 152,
  aux_sym__type_parameter_repeat1 = 153,
  aux_sym__type_parameter_repeat2 = 154,
  aux_sym_annotation_repeat1 = 155,
  aux_sym_val_declaration_repeat1 = 156,
  aux_sym_function_definition_repeat1 = 157,
  aux_sym_modifiers_repeat1 = 158,
  aux_sym_class_parameters_repeat1 = 159,
  aux_sym_parameters_repeat1 = 160,
  aux_sym__block_repeat1 = 161,
  aux_sym_compound_type_repeat1 = 162,
  aux_sym_tuple_type_repeat1 = 163,
  aux_sym_parameter_types_repeat1 = 164,
  aux_sym_case_class_pattern_repeat1 = 165,
  aux_sym_case_block_repeat1 = 166,
  aux_sym_tuple_expression_repeat1 = 167,
  aux_sym_string_repeat1 = 168,
  aux_sym_string_repeat2 = 169,
  alias_sym_type_identifier = 170,
};

static const char *ts_symbol_names[] = {
  [ts_builtin_sym_end] = "end",
  [sym_identifier] = "identifier",
  [anon_sym_package] = "package",
  [anon_sym_DOT] = ".",
  [anon_sym_object] = "object",
  [anon_sym_import] = "import",
  [anon_sym_COMMA] = ",",
  [anon_sym_LBRACE] = "{",
  [anon_sym_RBRACE] = "}",
  [anon_sym_EQ_GT] = "=>",
  [anon_sym_case] = "case",
  [anon_sym_class] = "class",
  [anon_sym_trait] = "trait",
  [anon_sym_LBRACK] = "[",
  [anon_sym_RBRACK] = "]",
  [anon_sym_PLUS] = "+",
  [anon_sym_DASH] = "-",
  [anon_sym_LT_COLON] = "<:",
  [anon_sym_GT_COLON] = ">:",
  [anon_sym_LT_PERCENT] = "<%",
  [anon_sym_COLON] = ":",
  [anon_sym_AT] = "@",
  [anon_sym_val] = "val",
  [anon_sym_EQ] = "=",
  [anon_sym_var] = "var",
  [anon_sym_type] = "type",
  [anon_sym_def] = "def",
  [anon_sym_abstract] = "abstract",
  [anon_sym_final] = "final",
  [anon_sym_sealed] = "sealed",
  [anon_sym_implicit] = "implicit",
  [anon_sym_lazy] = "lazy",
  [anon_sym_override] = "override",
  [anon_sym_private] = "private",
  [anon_sym_protected] = "protected",
  [anon_sym_extends] = "extends",
  [anon_sym_LPAREN] = "(",
  [anon_sym_RPAREN] = ")",
  [anon_sym_with] = "with",
  [anon_sym_POUND] = "#",
  [anon_sym_STAR] = "*",
  [anon_sym_PIPE] = "|",
  [anon_sym_if] = "if",
  [anon_sym_else] = "else",
  [anon_sym_match] = "match",
  [anon_sym_try] = "try",
  [anon_sym_catch] = "catch",
  [anon_sym_finally] = "finally",
  [anon_sym_new] = "new",
  [anon_sym_BANG] = "!",
  [anon_sym_TILDE] = "~",
  [sym_wildcard] = "wildcard",
  [sym_operator_identifier] = "operator_identifier",
  [sym_number] = "number",
  [anon_sym_DOLLAR] = "$",
  [anon_sym_SEMI] = ";",
  [sym_comment] = "comment",
  [sym__automatic_semicolon] = "_automatic_semicolon",
  [sym__simple_string] = "_simple_string",
  [sym__string_start] = "_string_start",
  [sym__string_middle] = "_string_middle",
  [sym__string_end] = "_string_end",
  [sym__multiline_string_start] = "_multiline_string_start",
  [sym__multiline_string_middle] = "_multiline_string_middle",
  [sym__multiline_string_end] = "_multiline_string_end",
  [sym_compilation_unit] = "compilation_unit",
  [sym_package_clause] = "package_clause",
  [sym_package_identifier] = "package_identifier",
  [sym_package_object] = "package_object",
  [sym_import_declaration] = "import_declaration",
  [sym__import_expression] = "_import_expression",
  [sym_import_selectors] = "import_selectors",
  [sym_renamed_identifier] = "renamed_identifier",
  [sym_object_definition] = "object_definition",
  [sym__object_definition] = "_object_definition",
  [sym_class_definition] = "class_definition",
  [sym_trait_definition] = "trait_definition",
  [sym_type_parameters] = "type_parameters",
  [sym__variant_type_parameter] = "_variant_type_parameter",
  [sym_covariant_type_parameter] = "covariant_type_parameter",
  [sym_contravariant_type_parameter] = "contravariant_type_parameter",
  [sym__type_parameter] = "_type_parameter",
  [sym_upper_bound] = "upper_bound",
  [sym_lower_bound] = "lower_bound",
  [sym_view_bound] = "view_bound",
  [sym_context_bound] = "context_bound",
  [sym_template_body] = "template_body",
  [sym_annotation] = "annotation",
  [sym_val_definition] = "val_definition",
  [sym_val_declaration] = "val_declaration",
  [sym_var_declaration] = "var_declaration",
  [sym_var_definition] = "var_definition",
  [sym_type_definition] = "type_definition",
  [sym_function_definition] = "function_definition",
  [sym_function_declaration] = "function_declaration",
  [sym_modifiers] = "modifiers",
  [sym_extends_clause] = "extends_clause",
  [sym_class_parameters] = "class_parameters",
  [sym_parameters] = "parameters",
  [sym_class_parameter] = "class_parameter",
  [sym_parameter] = "parameter",
  [sym__block] = "_block",
  [sym_block] = "block",
  [sym__type] = "_type",
  [sym__annotated_type] = "_annotated_type",
  [sym__simple_type] = "_simple_type",
  [sym_compound_type] = "compound_type",
  [sym_infix_type] = "infix_type",
  [sym_tuple_type] = "tuple_type",
  [sym_stable_type_identifier] = "stable_type_identifier",
  [sym_stable_identifier] = "stable_identifier",
  [sym_generic_type] = "generic_type",
  [sym_projected_type] = "projected_type",
  [sym_function_type] = "function_type",
  [sym_parameter_types] = "parameter_types",
  [sym_lazy_parameter_type] = "lazy_parameter_type",
  [sym_repeated_parameter_type] = "repeated_parameter_type",
  [sym_case_class_pattern] = "case_class_pattern",
  [sym_infix_pattern] = "infix_pattern",
  [sym_capture_pattern] = "capture_pattern",
  [sym_typed_pattern] = "typed_pattern",
  [sym_alternative_pattern] = "alternative_pattern",
  [sym_tuple_pattern] = "tuple_pattern",
  [sym__expression] = "_expression",
  [sym_if_expression] = "if_expression",
  [sym_match_expression] = "match_expression",
  [sym_try_expression] = "try_expression",
  [sym_catch_clause] = "catch_clause",
  [sym_finally_clause] = "finally_clause",
  [sym_case_block] = "case_block",
  [sym_case_clause] = "case_clause",
  [sym_guard] = "guard",
  [sym_assignment_expression] = "assignment_expression",
  [sym_generic_function] = "generic_function",
  [sym_call_expression] = "call_expression",
  [sym_field_expression] = "field_expression",
  [sym_instance_expression] = "instance_expression",
  [sym_infix_expression] = "infix_expression",
  [sym_prefix_expression] = "prefix_expression",
  [sym_tuple_expression] = "tuple_expression",
  [sym_parenthesized_expression] = "parenthesized_expression",
  [sym_type_arguments] = "type_arguments",
  [sym_arguments] = "arguments",
  [sym_string_transform_expression] = "string_transform_expression",
  [sym_string] = "string",
  [sym_interpolation] = "interpolation",
  [aux_sym_compilation_unit_repeat1] = "compilation_unit_repeat1",
  [aux_sym_package_identifier_repeat1] = "package_identifier_repeat1",
  [aux_sym_import_declaration_repeat1] = "import_declaration_repeat1",
  [aux_sym_import_selectors_repeat1] = "import_selectors_repeat1",
  [aux_sym_class_definition_repeat1] = "class_definition_repeat1",
  [aux_sym_class_definition_repeat2] = "class_definition_repeat2",
  [aux_sym_type_parameters_repeat1] = "type_parameters_repeat1",
  [aux_sym__type_parameter_repeat1] = "_type_parameter_repeat1",
  [aux_sym__type_parameter_repeat2] = "_type_parameter_repeat2",
  [aux_sym_annotation_repeat1] = "annotation_repeat1",
  [aux_sym_val_declaration_repeat1] = "val_declaration_repeat1",
  [aux_sym_function_definition_repeat1] = "function_definition_repeat1",
  [aux_sym_modifiers_repeat1] = "modifiers_repeat1",
  [aux_sym_class_parameters_repeat1] = "class_parameters_repeat1",
  [aux_sym_parameters_repeat1] = "parameters_repeat1",
  [aux_sym__block_repeat1] = "_block_repeat1",
  [aux_sym_compound_type_repeat1] = "compound_type_repeat1",
  [aux_sym_tuple_type_repeat1] = "tuple_type_repeat1",
  [aux_sym_parameter_types_repeat1] = "parameter_types_repeat1",
  [aux_sym_case_class_pattern_repeat1] = "case_class_pattern_repeat1",
  [aux_sym_case_block_repeat1] = "case_block_repeat1",
  [aux_sym_tuple_expression_repeat1] = "tuple_expression_repeat1",
  [aux_sym_string_repeat1] = "string_repeat1",
  [aux_sym_string_repeat2] = "string_repeat2",
  [alias_sym_type_identifier] = "type_identifier",
};

static TSSymbol ts_symbol_map[] = {
  [ts_builtin_sym_end] = ts_builtin_sym_end,
  [sym_identifier] = sym_identifier,
  [anon_sym_package] = anon_sym_package,
  [anon_sym_DOT] = anon_sym_DOT,
  [anon_sym_object] = anon_sym_object,
  [anon_sym_import] = anon_sym_import,
  [anon_sym_COMMA] = anon_sym_COMMA,
  [anon_sym_LBRACE] = anon_sym_LBRACE,
  [anon_sym_RBRACE] = anon_sym_RBRACE,
  [anon_sym_EQ_GT] = anon_sym_EQ_GT,
  [anon_sym_case] = anon_sym_case,
  [anon_sym_class] = anon_sym_class,
  [anon_sym_trait] = anon_sym_trait,
  [anon_sym_LBRACK] = anon_sym_LBRACK,
  [anon_sym_RBRACK] = anon_sym_RBRACK,
  [anon_sym_PLUS] = anon_sym_PLUS,
  [anon_sym_DASH] = anon_sym_DASH,
  [anon_sym_LT_COLON] = anon_sym_LT_COLON,
  [anon_sym_GT_COLON] = anon_sym_GT_COLON,
  [anon_sym_LT_PERCENT] = anon_sym_LT_PERCENT,
  [anon_sym_COLON] = anon_sym_COLON,
  [anon_sym_AT] = anon_sym_AT,
  [anon_sym_val] = anon_sym_val,
  [anon_sym_EQ] = anon_sym_EQ,
  [anon_sym_var] = anon_sym_var,
  [anon_sym_type] = anon_sym_type,
  [anon_sym_def] = anon_sym_def,
  [anon_sym_abstract] = anon_sym_abstract,
  [anon_sym_final] = anon_sym_final,
  [anon_sym_sealed] = anon_sym_sealed,
  [anon_sym_implicit] = anon_sym_implicit,
  [anon_sym_lazy] = anon_sym_lazy,
  [anon_sym_override] = anon_sym_override,
  [anon_sym_private] = anon_sym_private,
  [anon_sym_protected] = anon_sym_protected,
  [anon_sym_extends] = anon_sym_extends,
  [anon_sym_LPAREN] = anon_sym_LPAREN,
  [anon_sym_RPAREN] = anon_sym_RPAREN,
  [anon_sym_with] = anon_sym_with,
  [anon_sym_POUND] = anon_sym_POUND,
  [anon_sym_STAR] = anon_sym_STAR,
  [anon_sym_PIPE] = anon_sym_PIPE,
  [anon_sym_if] = anon_sym_if,
  [anon_sym_else] = anon_sym_else,
  [anon_sym_match] = anon_sym_match,
  [anon_sym_try] = anon_sym_try,
  [anon_sym_catch] = anon_sym_catch,
  [anon_sym_finally] = anon_sym_finally,
  [anon_sym_new] = anon_sym_new,
  [anon_sym_BANG] = anon_sym_BANG,
  [anon_sym_TILDE] = anon_sym_TILDE,
  [sym_wildcard] = sym_wildcard,
  [sym_operator_identifier] = sym_operator_identifier,
  [sym_number] = sym_number,
  [anon_sym_DOLLAR] = anon_sym_DOLLAR,
  [anon_sym_SEMI] = anon_sym_SEMI,
  [sym_comment] = sym_comment,
  [sym__automatic_semicolon] = sym__automatic_semicolon,
  [sym__simple_string] = sym__simple_string,
  [sym__string_start] = sym__string_start,
  [sym__string_middle] = sym__string_middle,
  [sym__string_end] = sym__string_end,
  [sym__multiline_string_start] = sym__multiline_string_start,
  [sym__multiline_string_middle] = sym__multiline_string_middle,
  [sym__multiline_string_end] = sym__multiline_string_end,
  [sym_compilation_unit] = sym_compilation_unit,
  [sym_package_clause] = sym_package_clause,
  [sym_package_identifier] = sym_package_identifier,
  [sym_package_object] = sym_package_object,
  [sym_import_declaration] = sym_import_declaration,
  [sym__import_expression] = sym__import_expression,
  [sym_import_selectors] = sym_import_selectors,
  [sym_renamed_identifier] = sym_renamed_identifier,
  [sym_object_definition] = sym_object_definition,
  [sym__object_definition] = sym__object_definition,
  [sym_class_definition] = sym_class_definition,
  [sym_trait_definition] = sym_trait_definition,
  [sym_type_parameters] = sym_type_parameters,
  [sym__variant_type_parameter] = sym__variant_type_parameter,
  [sym_covariant_type_parameter] = sym_covariant_type_parameter,
  [sym_contravariant_type_parameter] = sym_contravariant_type_parameter,
  [sym__type_parameter] = sym__type_parameter,
  [sym_upper_bound] = sym_upper_bound,
  [sym_lower_bound] = sym_lower_bound,
  [sym_view_bound] = sym_view_bound,
  [sym_context_bound] = sym_context_bound,
  [sym_template_body] = sym_template_body,
  [sym_annotation] = sym_annotation,
  [sym_val_definition] = sym_val_definition,
  [sym_val_declaration] = sym_val_declaration,
  [sym_var_declaration] = sym_var_declaration,
  [sym_var_definition] = sym_var_definition,
  [sym_type_definition] = sym_type_definition,
  [sym_function_definition] = sym_function_definition,
  [sym_function_declaration] = sym_function_declaration,
  [sym_modifiers] = sym_modifiers,
  [sym_extends_clause] = sym_extends_clause,
  [sym_class_parameters] = sym_class_parameters,
  [sym_parameters] = sym_parameters,
  [sym_class_parameter] = sym_class_parameter,
  [sym_parameter] = sym_parameter,
  [sym__block] = sym__block,
  [sym_block] = sym_block,
  [sym__type] = sym__type,
  [sym__annotated_type] = sym__annotated_type,
  [sym__simple_type] = sym__simple_type,
  [sym_compound_type] = sym_compound_type,
  [sym_infix_type] = sym_infix_type,
  [sym_tuple_type] = sym_tuple_type,
  [sym_stable_type_identifier] = sym_stable_type_identifier,
  [sym_stable_identifier] = sym_stable_identifier,
  [sym_generic_type] = sym_generic_type,
  [sym_projected_type] = sym_projected_type,
  [sym_function_type] = sym_function_type,
  [sym_parameter_types] = sym_parameter_types,
  [sym_lazy_parameter_type] = sym_lazy_parameter_type,
  [sym_repeated_parameter_type] = sym_repeated_parameter_type,
  [sym_case_class_pattern] = sym_case_class_pattern,
  [sym_infix_pattern] = sym_infix_pattern,
  [sym_capture_pattern] = sym_capture_pattern,
  [sym_typed_pattern] = sym_typed_pattern,
  [sym_alternative_pattern] = sym_alternative_pattern,
  [sym_tuple_pattern] = sym_tuple_pattern,
  [sym__expression] = sym__expression,
  [sym_if_expression] = sym_if_expression,
  [sym_match_expression] = sym_match_expression,
  [sym_try_expression] = sym_try_expression,
  [sym_catch_clause] = sym_catch_clause,
  [sym_finally_clause] = sym_finally_clause,
  [sym_case_block] = sym_case_block,
  [sym_case_clause] = sym_case_clause,
  [sym_guard] = sym_guard,
  [sym_assignment_expression] = sym_assignment_expression,
  [sym_generic_function] = sym_generic_function,
  [sym_call_expression] = sym_call_expression,
  [sym_field_expression] = sym_field_expression,
  [sym_instance_expression] = sym_instance_expression,
  [sym_infix_expression] = sym_infix_expression,
  [sym_prefix_expression] = sym_prefix_expression,
  [sym_tuple_expression] = sym_tuple_expression,
  [sym_parenthesized_expression] = sym_parenthesized_expression,
  [sym_type_arguments] = sym_type_arguments,
  [sym_arguments] = sym_arguments,
  [sym_string_transform_expression] = sym_string_transform_expression,
  [sym_string] = sym_string,
  [sym_interpolation] = sym_interpolation,
  [aux_sym_compilation_unit_repeat1] = aux_sym_compilation_unit_repeat1,
  [aux_sym_package_identifier_repeat1] = aux_sym_package_identifier_repeat1,
  [aux_sym_import_declaration_repeat1] = aux_sym_import_declaration_repeat1,
  [aux_sym_import_selectors_repeat1] = aux_sym_import_selectors_repeat1,
  [aux_sym_class_definition_repeat1] = aux_sym_class_definition_repeat1,
  [aux_sym_class_definition_repeat2] = aux_sym_class_definition_repeat2,
  [aux_sym_type_parameters_repeat1] = aux_sym_type_parameters_repeat1,
  [aux_sym__type_parameter_repeat1] = aux_sym__type_parameter_repeat1,
  [aux_sym__type_parameter_repeat2] = aux_sym__type_parameter_repeat2,
  [aux_sym_annotation_repeat1] = aux_sym_annotation_repeat1,
  [aux_sym_val_declaration_repeat1] = aux_sym_val_declaration_repeat1,
  [aux_sym_function_definition_repeat1] = aux_sym_function_definition_repeat1,
  [aux_sym_modifiers_repeat1] = aux_sym_modifiers_repeat1,
  [aux_sym_class_parameters_repeat1] = aux_sym_class_parameters_repeat1,
  [aux_sym_parameters_repeat1] = aux_sym_parameters_repeat1,
  [aux_sym__block_repeat1] = aux_sym__block_repeat1,
  [aux_sym_compound_type_repeat1] = aux_sym_compound_type_repeat1,
  [aux_sym_tuple_type_repeat1] = aux_sym_tuple_type_repeat1,
  [aux_sym_parameter_types_repeat1] = aux_sym_parameter_types_repeat1,
  [aux_sym_case_class_pattern_repeat1] = aux_sym_case_class_pattern_repeat1,
  [aux_sym_case_block_repeat1] = aux_sym_case_block_repeat1,
  [aux_sym_tuple_expression_repeat1] = aux_sym_tuple_expression_repeat1,
  [aux_sym_string_repeat1] = aux_sym_string_repeat1,
  [aux_sym_string_repeat2] = aux_sym_string_repeat2,
  [alias_sym_type_identifier] = alias_sym_type_identifier,
};

static const TSSymbolMetadata ts_symbol_metadata[] = {
  [ts_builtin_sym_end] = {
    .visible = false,
    .named = true,
  },
  [sym_identifier] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_package] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_object] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_import] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COMMA] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_case] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_class] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_trait] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_PERCENT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_val] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_var] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_type] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_def] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_abstract] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_final] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_sealed] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_implicit] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_lazy] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_override] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_private] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_protected] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_extends] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_with] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUND] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_if] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_else] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_match] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_try] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_catch] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_finally] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_new] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_TILDE] = {
    .visible = true,
    .named = false,
  },
  [sym_wildcard] = {
    .visible = true,
    .named = true,
  },
  [sym_operator_identifier] = {
    .visible = true,
    .named = true,
  },
  [sym_number] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_DOLLAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SEMI] = {
    .visible = true,
    .named = false,
  },
  [sym_comment] = {
    .visible = true,
    .named = true,
  },
  [sym__automatic_semicolon] = {
    .visible = false,
    .named = true,
  },
  [sym__simple_string] = {
    .visible = false,
    .named = true,
  },
  [sym__string_start] = {
    .visible = false,
    .named = true,
  },
  [sym__string_middle] = {
    .visible = false,
    .named = true,
  },
  [sym__string_end] = {
    .visible = false,
    .named = true,
  },
  [sym__multiline_string_start] = {
    .visible = false,
    .named = true,
  },
  [sym__multiline_string_middle] = {
    .visible = false,
    .named = true,
  },
  [sym__multiline_string_end] = {
    .visible = false,
    .named = true,
  },
  [sym_compilation_unit] = {
    .visible = true,
    .named = true,
  },
  [sym_package_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_package_identifier] = {
    .visible = true,
    .named = true,
  },
  [sym_package_object] = {
    .visible = true,
    .named = true,
  },
  [sym_import_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym__import_expression] = {
    .visible = false,
    .named = true,
  },
  [sym_import_selectors] = {
    .visible = true,
    .named = true,
  },
  [sym_renamed_identifier] = {
    .visible = true,
    .named = true,
  },
  [sym_object_definition] = {
    .visible = true,
    .named = true,
  },
  [sym__object_definition] = {
    .visible = false,
    .named = true,
  },
  [sym_class_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_trait_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_type_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym__variant_type_parameter] = {
    .visible = false,
    .named = true,
  },
  [sym_covariant_type_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_contravariant_type_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym__type_parameter] = {
    .visible = false,
    .named = true,
  },
  [sym_upper_bound] = {
    .visible = true,
    .named = true,
  },
  [sym_lower_bound] = {
    .visible = true,
    .named = true,
  },
  [sym_view_bound] = {
    .visible = true,
    .named = true,
  },
  [sym_context_bound] = {
    .visible = true,
    .named = true,
  },
  [sym_template_body] = {
    .visible = true,
    .named = true,
  },
  [sym_annotation] = {
    .visible = true,
    .named = true,
  },
  [sym_val_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_val_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_var_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_var_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_type_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_function_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_function_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_modifiers] = {
    .visible = true,
    .named = true,
  },
  [sym_extends_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_class_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_class_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym__block] = {
    .visible = false,
    .named = true,
  },
  [sym_block] = {
    .visible = true,
    .named = true,
  },
  [sym__type] = {
    .visible = false,
    .named = true,
  },
  [sym__annotated_type] = {
    .visible = false,
    .named = true,
  },
  [sym__simple_type] = {
    .visible = false,
    .named = true,
  },
  [sym_compound_type] = {
    .visible = true,
    .named = true,
  },
  [sym_infix_type] = {
    .visible = true,
    .named = true,
  },
  [sym_tuple_type] = {
    .visible = true,
    .named = true,
  },
  [sym_stable_type_identifier] = {
    .visible = true,
    .named = true,
  },
  [sym_stable_identifier] = {
    .visible = true,
    .named = true,
  },
  [sym_generic_type] = {
    .visible = true,
    .named = true,
  },
  [sym_projected_type] = {
    .visible = true,
    .named = true,
  },
  [sym_function_type] = {
    .visible = true,
    .named = true,
  },
  [sym_parameter_types] = {
    .visible = true,
    .named = true,
  },
  [sym_lazy_parameter_type] = {
    .visible = true,
    .named = true,
  },
  [sym_repeated_parameter_type] = {
    .visible = true,
    .named = true,
  },
  [sym_case_class_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_infix_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_capture_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_typed_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_alternative_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_tuple_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym__expression] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_if_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_match_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_try_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_catch_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_finally_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_case_block] = {
    .visible = true,
    .named = true,
  },
  [sym_case_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_guard] = {
    .visible = true,
    .named = true,
  },
  [sym_assignment_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_generic_function] = {
    .visible = true,
    .named = true,
  },
  [sym_call_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_field_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_instance_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_infix_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_prefix_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_tuple_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_parenthesized_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_type_arguments] = {
    .visible = true,
    .named = true,
  },
  [sym_arguments] = {
    .visible = true,
    .named = true,
  },
  [sym_string_transform_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_string] = {
    .visible = true,
    .named = true,
  },
  [sym_interpolation] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_compilation_unit_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_package_identifier_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_import_declaration_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_import_selectors_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_class_definition_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_class_definition_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_type_parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__type_parameter_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__type_parameter_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_annotation_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_val_declaration_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_function_definition_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_modifiers_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_class_parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__block_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_compound_type_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_tuple_type_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_parameter_types_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_case_class_pattern_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_case_block_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_tuple_expression_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_string_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_string_repeat2] = {
    .visible = false,
    .named = false,
  },
  [alias_sym_type_identifier] = {
    .visible = true,
    .named = true,
  },
};

enum {
  field_alias = 1,
  field_alternative = 2,
  field_arguments = 3,
  field_base = 4,
  field_body = 5,
  field_bound = 6,
  field_class_parameters = 7,
  field_condition = 8,
  field_consequence = 9,
  field_default_value = 10,
  field_extend = 11,
  field_extra = 12,
  field_field = 13,
  field_function = 14,
  field_left = 15,
  field_name = 16,
  field_operator = 17,
  field_parameter_types = 18,
  field_parameters = 19,
  field_path = 20,
  field_pattern = 21,
  field_return_type = 22,
  field_right = 23,
  field_selector = 24,
  field_type = 25,
  field_type_arguments = 26,
  field_type_parameters = 27,
  field_value = 28,
};

static const char *ts_field_names[] = {
  [0] = NULL,
  [field_alias] = "alias",
  [field_alternative] = "alternative",
  [field_arguments] = "arguments",
  [field_base] = "base",
  [field_body] = "body",
  [field_bound] = "bound",
  [field_class_parameters] = "class_parameters",
  [field_condition] = "condition",
  [field_consequence] = "consequence",
  [field_default_value] = "default_value",
  [field_extend] = "extend",
  [field_extra] = "extra",
  [field_field] = "field",
  [field_function] = "function",
  [field_left] = "left",
  [field_name] = "name",
  [field_operator] = "operator",
  [field_parameter_types] = "parameter_types",
  [field_parameters] = "parameters",
  [field_path] = "path",
  [field_pattern] = "pattern",
  [field_return_type] = "return_type",
  [field_right] = "right",
  [field_selector] = "selector",
  [field_type] = "type",
  [field_type_arguments] = "type_arguments",
  [field_type_parameters] = "type_parameters",
  [field_value] = "value",
};

static const TSFieldMapSlice ts_field_map_slices[PRODUCTION_ID_COUNT] = {
  [1] = {.index = 0, .length = 1},
  [2] = {.index = 1, .length = 1},
  [3] = {.index = 2, .length = 3},
  [4] = {.index = 5, .length = 1},
  [5] = {.index = 6, .length = 1},
  [7] = {.index = 7, .length = 3},
  [8] = {.index = 10, .length = 2},
  [9] = {.index = 12, .length = 2},
  [10] = {.index = 14, .length = 2},
  [11] = {.index = 16, .length = 2},
  [12] = {.index = 18, .length = 1},
  [13] = {.index = 19, .length = 2},
  [14] = {.index = 21, .length = 2},
  [15] = {.index = 23, .length = 2},
  [16] = {.index = 25, .length = 2},
  [17] = {.index = 27, .length = 2},
  [18] = {.index = 29, .length = 2},
  [19] = {.index = 31, .length = 1},
  [20] = {.index = 32, .length = 3},
  [21] = {.index = 35, .length = 2},
  [22] = {.index = 37, .length = 2},
  [23] = {.index = 39, .length = 2},
  [24] = {.index = 41, .length = 2},
  [25] = {.index = 43, .length = 2},
  [26] = {.index = 45, .length = 3},
  [27] = {.index = 48, .length = 3},
  [28] = {.index = 51, .length = 3},
  [29] = {.index = 54, .length = 3},
  [30] = {.index = 57, .length = 3},
  [31] = {.index = 60, .length = 3},
  [32] = {.index = 63, .length = 2},
  [34] = {.index = 65, .length = 2},
  [35] = {.index = 67, .length = 2},
  [36] = {.index = 69, .length = 2},
  [37] = {.index = 71, .length = 2},
  [38] = {.index = 73, .length = 2},
  [39] = {.index = 75, .length = 1},
  [40] = {.index = 76, .length = 3},
  [41] = {.index = 79, .length = 2},
  [42] = {.index = 75, .length = 1},
  [43] = {.index = 69, .length = 2},
  [44] = {.index = 81, .length = 2},
  [45] = {.index = 83, .length = 2},
  [46] = {.index = 85, .length = 3},
  [47] = {.index = 88, .length = 3},
  [48] = {.index = 91, .length = 1},
  [49] = {.index = 92, .length = 2},
  [50] = {.index = 94, .length = 1},
  [51] = {.index = 95, .length = 2},
  [52] = {.index = 97, .length = 2},
  [53] = {.index = 99, .length = 3},
  [54] = {.index = 102, .length = 3},
  [55] = {.index = 105, .length = 3},
  [56] = {.index = 108, .length = 3},
  [57] = {.index = 111, .length = 3},
  [58] = {.index = 114, .length = 3},
  [59] = {.index = 117, .length = 3},
  [60] = {.index = 120, .length = 2},
  [61] = {.index = 122, .length = 2},
  [62] = {.index = 124, .length = 4},
  [63] = {.index = 128, .length = 4},
  [64] = {.index = 132, .length = 4},
  [65] = {.index = 136, .length = 4},
  [66] = {.index = 140, .length = 1},
  [67] = {.index = 141, .length = 2},
  [68] = {.index = 143, .length = 2},
  [69] = {.index = 145, .length = 2},
  [70] = {.index = 147, .length = 3},
  [71] = {.index = 145, .length = 2},
  [72] = {.index = 150, .length = 3},
  [73] = {.index = 153, .length = 3},
  [74] = {.index = 156, .length = 3},
  [75] = {.index = 159, .length = 3},
  [76] = {.index = 162, .length = 4},
  [77] = {.index = 166, .length = 3},
  [78] = {.index = 169, .length = 3},
  [79] = {.index = 172, .length = 2},
  [80] = {.index = 174, .length = 2},
  [81] = {.index = 176, .length = 2},
  [82] = {.index = 178, .length = 2},
  [83] = {.index = 180, .length = 2},
  [84] = {.index = 182, .length = 2},
  [85] = {.index = 180, .length = 2},
  [86] = {.index = 184, .length = 2},
  [87] = {.index = 186, .length = 2},
  [88] = {.index = 188, .length = 3},
  [89] = {.index = 191, .length = 3},
  [90] = {.index = 194, .length = 1},
  [91] = {.index = 195, .length = 2},
  [92] = {.index = 197, .length = 2},
  [93] = {.index = 199, .length = 2},
  [94] = {.index = 201, .length = 2},
  [95] = {.index = 203, .length = 2},
  [96] = {.index = 205, .length = 3},
  [97] = {.index = 208, .length = 4},
  [98] = {.index = 212, .length = 4},
  [99] = {.index = 216, .length = 4},
  [100] = {.index = 220, .length = 4},
  [101] = {.index = 224, .length = 3},
  [102] = {.index = 227, .length = 3},
  [103] = {.index = 230, .length = 6},
  [104] = {.index = 236, .length = 6},
  [105] = {.index = 242, .length = 2},
  [106] = {.index = 244, .length = 2},
  [107] = {.index = 246, .length = 5},
  [108] = {.index = 251, .length = 3},
  [109] = {.index = 254, .length = 3},
  [110] = {.index = 254, .length = 3},
  [111] = {.index = 257, .length = 3},
  [112] = {.index = 260, .length = 4},
  [113] = {.index = 264, .length = 4},
  [114] = {.index = 268, .length = 4},
  [115] = {.index = 272, .length = 4},
  [116] = {.index = 276, .length = 3},
  [117] = {.index = 279, .length = 3},
  [118] = {.index = 282, .length = 3},
  [119] = {.index = 285, .length = 3},
  [120] = {.index = 288, .length = 3},
  [121] = {.index = 291, .length = 3},
  [122] = {.index = 294, .length = 3},
  [123] = {.index = 297, .length = 3},
  [124] = {.index = 300, .length = 3},
  [125] = {.index = 303, .length = 3},
  [126] = {.index = 306, .length = 3},
  [127] = {.index = 309, .length = 4},
  [128] = {.index = 313, .length = 3},
  [129] = {.index = 316, .length = 3},
  [130] = {.index = 319, .length = 2},
  [131] = {.index = 321, .length = 2},
  [132] = {.index = 323, .length = 2},
  [133] = {.index = 325, .length = 2},
  [134] = {.index = 327, .length = 2},
  [135] = {.index = 329, .length = 2},
  [136] = {.index = 327, .length = 2},
  [137] = {.index = 331, .length = 2},
  [138] = {.index = 333, .length = 2},
  [139] = {.index = 335, .length = 3},
  [140] = {.index = 338, .length = 3},
  [141] = {.index = 341, .length = 1},
  [142] = {.index = 342, .length = 2},
  [143] = {.index = 344, .length = 5},
  [144] = {.index = 349, .length = 4},
  [145] = {.index = 353, .length = 4},
  [146] = {.index = 357, .length = 2},
  [147] = {.index = 359, .length = 4},
  [148] = {.index = 363, .length = 5},
  [149] = {.index = 368, .length = 4},
  [150] = {.index = 372, .length = 4},
  [151] = {.index = 376, .length = 4},
  [152] = {.index = 380, .length = 4},
  [153] = {.index = 384, .length = 4},
  [154] = {.index = 388, .length = 3},
  [155] = {.index = 391, .length = 3},
  [156] = {.index = 394, .length = 4},
  [157] = {.index = 398, .length = 4},
  [158] = {.index = 402, .length = 4},
  [159] = {.index = 406, .length = 4},
  [160] = {.index = 410, .length = 3},
  [161] = {.index = 413, .length = 3},
  [162] = {.index = 416, .length = 3},
  [163] = {.index = 419, .length = 3},
  [164] = {.index = 422, .length = 3},
  [165] = {.index = 425, .length = 3},
  [166] = {.index = 428, .length = 3},
  [167] = {.index = 431, .length = 3},
  [168] = {.index = 434, .length = 3},
  [169] = {.index = 437, .length = 3},
  [170] = {.index = 440, .length = 3},
  [171] = {.index = 443, .length = 4},
  [172] = {.index = 447, .length = 3},
  [173] = {.index = 450, .length = 3},
  [174] = {.index = 453, .length = 2},
  [175] = {.index = 455, .length = 1},
  [176] = {.index = 456, .length = 3},
  [177] = {.index = 459, .length = 5},
  [178] = {.index = 464, .length = 5},
  [179] = {.index = 469, .length = 3},
  [180] = {.index = 472, .length = 2},
  [181] = {.index = 474, .length = 5},
  [182] = {.index = 479, .length = 5},
  [183] = {.index = 484, .length = 4},
  [184] = {.index = 488, .length = 5},
  [185] = {.index = 493, .length = 4},
  [186] = {.index = 497, .length = 4},
  [187] = {.index = 501, .length = 4},
  [188] = {.index = 505, .length = 4},
  [189] = {.index = 509, .length = 4},
  [190] = {.index = 513, .length = 3},
  [191] = {.index = 516, .length = 3},
  [192] = {.index = 519, .length = 4},
  [193] = {.index = 523, .length = 4},
  [194] = {.index = 527, .length = 4},
  [195] = {.index = 531, .length = 4},
  [196] = {.index = 535, .length = 2},
  [197] = {.index = 537, .length = 6},
  [198] = {.index = 543, .length = 3},
  [199] = {.index = 546, .length = 5},
  [200] = {.index = 551, .length = 5},
  [201] = {.index = 556, .length = 4},
  [202] = {.index = 560, .length = 5},
  [203] = {.index = 565, .length = 4},
  [204] = {.index = 569, .length = 3},
  [205] = {.index = 572, .length = 5},
};

static const TSFieldMapEntry ts_field_map_entries[] = {
  [0] =
    {field_name, 1},
  [1] =
    {field_name, 0},
  [2] =
    {field_body, 1, .inherited = true},
    {field_extend, 1, .inherited = true},
    {field_name, 1, .inherited = true},
  [5] =
    {field_path, 0},
  [6] =
    {field_path, 1, .inherited = true},
  [7] =
    {field_body, 2, .inherited = true},
    {field_extend, 2, .inherited = true},
    {field_name, 2, .inherited = true},
  [10] =
    {field_body, 2},
    {field_name, 1},
  [12] =
    {field_body, 1},
    {field_name, 0},
  [14] =
    {field_extend, 1},
    {field_name, 0},
  [16] =
    {field_path, 1, .inherited = true},
    {field_path, 2, .inherited = true},
  [18] =
    {field_name, 2},
  [19] =
    {field_name, 1},
    {field_type_parameters, 2},
  [21] =
    {field_extend, 2},
    {field_name, 1},
  [23] =
    {field_class_parameters, 2},
    {field_name, 1},
  [25] =
    {field_type, 0},
    {field_type_arguments, 1},
  [27] =
    {field_arguments, 2},
    {field_name, 1},
  [29] =
    {field_name, 1},
    {field_parameters, 2},
  [31] =
    {field_type, 1},
  [32] =
    {field_body, 2},
    {field_extend, 1},
    {field_name, 0},
  [35] =
    {field_path, 0, .inherited = true},
    {field_path, 1, .inherited = true},
  [37] =
    {field_name, 2},
    {field_type_parameters, 3},
  [39] =
    {field_body, 3},
    {field_name, 2},
  [41] =
    {field_extend, 3},
    {field_name, 2},
  [43] =
    {field_class_parameters, 3},
    {field_name, 2},
  [45] =
    {field_body, 3},
    {field_name, 1},
    {field_type_parameters, 2},
  [48] =
    {field_extend, 3},
    {field_name, 1},
    {field_type_parameters, 2},
  [51] =
    {field_class_parameters, 3},
    {field_name, 1},
    {field_type_parameters, 2},
  [54] =
    {field_body, 3},
    {field_extend, 2},
    {field_name, 1},
  [57] =
    {field_body, 3},
    {field_class_parameters, 2},
    {field_name, 1},
  [60] =
    {field_class_parameters, 2},
    {field_extend, 3},
    {field_name, 1},
  [63] =
    {field_base, 0},
    {field_extra, 1, .inherited = true},
  [65] =
    {field_selector, 2},
    {field_type, 0},
  [67] =
    {field_pattern, 0},
    {field_type, 2},
  [69] =
    {field_name, 1},
    {field_type, 3},
  [71] =
    {field_name, 0},
    {field_pattern, 2},
  [73] =
    {field_pattern, 1},
    {field_value, 3},
  [75] =
    {field_type, 0},
  [76] =
    {field_left, 0},
    {field_operator, 1},
    {field_right, 2},
  [79] =
    {field_name, 0, .inherited = true},
    {field_name, 1, .inherited = true},
  [81] =
    {field_name, 1},
    {field_return_type, 3},
  [83] =
    {field_body, 3},
    {field_name, 1},
  [85] =
    {field_name, 1},
    {field_parameters, 3},
    {field_type_parameters, 2},
  [88] =
    {field_body, 3},
    {field_name, 1},
    {field_parameters, 2},
  [91] =
    {field_name, 3},
  [92] =
    {field_name, 2},
    {field_parameters, 3},
  [94] =
    {field_body, 1},
  [95] =
    {field_function, 0},
    {field_type_arguments, 1},
  [97] =
    {field_arguments, 1},
    {field_function, 0},
  [99] =
    {field_body, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [102] =
    {field_extend, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [105] =
    {field_class_parameters, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [108] =
    {field_body, 4},
    {field_extend, 3},
    {field_name, 2},
  [111] =
    {field_body, 4},
    {field_class_parameters, 3},
    {field_name, 2},
  [114] =
    {field_class_parameters, 3},
    {field_extend, 4},
    {field_name, 2},
  [117] =
    {field_bound, 1, .inherited = true},
    {field_name, 1, .inherited = true},
    {field_type_parameters, 1, .inherited = true},
  [120] =
    {field_name, 0},
    {field_type_parameters, 1},
  [122] =
    {field_bound, 1},
    {field_name, 0},
  [124] =
    {field_body, 4},
    {field_extend, 3},
    {field_name, 1},
    {field_type_parameters, 2},
  [128] =
    {field_body, 4},
    {field_class_parameters, 3},
    {field_name, 1},
    {field_type_parameters, 2},
  [132] =
    {field_class_parameters, 3},
    {field_extend, 4},
    {field_name, 1},
    {field_type_parameters, 2},
  [136] =
    {field_body, 4},
    {field_class_parameters, 2},
    {field_extend, 3},
    {field_name, 1},
  [140] =
    {field_extra, 1},
  [141] =
    {field_extra, 0, .inherited = true},
    {field_extra, 1, .inherited = true},
  [143] =
    {field_parameter_types, 0},
    {field_return_type, 2},
  [145] =
    {field_pattern, 2},
    {field_type, 0},
  [147] =
    {field_name, 1},
    {field_name, 2, .inherited = true},
    {field_type, 4},
  [150] =
    {field_name, 1},
    {field_type, 4},
    {field_type_parameters, 2},
  [153] =
    {field_body, 4},
    {field_name, 1},
    {field_return_type, 3},
  [156] =
    {field_name, 1},
    {field_return_type, 4},
    {field_type_parameters, 2},
  [159] =
    {field_body, 4},
    {field_name, 1},
    {field_type_parameters, 2},
  [162] =
    {field_body, 4},
    {field_name, 1},
    {field_parameters, 3},
    {field_type_parameters, 2},
  [166] =
    {field_name, 1},
    {field_parameters, 2},
    {field_return_type, 4},
  [169] =
    {field_body, 4},
    {field_name, 1},
    {field_parameters, 2},
  [172] =
    {field_name, 3},
    {field_type_parameters, 4},
  [174] =
    {field_body, 4},
    {field_name, 3},
  [176] =
    {field_extend, 4},
    {field_name, 3},
  [178] =
    {field_class_parameters, 4},
    {field_name, 3},
  [180] =
    {field_name, 2},
    {field_type, 4},
  [182] =
    {field_pattern, 2},
    {field_value, 4},
  [184] =
    {field_name, 2},
    {field_return_type, 4},
  [186] =
    {field_body, 4},
    {field_name, 2},
  [188] =
    {field_name, 2},
    {field_parameters, 4},
    {field_type_parameters, 3},
  [191] =
    {field_body, 4},
    {field_name, 2},
    {field_parameters, 3},
  [194] =
    {field_name, 4},
  [195] =
    {field_name, 3},
    {field_parameters, 4},
  [197] =
    {field_condition, 1},
    {field_consequence, 2},
  [199] =
    {field_field, 2},
    {field_value, 0},
  [201] =
    {field_left, 0},
    {field_right, 2},
  [203] =
    {field_body, 2},
    {field_value, 0},
  [205] =
    {field_arguments, 1},
    {field_body, 2},
    {field_function, 0},
  [208] =
    {field_body, 5},
    {field_extend, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [212] =
    {field_body, 5},
    {field_class_parameters, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [216] =
    {field_class_parameters, 4},
    {field_extend, 5},
    {field_name, 2},
    {field_type_parameters, 3},
  [220] =
    {field_body, 5},
    {field_class_parameters, 3},
    {field_extend, 4},
    {field_name, 2},
  [224] =
    {field_bound, 2},
    {field_name, 0},
    {field_type_parameters, 1},
  [227] =
    {field_bound, 1},
    {field_bound, 2},
    {field_name, 0},
  [230] =
    {field_bound, 1, .inherited = true},
    {field_bound, 2, .inherited = true},
    {field_name, 1, .inherited = true},
    {field_name, 2, .inherited = true},
    {field_type_parameters, 1, .inherited = true},
    {field_type_parameters, 2, .inherited = true},
  [236] =
    {field_bound, 0, .inherited = true},
    {field_bound, 1, .inherited = true},
    {field_name, 0, .inherited = true},
    {field_name, 1, .inherited = true},
    {field_type_parameters, 0, .inherited = true},
    {field_type_parameters, 1, .inherited = true},
  [242] =
    {field_name, 0},
    {field_type, 2},
  [244] =
    {field_default_value, 2},
    {field_name, 0},
  [246] =
    {field_body, 5},
    {field_class_parameters, 3},
    {field_extend, 4},
    {field_name, 1},
    {field_type_parameters, 2},
  [251] =
    {field_pattern, 1},
    {field_type, 3},
    {field_value, 5},
  [254] =
    {field_pattern, 2},
    {field_pattern, 3},
    {field_type, 0},
  [257] =
    {field_body, 5},
    {field_name, 1},
    {field_return_type, 3},
  [260] =
    {field_body, 5},
    {field_name, 1},
    {field_return_type, 4},
    {field_type_parameters, 2},
  [264] =
    {field_name, 1},
    {field_parameters, 3},
    {field_return_type, 5},
    {field_type_parameters, 2},
  [268] =
    {field_body, 5},
    {field_name, 1},
    {field_parameters, 3},
    {field_type_parameters, 2},
  [272] =
    {field_body, 5},
    {field_name, 1},
    {field_parameters, 2},
    {field_return_type, 4},
  [276] =
    {field_body, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [279] =
    {field_extend, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [282] =
    {field_class_parameters, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [285] =
    {field_body, 5},
    {field_extend, 4},
    {field_name, 3},
  [288] =
    {field_body, 5},
    {field_class_parameters, 4},
    {field_name, 3},
  [291] =
    {field_class_parameters, 4},
    {field_extend, 5},
    {field_name, 3},
  [294] =
    {field_name, 2},
    {field_name, 3, .inherited = true},
    {field_type, 5},
  [297] =
    {field_name, 2},
    {field_type, 5},
    {field_type_parameters, 3},
  [300] =
    {field_body, 5},
    {field_name, 2},
    {field_return_type, 4},
  [303] =
    {field_name, 2},
    {field_return_type, 5},
    {field_type_parameters, 3},
  [306] =
    {field_body, 5},
    {field_name, 2},
    {field_type_parameters, 3},
  [309] =
    {field_body, 5},
    {field_name, 2},
    {field_parameters, 4},
    {field_type_parameters, 3},
  [313] =
    {field_name, 2},
    {field_parameters, 3},
    {field_return_type, 5},
  [316] =
    {field_body, 5},
    {field_name, 2},
    {field_parameters, 3},
  [319] =
    {field_name, 4},
    {field_type_parameters, 5},
  [321] =
    {field_body, 5},
    {field_name, 4},
  [323] =
    {field_extend, 5},
    {field_name, 4},
  [325] =
    {field_class_parameters, 5},
    {field_name, 4},
  [327] =
    {field_name, 3},
    {field_type, 5},
  [329] =
    {field_pattern, 3},
    {field_value, 5},
  [331] =
    {field_name, 3},
    {field_return_type, 5},
  [333] =
    {field_body, 5},
    {field_name, 3},
  [335] =
    {field_name, 3},
    {field_parameters, 5},
    {field_type_parameters, 4},
  [338] =
    {field_body, 5},
    {field_name, 3},
    {field_parameters, 4},
  [341] =
    {field_pattern, 1},
  [342] =
    {field_alias, 2},
    {field_name, 0},
  [344] =
    {field_body, 6},
    {field_class_parameters, 4},
    {field_extend, 5},
    {field_name, 2},
    {field_type_parameters, 3},
  [349] =
    {field_bound, 2},
    {field_bound, 3},
    {field_name, 0},
    {field_type_parameters, 1},
  [353] =
    {field_bound, 1},
    {field_bound, 2},
    {field_bound, 3},
    {field_name, 0},
  [357] =
    {field_default_value, 3},
    {field_name, 1},
  [359] =
    {field_body, 6},
    {field_name, 1},
    {field_return_type, 4},
    {field_type_parameters, 2},
  [363] =
    {field_body, 6},
    {field_name, 1},
    {field_parameters, 3},
    {field_return_type, 5},
    {field_type_parameters, 2},
  [368] =
    {field_body, 6},
    {field_name, 1},
    {field_parameters, 2},
    {field_return_type, 4},
  [372] =
    {field_body, 6},
    {field_extend, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [376] =
    {field_body, 6},
    {field_class_parameters, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [380] =
    {field_class_parameters, 5},
    {field_extend, 6},
    {field_name, 3},
    {field_type_parameters, 4},
  [384] =
    {field_body, 6},
    {field_class_parameters, 4},
    {field_extend, 5},
    {field_name, 3},
  [388] =
    {field_pattern, 2},
    {field_type, 4},
    {field_value, 6},
  [391] =
    {field_body, 6},
    {field_name, 2},
    {field_return_type, 4},
  [394] =
    {field_body, 6},
    {field_name, 2},
    {field_return_type, 5},
    {field_type_parameters, 3},
  [398] =
    {field_name, 2},
    {field_parameters, 4},
    {field_return_type, 6},
    {field_type_parameters, 3},
  [402] =
    {field_body, 6},
    {field_name, 2},
    {field_parameters, 4},
    {field_type_parameters, 3},
  [406] =
    {field_body, 6},
    {field_name, 2},
    {field_parameters, 3},
    {field_return_type, 5},
  [410] =
    {field_body, 6},
    {field_name, 4},
    {field_type_parameters, 5},
  [413] =
    {field_extend, 6},
    {field_name, 4},
    {field_type_parameters, 5},
  [416] =
    {field_class_parameters, 6},
    {field_name, 4},
    {field_type_parameters, 5},
  [419] =
    {field_body, 6},
    {field_extend, 5},
    {field_name, 4},
  [422] =
    {field_body, 6},
    {field_class_parameters, 5},
    {field_name, 4},
  [425] =
    {field_class_parameters, 5},
    {field_extend, 6},
    {field_name, 4},
  [428] =
    {field_name, 3},
    {field_name, 4, .inherited = true},
    {field_type, 6},
  [431] =
    {field_name, 3},
    {field_type, 6},
    {field_type_parameters, 4},
  [434] =
    {field_body, 6},
    {field_name, 3},
    {field_return_type, 5},
  [437] =
    {field_name, 3},
    {field_return_type, 6},
    {field_type_parameters, 4},
  [440] =
    {field_body, 6},
    {field_name, 3},
    {field_type_parameters, 4},
  [443] =
    {field_body, 6},
    {field_name, 3},
    {field_parameters, 5},
    {field_type_parameters, 4},
  [447] =
    {field_name, 3},
    {field_parameters, 4},
    {field_return_type, 6},
  [450] =
    {field_body, 6},
    {field_name, 3},
    {field_parameters, 4},
  [453] =
    {field_body, 3},
    {field_pattern, 1},
  [455] =
    {field_condition, 1},
  [456] =
    {field_alternative, 4},
    {field_condition, 1},
    {field_consequence, 2},
  [459] =
    {field_bound, 2},
    {field_bound, 3},
    {field_bound, 4},
    {field_name, 0},
    {field_type_parameters, 1},
  [464] =
    {field_bound, 1},
    {field_bound, 2},
    {field_bound, 3},
    {field_bound, 4},
    {field_name, 0},
  [469] =
    {field_default_value, 4},
    {field_name, 0},
    {field_type, 2},
  [472] =
    {field_default_value, 4},
    {field_name, 2},
  [474] =
    {field_body, 7},
    {field_name, 1},
    {field_parameters, 3},
    {field_return_type, 5},
    {field_type_parameters, 2},
  [479] =
    {field_body, 7},
    {field_class_parameters, 5},
    {field_extend, 6},
    {field_name, 3},
    {field_type_parameters, 4},
  [484] =
    {field_body, 7},
    {field_name, 2},
    {field_return_type, 5},
    {field_type_parameters, 3},
  [488] =
    {field_body, 7},
    {field_name, 2},
    {field_parameters, 4},
    {field_return_type, 6},
    {field_type_parameters, 3},
  [493] =
    {field_body, 7},
    {field_name, 2},
    {field_parameters, 3},
    {field_return_type, 5},
  [497] =
    {field_body, 7},
    {field_extend, 6},
    {field_name, 4},
    {field_type_parameters, 5},
  [501] =
    {field_body, 7},
    {field_class_parameters, 6},
    {field_name, 4},
    {field_type_parameters, 5},
  [505] =
    {field_class_parameters, 6},
    {field_extend, 7},
    {field_name, 4},
    {field_type_parameters, 5},
  [509] =
    {field_body, 7},
    {field_class_parameters, 5},
    {field_extend, 6},
    {field_name, 4},
  [513] =
    {field_pattern, 3},
    {field_type, 5},
    {field_value, 7},
  [516] =
    {field_body, 7},
    {field_name, 3},
    {field_return_type, 5},
  [519] =
    {field_body, 7},
    {field_name, 3},
    {field_return_type, 6},
    {field_type_parameters, 4},
  [523] =
    {field_name, 3},
    {field_parameters, 5},
    {field_return_type, 7},
    {field_type_parameters, 4},
  [527] =
    {field_body, 7},
    {field_name, 3},
    {field_parameters, 5},
    {field_type_parameters, 4},
  [531] =
    {field_body, 7},
    {field_name, 3},
    {field_parameters, 4},
    {field_return_type, 6},
  [535] =
    {field_body, 4},
    {field_pattern, 1},
  [537] =
    {field_bound, 2},
    {field_bound, 3},
    {field_bound, 4},
    {field_bound, 5},
    {field_name, 0},
    {field_type_parameters, 1},
  [543] =
    {field_default_value, 5},
    {field_name, 1},
    {field_type, 3},
  [546] =
    {field_body, 8},
    {field_name, 2},
    {field_parameters, 4},
    {field_return_type, 6},
    {field_type_parameters, 3},
  [551] =
    {field_body, 8},
    {field_class_parameters, 6},
    {field_extend, 7},
    {field_name, 4},
    {field_type_parameters, 5},
  [556] =
    {field_body, 8},
    {field_name, 3},
    {field_return_type, 6},
    {field_type_parameters, 4},
  [560] =
    {field_body, 8},
    {field_name, 3},
    {field_parameters, 5},
    {field_return_type, 7},
    {field_type_parameters, 4},
  [565] =
    {field_body, 8},
    {field_name, 3},
    {field_parameters, 4},
    {field_return_type, 6},
  [569] =
    {field_default_value, 6},
    {field_name, 2},
    {field_type, 4},
  [572] =
    {field_body, 9},
    {field_name, 3},
    {field_parameters, 5},
    {field_return_type, 7},
    {field_type_parameters, 4},
};

static TSSymbol ts_alias_sequences[PRODUCTION_ID_COUNT][MAX_ALIAS_SEQUENCE_LENGTH] = {
  [0] = {0},
  [6] = {
    [0] = alias_sym_type_identifier,
  },
  [33] = {
    [2] = alias_sym_type_identifier,
  },
  [34] = {
    [2] = alias_sym_type_identifier,
  },
  [39] = {
    [0] = alias_sym_type_identifier,
  },
  [43] = {
    [1] = alias_sym_type_identifier,
  },
  [69] = {
    [0] = alias_sym_type_identifier,
  },
  [72] = {
    [1] = alias_sym_type_identifier,
  },
  [85] = {
    [2] = alias_sym_type_identifier,
  },
  [109] = {
    [0] = alias_sym_type_identifier,
  },
  [123] = {
    [2] = alias_sym_type_identifier,
  },
  [136] = {
    [3] = alias_sym_type_identifier,
  },
  [167] = {
    [3] = alias_sym_type_identifier,
  },
};

static uint16_t ts_non_terminal_alias_map[] = {
  0,
};

static inline bool sym_operator_identifier_character_set_1(int32_t c) {
  return (c < ';'
    ? (c < '\''
      ? (c < '"'
        ? c == 0
        : c <= '"')
      : (c <= '\'' || (c >= '.' && c <= '9')))
    : (c <= ';' || (c < '`'
      ? (c < ']'
        ? c == '['
        : c <= ']')
      : (c <= '{' || c == '}'))));
}

static inline bool sym_operator_identifier_character_set_2(int32_t c) {
  return (c < ';'
    ? (c < '\''
      ? (c < '"'
        ? c == 0
        : c <= '"')
      : (c <= '(' || (c >= '.' && c <= '9')))
    : (c <= ';' || (c < '`'
      ? (c < ']'
        ? c == '['
        : c <= ']')
      : (c <= '{' || c == '}'))));
}

static inline bool sym_operator_identifier_character_set_4(int32_t c) {
  return (c < '.'
    ? (c < ' '
      ? (c < '\t'
        ? c == 0
        : (c <= '\n' || c == '\r'))
      : (c <= ' ' || (c < '\''
        ? c == '"'
        : (c <= ')' || c == ','))))
    : (c <= '.' || (c < ']'
      ? (c < ';'
        ? (c >= '0' && c <= '9')
        : (c <= ';' || (c >= 'A' && c <= '[')))
      : (c <= ']' || (c < '}'
        ? (c >= '_' && c <= '{')
        : c <= '}')))));
}

static bool ts_lex(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (eof) ADVANCE(41);
      if (lookahead == '!') ADVANCE(73);
      if (lookahead == '#') ADVANCE(67);
      if (lookahead == '$') ADVANCE(85);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == '*') ADVANCE(69);
      if (lookahead == '+') ADVANCE(51);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '-') ADVANCE(52);
      if (lookahead == '.') ADVANCE(43);
      if (lookahead == '/') ADVANCE(23);
      if (lookahead == ':') ADVANCE(58);
      if (lookahead == ';') ADVANCE(86);
      if (lookahead == '<') ADVANCE(8);
      if (lookahead == '=') ADVANCE(62);
      if (lookahead == '>') ADVANCE(31);
      if (lookahead == '@') ADVANCE(60);
      if (lookahead == '[') ADVANCE(49);
      if (lookahead == ']') ADVANCE(50);
      if (lookahead == '{') ADVANCE(45);
      if (lookahead == '|') ADVANCE(71);
      if (lookahead == '}') ADVANCE(46);
      if (lookahead == '~') ADVANCE(74);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(0)
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(84);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      END_STATE();
    case 1:
      if (lookahead == '!') ADVANCE(73);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == '+') ADVANCE(51);
      if (lookahead == '-') ADVANCE(52);
      if (lookahead == '/') ADVANCE(23);
      if (lookahead == '@') ADVANCE(60);
      if (lookahead == '{') ADVANCE(45);
      if (lookahead == '}') ADVANCE(46);
      if (lookahead == '~') ADVANCE(74);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(1)
      if (('.' <= lookahead && lookahead <= '9')) ADVANCE(84);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      END_STATE();
    case 2:
      if (lookahead == '#') ADVANCE(68);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == '*') ADVANCE(70);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == '=') ADVANCE(63);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '[') ADVANCE(49);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(2)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\'' &&
          (lookahead < '0' || '9' < lookahead) &&
          lookahead != ';' &&
          lookahead != ']' &&
          (lookahead < '`' || '{' < lookahead) &&
          lookahead != '}') ADVANCE(83);
      END_STATE();
    case 3:
      if (lookahead == '#') ADVANCE(68);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == '*') ADVANCE(70);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '[') ADVANCE(49);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(3)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\'' &&
          (lookahead < '0' || '9' < lookahead) &&
          lookahead != ';' &&
          lookahead != ']' &&
          (lookahead < '`' || '{' < lookahead) &&
          lookahead != '}') ADVANCE(83);
      END_STATE();
    case 4:
      if (lookahead == '#') ADVANCE(68);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '[') ADVANCE(49);
      if (lookahead == '|') ADVANCE(72);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(4)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\'' &&
          (lookahead < '0' || ';' < lookahead) &&
          lookahead != ']' &&
          (lookahead < '`' || '}' < lookahead)) ADVANCE(83);
      END_STATE();
    case 5:
      if (lookahead == '#') ADVANCE(68);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == '<') ADVANCE(76);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == '>') ADVANCE(80);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '[') ADVANCE(49);
      if (lookahead == ']') ADVANCE(50);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(5)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          (lookahead < '0' || ';' < lookahead) &&
          (lookahead < '`' || '{' < lookahead) &&
          lookahead != '}') ADVANCE(83);
      END_STATE();
    case 6:
      if (lookahead == '#') ADVANCE(68);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == '<') ADVANCE(76);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '[') ADVANCE(49);
      if (lookahead == ']') ADVANCE(50);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(6)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          (lookahead < '0' || ';' < lookahead) &&
          (lookahead < '`' || '{' < lookahead) &&
          lookahead != '}') ADVANCE(83);
      END_STATE();
    case 7:
      if (lookahead == '#') ADVANCE(68);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '[') ADVANCE(49);
      if (lookahead == ']') ADVANCE(50);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(7)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          (lookahead < '0' || ';' < lookahead) &&
          (lookahead < '`' || '{' < lookahead) &&
          lookahead != '}') ADVANCE(83);
      END_STATE();
    case 8:
      if (lookahead == '%') ADVANCE(56);
      if (lookahead == ':') ADVANCE(53);
      END_STATE();
    case 9:
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == '*') ADVANCE(70);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == '=') ADVANCE(63);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(9)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (!sym_operator_identifier_character_set_1(lookahead)) ADVANCE(83);
      END_STATE();
    case 10:
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == '*') ADVANCE(70);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(10)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (!sym_operator_identifier_character_set_1(lookahead)) ADVANCE(83);
      END_STATE();
    case 11:
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '|') ADVANCE(72);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(11)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\'' &&
          (lookahead < '0' || ';' < lookahead) &&
          lookahead != '[' &&
          lookahead != ']' &&
          (lookahead < '`' || '}' < lookahead)) ADVANCE(83);
      END_STATE();
    case 12:
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '|') ADVANCE(72);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(12)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\'' &&
          (lookahead < '0' || ';' < lookahead) &&
          lookahead != '[' &&
          lookahead != ']' &&
          (lookahead < '`' || '}' < lookahead)) ADVANCE(83);
      END_STATE();
    case 13:
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ';') ADVANCE(86);
      if (lookahead == '=') ADVANCE(64);
      if (lookahead == '[') ADVANCE(49);
      if (lookahead == '{') ADVANCE(45);
      if (lookahead == '}') ADVANCE(46);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(13)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\'' &&
          (lookahead < '0' || '9' < lookahead) &&
          lookahead != ']' &&
          lookahead != '`') ADVANCE(83);
      END_STATE();
    case 14:
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ';') ADVANCE(86);
      if (lookahead == '=') ADVANCE(63);
      if (lookahead == '[') ADVANCE(49);
      if (lookahead == '{') ADVANCE(45);
      if (lookahead == '}') ADVANCE(46);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(14)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\'' &&
          (lookahead < '0' || '9' < lookahead) &&
          lookahead != ']' &&
          lookahead != '`') ADVANCE(83);
      END_STATE();
    case 15:
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ';') ADVANCE(86);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == ']') ADVANCE(50);
      if (lookahead == '{') ADVANCE(45);
      if (lookahead == '}') ADVANCE(46);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(15)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\'' &&
          (lookahead < '.' || '9' < lookahead) &&
          lookahead != '[' &&
          lookahead != '`') ADVANCE(83);
      END_STATE();
    case 16:
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == '<') ADVANCE(76);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == '>') ADVANCE(80);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == ']') ADVANCE(50);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(16)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          (lookahead < '.' || ';' < lookahead) &&
          lookahead != '[' &&
          (lookahead < '`' || '{' < lookahead) &&
          lookahead != '}') ADVANCE(83);
      END_STATE();
    case 17:
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == '<') ADVANCE(76);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == ']') ADVANCE(50);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(17)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          (lookahead < '.' || ';' < lookahead) &&
          lookahead != '[' &&
          (lookahead < '`' || '{' < lookahead) &&
          lookahead != '}') ADVANCE(83);
      END_STATE();
    case 18:
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == ']') ADVANCE(50);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(18)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          (lookahead < '.' || ';' < lookahead) &&
          lookahead != '[' &&
          (lookahead < '`' || '{' < lookahead) &&
          lookahead != '}') ADVANCE(83);
      END_STATE();
    case 19:
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == '*') ADVANCE(70);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == '=') ADVANCE(63);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(19)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (!sym_operator_identifier_character_set_2(lookahead)) ADVANCE(83);
      END_STATE();
    case 20:
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == '*') ADVANCE(70);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(20)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (!sym_operator_identifier_character_set_2(lookahead)) ADVANCE(83);
      END_STATE();
    case 21:
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == '|') ADVANCE(72);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(21)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\'' &&
          lookahead != '(' &&
          (lookahead < '.' || ';' < lookahead) &&
          lookahead != '[' &&
          lookahead != ']' &&
          (lookahead < '`' || '}' < lookahead)) ADVANCE(83);
      END_STATE();
    case 22:
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == '|') ADVANCE(72);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(22)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\'' &&
          lookahead != '(' &&
          (lookahead < '.' || ';' < lookahead) &&
          lookahead != '[' &&
          lookahead != ']' &&
          (lookahead < '`' || '}' < lookahead)) ADVANCE(83);
      END_STATE();
    case 23:
      if (lookahead == '*') ADVANCE(25);
      if (lookahead == '/') ADVANCE(88);
      END_STATE();
    case 24:
      if (lookahead == '*') ADVANCE(24);
      if (lookahead == '/') ADVANCE(87);
      if (lookahead != 0) ADVANCE(25);
      END_STATE();
    case 25:
      if (lookahead == '*') ADVANCE(24);
      if (lookahead != 0) ADVANCE(25);
      END_STATE();
    case 26:
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == '<') ADVANCE(76);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == '>') ADVANCE(80);
      if (lookahead == ']') ADVANCE(50);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(26)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          (lookahead < '.' || ';' < lookahead) &&
          lookahead != '[' &&
          (lookahead < '`' || '{' < lookahead) &&
          lookahead != '}') ADVANCE(83);
      END_STATE();
    case 27:
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == '<') ADVANCE(76);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == ']') ADVANCE(50);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(27)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          (lookahead < '.' || ';' < lookahead) &&
          lookahead != '[' &&
          (lookahead < '`' || '{' < lookahead) &&
          lookahead != '}') ADVANCE(83);
      END_STATE();
    case 28:
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == ']') ADVANCE(50);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(28)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          (lookahead < '.' || ';' < lookahead) &&
          lookahead != '[' &&
          (lookahead < '`' || '{' < lookahead) &&
          lookahead != '}') ADVANCE(83);
      END_STATE();
    case 29:
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == ';') ADVANCE(86);
      if (lookahead == '=') ADVANCE(64);
      if (lookahead == '|') ADVANCE(72);
      if (lookahead == '}') ADVANCE(46);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(29)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          lookahead != ',' &&
          (lookahead < '.' || '9' < lookahead) &&
          lookahead != '[' &&
          lookahead != ']' &&
          (lookahead < '`' || '{' < lookahead)) ADVANCE(83);
      END_STATE();
    case 30:
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == ';') ADVANCE(86);
      if (lookahead == '=') ADVANCE(63);
      if (lookahead == '|') ADVANCE(72);
      if (lookahead == '}') ADVANCE(46);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(30)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          lookahead != ',' &&
          (lookahead < '.' || '9' < lookahead) &&
          lookahead != '[' &&
          lookahead != ']' &&
          (lookahead < '`' || '{' < lookahead)) ADVANCE(83);
      END_STATE();
    case 31:
      if (lookahead == ':') ADVANCE(54);
      END_STATE();
    case 32:
      if (eof) ADVANCE(41);
      if (lookahead == '#') ADVANCE(67);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == '+') ADVANCE(51);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '-') ADVANCE(52);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(23);
      if (lookahead == ';') ADVANCE(86);
      if (lookahead == '@') ADVANCE(60);
      if (lookahead == '[') ADVANCE(49);
      if (lookahead == '{') ADVANCE(45);
      if (lookahead == '}') ADVANCE(46);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(32)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      END_STATE();
    case 33:
      if (eof) ADVANCE(41);
      if (lookahead == '#') ADVANCE(68);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ';') ADVANCE(86);
      if (lookahead == '=') ADVANCE(63);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '[') ADVANCE(49);
      if (lookahead == '{') ADVANCE(45);
      if (lookahead == '}') ADVANCE(46);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(33)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\'' &&
          (lookahead < '0' || '9' < lookahead) &&
          lookahead != ']' &&
          lookahead != '`') ADVANCE(83);
      END_STATE();
    case 34:
      if (eof) ADVANCE(41);
      if (lookahead == '#') ADVANCE(68);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ';') ADVANCE(86);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '[') ADVANCE(49);
      if (lookahead == ']') ADVANCE(50);
      if (lookahead == '{') ADVANCE(45);
      if (lookahead == '}') ADVANCE(46);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(34)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\'' &&
          (lookahead < '0' || '9' < lookahead) &&
          lookahead != '`') ADVANCE(83);
      END_STATE();
    case 35:
      if (eof) ADVANCE(41);
      if (lookahead == '#') ADVANCE(68);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == ';') ADVANCE(86);
      if (lookahead == '=') ADVANCE(63);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '[') ADVANCE(49);
      if (lookahead == '|') ADVANCE(72);
      if (lookahead == '}') ADVANCE(46);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(35)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          lookahead != ',' &&
          (lookahead < '0' || '9' < lookahead) &&
          lookahead != ']' &&
          (lookahead < '`' || '{' < lookahead)) ADVANCE(83);
      END_STATE();
    case 36:
      if (eof) ADVANCE(41);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ';') ADVANCE(86);
      if (lookahead == '=') ADVANCE(63);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '{') ADVANCE(45);
      if (lookahead == '}') ADVANCE(46);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(36)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\'' &&
          (lookahead < '.' || '9' < lookahead) &&
          lookahead != '[' &&
          lookahead != ']' &&
          lookahead != '`') ADVANCE(83);
      END_STATE();
    case 37:
      if (eof) ADVANCE(41);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ')') ADVANCE(66);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ';') ADVANCE(86);
      if (lookahead == '=') ADVANCE(81);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == ']') ADVANCE(50);
      if (lookahead == '{') ADVANCE(45);
      if (lookahead == '}') ADVANCE(46);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(37)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\'' &&
          (lookahead < '.' || '9' < lookahead) &&
          lookahead != '[' &&
          lookahead != '`') ADVANCE(83);
      END_STATE();
    case 38:
      if (eof) ADVANCE(41);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == '=') ADVANCE(64);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '|') ADVANCE(72);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(38)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          (lookahead < '0' || ';' < lookahead) &&
          lookahead != '[' &&
          lookahead != ']' &&
          (lookahead < '`' || '}' < lookahead)) ADVANCE(83);
      END_STATE();
    case 39:
      if (eof) ADVANCE(41);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == '.') ADVANCE(42);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == '=') ADVANCE(64);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '[') ADVANCE(49);
      if (lookahead == '{') ADVANCE(45);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(39)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          lookahead != ',' &&
          (lookahead < '0' || '9' < lookahead) &&
          lookahead != ';' &&
          lookahead != ']' &&
          lookahead != '`' &&
          lookahead != '}') ADVANCE(83);
      END_STATE();
    case 40:
      if (eof) ADVANCE(41);
      if (lookahead == '(') ADVANCE(65);
      if (lookahead == '/') ADVANCE(77);
      if (lookahead == ':') ADVANCE(59);
      if (lookahead == ';') ADVANCE(86);
      if (lookahead == '=') ADVANCE(63);
      if (lookahead == '@') ADVANCE(61);
      if (lookahead == '|') ADVANCE(72);
      if (lookahead == '}') ADVANCE(46);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(40)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      if (lookahead != 0 &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          lookahead != ',' &&
          (lookahead < '.' || '9' < lookahead) &&
          lookahead != '[' &&
          lookahead != ']' &&
          (lookahead < '`' || '{' < lookahead)) ADVANCE(83);
      END_STATE();
    case 41:
      ACCEPT_TOKEN(ts_builtin_sym_end);
      END_STATE();
    case 42:
      ACCEPT_TOKEN(anon_sym_DOT);
      END_STATE();
    case 43:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.' ||
          ('0' <= lookahead && lookahead <= '9')) ADVANCE(84);
      END_STATE();
    case 44:
      ACCEPT_TOKEN(anon_sym_COMMA);
      END_STATE();
    case 45:
      ACCEPT_TOKEN(anon_sym_LBRACE);
      END_STATE();
    case 46:
      ACCEPT_TOKEN(anon_sym_RBRACE);
      END_STATE();
    case 47:
      ACCEPT_TOKEN(anon_sym_EQ_GT);
      END_STATE();
    case 48:
      ACCEPT_TOKEN(anon_sym_EQ_GT);
      if (!sym_operator_identifier_character_set_4(lookahead)) ADVANCE(83);
      END_STATE();
    case 49:
      ACCEPT_TOKEN(anon_sym_LBRACK);
      END_STATE();
    case 50:
      ACCEPT_TOKEN(anon_sym_RBRACK);
      END_STATE();
    case 51:
      ACCEPT_TOKEN(anon_sym_PLUS);
      END_STATE();
    case 52:
      ACCEPT_TOKEN(anon_sym_DASH);
      END_STATE();
    case 53:
      ACCEPT_TOKEN(anon_sym_LT_COLON);
      END_STATE();
    case 54:
      ACCEPT_TOKEN(anon_sym_GT_COLON);
      END_STATE();
    case 55:
      ACCEPT_TOKEN(anon_sym_GT_COLON);
      if (!sym_operator_identifier_character_set_4(lookahead)) ADVANCE(83);
      END_STATE();
    case 56:
      ACCEPT_TOKEN(anon_sym_LT_PERCENT);
      END_STATE();
    case 57:
      ACCEPT_TOKEN(anon_sym_LT_PERCENT);
      if (!sym_operator_identifier_character_set_4(lookahead)) ADVANCE(83);
      END_STATE();
    case 58:
      ACCEPT_TOKEN(anon_sym_COLON);
      END_STATE();
    case 59:
      ACCEPT_TOKEN(anon_sym_COLON);
      if (!sym_operator_identifier_character_set_4(lookahead)) ADVANCE(83);
      END_STATE();
    case 60:
      ACCEPT_TOKEN(anon_sym_AT);
      END_STATE();
    case 61:
      ACCEPT_TOKEN(anon_sym_AT);
      if (!sym_operator_identifier_character_set_4(lookahead)) ADVANCE(83);
      END_STATE();
    case 62:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '>') ADVANCE(47);
      END_STATE();
    case 63:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '>') ADVANCE(48);
      if (!sym_operator_identifier_character_set_4(lookahead)) ADVANCE(83);
      END_STATE();
    case 64:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (!sym_operator_identifier_character_set_4(lookahead)) ADVANCE(83);
      END_STATE();
    case 65:
      ACCEPT_TOKEN(anon_sym_LPAREN);
      END_STATE();
    case 66:
      ACCEPT_TOKEN(anon_sym_RPAREN);
      END_STATE();
    case 67:
      ACCEPT_TOKEN(anon_sym_POUND);
      END_STATE();
    case 68:
      ACCEPT_TOKEN(anon_sym_POUND);
      if (!sym_operator_identifier_character_set_4(lookahead)) ADVANCE(83);
      END_STATE();
    case 69:
      ACCEPT_TOKEN(anon_sym_STAR);
      END_STATE();
    case 70:
      ACCEPT_TOKEN(anon_sym_STAR);
      if (!sym_operator_identifier_character_set_4(lookahead)) ADVANCE(83);
      END_STATE();
    case 71:
      ACCEPT_TOKEN(anon_sym_PIPE);
      END_STATE();
    case 72:
      ACCEPT_TOKEN(anon_sym_PIPE);
      if (!sym_operator_identifier_character_set_4(lookahead)) ADVANCE(83);
      END_STATE();
    case 73:
      ACCEPT_TOKEN(anon_sym_BANG);
      END_STATE();
    case 74:
      ACCEPT_TOKEN(anon_sym_TILDE);
      END_STATE();
    case 75:
      ACCEPT_TOKEN(sym_identifier);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(75);
      END_STATE();
    case 76:
      ACCEPT_TOKEN(sym_operator_identifier);
      if (lookahead == '%') ADVANCE(57);
      if (!sym_operator_identifier_character_set_4(lookahead)) ADVANCE(83);
      END_STATE();
    case 77:
      ACCEPT_TOKEN(sym_operator_identifier);
      if (lookahead == '*') ADVANCE(79);
      if (lookahead == '/') ADVANCE(82);
      if (lookahead != 0 &&
          lookahead != '\t' &&
          lookahead != '\n' &&
          lookahead != '\r' &&
          lookahead != ' ' &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          lookahead != ',' &&
          (lookahead < '.' || '9' < lookahead) &&
          lookahead != ';' &&
          (lookahead < 'A' || '[' < lookahead) &&
          lookahead != ']' &&
          (lookahead < '_' || '{' < lookahead) &&
          lookahead != '}') ADVANCE(83);
      END_STATE();
    case 78:
      ACCEPT_TOKEN(sym_operator_identifier);
      if (lookahead == '*') ADVANCE(78);
      if (lookahead == '/') ADVANCE(83);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == '"' ||
          ('\'' <= lookahead && lookahead <= ')') ||
          lookahead == ',' ||
          ('.' <= lookahead && lookahead <= '9') ||
          lookahead == ';' ||
          ('A' <= lookahead && lookahead <= '[') ||
          lookahead == ']' ||
          ('_' <= lookahead && lookahead <= '{') ||
          lookahead == '}') ADVANCE(25);
      if (lookahead != 0) ADVANCE(79);
      END_STATE();
    case 79:
      ACCEPT_TOKEN(sym_operator_identifier);
      if (lookahead == '*') ADVANCE(78);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == '"' ||
          ('\'' <= lookahead && lookahead <= ')') ||
          lookahead == ',' ||
          lookahead == '.' ||
          ('0' <= lookahead && lookahead <= '9') ||
          lookahead == ';' ||
          ('A' <= lookahead && lookahead <= '[') ||
          lookahead == ']' ||
          ('_' <= lookahead && lookahead <= '{') ||
          lookahead == '}') ADVANCE(25);
      if (lookahead != 0) ADVANCE(79);
      END_STATE();
    case 80:
      ACCEPT_TOKEN(sym_operator_identifier);
      if (lookahead == ':') ADVANCE(55);
      if (lookahead != 0 &&
          lookahead != '\t' &&
          lookahead != '\n' &&
          lookahead != '\r' &&
          lookahead != ' ' &&
          lookahead != '"' &&
          (lookahead < '\'' || ')' < lookahead) &&
          lookahead != ',' &&
          lookahead != '.' &&
          (lookahead < '0' || ';' < lookahead) &&
          (lookahead < 'A' || '[' < lookahead) &&
          lookahead != ']' &&
          (lookahead < '_' || '{' < lookahead) &&
          lookahead != '}') ADVANCE(83);
      END_STATE();
    case 81:
      ACCEPT_TOKEN(sym_operator_identifier);
      if (lookahead == '>') ADVANCE(48);
      if (!sym_operator_identifier_character_set_4(lookahead)) ADVANCE(83);
      END_STATE();
    case 82:
      ACCEPT_TOKEN(sym_operator_identifier);
      if (lookahead == '\t' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == '"' ||
          ('\'' <= lookahead && lookahead <= ')') ||
          lookahead == ',' ||
          lookahead == '.' ||
          ('0' <= lookahead && lookahead <= '9') ||
          lookahead == ';' ||
          ('A' <= lookahead && lookahead <= '[') ||
          lookahead == ']' ||
          ('_' <= lookahead && lookahead <= '{') ||
          lookahead == '}') ADVANCE(88);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(82);
      END_STATE();
    case 83:
      ACCEPT_TOKEN(sym_operator_identifier);
      if (!sym_operator_identifier_character_set_4(lookahead)) ADVANCE(83);
      END_STATE();
    case 84:
      ACCEPT_TOKEN(sym_number);
      if (lookahead == '.' ||
          ('0' <= lookahead && lookahead <= '9')) ADVANCE(84);
      END_STATE();
    case 85:
      ACCEPT_TOKEN(anon_sym_DOLLAR);
      END_STATE();
    case 86:
      ACCEPT_TOKEN(anon_sym_SEMI);
      END_STATE();
    case 87:
      ACCEPT_TOKEN(sym_comment);
      END_STATE();
    case 88:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(88);
      END_STATE();
    default:
      return false;
  }
}

static bool ts_lex_keywords(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (lookahead == '_') ADVANCE(1);
      if (lookahead == 'a') ADVANCE(2);
      if (lookahead == 'c') ADVANCE(3);
      if (lookahead == 'd') ADVANCE(4);
      if (lookahead == 'e') ADVANCE(5);
      if (lookahead == 'f') ADVANCE(6);
      if (lookahead == 'i') ADVANCE(7);
      if (lookahead == 'l') ADVANCE(8);
      if (lookahead == 'm') ADVANCE(9);
      if (lookahead == 'n') ADVANCE(10);
      if (lookahead == 'o') ADVANCE(11);
      if (lookahead == 'p') ADVANCE(12);
      if (lookahead == 's') ADVANCE(13);
      if (lookahead == 't') ADVANCE(14);
      if (lookahead == 'v') ADVANCE(15);
      if (lookahead == 'w') ADVANCE(16);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(0)
      END_STATE();
    case 1:
      ACCEPT_TOKEN(sym_wildcard);
      END_STATE();
    case 2:
      if (lookahead == 'b') ADVANCE(17);
      END_STATE();
    case 3:
      if (lookahead == 'a') ADVANCE(18);
      if (lookahead == 'l') ADVANCE(19);
      END_STATE();
    case 4:
      if (lookahead == 'e') ADVANCE(20);
      END_STATE();
    case 5:
      if (lookahead == 'l') ADVANCE(21);
      if (lookahead == 'x') ADVANCE(22);
      END_STATE();
    case 6:
      if (lookahead == 'i') ADVANCE(23);
      END_STATE();
    case 7:
      if (lookahead == 'f') ADVANCE(24);
      if (lookahead == 'm') ADVANCE(25);
      END_STATE();
    case 8:
      if (lookahead == 'a') ADVANCE(26);
      END_STATE();
    case 9:
      if (lookahead == 'a') ADVANCE(27);
      END_STATE();
    case 10:
      if (lookahead == 'e') ADVANCE(28);
      END_STATE();
    case 11:
      if (lookahead == 'b') ADVANCE(29);
      if (lookahead == 'v') ADVANCE(30);
      END_STATE();
    case 12:
      if (lookahead == 'a') ADVANCE(31);
      if (lookahead == 'r') ADVANCE(32);
      END_STATE();
    case 13:
      if (lookahead == 'e') ADVANCE(33);
      END_STATE();
    case 14:
      if (lookahead == 'r') ADVANCE(34);
      if (lookahead == 'y') ADVANCE(35);
      END_STATE();
    case 15:
      if (lookahead == 'a') ADVANCE(36);
      END_STATE();
    case 16:
      if (lookahead == 'i') ADVANCE(37);
      END_STATE();
    case 17:
      if (lookahead == 's') ADVANCE(38);
      END_STATE();
    case 18:
      if (lookahead == 's') ADVANCE(39);
      if (lookahead == 't') ADVANCE(40);
      END_STATE();
    case 19:
      if (lookahead == 'a') ADVANCE(41);
      END_STATE();
    case 20:
      if (lookahead == 'f') ADVANCE(42);
      END_STATE();
    case 21:
      if (lookahead == 's') ADVANCE(43);
      END_STATE();
    case 22:
      if (lookahead == 't') ADVANCE(44);
      END_STATE();
    case 23:
      if (lookahead == 'n') ADVANCE(45);
      END_STATE();
    case 24:
      ACCEPT_TOKEN(anon_sym_if);
      END_STATE();
    case 25:
      if (lookahead == 'p') ADVANCE(46);
      END_STATE();
    case 26:
      if (lookahead == 'z') ADVANCE(47);
      END_STATE();
    case 27:
      if (lookahead == 't') ADVANCE(48);
      END_STATE();
    case 28:
      if (lookahead == 'w') ADVANCE(49);
      END_STATE();
    case 29:
      if (lookahead == 'j') ADVANCE(50);
      END_STATE();
    case 30:
      if (lookahead == 'e') ADVANCE(51);
      END_STATE();
    case 31:
      if (lookahead == 'c') ADVANCE(52);
      END_STATE();
    case 32:
      if (lookahead == 'i') ADVANCE(53);
      if (lookahead == 'o') ADVANCE(54);
      END_STATE();
    case 33:
      if (lookahead == 'a') ADVANCE(55);
      END_STATE();
    case 34:
      if (lookahead == 'a') ADVANCE(56);
      if (lookahead == 'y') ADVANCE(57);
      END_STATE();
    case 35:
      if (lookahead == 'p') ADVANCE(58);
      END_STATE();
    case 36:
      if (lookahead == 'l') ADVANCE(59);
      if (lookahead == 'r') ADVANCE(60);
      END_STATE();
    case 37:
      if (lookahead == 't') ADVANCE(61);
      END_STATE();
    case 38:
      if (lookahead == 't') ADVANCE(62);
      END_STATE();
    case 39:
      if (lookahead == 'e') ADVANCE(63);
      END_STATE();
    case 40:
      if (lookahead == 'c') ADVANCE(64);
      END_STATE();
    case 41:
      if (lookahead == 's') ADVANCE(65);
      END_STATE();
    case 42:
      ACCEPT_TOKEN(anon_sym_def);
      END_STATE();
    case 43:
      if (lookahead == 'e') ADVANCE(66);
      END_STATE();
    case 44:
      if (lookahead == 'e') ADVANCE(67);
      END_STATE();
    case 45:
      if (lookahead == 'a') ADVANCE(68);
      END_STATE();
    case 46:
      if (lookahead == 'l') ADVANCE(69);
      if (lookahead == 'o') ADVANCE(70);
      END_STATE();
    case 47:
      if (lookahead == 'y') ADVANCE(71);
      END_STATE();
    case 48:
      if (lookahead == 'c') ADVANCE(72);
      END_STATE();
    case 49:
      ACCEPT_TOKEN(anon_sym_new);
      END_STATE();
    case 50:
      if (lookahead == 'e') ADVANCE(73);
      END_STATE();
    case 51:
      if (lookahead == 'r') ADVANCE(74);
      END_STATE();
    case 52:
      if (lookahead == 'k') ADVANCE(75);
      END_STATE();
    case 53:
      if (lookahead == 'v') ADVANCE(76);
      END_STATE();
    case 54:
      if (lookahead == 't') ADVANCE(77);
      END_STATE();
    case 55:
      if (lookahead == 'l') ADVANCE(78);
      END_STATE();
    case 56:
      if (lookahead == 'i') ADVANCE(79);
      END_STATE();
    case 57:
      ACCEPT_TOKEN(anon_sym_try);
      END_STATE();
    case 58:
      if (lookahead == 'e') ADVANCE(80);
      END_STATE();
    case 59:
      ACCEPT_TOKEN(anon_sym_val);
      END_STATE();
    case 60:
      ACCEPT_TOKEN(anon_sym_var);
      END_STATE();
    case 61:
      if (lookahead == 'h') ADVANCE(81);
      END_STATE();
    case 62:
      if (lookahead == 'r') ADVANCE(82);
      END_STATE();
    case 63:
      ACCEPT_TOKEN(anon_sym_case);
      END_STATE();
    case 64:
      if (lookahead == 'h') ADVANCE(83);
      END_STATE();
    case 65:
      if (lookahead == 's') ADVANCE(84);
      END_STATE();
    case 66:
      ACCEPT_TOKEN(anon_sym_else);
      END_STATE();
    case 67:
      if (lookahead == 'n') ADVANCE(85);
      END_STATE();
    case 68:
      if (lookahead == 'l') ADVANCE(86);
      END_STATE();
    case 69:
      if (lookahead == 'i') ADVANCE(87);
      END_STATE();
    case 70:
      if (lookahead == 'r') ADVANCE(88);
      END_STATE();
    case 71:
      ACCEPT_TOKEN(anon_sym_lazy);
      END_STATE();
    case 72:
      if (lookahead == 'h') ADVANCE(89);
      END_STATE();
    case 73:
      if (lookahead == 'c') ADVANCE(90);
      END_STATE();
    case 74:
      if (lookahead == 'r') ADVANCE(91);
      END_STATE();
    case 75:
      if (lookahead == 'a') ADVANCE(92);
      END_STATE();
    case 76:
      if (lookahead == 'a') ADVANCE(93);
      END_STATE();
    case 77:
      if (lookahead == 'e') ADVANCE(94);
      END_STATE();
    case 78:
      if (lookahead == 'e') ADVANCE(95);
      END_STATE();
    case 79:
      if (lookahead == 't') ADVANCE(96);
      END_STATE();
    case 80:
      ACCEPT_TOKEN(anon_sym_type);
      END_STATE();
    case 81:
      ACCEPT_TOKEN(anon_sym_with);
      END_STATE();
    case 82:
      if (lookahead == 'a') ADVANCE(97);
      END_STATE();
    case 83:
      ACCEPT_TOKEN(anon_sym_catch);
      END_STATE();
    case 84:
      ACCEPT_TOKEN(anon_sym_class);
      END_STATE();
    case 85:
      if (lookahead == 'd') ADVANCE(98);
      END_STATE();
    case 86:
      ACCEPT_TOKEN(anon_sym_final);
      if (lookahead == 'l') ADVANCE(99);
      END_STATE();
    case 87:
      if (lookahead == 'c') ADVANCE(100);
      END_STATE();
    case 88:
      if (lookahead == 't') ADVANCE(101);
      END_STATE();
    case 89:
      ACCEPT_TOKEN(anon_sym_match);
      END_STATE();
    case 90:
      if (lookahead == 't') ADVANCE(102);
      END_STATE();
    case 91:
      if (lookahead == 'i') ADVANCE(103);
      END_STATE();
    case 92:
      if (lookahead == 'g') ADVANCE(104);
      END_STATE();
    case 93:
      if (lookahead == 't') ADVANCE(105);
      END_STATE();
    case 94:
      if (lookahead == 'c') ADVANCE(106);
      END_STATE();
    case 95:
      if (lookahead == 'd') ADVANCE(107);
      END_STATE();
    case 96:
      ACCEPT_TOKEN(anon_sym_trait);
      END_STATE();
    case 97:
      if (lookahead == 'c') ADVANCE(108);
      END_STATE();
    case 98:
      if (lookahead == 's') ADVANCE(109);
      END_STATE();
    case 99:
      if (lookahead == 'y') ADVANCE(110);
      END_STATE();
    case 100:
      if (lookahead == 'i') ADVANCE(111);
      END_STATE();
    case 101:
      ACCEPT_TOKEN(anon_sym_import);
      END_STATE();
    case 102:
      ACCEPT_TOKEN(anon_sym_object);
      END_STATE();
    case 103:
      if (lookahead == 'd') ADVANCE(112);
      END_STATE();
    case 104:
      if (lookahead == 'e') ADVANCE(113);
      END_STATE();
    case 105:
      if (lookahead == 'e') ADVANCE(114);
      END_STATE();
    case 106:
      if (lookahead == 't') ADVANCE(115);
      END_STATE();
    case 107:
      ACCEPT_TOKEN(anon_sym_sealed);
      END_STATE();
    case 108:
      if (lookahead == 't') ADVANCE(116);
      END_STATE();
    case 109:
      ACCEPT_TOKEN(anon_sym_extends);
      END_STATE();
    case 110:
      ACCEPT_TOKEN(anon_sym_finally);
      END_STATE();
    case 111:
      if (lookahead == 't') ADVANCE(117);
      END_STATE();
    case 112:
      if (lookahead == 'e') ADVANCE(118);
      END_STATE();
    case 113:
      ACCEPT_TOKEN(anon_sym_package);
      END_STATE();
    case 114:
      ACCEPT_TOKEN(anon_sym_private);
      END_STATE();
    case 115:
      if (lookahead == 'e') ADVANCE(119);
      END_STATE();
    case 116:
      ACCEPT_TOKEN(anon_sym_abstract);
      END_STATE();
    case 117:
      ACCEPT_TOKEN(anon_sym_implicit);
      END_STATE();
    case 118:
      ACCEPT_TOKEN(anon_sym_override);
      END_STATE();
    case 119:
      if (lookahead == 'd') ADVANCE(120);
      END_STATE();
    case 120:
      ACCEPT_TOKEN(anon_sym_protected);
      END_STATE();
    default:
      return false;
  }
}

static TSLexMode ts_lex_modes[STATE_COUNT] = {
  [0] = {.lex_state = 0, .external_lex_state = 1},
  [1] = {.lex_state = 0},
  [2] = {.lex_state = 1, .external_lex_state = 2},
  [3] = {.lex_state = 1, .external_lex_state = 2},
  [4] = {.lex_state = 1, .external_lex_state = 2},
  [5] = {.lex_state = 1, .external_lex_state = 2},
  [6] = {.lex_state = 1, .external_lex_state = 2},
  [7] = {.lex_state = 1, .external_lex_state = 2},
  [8] = {.lex_state = 1, .external_lex_state = 2},
  [9] = {.lex_state = 1, .external_lex_state = 2},
  [10] = {.lex_state = 1, .external_lex_state = 2},
  [11] = {.lex_state = 1, .external_lex_state = 2},
  [12] = {.lex_state = 1, .external_lex_state = 2},
  [13] = {.lex_state = 1, .external_lex_state = 2},
  [14] = {.lex_state = 1, .external_lex_state = 2},
  [15] = {.lex_state = 1, .external_lex_state = 2},
  [16] = {.lex_state = 1, .external_lex_state = 2},
  [17] = {.lex_state = 1, .external_lex_state = 2},
  [18] = {.lex_state = 1, .external_lex_state = 2},
  [19] = {.lex_state = 1, .external_lex_state = 2},
  [20] = {.lex_state = 1, .external_lex_state = 2},
  [21] = {.lex_state = 1, .external_lex_state = 2},
  [22] = {.lex_state = 1, .external_lex_state = 2},
  [23] = {.lex_state = 1, .external_lex_state = 2},
  [24] = {.lex_state = 1, .external_lex_state = 2},
  [25] = {.lex_state = 0},
  [26] = {.lex_state = 0},
  [27] = {.lex_state = 39, .external_lex_state = 3},
  [28] = {.lex_state = 39, .external_lex_state = 3},
  [29] = {.lex_state = 39, .external_lex_state = 4},
  [30] = {.lex_state = 39},
  [31] = {.lex_state = 35},
  [32] = {.lex_state = 39, .external_lex_state = 2},
  [33] = {.lex_state = 39, .external_lex_state = 3},
  [34] = {.lex_state = 39},
  [35] = {.lex_state = 1, .external_lex_state = 2},
  [36] = {.lex_state = 1, .external_lex_state = 2},
  [37] = {.lex_state = 39, .external_lex_state = 3},
  [38] = {.lex_state = 39, .external_lex_state = 3},
  [39] = {.lex_state = 39, .external_lex_state = 3},
  [40] = {.lex_state = 39, .external_lex_state = 3},
  [41] = {.lex_state = 39, .external_lex_state = 3},
  [42] = {.lex_state = 39, .external_lex_state = 3},
  [43] = {.lex_state = 39, .external_lex_state = 4},
  [44] = {.lex_state = 33},
  [45] = {.lex_state = 1, .external_lex_state = 2},
  [46] = {.lex_state = 39, .external_lex_state = 3},
  [47] = {.lex_state = 1, .external_lex_state = 2},
  [48] = {.lex_state = 1, .external_lex_state = 2},
  [49] = {.lex_state = 1, .external_lex_state = 2},
  [50] = {.lex_state = 1, .external_lex_state = 2},
  [51] = {.lex_state = 1, .external_lex_state = 2},
  [52] = {.lex_state = 39},
  [53] = {.lex_state = 39, .external_lex_state = 3},
  [54] = {.lex_state = 1, .external_lex_state = 2},
  [55] = {.lex_state = 1, .external_lex_state = 2},
  [56] = {.lex_state = 1, .external_lex_state = 2},
  [57] = {.lex_state = 1, .external_lex_state = 2},
  [58] = {.lex_state = 1, .external_lex_state = 2},
  [59] = {.lex_state = 1, .external_lex_state = 2},
  [60] = {.lex_state = 1, .external_lex_state = 2},
  [61] = {.lex_state = 1, .external_lex_state = 2},
  [62] = {.lex_state = 1, .external_lex_state = 2},
  [63] = {.lex_state = 1, .external_lex_state = 2},
  [64] = {.lex_state = 1, .external_lex_state = 2},
  [65] = {.lex_state = 1, .external_lex_state = 2},
  [66] = {.lex_state = 1, .external_lex_state = 2},
  [67] = {.lex_state = 1, .external_lex_state = 2},
  [68] = {.lex_state = 1, .external_lex_state = 2},
  [69] = {.lex_state = 1, .external_lex_state = 2},
  [70] = {.lex_state = 35},
  [71] = {.lex_state = 1, .external_lex_state = 2},
  [72] = {.lex_state = 1, .external_lex_state = 2},
  [73] = {.lex_state = 1, .external_lex_state = 2},
  [74] = {.lex_state = 1, .external_lex_state = 2},
  [75] = {.lex_state = 1, .external_lex_state = 2},
  [76] = {.lex_state = 1, .external_lex_state = 2},
  [77] = {.lex_state = 1, .external_lex_state = 2},
  [78] = {.lex_state = 1, .external_lex_state = 2},
  [79] = {.lex_state = 1, .external_lex_state = 2},
  [80] = {.lex_state = 1, .external_lex_state = 2},
  [81] = {.lex_state = 1, .external_lex_state = 2},
  [82] = {.lex_state = 1, .external_lex_state = 2},
  [83] = {.lex_state = 1, .external_lex_state = 2},
  [84] = {.lex_state = 1, .external_lex_state = 2},
  [85] = {.lex_state = 1, .external_lex_state = 2},
  [86] = {.lex_state = 34},
  [87] = {.lex_state = 39, .external_lex_state = 2},
  [88] = {.lex_state = 1, .external_lex_state = 2},
  [89] = {.lex_state = 1, .external_lex_state = 2},
  [90] = {.lex_state = 1, .external_lex_state = 2},
  [91] = {.lex_state = 1, .external_lex_state = 2},
  [92] = {.lex_state = 1, .external_lex_state = 2},
  [93] = {.lex_state = 1, .external_lex_state = 2},
  [94] = {.lex_state = 34},
  [95] = {.lex_state = 39},
  [96] = {.lex_state = 39},
  [97] = {.lex_state = 1, .external_lex_state = 2},
  [98] = {.lex_state = 1, .external_lex_state = 2},
  [99] = {.lex_state = 1, .external_lex_state = 2},
  [100] = {.lex_state = 1, .external_lex_state = 2},
  [101] = {.lex_state = 1, .external_lex_state = 2},
  [102] = {.lex_state = 1, .external_lex_state = 2},
  [103] = {.lex_state = 1, .external_lex_state = 2},
  [104] = {.lex_state = 39},
  [105] = {.lex_state = 1, .external_lex_state = 2},
  [106] = {.lex_state = 1, .external_lex_state = 2},
  [107] = {.lex_state = 1, .external_lex_state = 2},
  [108] = {.lex_state = 1, .external_lex_state = 2},
  [109] = {.lex_state = 1, .external_lex_state = 2},
  [110] = {.lex_state = 1, .external_lex_state = 2},
  [111] = {.lex_state = 39},
  [112] = {.lex_state = 1, .external_lex_state = 2},
  [113] = {.lex_state = 1, .external_lex_state = 2},
  [114] = {.lex_state = 1, .external_lex_state = 2},
  [115] = {.lex_state = 1, .external_lex_state = 2},
  [116] = {.lex_state = 1, .external_lex_state = 2},
  [117] = {.lex_state = 1, .external_lex_state = 2},
  [118] = {.lex_state = 1, .external_lex_state = 2},
  [119] = {.lex_state = 1, .external_lex_state = 2},
  [120] = {.lex_state = 1, .external_lex_state = 2},
  [121] = {.lex_state = 39},
  [122] = {.lex_state = 1, .external_lex_state = 2},
  [123] = {.lex_state = 1, .external_lex_state = 2},
  [124] = {.lex_state = 1, .external_lex_state = 2},
  [125] = {.lex_state = 1, .external_lex_state = 2},
  [126] = {.lex_state = 1, .external_lex_state = 2},
  [127] = {.lex_state = 1, .external_lex_state = 2},
  [128] = {.lex_state = 39},
  [129] = {.lex_state = 1, .external_lex_state = 2},
  [130] = {.lex_state = 1, .external_lex_state = 2},
  [131] = {.lex_state = 1, .external_lex_state = 2},
  [132] = {.lex_state = 1, .external_lex_state = 2},
  [133] = {.lex_state = 1, .external_lex_state = 2},
  [134] = {.lex_state = 1, .external_lex_state = 2},
  [135] = {.lex_state = 1, .external_lex_state = 2},
  [136] = {.lex_state = 1, .external_lex_state = 2},
  [137] = {.lex_state = 1, .external_lex_state = 2},
  [138] = {.lex_state = 1, .external_lex_state = 2},
  [139] = {.lex_state = 1, .external_lex_state = 2},
  [140] = {.lex_state = 1, .external_lex_state = 2},
  [141] = {.lex_state = 1, .external_lex_state = 2},
  [142] = {.lex_state = 1, .external_lex_state = 2},
  [143] = {.lex_state = 1, .external_lex_state = 2},
  [144] = {.lex_state = 1, .external_lex_state = 2},
  [145] = {.lex_state = 35},
  [146] = {.lex_state = 1, .external_lex_state = 2},
  [147] = {.lex_state = 1, .external_lex_state = 2},
  [148] = {.lex_state = 1, .external_lex_state = 2},
  [149] = {.lex_state = 1, .external_lex_state = 2},
  [150] = {.lex_state = 1, .external_lex_state = 2},
  [151] = {.lex_state = 1, .external_lex_state = 2},
  [152] = {.lex_state = 1, .external_lex_state = 2},
  [153] = {.lex_state = 1, .external_lex_state = 2},
  [154] = {.lex_state = 1, .external_lex_state = 2},
  [155] = {.lex_state = 1, .external_lex_state = 2},
  [156] = {.lex_state = 1, .external_lex_state = 2},
  [157] = {.lex_state = 39, .external_lex_state = 3},
  [158] = {.lex_state = 1, .external_lex_state = 2},
  [159] = {.lex_state = 1, .external_lex_state = 2},
  [160] = {.lex_state = 1, .external_lex_state = 2},
  [161] = {.lex_state = 1, .external_lex_state = 2},
  [162] = {.lex_state = 1, .external_lex_state = 2},
  [163] = {.lex_state = 35},
  [164] = {.lex_state = 1, .external_lex_state = 2},
  [165] = {.lex_state = 1, .external_lex_state = 2},
  [166] = {.lex_state = 1, .external_lex_state = 2},
  [167] = {.lex_state = 1, .external_lex_state = 2},
  [168] = {.lex_state = 1, .external_lex_state = 2},
  [169] = {.lex_state = 1, .external_lex_state = 2},
  [170] = {.lex_state = 1, .external_lex_state = 2},
  [171] = {.lex_state = 1, .external_lex_state = 2},
  [172] = {.lex_state = 1, .external_lex_state = 2},
  [173] = {.lex_state = 1, .external_lex_state = 2},
  [174] = {.lex_state = 1, .external_lex_state = 2},
  [175] = {.lex_state = 1, .external_lex_state = 2},
  [176] = {.lex_state = 1, .external_lex_state = 2},
  [177] = {.lex_state = 1, .external_lex_state = 2},
  [178] = {.lex_state = 1, .external_lex_state = 2},
  [179] = {.lex_state = 1, .external_lex_state = 2},
  [180] = {.lex_state = 1, .external_lex_state = 2},
  [181] = {.lex_state = 1, .external_lex_state = 2},
  [182] = {.lex_state = 1, .external_lex_state = 2},
  [183] = {.lex_state = 1, .external_lex_state = 2},
  [184] = {.lex_state = 1, .external_lex_state = 2},
  [185] = {.lex_state = 1, .external_lex_state = 2},
  [186] = {.lex_state = 1, .external_lex_state = 2},
  [187] = {.lex_state = 1, .external_lex_state = 2},
  [188] = {.lex_state = 1, .external_lex_state = 2},
  [189] = {.lex_state = 1, .external_lex_state = 2},
  [190] = {.lex_state = 1, .external_lex_state = 2},
  [191] = {.lex_state = 1, .external_lex_state = 2},
  [192] = {.lex_state = 1, .external_lex_state = 2},
  [193] = {.lex_state = 1, .external_lex_state = 2},
  [194] = {.lex_state = 1, .external_lex_state = 2},
  [195] = {.lex_state = 1, .external_lex_state = 2},
  [196] = {.lex_state = 1, .external_lex_state = 2},
  [197] = {.lex_state = 1, .external_lex_state = 2},
  [198] = {.lex_state = 1, .external_lex_state = 2},
  [199] = {.lex_state = 1, .external_lex_state = 2},
  [200] = {.lex_state = 1, .external_lex_state = 2},
  [201] = {.lex_state = 1, .external_lex_state = 2},
  [202] = {.lex_state = 1, .external_lex_state = 2},
  [203] = {.lex_state = 1, .external_lex_state = 2},
  [204] = {.lex_state = 1, .external_lex_state = 2},
  [205] = {.lex_state = 1, .external_lex_state = 2},
  [206] = {.lex_state = 1, .external_lex_state = 2},
  [207] = {.lex_state = 1, .external_lex_state = 2},
  [208] = {.lex_state = 1, .external_lex_state = 2},
  [209] = {.lex_state = 1, .external_lex_state = 2},
  [210] = {.lex_state = 1, .external_lex_state = 2},
  [211] = {.lex_state = 1, .external_lex_state = 2},
  [212] = {.lex_state = 33},
  [213] = {.lex_state = 1, .external_lex_state = 2},
  [214] = {.lex_state = 1, .external_lex_state = 2},
  [215] = {.lex_state = 1, .external_lex_state = 2},
  [216] = {.lex_state = 1, .external_lex_state = 2},
  [217] = {.lex_state = 1, .external_lex_state = 2},
  [218] = {.lex_state = 1, .external_lex_state = 2},
  [219] = {.lex_state = 1, .external_lex_state = 2},
  [220] = {.lex_state = 1, .external_lex_state = 2},
  [221] = {.lex_state = 1, .external_lex_state = 2},
  [222] = {.lex_state = 1, .external_lex_state = 2},
  [223] = {.lex_state = 1, .external_lex_state = 2},
  [224] = {.lex_state = 1, .external_lex_state = 2},
  [225] = {.lex_state = 1, .external_lex_state = 2},
  [226] = {.lex_state = 1, .external_lex_state = 2},
  [227] = {.lex_state = 1, .external_lex_state = 2},
  [228] = {.lex_state = 1, .external_lex_state = 2},
  [229] = {.lex_state = 1, .external_lex_state = 2},
  [230] = {.lex_state = 39, .external_lex_state = 3},
  [231] = {.lex_state = 39, .external_lex_state = 3},
  [232] = {.lex_state = 39, .external_lex_state = 3},
  [233] = {.lex_state = 1, .external_lex_state = 2},
  [234] = {.lex_state = 1, .external_lex_state = 2},
  [235] = {.lex_state = 1, .external_lex_state = 2},
  [236] = {.lex_state = 1, .external_lex_state = 2},
  [237] = {.lex_state = 1, .external_lex_state = 2},
  [238] = {.lex_state = 1, .external_lex_state = 2},
  [239] = {.lex_state = 1, .external_lex_state = 2},
  [240] = {.lex_state = 1, .external_lex_state = 2},
  [241] = {.lex_state = 1, .external_lex_state = 2},
  [242] = {.lex_state = 1, .external_lex_state = 2},
  [243] = {.lex_state = 1, .external_lex_state = 2},
  [244] = {.lex_state = 1, .external_lex_state = 2},
  [245] = {.lex_state = 39, .external_lex_state = 3},
  [246] = {.lex_state = 1, .external_lex_state = 2},
  [247] = {.lex_state = 1, .external_lex_state = 2},
  [248] = {.lex_state = 1, .external_lex_state = 2},
  [249] = {.lex_state = 1, .external_lex_state = 2},
  [250] = {.lex_state = 1, .external_lex_state = 2},
  [251] = {.lex_state = 1, .external_lex_state = 2},
  [252] = {.lex_state = 1, .external_lex_state = 2},
  [253] = {.lex_state = 1, .external_lex_state = 2},
  [254] = {.lex_state = 1, .external_lex_state = 2},
  [255] = {.lex_state = 1, .external_lex_state = 2},
  [256] = {.lex_state = 1, .external_lex_state = 2},
  [257] = {.lex_state = 1, .external_lex_state = 2},
  [258] = {.lex_state = 1, .external_lex_state = 2},
  [259] = {.lex_state = 1, .external_lex_state = 2},
  [260] = {.lex_state = 1, .external_lex_state = 2},
  [261] = {.lex_state = 1, .external_lex_state = 2},
  [262] = {.lex_state = 1, .external_lex_state = 2},
  [263] = {.lex_state = 1, .external_lex_state = 2},
  [264] = {.lex_state = 1, .external_lex_state = 2},
  [265] = {.lex_state = 1, .external_lex_state = 2},
  [266] = {.lex_state = 1, .external_lex_state = 2},
  [267] = {.lex_state = 0},
  [268] = {.lex_state = 33},
  [269] = {.lex_state = 39, .external_lex_state = 3},
  [270] = {.lex_state = 39, .external_lex_state = 3},
  [271] = {.lex_state = 35},
  [272] = {.lex_state = 39, .external_lex_state = 3},
  [273] = {.lex_state = 39, .external_lex_state = 3},
  [274] = {.lex_state = 39, .external_lex_state = 3},
  [275] = {.lex_state = 39, .external_lex_state = 3},
  [276] = {.lex_state = 35},
  [277] = {.lex_state = 39, .external_lex_state = 3},
  [278] = {.lex_state = 39, .external_lex_state = 3},
  [279] = {.lex_state = 39, .external_lex_state = 3},
  [280] = {.lex_state = 39, .external_lex_state = 3},
  [281] = {.lex_state = 39, .external_lex_state = 3},
  [282] = {.lex_state = 39, .external_lex_state = 3},
  [283] = {.lex_state = 33},
  [284] = {.lex_state = 39, .external_lex_state = 3},
  [285] = {.lex_state = 39, .external_lex_state = 3},
  [286] = {.lex_state = 39, .external_lex_state = 3},
  [287] = {.lex_state = 0},
  [288] = {.lex_state = 39},
  [289] = {.lex_state = 40},
  [290] = {.lex_state = 39, .external_lex_state = 3},
  [291] = {.lex_state = 39},
  [292] = {.lex_state = 39, .external_lex_state = 3},
  [293] = {.lex_state = 39, .external_lex_state = 3},
  [294] = {.lex_state = 39, .external_lex_state = 3},
  [295] = {.lex_state = 39, .external_lex_state = 3},
  [296] = {.lex_state = 39},
  [297] = {.lex_state = 39, .external_lex_state = 3},
  [298] = {.lex_state = 39, .external_lex_state = 3},
  [299] = {.lex_state = 39, .external_lex_state = 3},
  [300] = {.lex_state = 35},
  [301] = {.lex_state = 40},
  [302] = {.lex_state = 39, .external_lex_state = 3},
  [303] = {.lex_state = 35},
  [304] = {.lex_state = 0},
  [305] = {.lex_state = 35},
  [306] = {.lex_state = 39, .external_lex_state = 3},
  [307] = {.lex_state = 39, .external_lex_state = 3},
  [308] = {.lex_state = 39, .external_lex_state = 3},
  [309] = {.lex_state = 35},
  [310] = {.lex_state = 39, .external_lex_state = 3},
  [311] = {.lex_state = 39},
  [312] = {.lex_state = 39, .external_lex_state = 3},
  [313] = {.lex_state = 39},
  [314] = {.lex_state = 39},
  [315] = {.lex_state = 33},
  [316] = {.lex_state = 39},
  [317] = {.lex_state = 40},
  [318] = {.lex_state = 39},
  [319] = {.lex_state = 39},
  [320] = {.lex_state = 39},
  [321] = {.lex_state = 39},
  [322] = {.lex_state = 39, .external_lex_state = 3},
  [323] = {.lex_state = 39},
  [324] = {.lex_state = 39},
  [325] = {.lex_state = 39, .external_lex_state = 3},
  [326] = {.lex_state = 39},
  [327] = {.lex_state = 39},
  [328] = {.lex_state = 0},
  [329] = {.lex_state = 39, .external_lex_state = 3},
  [330] = {.lex_state = 39},
  [331] = {.lex_state = 39},
  [332] = {.lex_state = 39},
  [333] = {.lex_state = 39},
  [334] = {.lex_state = 0},
  [335] = {.lex_state = 34},
  [336] = {.lex_state = 33},
  [337] = {.lex_state = 36},
  [338] = {.lex_state = 0},
  [339] = {.lex_state = 35},
  [340] = {.lex_state = 33},
  [341] = {.lex_state = 39},
  [342] = {.lex_state = 33},
  [343] = {.lex_state = 33},
  [344] = {.lex_state = 33},
  [345] = {.lex_state = 39},
  [346] = {.lex_state = 39},
  [347] = {.lex_state = 39},
  [348] = {.lex_state = 39},
  [349] = {.lex_state = 39},
  [350] = {.lex_state = 39},
  [351] = {.lex_state = 40},
  [352] = {.lex_state = 34},
  [353] = {.lex_state = 39},
  [354] = {.lex_state = 0},
  [355] = {.lex_state = 39},
  [356] = {.lex_state = 39},
  [357] = {.lex_state = 39},
  [358] = {.lex_state = 39},
  [359] = {.lex_state = 0},
  [360] = {.lex_state = 39},
  [361] = {.lex_state = 39},
  [362] = {.lex_state = 39},
  [363] = {.lex_state = 39},
  [364] = {.lex_state = 36},
  [365] = {.lex_state = 39},
  [366] = {.lex_state = 39},
  [367] = {.lex_state = 39},
  [368] = {.lex_state = 39},
  [369] = {.lex_state = 39},
  [370] = {.lex_state = 39},
  [371] = {.lex_state = 39},
  [372] = {.lex_state = 39},
  [373] = {.lex_state = 39},
  [374] = {.lex_state = 39},
  [375] = {.lex_state = 39},
  [376] = {.lex_state = 39},
  [377] = {.lex_state = 39},
  [378] = {.lex_state = 39},
  [379] = {.lex_state = 39},
  [380] = {.lex_state = 39},
  [381] = {.lex_state = 39},
  [382] = {.lex_state = 39},
  [383] = {.lex_state = 39},
  [384] = {.lex_state = 39},
  [385] = {.lex_state = 39},
  [386] = {.lex_state = 39},
  [387] = {.lex_state = 39},
  [388] = {.lex_state = 39},
  [389] = {.lex_state = 39},
  [390] = {.lex_state = 39},
  [391] = {.lex_state = 39},
  [392] = {.lex_state = 0},
  [393] = {.lex_state = 39},
  [394] = {.lex_state = 39},
  [395] = {.lex_state = 39},
  [396] = {.lex_state = 39},
  [397] = {.lex_state = 39},
  [398] = {.lex_state = 39},
  [399] = {.lex_state = 0},
  [400] = {.lex_state = 34},
  [401] = {.lex_state = 39, .external_lex_state = 3},
  [402] = {.lex_state = 39, .external_lex_state = 3},
  [403] = {.lex_state = 39, .external_lex_state = 3},
  [404] = {.lex_state = 39, .external_lex_state = 3},
  [405] = {.lex_state = 33},
  [406] = {.lex_state = 36},
  [407] = {.lex_state = 36},
  [408] = {.lex_state = 39, .external_lex_state = 3},
  [409] = {.lex_state = 40},
  [410] = {.lex_state = 39, .external_lex_state = 3},
  [411] = {.lex_state = 34},
  [412] = {.lex_state = 34},
  [413] = {.lex_state = 37},
  [414] = {.lex_state = 39, .external_lex_state = 3},
  [415] = {.lex_state = 37},
  [416] = {.lex_state = 39, .external_lex_state = 3},
  [417] = {.lex_state = 39, .external_lex_state = 3},
  [418] = {.lex_state = 34},
  [419] = {.lex_state = 40},
  [420] = {.lex_state = 39, .external_lex_state = 3},
  [421] = {.lex_state = 39, .external_lex_state = 3},
  [422] = {.lex_state = 39},
  [423] = {.lex_state = 39},
  [424] = {.lex_state = 37},
  [425] = {.lex_state = 39, .external_lex_state = 3},
  [426] = {.lex_state = 39, .external_lex_state = 3},
  [427] = {.lex_state = 34},
  [428] = {.lex_state = 39, .external_lex_state = 3},
  [429] = {.lex_state = 39, .external_lex_state = 3},
  [430] = {.lex_state = 39, .external_lex_state = 3},
  [431] = {.lex_state = 40},
  [432] = {.lex_state = 39, .external_lex_state = 3},
  [433] = {.lex_state = 40},
  [434] = {.lex_state = 40},
  [435] = {.lex_state = 39, .external_lex_state = 3},
  [436] = {.lex_state = 39, .external_lex_state = 3},
  [437] = {.lex_state = 37},
  [438] = {.lex_state = 40},
  [439] = {.lex_state = 34},
  [440] = {.lex_state = 39},
  [441] = {.lex_state = 40},
  [442] = {.lex_state = 34},
  [443] = {.lex_state = 39},
  [444] = {.lex_state = 0},
  [445] = {.lex_state = 37},
  [446] = {.lex_state = 40},
  [447] = {.lex_state = 0},
  [448] = {.lex_state = 39},
  [449] = {.lex_state = 0},
  [450] = {.lex_state = 0},
  [451] = {.lex_state = 0},
  [452] = {.lex_state = 37},
  [453] = {.lex_state = 0},
  [454] = {.lex_state = 0},
  [455] = {.lex_state = 0},
  [456] = {.lex_state = 0},
  [457] = {.lex_state = 39},
  [458] = {.lex_state = 39},
  [459] = {.lex_state = 39},
  [460] = {.lex_state = 36},
  [461] = {.lex_state = 0},
  [462] = {.lex_state = 39},
  [463] = {.lex_state = 36},
  [464] = {.lex_state = 0},
  [465] = {.lex_state = 37},
  [466] = {.lex_state = 39},
  [467] = {.lex_state = 36},
  [468] = {.lex_state = 39},
  [469] = {.lex_state = 36},
  [470] = {.lex_state = 0},
  [471] = {.lex_state = 0},
  [472] = {.lex_state = 0},
  [473] = {.lex_state = 37},
  [474] = {.lex_state = 39},
  [475] = {.lex_state = 39},
  [476] = {.lex_state = 36},
  [477] = {.lex_state = 39},
  [478] = {.lex_state = 0},
  [479] = {.lex_state = 39},
  [480] = {.lex_state = 0},
  [481] = {.lex_state = 39},
  [482] = {.lex_state = 39},
  [483] = {.lex_state = 0},
  [484] = {.lex_state = 0},
  [485] = {.lex_state = 39},
  [486] = {.lex_state = 0},
  [487] = {.lex_state = 36},
  [488] = {.lex_state = 39},
  [489] = {.lex_state = 39},
  [490] = {.lex_state = 0},
  [491] = {.lex_state = 36},
  [492] = {.lex_state = 0},
  [493] = {.lex_state = 39},
  [494] = {.lex_state = 39},
  [495] = {.lex_state = 36},
  [496] = {.lex_state = 40},
  [497] = {.lex_state = 40},
  [498] = {.lex_state = 0},
  [499] = {.lex_state = 0},
  [500] = {.lex_state = 37},
  [501] = {.lex_state = 37},
  [502] = {.lex_state = 37},
  [503] = {.lex_state = 37},
  [504] = {.lex_state = 38},
  [505] = {.lex_state = 37},
  [506] = {.lex_state = 38},
  [507] = {.lex_state = 38},
  [508] = {.lex_state = 36},
  [509] = {.lex_state = 0},
  [510] = {.lex_state = 38},
  [511] = {.lex_state = 38},
  [512] = {.lex_state = 37},
  [513] = {.lex_state = 36},
  [514] = {.lex_state = 38},
  [515] = {.lex_state = 38},
  [516] = {.lex_state = 0},
  [517] = {.lex_state = 0},
  [518] = {.lex_state = 0},
  [519] = {.lex_state = 0},
  [520] = {.lex_state = 0},
  [521] = {.lex_state = 0},
  [522] = {.lex_state = 32},
  [523] = {.lex_state = 0},
  [524] = {.lex_state = 0},
  [525] = {.lex_state = 0},
  [526] = {.lex_state = 32},
  [527] = {.lex_state = 0},
  [528] = {.lex_state = 0},
  [529] = {.lex_state = 0},
  [530] = {.lex_state = 0},
  [531] = {.lex_state = 0},
  [532] = {.lex_state = 0},
  [533] = {.lex_state = 0},
  [534] = {.lex_state = 0},
  [535] = {.lex_state = 0},
  [536] = {.lex_state = 32},
  [537] = {.lex_state = 0},
  [538] = {.lex_state = 0},
  [539] = {.lex_state = 0},
  [540] = {.lex_state = 32},
  [541] = {.lex_state = 0},
  [542] = {.lex_state = 32},
  [543] = {.lex_state = 0},
  [544] = {.lex_state = 0},
  [545] = {.lex_state = 0},
  [546] = {.lex_state = 0},
  [547] = {.lex_state = 0},
  [548] = {.lex_state = 32},
  [549] = {.lex_state = 0},
  [550] = {.lex_state = 0},
  [551] = {.lex_state = 0},
  [552] = {.lex_state = 0},
  [553] = {.lex_state = 0},
  [554] = {.lex_state = 0},
  [555] = {.lex_state = 0},
  [556] = {.lex_state = 0},
  [557] = {.lex_state = 0},
  [558] = {.lex_state = 0},
  [559] = {.lex_state = 0},
  [560] = {.lex_state = 0},
  [561] = {.lex_state = 0},
  [562] = {.lex_state = 0},
  [563] = {.lex_state = 0},
  [564] = {.lex_state = 0},
  [565] = {.lex_state = 32},
  [566] = {.lex_state = 0},
  [567] = {.lex_state = 0},
  [568] = {.lex_state = 0},
  [569] = {.lex_state = 0},
  [570] = {.lex_state = 0},
  [571] = {.lex_state = 0},
  [572] = {.lex_state = 0},
  [573] = {.lex_state = 0},
  [574] = {.lex_state = 0},
  [575] = {.lex_state = 0},
  [576] = {.lex_state = 32},
  [577] = {.lex_state = 0},
  [578] = {.lex_state = 0},
  [579] = {.lex_state = 0},
  [580] = {.lex_state = 0},
  [581] = {.lex_state = 0},
  [582] = {.lex_state = 0},
  [583] = {.lex_state = 0},
  [584] = {.lex_state = 0},
  [585] = {.lex_state = 0},
  [586] = {.lex_state = 0},
  [587] = {.lex_state = 0},
  [588] = {.lex_state = 0},
  [589] = {.lex_state = 0},
  [590] = {.lex_state = 0},
  [591] = {.lex_state = 0},
  [592] = {.lex_state = 0},
  [593] = {.lex_state = 0},
  [594] = {.lex_state = 0},
  [595] = {.lex_state = 0},
  [596] = {.lex_state = 0},
  [597] = {.lex_state = 0},
  [598] = {.lex_state = 0},
  [599] = {.lex_state = 0},
  [600] = {.lex_state = 0},
  [601] = {.lex_state = 0},
  [602] = {.lex_state = 0},
  [603] = {.lex_state = 0},
  [604] = {.lex_state = 0},
  [605] = {.lex_state = 0},
  [606] = {.lex_state = 0},
  [607] = {.lex_state = 0},
  [608] = {.lex_state = 0},
  [609] = {.lex_state = 0},
  [610] = {.lex_state = 0},
  [611] = {.lex_state = 0},
  [612] = {.lex_state = 0},
  [613] = {.lex_state = 0},
  [614] = {.lex_state = 0},
  [615] = {.lex_state = 0},
  [616] = {.lex_state = 0},
  [617] = {.lex_state = 0},
  [618] = {.lex_state = 0},
  [619] = {.lex_state = 0},
  [620] = {.lex_state = 0},
  [621] = {.lex_state = 0},
  [622] = {.lex_state = 0},
  [623] = {.lex_state = 0},
  [624] = {.lex_state = 0},
  [625] = {.lex_state = 0},
  [626] = {.lex_state = 0},
  [627] = {.lex_state = 0},
  [628] = {.lex_state = 0},
  [629] = {.lex_state = 0},
  [630] = {.lex_state = 0},
  [631] = {.lex_state = 0},
  [632] = {.lex_state = 0},
  [633] = {.lex_state = 0},
  [634] = {.lex_state = 0},
  [635] = {.lex_state = 0},
  [636] = {.lex_state = 0},
  [637] = {.lex_state = 0},
  [638] = {.lex_state = 0},
  [639] = {.lex_state = 0},
  [640] = {.lex_state = 0},
  [641] = {.lex_state = 0},
  [642] = {.lex_state = 0},
  [643] = {.lex_state = 0},
  [644] = {.lex_state = 0},
  [645] = {.lex_state = 0},
  [646] = {.lex_state = 0},
  [647] = {.lex_state = 0},
  [648] = {.lex_state = 0},
  [649] = {.lex_state = 0},
  [650] = {.lex_state = 0},
  [651] = {.lex_state = 0},
  [652] = {.lex_state = 0},
  [653] = {.lex_state = 0},
  [654] = {.lex_state = 0},
  [655] = {.lex_state = 0},
  [656] = {.lex_state = 0},
  [657] = {.lex_state = 0},
  [658] = {.lex_state = 0},
  [659] = {.lex_state = 0},
  [660] = {.lex_state = 0},
  [661] = {.lex_state = 0},
  [662] = {.lex_state = 0},
  [663] = {.lex_state = 0},
  [664] = {.lex_state = 0},
  [665] = {.lex_state = 0},
  [666] = {.lex_state = 0},
  [667] = {.lex_state = 0},
  [668] = {.lex_state = 0},
  [669] = {.lex_state = 0},
  [670] = {.lex_state = 0},
  [671] = {.lex_state = 0},
  [672] = {.lex_state = 1, .external_lex_state = 2},
  [673] = {.lex_state = 0},
  [674] = {.lex_state = 13, .external_lex_state = 5},
  [675] = {.lex_state = 0},
  [676] = {.lex_state = 0},
  [677] = {.lex_state = 0},
  [678] = {.lex_state = 0},
  [679] = {.lex_state = 0},
  [680] = {.lex_state = 0},
  [681] = {.lex_state = 13, .external_lex_state = 6},
  [682] = {.lex_state = 0},
  [683] = {.lex_state = 0},
  [684] = {.lex_state = 0},
  [685] = {.lex_state = 35, .external_lex_state = 7},
  [686] = {.lex_state = 0},
  [687] = {.lex_state = 0},
  [688] = {.lex_state = 0},
  [689] = {.lex_state = 0},
  [690] = {.lex_state = 1, .external_lex_state = 2},
  [691] = {.lex_state = 35, .external_lex_state = 7},
  [692] = {.lex_state = 1, .external_lex_state = 2},
  [693] = {.lex_state = 0},
  [694] = {.lex_state = 1, .external_lex_state = 2},
  [695] = {.lex_state = 0},
  [696] = {.lex_state = 0},
  [697] = {.lex_state = 0},
  [698] = {.lex_state = 13, .external_lex_state = 5},
  [699] = {.lex_state = 13, .external_lex_state = 8},
  [700] = {.lex_state = 1, .external_lex_state = 2},
  [701] = {.lex_state = 33, .external_lex_state = 7},
  [702] = {.lex_state = 1, .external_lex_state = 2},
  [703] = {.lex_state = 13, .external_lex_state = 7},
  [704] = {.lex_state = 1, .external_lex_state = 2},
  [705] = {.lex_state = 0},
  [706] = {.lex_state = 1, .external_lex_state = 2},
  [707] = {.lex_state = 1, .external_lex_state = 2},
  [708] = {.lex_state = 1, .external_lex_state = 2},
  [709] = {.lex_state = 1, .external_lex_state = 2},
  [710] = {.lex_state = 35, .external_lex_state = 7},
  [711] = {.lex_state = 4},
  [712] = {.lex_state = 33, .external_lex_state = 7},
  [713] = {.lex_state = 1, .external_lex_state = 2},
  [714] = {.lex_state = 1, .external_lex_state = 2},
  [715] = {.lex_state = 13, .external_lex_state = 5},
  [716] = {.lex_state = 1, .external_lex_state = 2},
  [717] = {.lex_state = 13, .external_lex_state = 5},
  [718] = {.lex_state = 34, .external_lex_state = 7},
  [719] = {.lex_state = 13, .external_lex_state = 5},
  [720] = {.lex_state = 1, .external_lex_state = 2},
  [721] = {.lex_state = 5},
  [722] = {.lex_state = 13, .external_lex_state = 5},
  [723] = {.lex_state = 1, .external_lex_state = 2},
  [724] = {.lex_state = 13, .external_lex_state = 3},
  [725] = {.lex_state = 1, .external_lex_state = 2},
  [726] = {.lex_state = 1, .external_lex_state = 2},
  [727] = {.lex_state = 1, .external_lex_state = 2},
  [728] = {.lex_state = 13, .external_lex_state = 5},
  [729] = {.lex_state = 1, .external_lex_state = 2},
  [730] = {.lex_state = 1, .external_lex_state = 2},
  [731] = {.lex_state = 1, .external_lex_state = 2},
  [732] = {.lex_state = 1, .external_lex_state = 2},
  [733] = {.lex_state = 1, .external_lex_state = 2},
  [734] = {.lex_state = 1, .external_lex_state = 2},
  [735] = {.lex_state = 35, .external_lex_state = 7},
  [736] = {.lex_state = 13, .external_lex_state = 5},
  [737] = {.lex_state = 13, .external_lex_state = 4},
  [738] = {.lex_state = 1, .external_lex_state = 2},
  [739] = {.lex_state = 1, .external_lex_state = 2},
  [740] = {.lex_state = 1, .external_lex_state = 2},
  [741] = {.lex_state = 1, .external_lex_state = 2},
  [742] = {.lex_state = 1, .external_lex_state = 2},
  [743] = {.lex_state = 13, .external_lex_state = 7},
  [744] = {.lex_state = 1, .external_lex_state = 2},
  [745] = {.lex_state = 1, .external_lex_state = 2},
  [746] = {.lex_state = 13, .external_lex_state = 5},
  [747] = {.lex_state = 34, .external_lex_state = 7},
  [748] = {.lex_state = 13, .external_lex_state = 5},
  [749] = {.lex_state = 1, .external_lex_state = 2},
  [750] = {.lex_state = 4},
  [751] = {.lex_state = 13, .external_lex_state = 7},
  [752] = {.lex_state = 13, .external_lex_state = 7},
  [753] = {.lex_state = 33, .external_lex_state = 7},
  [754] = {.lex_state = 13, .external_lex_state = 5},
  [755] = {.lex_state = 13, .external_lex_state = 3},
  [756] = {.lex_state = 35, .external_lex_state = 7},
  [757] = {.lex_state = 13, .external_lex_state = 5},
  [758] = {.lex_state = 13, .external_lex_state = 5},
  [759] = {.lex_state = 35, .external_lex_state = 7},
  [760] = {.lex_state = 33, .external_lex_state = 7},
  [761] = {.lex_state = 0},
  [762] = {.lex_state = 35, .external_lex_state = 7},
  [763] = {.lex_state = 40, .external_lex_state = 7},
  [764] = {.lex_state = 35, .external_lex_state = 7},
  [765] = {.lex_state = 13, .external_lex_state = 7},
  [766] = {.lex_state = 0},
  [767] = {.lex_state = 35, .external_lex_state = 7},
  [768] = {.lex_state = 13},
  [769] = {.lex_state = 14, .external_lex_state = 4},
  [770] = {.lex_state = 6},
  [771] = {.lex_state = 13, .external_lex_state = 5},
  [772] = {.lex_state = 40, .external_lex_state = 7},
  [773] = {.lex_state = 35, .external_lex_state = 7},
  [774] = {.lex_state = 13, .external_lex_state = 7},
  [775] = {.lex_state = 14, .external_lex_state = 3},
  [776] = {.lex_state = 2},
  [777] = {.lex_state = 13, .external_lex_state = 7},
  [778] = {.lex_state = 13, .external_lex_state = 7},
  [779] = {.lex_state = 13, .external_lex_state = 2},
  [780] = {.lex_state = 5},
  [781] = {.lex_state = 13, .external_lex_state = 3},
  [782] = {.lex_state = 33, .external_lex_state = 7},
  [783] = {.lex_state = 0},
  [784] = {.lex_state = 0},
  [785] = {.lex_state = 0},
  [786] = {.lex_state = 0},
  [787] = {.lex_state = 13, .external_lex_state = 5},
  [788] = {.lex_state = 0},
  [789] = {.lex_state = 0},
  [790] = {.lex_state = 0},
  [791] = {.lex_state = 0},
  [792] = {.lex_state = 5},
  [793] = {.lex_state = 0},
  [794] = {.lex_state = 14, .external_lex_state = 2},
  [795] = {.lex_state = 13},
  [796] = {.lex_state = 0},
  [797] = {.lex_state = 0},
  [798] = {.lex_state = 6},
  [799] = {.lex_state = 13, .external_lex_state = 7},
  [800] = {.lex_state = 0},
  [801] = {.lex_state = 0},
  [802] = {.lex_state = 0},
  [803] = {.lex_state = 0},
  [804] = {.lex_state = 0},
  [805] = {.lex_state = 0},
  [806] = {.lex_state = 0},
  [807] = {.lex_state = 5},
  [808] = {.lex_state = 0},
  [809] = {.lex_state = 0},
  [810] = {.lex_state = 0},
  [811] = {.lex_state = 0},
  [812] = {.lex_state = 0},
  [813] = {.lex_state = 0},
  [814] = {.lex_state = 0},
  [815] = {.lex_state = 0},
  [816] = {.lex_state = 0},
  [817] = {.lex_state = 0},
  [818] = {.lex_state = 0},
  [819] = {.lex_state = 13, .external_lex_state = 5},
  [820] = {.lex_state = 0},
  [821] = {.lex_state = 0},
  [822] = {.lex_state = 0},
  [823] = {.lex_state = 14, .external_lex_state = 3},
  [824] = {.lex_state = 0},
  [825] = {.lex_state = 0},
  [826] = {.lex_state = 0},
  [827] = {.lex_state = 0},
  [828] = {.lex_state = 0},
  [829] = {.lex_state = 0},
  [830] = {.lex_state = 0},
  [831] = {.lex_state = 0},
  [832] = {.lex_state = 0},
  [833] = {.lex_state = 13, .external_lex_state = 5},
  [834] = {.lex_state = 0},
  [835] = {.lex_state = 0},
  [836] = {.lex_state = 4},
  [837] = {.lex_state = 0},
  [838] = {.lex_state = 13, .external_lex_state = 5},
  [839] = {.lex_state = 0},
  [840] = {.lex_state = 4},
  [841] = {.lex_state = 0},
  [842] = {.lex_state = 0},
  [843] = {.lex_state = 0},
  [844] = {.lex_state = 0},
  [845] = {.lex_state = 13, .external_lex_state = 3},
  [846] = {.lex_state = 0},
  [847] = {.lex_state = 0},
  [848] = {.lex_state = 0},
  [849] = {.lex_state = 0},
  [850] = {.lex_state = 0},
  [851] = {.lex_state = 0},
  [852] = {.lex_state = 13, .external_lex_state = 7},
  [853] = {.lex_state = 7},
  [854] = {.lex_state = 0},
  [855] = {.lex_state = 0},
  [856] = {.lex_state = 0},
  [857] = {.lex_state = 0},
  [858] = {.lex_state = 0},
  [859] = {.lex_state = 0},
  [860] = {.lex_state = 14},
  [861] = {.lex_state = 0},
  [862] = {.lex_state = 0},
  [863] = {.lex_state = 0},
  [864] = {.lex_state = 13, .external_lex_state = 7},
  [865] = {.lex_state = 13, .external_lex_state = 7},
  [866] = {.lex_state = 0},
  [867] = {.lex_state = 0},
  [868] = {.lex_state = 0},
  [869] = {.lex_state = 0},
  [870] = {.lex_state = 13, .external_lex_state = 3},
  [871] = {.lex_state = 13, .external_lex_state = 3},
  [872] = {.lex_state = 0},
  [873] = {.lex_state = 0},
  [874] = {.lex_state = 0},
  [875] = {.lex_state = 13, .external_lex_state = 3},
  [876] = {.lex_state = 0},
  [877] = {.lex_state = 0},
  [878] = {.lex_state = 13, .external_lex_state = 3},
  [879] = {.lex_state = 0},
  [880] = {.lex_state = 0},
  [881] = {.lex_state = 0},
  [882] = {.lex_state = 13, .external_lex_state = 5},
  [883] = {.lex_state = 0},
  [884] = {.lex_state = 0},
  [885] = {.lex_state = 0},
  [886] = {.lex_state = 0},
  [887] = {.lex_state = 0},
  [888] = {.lex_state = 0},
  [889] = {.lex_state = 0},
  [890] = {.lex_state = 33},
  [891] = {.lex_state = 0},
  [892] = {.lex_state = 0},
  [893] = {.lex_state = 0},
  [894] = {.lex_state = 13, .external_lex_state = 3},
  [895] = {.lex_state = 0},
  [896] = {.lex_state = 2},
  [897] = {.lex_state = 0},
  [898] = {.lex_state = 0},
  [899] = {.lex_state = 0},
  [900] = {.lex_state = 0},
  [901] = {.lex_state = 35},
  [902] = {.lex_state = 0},
  [903] = {.lex_state = 13, .external_lex_state = 5},
  [904] = {.lex_state = 0},
  [905] = {.lex_state = 0},
  [906] = {.lex_state = 13, .external_lex_state = 5},
  [907] = {.lex_state = 0},
  [908] = {.lex_state = 13, .external_lex_state = 5},
  [909] = {.lex_state = 13, .external_lex_state = 5},
  [910] = {.lex_state = 33, .external_lex_state = 7},
  [911] = {.lex_state = 0},
  [912] = {.lex_state = 13, .external_lex_state = 5},
  [913] = {.lex_state = 0},
  [914] = {.lex_state = 0},
  [915] = {.lex_state = 13, .external_lex_state = 5},
  [916] = {.lex_state = 13, .external_lex_state = 5},
  [917] = {.lex_state = 0},
  [918] = {.lex_state = 33, .external_lex_state = 7},
  [919] = {.lex_state = 0},
  [920] = {.lex_state = 0},
  [921] = {.lex_state = 0},
  [922] = {.lex_state = 0},
  [923] = {.lex_state = 0},
  [924] = {.lex_state = 0},
  [925] = {.lex_state = 13, .external_lex_state = 5},
  [926] = {.lex_state = 0},
  [927] = {.lex_state = 0},
  [928] = {.lex_state = 0},
  [929] = {.lex_state = 0},
  [930] = {.lex_state = 13, .external_lex_state = 5},
  [931] = {.lex_state = 13, .external_lex_state = 5},
  [932] = {.lex_state = 0},
  [933] = {.lex_state = 13, .external_lex_state = 5},
  [934] = {.lex_state = 0},
  [935] = {.lex_state = 13, .external_lex_state = 5},
  [936] = {.lex_state = 0},
  [937] = {.lex_state = 0},
  [938] = {.lex_state = 0},
  [939] = {.lex_state = 0},
  [940] = {.lex_state = 0},
  [941] = {.lex_state = 0},
  [942] = {.lex_state = 0},
  [943] = {.lex_state = 0},
  [944] = {.lex_state = 0},
  [945] = {.lex_state = 13, .external_lex_state = 5},
  [946] = {.lex_state = 1, .external_lex_state = 2},
  [947] = {.lex_state = 0},
  [948] = {.lex_state = 0},
  [949] = {.lex_state = 0},
  [950] = {.lex_state = 13, .external_lex_state = 5},
  [951] = {.lex_state = 13, .external_lex_state = 5},
  [952] = {.lex_state = 34, .external_lex_state = 7},
  [953] = {.lex_state = 34},
  [954] = {.lex_state = 0},
  [955] = {.lex_state = 0},
  [956] = {.lex_state = 40, .external_lex_state = 7},
  [957] = {.lex_state = 0},
  [958] = {.lex_state = 34, .external_lex_state = 7},
  [959] = {.lex_state = 13, .external_lex_state = 3},
  [960] = {.lex_state = 0},
  [961] = {.lex_state = 3},
  [962] = {.lex_state = 35, .external_lex_state = 7},
  [963] = {.lex_state = 0},
  [964] = {.lex_state = 0},
  [965] = {.lex_state = 0},
  [966] = {.lex_state = 0},
  [967] = {.lex_state = 0},
  [968] = {.lex_state = 36, .external_lex_state = 7},
  [969] = {.lex_state = 40, .external_lex_state = 7},
  [970] = {.lex_state = 0},
  [971] = {.lex_state = 0},
  [972] = {.lex_state = 13, .external_lex_state = 7},
  [973] = {.lex_state = 0},
  [974] = {.lex_state = 0},
  [975] = {.lex_state = 33, .external_lex_state = 7},
  [976] = {.lex_state = 0},
  [977] = {.lex_state = 0},
  [978] = {.lex_state = 0},
  [979] = {.lex_state = 33, .external_lex_state = 7},
  [980] = {.lex_state = 0},
  [981] = {.lex_state = 36, .external_lex_state = 7},
  [982] = {.lex_state = 0},
  [983] = {.lex_state = 0},
  [984] = {.lex_state = 0},
  [985] = {.lex_state = 33, .external_lex_state = 7},
  [986] = {.lex_state = 0},
  [987] = {.lex_state = 0},
  [988] = {.lex_state = 0},
  [989] = {.lex_state = 13, .external_lex_state = 7},
  [990] = {.lex_state = 2},
  [991] = {.lex_state = 14, .external_lex_state = 3},
  [992] = {.lex_state = 13, .external_lex_state = 7},
  [993] = {.lex_state = 13, .external_lex_state = 7},
  [994] = {.lex_state = 13, .external_lex_state = 7},
  [995] = {.lex_state = 14, .external_lex_state = 3},
  [996] = {.lex_state = 13, .external_lex_state = 3},
  [997] = {.lex_state = 6},
  [998] = {.lex_state = 13, .external_lex_state = 7},
  [999] = {.lex_state = 13, .external_lex_state = 7},
  [1000] = {.lex_state = 7},
  [1001] = {.lex_state = 13, .external_lex_state = 7},
  [1002] = {.lex_state = 13, .external_lex_state = 7},
  [1003] = {.lex_state = 6},
  [1004] = {.lex_state = 13, .external_lex_state = 7},
  [1005] = {.lex_state = 13, .external_lex_state = 7},
  [1006] = {.lex_state = 5},
  [1007] = {.lex_state = 13, .external_lex_state = 7},
  [1008] = {.lex_state = 3},
  [1009] = {.lex_state = 14},
  [1010] = {.lex_state = 16},
  [1011] = {.lex_state = 5},
  [1012] = {.lex_state = 5},
  [1013] = {.lex_state = 16},
  [1014] = {.lex_state = 36, .external_lex_state = 7},
  [1015] = {.lex_state = 5},
  [1016] = {.lex_state = 13, .external_lex_state = 7},
  [1017] = {.lex_state = 13, .external_lex_state = 7},
  [1018] = {.lex_state = 13, .external_lex_state = 7},
  [1019] = {.lex_state = 0, .external_lex_state = 7},
  [1020] = {.lex_state = 13, .external_lex_state = 7},
  [1021] = {.lex_state = 13, .external_lex_state = 7},
  [1022] = {.lex_state = 33, .external_lex_state = 7},
  [1023] = {.lex_state = 34},
  [1024] = {.lex_state = 4},
  [1025] = {.lex_state = 13, .external_lex_state = 7},
  [1026] = {.lex_state = 13, .external_lex_state = 3},
  [1027] = {.lex_state = 36, .external_lex_state = 7},
  [1028] = {.lex_state = 13, .external_lex_state = 7},
  [1029] = {.lex_state = 13},
  [1030] = {.lex_state = 13},
  [1031] = {.lex_state = 11},
  [1032] = {.lex_state = 4},
  [1033] = {.lex_state = 0, .external_lex_state = 7},
  [1034] = {.lex_state = 13, .external_lex_state = 7},
  [1035] = {.lex_state = 4},
  [1036] = {.lex_state = 13, .external_lex_state = 7},
  [1037] = {.lex_state = 5},
  [1038] = {.lex_state = 11},
  [1039] = {.lex_state = 4},
  [1040] = {.lex_state = 0, .external_lex_state = 7},
  [1041] = {.lex_state = 0},
  [1042] = {.lex_state = 14, .external_lex_state = 3},
  [1043] = {.lex_state = 5},
  [1044] = {.lex_state = 13},
  [1045] = {.lex_state = 14, .external_lex_state = 3},
  [1046] = {.lex_state = 13, .external_lex_state = 7},
  [1047] = {.lex_state = 13, .external_lex_state = 7},
  [1048] = {.lex_state = 13, .external_lex_state = 7},
  [1049] = {.lex_state = 13, .external_lex_state = 3},
  [1050] = {.lex_state = 0, .external_lex_state = 7},
  [1051] = {.lex_state = 13, .external_lex_state = 7},
  [1052] = {.lex_state = 14, .external_lex_state = 3},
  [1053] = {.lex_state = 34, .external_lex_state = 7},
  [1054] = {.lex_state = 13, .external_lex_state = 7},
  [1055] = {.lex_state = 13},
  [1056] = {.lex_state = 13, .external_lex_state = 7},
  [1057] = {.lex_state = 14, .external_lex_state = 3},
  [1058] = {.lex_state = 13, .external_lex_state = 7},
  [1059] = {.lex_state = 13, .external_lex_state = 7},
  [1060] = {.lex_state = 37, .external_lex_state = 7},
  [1061] = {.lex_state = 13, .external_lex_state = 7},
  [1062] = {.lex_state = 13, .external_lex_state = 7},
  [1063] = {.lex_state = 13, .external_lex_state = 7},
  [1064] = {.lex_state = 33},
  [1065] = {.lex_state = 34, .external_lex_state = 7},
  [1066] = {.lex_state = 13, .external_lex_state = 7},
  [1067] = {.lex_state = 13, .external_lex_state = 7},
  [1068] = {.lex_state = 13, .external_lex_state = 7},
  [1069] = {.lex_state = 13, .external_lex_state = 7},
  [1070] = {.lex_state = 4},
  [1071] = {.lex_state = 13},
  [1072] = {.lex_state = 13, .external_lex_state = 7},
  [1073] = {.lex_state = 13, .external_lex_state = 3},
  [1074] = {.lex_state = 0, .external_lex_state = 7},
  [1075] = {.lex_state = 13, .external_lex_state = 7},
  [1076] = {.lex_state = 4},
  [1077] = {.lex_state = 40, .external_lex_state = 7},
  [1078] = {.lex_state = 13, .external_lex_state = 7},
  [1079] = {.lex_state = 40, .external_lex_state = 7},
  [1080] = {.lex_state = 40, .external_lex_state = 7},
  [1081] = {.lex_state = 34, .external_lex_state = 7},
  [1082] = {.lex_state = 13, .external_lex_state = 7},
  [1083] = {.lex_state = 37, .external_lex_state = 7},
  [1084] = {.lex_state = 13, .external_lex_state = 7},
  [1085] = {.lex_state = 13, .external_lex_state = 7},
  [1086] = {.lex_state = 34, .external_lex_state = 7},
  [1087] = {.lex_state = 13, .external_lex_state = 7},
  [1088] = {.lex_state = 13, .external_lex_state = 7},
  [1089] = {.lex_state = 0, .external_lex_state = 7},
  [1090] = {.lex_state = 37, .external_lex_state = 7},
  [1091] = {.lex_state = 13, .external_lex_state = 7},
  [1092] = {.lex_state = 13, .external_lex_state = 7},
  [1093] = {.lex_state = 13},
  [1094] = {.lex_state = 13, .external_lex_state = 7},
  [1095] = {.lex_state = 0, .external_lex_state = 7},
  [1096] = {.lex_state = 34, .external_lex_state = 7},
  [1097] = {.lex_state = 13, .external_lex_state = 7},
  [1098] = {.lex_state = 14, .external_lex_state = 3},
  [1099] = {.lex_state = 13, .external_lex_state = 7},
  [1100] = {.lex_state = 34, .external_lex_state = 7},
  [1101] = {.lex_state = 13, .external_lex_state = 7},
  [1102] = {.lex_state = 13, .external_lex_state = 7},
  [1103] = {.lex_state = 13, .external_lex_state = 7},
  [1104] = {.lex_state = 13, .external_lex_state = 7},
  [1105] = {.lex_state = 13, .external_lex_state = 7},
  [1106] = {.lex_state = 34, .external_lex_state = 7},
  [1107] = {.lex_state = 13, .external_lex_state = 7},
  [1108] = {.lex_state = 13, .external_lex_state = 7},
  [1109] = {.lex_state = 14, .external_lex_state = 3},
  [1110] = {.lex_state = 2},
  [1111] = {.lex_state = 13, .external_lex_state = 7},
  [1112] = {.lex_state = 13, .external_lex_state = 7},
  [1113] = {.lex_state = 13, .external_lex_state = 7},
  [1114] = {.lex_state = 35},
  [1115] = {.lex_state = 37, .external_lex_state = 7},
  [1116] = {.lex_state = 13},
  [1117] = {.lex_state = 13, .external_lex_state = 3},
  [1118] = {.lex_state = 13},
  [1119] = {.lex_state = 9},
  [1120] = {.lex_state = 17},
  [1121] = {.lex_state = 6},
  [1122] = {.lex_state = 13},
  [1123] = {.lex_state = 6},
  [1124] = {.lex_state = 17},
  [1125] = {.lex_state = 2},
  [1126] = {.lex_state = 6},
  [1127] = {.lex_state = 13},
  [1128] = {.lex_state = 2},
  [1129] = {.lex_state = 13},
  [1130] = {.lex_state = 14},
  [1131] = {.lex_state = 2},
  [1132] = {.lex_state = 14},
  [1133] = {.lex_state = 9},
  [1134] = {.lex_state = 2},
  [1135] = {.lex_state = 13},
  [1136] = {.lex_state = 14},
  [1137] = {.lex_state = 14},
  [1138] = {.lex_state = 13},
  [1139] = {.lex_state = 13},
  [1140] = {.lex_state = 13, .external_lex_state = 3},
  [1141] = {.lex_state = 2},
  [1142] = {.lex_state = 13},
  [1143] = {.lex_state = 14},
  [1144] = {.lex_state = 14},
  [1145] = {.lex_state = 13, .external_lex_state = 3},
  [1146] = {.lex_state = 34},
  [1147] = {.lex_state = 13},
  [1148] = {.lex_state = 13},
  [1149] = {.lex_state = 16},
  [1150] = {.lex_state = 2},
  [1151] = {.lex_state = 13, .external_lex_state = 3},
  [1152] = {.lex_state = 13, .external_lex_state = 3},
  [1153] = {.lex_state = 13},
  [1154] = {.lex_state = 6},
  [1155] = {.lex_state = 6},
  [1156] = {.lex_state = 13},
  [1157] = {.lex_state = 13},
  [1158] = {.lex_state = 13},
  [1159] = {.lex_state = 13},
  [1160] = {.lex_state = 13},
  [1161] = {.lex_state = 13},
  [1162] = {.lex_state = 6},
  [1163] = {.lex_state = 5},
  [1164] = {.lex_state = 13},
  [1165] = {.lex_state = 13},
  [1166] = {.lex_state = 13},
  [1167] = {.lex_state = 13},
  [1168] = {.lex_state = 13, .external_lex_state = 3},
  [1169] = {.lex_state = 13},
  [1170] = {.lex_state = 13},
  [1171] = {.lex_state = 34},
  [1172] = {.lex_state = 16},
  [1173] = {.lex_state = 33},
  [1174] = {.lex_state = 14, .external_lex_state = 3},
  [1175] = {.lex_state = 13},
  [1176] = {.lex_state = 13, .external_lex_state = 3},
  [1177] = {.lex_state = 11},
  [1178] = {.lex_state = 13},
  [1179] = {.lex_state = 13},
  [1180] = {.lex_state = 30, .external_lex_state = 7},
  [1181] = {.lex_state = 35},
  [1182] = {.lex_state = 13},
  [1183] = {.lex_state = 30, .external_lex_state = 7},
  [1184] = {.lex_state = 13},
  [1185] = {.lex_state = 30, .external_lex_state = 7},
  [1186] = {.lex_state = 13, .external_lex_state = 3},
  [1187] = {.lex_state = 13},
  [1188] = {.lex_state = 13},
  [1189] = {.lex_state = 7},
  [1190] = {.lex_state = 13},
  [1191] = {.lex_state = 13, .external_lex_state = 3},
  [1192] = {.lex_state = 4},
  [1193] = {.lex_state = 13},
  [1194] = {.lex_state = 13},
  [1195] = {.lex_state = 35},
  [1196] = {.lex_state = 13},
  [1197] = {.lex_state = 13},
  [1198] = {.lex_state = 3},
  [1199] = {.lex_state = 13},
  [1200] = {.lex_state = 13, .external_lex_state = 3},
  [1201] = {.lex_state = 0},
  [1202] = {.lex_state = 11},
  [1203] = {.lex_state = 13},
  [1204] = {.lex_state = 13, .external_lex_state = 3},
  [1205] = {.lex_state = 13},
  [1206] = {.lex_state = 14, .external_lex_state = 3},
  [1207] = {.lex_state = 13, .external_lex_state = 3},
  [1208] = {.lex_state = 13},
  [1209] = {.lex_state = 7},
  [1210] = {.lex_state = 14, .external_lex_state = 3},
  [1211] = {.lex_state = 13},
  [1212] = {.lex_state = 13},
  [1213] = {.lex_state = 36, .external_lex_state = 7},
  [1214] = {.lex_state = 14, .external_lex_state = 3},
  [1215] = {.lex_state = 33},
  [1216] = {.lex_state = 13, .external_lex_state = 3},
  [1217] = {.lex_state = 13, .external_lex_state = 3},
  [1218] = {.lex_state = 30, .external_lex_state = 7},
  [1219] = {.lex_state = 36, .external_lex_state = 7},
  [1220] = {.lex_state = 13, .external_lex_state = 3},
  [1221] = {.lex_state = 13, .external_lex_state = 3},
  [1222] = {.lex_state = 13, .external_lex_state = 3},
  [1223] = {.lex_state = 3},
  [1224] = {.lex_state = 13, .external_lex_state = 3},
  [1225] = {.lex_state = 13, .external_lex_state = 3},
  [1226] = {.lex_state = 36, .external_lex_state = 7},
  [1227] = {.lex_state = 13, .external_lex_state = 3},
  [1228] = {.lex_state = 0},
  [1229] = {.lex_state = 33},
  [1230] = {.lex_state = 13},
  [1231] = {.lex_state = 0},
  [1232] = {.lex_state = 0, .external_lex_state = 7},
  [1233] = {.lex_state = 14, .external_lex_state = 3},
  [1234] = {.lex_state = 3},
  [1235] = {.lex_state = 14, .external_lex_state = 3},
  [1236] = {.lex_state = 13},
  [1237] = {.lex_state = 13},
  [1238] = {.lex_state = 0},
  [1239] = {.lex_state = 0, .external_lex_state = 7},
  [1240] = {.lex_state = 7},
  [1241] = {.lex_state = 35},
  [1242] = {.lex_state = 18},
  [1243] = {.lex_state = 7},
  [1244] = {.lex_state = 10},
  [1245] = {.lex_state = 3},
  [1246] = {.lex_state = 10},
  [1247] = {.lex_state = 7},
  [1248] = {.lex_state = 0},
  [1249] = {.lex_state = 13},
  [1250] = {.lex_state = 33},
  [1251] = {.lex_state = 18},
  [1252] = {.lex_state = 13},
  [1253] = {.lex_state = 3},
  [1254] = {.lex_state = 0},
  [1255] = {.lex_state = 7},
  [1256] = {.lex_state = 0, .external_lex_state = 7},
  [1257] = {.lex_state = 0, .external_lex_state = 7},
  [1258] = {.lex_state = 0},
  [1259] = {.lex_state = 14, .external_lex_state = 3},
  [1260] = {.lex_state = 34},
  [1261] = {.lex_state = 33},
  [1262] = {.lex_state = 40},
  [1263] = {.lex_state = 35},
  [1264] = {.lex_state = 0},
  [1265] = {.lex_state = 0, .external_lex_state = 7},
  [1266] = {.lex_state = 34},
  [1267] = {.lex_state = 35},
  [1268] = {.lex_state = 0},
  [1269] = {.lex_state = 13},
  [1270] = {.lex_state = 0, .external_lex_state = 7},
  [1271] = {.lex_state = 37},
  [1272] = {.lex_state = 14, .external_lex_state = 3},
  [1273] = {.lex_state = 14, .external_lex_state = 3},
  [1274] = {.lex_state = 0, .external_lex_state = 7},
  [1275] = {.lex_state = 35},
  [1276] = {.lex_state = 0, .external_lex_state = 7},
  [1277] = {.lex_state = 14, .external_lex_state = 3},
  [1278] = {.lex_state = 13},
  [1279] = {.lex_state = 33},
  [1280] = {.lex_state = 30, .external_lex_state = 7},
  [1281] = {.lex_state = 0},
  [1282] = {.lex_state = 14, .external_lex_state = 3},
  [1283] = {.lex_state = 36},
  [1284] = {.lex_state = 13},
  [1285] = {.lex_state = 14, .external_lex_state = 3},
  [1286] = {.lex_state = 13},
  [1287] = {.lex_state = 11},
  [1288] = {.lex_state = 33},
  [1289] = {.lex_state = 13},
  [1290] = {.lex_state = 37, .external_lex_state = 7},
  [1291] = {.lex_state = 0, .external_lex_state = 7},
  [1292] = {.lex_state = 14},
  [1293] = {.lex_state = 13},
  [1294] = {.lex_state = 3},
  [1295] = {.lex_state = 13},
  [1296] = {.lex_state = 13},
  [1297] = {.lex_state = 0, .external_lex_state = 7},
  [1298] = {.lex_state = 0, .external_lex_state = 7},
  [1299] = {.lex_state = 0, .external_lex_state = 7},
  [1300] = {.lex_state = 9},
  [1301] = {.lex_state = 14, .external_lex_state = 3},
  [1302] = {.lex_state = 37, .external_lex_state = 7},
  [1303] = {.lex_state = 14, .external_lex_state = 3},
  [1304] = {.lex_state = 14},
  [1305] = {.lex_state = 13},
  [1306] = {.lex_state = 40},
  [1307] = {.lex_state = 14, .external_lex_state = 3},
  [1308] = {.lex_state = 37, .external_lex_state = 7},
  [1309] = {.lex_state = 14, .external_lex_state = 3},
  [1310] = {.lex_state = 3},
  [1311] = {.lex_state = 13},
  [1312] = {.lex_state = 0},
  [1313] = {.lex_state = 13},
  [1314] = {.lex_state = 14, .external_lex_state = 7},
  [1315] = {.lex_state = 0, .external_lex_state = 7},
  [1316] = {.lex_state = 0, .external_lex_state = 7},
  [1317] = {.lex_state = 13},
  [1318] = {.lex_state = 13},
  [1319] = {.lex_state = 14, .external_lex_state = 7},
  [1320] = {.lex_state = 0, .external_lex_state = 7},
  [1321] = {.lex_state = 13},
  [1322] = {.lex_state = 14, .external_lex_state = 3},
  [1323] = {.lex_state = 0},
  [1324] = {.lex_state = 0},
  [1325] = {.lex_state = 15, .external_lex_state = 7},
  [1326] = {.lex_state = 0, .external_lex_state = 7},
  [1327] = {.lex_state = 0},
  [1328] = {.lex_state = 13},
  [1329] = {.lex_state = 14},
  [1330] = {.lex_state = 14, .external_lex_state = 3},
  [1331] = {.lex_state = 33},
  [1332] = {.lex_state = 0, .external_lex_state = 7},
  [1333] = {.lex_state = 0, .external_lex_state = 7},
  [1334] = {.lex_state = 14, .external_lex_state = 7},
  [1335] = {.lex_state = 3},
  [1336] = {.lex_state = 36},
  [1337] = {.lex_state = 14, .external_lex_state = 7},
  [1338] = {.lex_state = 0},
  [1339] = {.lex_state = 2},
  [1340] = {.lex_state = 14, .external_lex_state = 3},
  [1341] = {.lex_state = 0},
  [1342] = {.lex_state = 0, .external_lex_state = 7},
  [1343] = {.lex_state = 15, .external_lex_state = 7},
  [1344] = {.lex_state = 13},
  [1345] = {.lex_state = 0, .external_lex_state = 7},
  [1346] = {.lex_state = 13},
  [1347] = {.lex_state = 13},
  [1348] = {.lex_state = 34},
  [1349] = {.lex_state = 34},
  [1350] = {.lex_state = 17},
  [1351] = {.lex_state = 13},
  [1352] = {.lex_state = 0},
  [1353] = {.lex_state = 34},
  [1354] = {.lex_state = 14, .external_lex_state = 3},
  [1355] = {.lex_state = 14, .external_lex_state = 3},
  [1356] = {.lex_state = 0},
  [1357] = {.lex_state = 34},
  [1358] = {.lex_state = 37},
  [1359] = {.lex_state = 6},
  [1360] = {.lex_state = 13},
  [1361] = {.lex_state = 35},
  [1362] = {.lex_state = 13},
  [1363] = {.lex_state = 13},
  [1364] = {.lex_state = 14, .external_lex_state = 3},
  [1365] = {.lex_state = 14},
  [1366] = {.lex_state = 9},
  [1367] = {.lex_state = 15, .external_lex_state = 7},
  [1368] = {.lex_state = 35},
  [1369] = {.lex_state = 13},
  [1370] = {.lex_state = 13},
  [1371] = {.lex_state = 11},
  [1372] = {.lex_state = 16},
  [1373] = {.lex_state = 17},
  [1374] = {.lex_state = 13},
  [1375] = {.lex_state = 15, .external_lex_state = 7},
  [1376] = {.lex_state = 0},
  [1377] = {.lex_state = 14, .external_lex_state = 3},
  [1378] = {.lex_state = 13},
  [1379] = {.lex_state = 0, .external_lex_state = 7},
  [1380] = {.lex_state = 7},
  [1381] = {.lex_state = 16},
  [1382] = {.lex_state = 16},
  [1383] = {.lex_state = 14, .external_lex_state = 3},
  [1384] = {.lex_state = 11},
  [1385] = {.lex_state = 7},
  [1386] = {.lex_state = 0},
  [1387] = {.lex_state = 0},
  [1388] = {.lex_state = 37},
  [1389] = {.lex_state = 12},
  [1390] = {.lex_state = 0, .external_lex_state = 7},
  [1391] = {.lex_state = 14},
  [1392] = {.lex_state = 14},
  [1393] = {.lex_state = 14},
  [1394] = {.lex_state = 38},
  [1395] = {.lex_state = 14},
  [1396] = {.lex_state = 14},
  [1397] = {.lex_state = 15, .external_lex_state = 7},
  [1398] = {.lex_state = 14},
  [1399] = {.lex_state = 38},
  [1400] = {.lex_state = 14},
  [1401] = {.lex_state = 14},
  [1402] = {.lex_state = 40},
  [1403] = {.lex_state = 14},
  [1404] = {.lex_state = 12},
  [1405] = {.lex_state = 35},
  [1406] = {.lex_state = 14},
  [1407] = {.lex_state = 14},
  [1408] = {.lex_state = 0},
  [1409] = {.lex_state = 37},
  [1410] = {.lex_state = 12},
  [1411] = {.lex_state = 14},
  [1412] = {.lex_state = 14},
  [1413] = {.lex_state = 12},
  [1414] = {.lex_state = 0},
  [1415] = {.lex_state = 12},
  [1416] = {.lex_state = 10},
  [1417] = {.lex_state = 14},
  [1418] = {.lex_state = 30, .external_lex_state = 7},
  [1419] = {.lex_state = 14},
  [1420] = {.lex_state = 34},
  [1421] = {.lex_state = 38},
  [1422] = {.lex_state = 18},
  [1423] = {.lex_state = 14},
  [1424] = {.lex_state = 38},
  [1425] = {.lex_state = 40},
  [1426] = {.lex_state = 3},
  [1427] = {.lex_state = 12},
  [1428] = {.lex_state = 38},
  [1429] = {.lex_state = 14, .external_lex_state = 7},
  [1430] = {.lex_state = 38},
  [1431] = {.lex_state = 36},
  [1432] = {.lex_state = 26},
  [1433] = {.lex_state = 0, .external_lex_state = 7},
  [1434] = {.lex_state = 14},
  [1435] = {.lex_state = 26},
  [1436] = {.lex_state = 7},
  [1437] = {.lex_state = 38},
  [1438] = {.lex_state = 14},
  [1439] = {.lex_state = 14},
  [1440] = {.lex_state = 38},
  [1441] = {.lex_state = 38},
  [1442] = {.lex_state = 26},
  [1443] = {.lex_state = 10},
  [1444] = {.lex_state = 38},
  [1445] = {.lex_state = 9},
  [1446] = {.lex_state = 0},
  [1447] = {.lex_state = 14},
  [1448] = {.lex_state = 9},
  [1449] = {.lex_state = 26},
  [1450] = {.lex_state = 38},
  [1451] = {.lex_state = 33},
  [1452] = {.lex_state = 9},
  [1453] = {.lex_state = 14},
  [1454] = {.lex_state = 11},
  [1455] = {.lex_state = 38},
  [1456] = {.lex_state = 36},
  [1457] = {.lex_state = 17},
  [1458] = {.lex_state = 13},
  [1459] = {.lex_state = 30, .external_lex_state = 7},
  [1460] = {.lex_state = 21},
  [1461] = {.lex_state = 17},
  [1462] = {.lex_state = 17},
  [1463] = {.lex_state = 21},
  [1464] = {.lex_state = 18},
  [1465] = {.lex_state = 21},
  [1466] = {.lex_state = 21},
  [1467] = {.lex_state = 18},
  [1468] = {.lex_state = 18},
  [1469] = {.lex_state = 18},
  [1470] = {.lex_state = 29, .external_lex_state = 7},
  [1471] = {.lex_state = 37},
  [1472] = {.lex_state = 0},
  [1473] = {.lex_state = 0},
  [1474] = {.lex_state = 36},
  [1475] = {.lex_state = 36},
  [1476] = {.lex_state = 12},
  [1477] = {.lex_state = 12},
  [1478] = {.lex_state = 36},
  [1479] = {.lex_state = 21},
  [1480] = {.lex_state = 12},
  [1481] = {.lex_state = 12},
  [1482] = {.lex_state = 0},
  [1483] = {.lex_state = 0},
  [1484] = {.lex_state = 12},
  [1485] = {.lex_state = 29, .external_lex_state = 7},
  [1486] = {.lex_state = 26},
  [1487] = {.lex_state = 0, .external_lex_state = 7},
  [1488] = {.lex_state = 11},
  [1489] = {.lex_state = 29, .external_lex_state = 7},
  [1490] = {.lex_state = 11},
  [1491] = {.lex_state = 29, .external_lex_state = 7},
  [1492] = {.lex_state = 14, .external_lex_state = 7},
  [1493] = {.lex_state = 0},
  [1494] = {.lex_state = 0},
  [1495] = {.lex_state = 29, .external_lex_state = 7},
  [1496] = {.lex_state = 0},
  [1497] = {.lex_state = 27},
  [1498] = {.lex_state = 19},
  [1499] = {.lex_state = 27},
  [1500] = {.lex_state = 11},
  [1501] = {.lex_state = 0},
  [1502] = {.lex_state = 0, .external_lex_state = 7},
  [1503] = {.lex_state = 0},
  [1504] = {.lex_state = 19},
  [1505] = {.lex_state = 14, .external_lex_state = 7},
  [1506] = {.lex_state = 0},
  [1507] = {.lex_state = 27},
  [1508] = {.lex_state = 29, .external_lex_state = 7},
  [1509] = {.lex_state = 37},
  [1510] = {.lex_state = 29, .external_lex_state = 7},
  [1511] = {.lex_state = 0},
  [1512] = {.lex_state = 0},
  [1513] = {.lex_state = 12},
  [1514] = {.lex_state = 19},
  [1515] = {.lex_state = 0},
  [1516] = {.lex_state = 19},
  [1517] = {.lex_state = 12},
  [1518] = {.lex_state = 10},
  [1519] = {.lex_state = 0},
  [1520] = {.lex_state = 37},
  [1521] = {.lex_state = 15, .external_lex_state = 7},
  [1522] = {.lex_state = 10},
  [1523] = {.lex_state = 0},
  [1524] = {.lex_state = 40},
  [1525] = {.lex_state = 0},
  [1526] = {.lex_state = 0},
  [1527] = {.lex_state = 27},
  [1528] = {.lex_state = 15, .external_lex_state = 7},
  [1529] = {.lex_state = 40},
  [1530] = {.lex_state = 0},
  [1531] = {.lex_state = 40},
  [1532] = {.lex_state = 0, .external_lex_state = 7},
  [1533] = {.lex_state = 10},
  [1534] = {.lex_state = 0},
  [1535] = {.lex_state = 0},
  [1536] = {.lex_state = 15},
  [1537] = {.lex_state = 0},
  [1538] = {.lex_state = 0},
  [1539] = {.lex_state = 0},
  [1540] = {.lex_state = 0},
  [1541] = {.lex_state = 38},
  [1542] = {.lex_state = 0},
  [1543] = {.lex_state = 0},
  [1544] = {.lex_state = 0},
  [1545] = {.lex_state = 0, .external_lex_state = 7},
  [1546] = {.lex_state = 15},
  [1547] = {.lex_state = 38},
  [1548] = {.lex_state = 0},
  [1549] = {.lex_state = 30},
  [1550] = {.lex_state = 28},
  [1551] = {.lex_state = 30},
  [1552] = {.lex_state = 15},
  [1553] = {.lex_state = 38},
  [1554] = {.lex_state = 0},
  [1555] = {.lex_state = 0, .external_lex_state = 7},
  [1556] = {.lex_state = 0},
  [1557] = {.lex_state = 30},
  [1558] = {.lex_state = 0},
  [1559] = {.lex_state = 0},
  [1560] = {.lex_state = 28},
  [1561] = {.lex_state = 0},
  [1562] = {.lex_state = 0},
  [1563] = {.lex_state = 0},
  [1564] = {.lex_state = 27},
  [1565] = {.lex_state = 0, .external_lex_state = 7},
  [1566] = {.lex_state = 19},
  [1567] = {.lex_state = 0},
  [1568] = {.lex_state = 0},
  [1569] = {.lex_state = 0},
  [1570] = {.lex_state = 0, .external_lex_state = 7},
  [1571] = {.lex_state = 30},
  [1572] = {.lex_state = 26},
  [1573] = {.lex_state = 0, .external_lex_state = 7},
  [1574] = {.lex_state = 28},
  [1575] = {.lex_state = 28},
  [1576] = {.lex_state = 0},
  [1577] = {.lex_state = 0},
  [1578] = {.lex_state = 0},
  [1579] = {.lex_state = 26},
  [1580] = {.lex_state = 0},
  [1581] = {.lex_state = 0},
  [1582] = {.lex_state = 20},
  [1583] = {.lex_state = 0, .external_lex_state = 7},
  [1584] = {.lex_state = 21},
  [1585] = {.lex_state = 20},
  [1586] = {.lex_state = 14},
  [1587] = {.lex_state = 0},
  [1588] = {.lex_state = 0},
  [1589] = {.lex_state = 14},
  [1590] = {.lex_state = 0},
  [1591] = {.lex_state = 0},
  [1592] = {.lex_state = 20},
  [1593] = {.lex_state = 14},
  [1594] = {.lex_state = 0},
  [1595] = {.lex_state = 20},
  [1596] = {.lex_state = 21},
  [1597] = {.lex_state = 0},
  [1598] = {.lex_state = 14},
  [1599] = {.lex_state = 0},
  [1600] = {.lex_state = 15},
  [1601] = {.lex_state = 0, .external_lex_state = 7},
  [1602] = {.lex_state = 0, .external_lex_state = 7},
  [1603] = {.lex_state = 32, .external_lex_state = 7},
  [1604] = {.lex_state = 22},
  [1605] = {.lex_state = 19},
  [1606] = {.lex_state = 22},
  [1607] = {.lex_state = 0},
  [1608] = {.lex_state = 15},
  [1609] = {.lex_state = 0, .external_lex_state = 7},
  [1610] = {.lex_state = 30},
  [1611] = {.lex_state = 0},
  [1612] = {.lex_state = 0},
  [1613] = {.lex_state = 22},
  [1614] = {.lex_state = 32, .external_lex_state = 7},
  [1615] = {.lex_state = 22},
  [1616] = {.lex_state = 22},
  [1617] = {.lex_state = 0, .external_lex_state = 7},
  [1618] = {.lex_state = 0, .external_lex_state = 7},
  [1619] = {.lex_state = 0, .external_lex_state = 7},
  [1620] = {.lex_state = 22},
  [1621] = {.lex_state = 27},
  [1622] = {.lex_state = 0, .external_lex_state = 7},
  [1623] = {.lex_state = 14},
  [1624] = {.lex_state = 19},
  [1625] = {.lex_state = 0, .external_lex_state = 7},
  [1626] = {.lex_state = 27},
  [1627] = {.lex_state = 0, .external_lex_state = 7},
  [1628] = {.lex_state = 32, .external_lex_state = 7},
  [1629] = {.lex_state = 0, .external_lex_state = 7},
  [1630] = {.lex_state = 21},
  [1631] = {.lex_state = 0, .external_lex_state = 7},
  [1632] = {.lex_state = 28},
  [1633] = {.lex_state = 0, .external_lex_state = 7},
  [1634] = {.lex_state = 0, .external_lex_state = 7},
  [1635] = {.lex_state = 0, .external_lex_state = 7},
  [1636] = {.lex_state = 0, .external_lex_state = 7},
  [1637] = {.lex_state = 20},
  [1638] = {.lex_state = 21},
  [1639] = {.lex_state = 0, .external_lex_state = 7},
  [1640] = {.lex_state = 0, .external_lex_state = 7},
  [1641] = {.lex_state = 0, .external_lex_state = 7},
  [1642] = {.lex_state = 0, .external_lex_state = 7},
  [1643] = {.lex_state = 0, .external_lex_state = 7},
  [1644] = {.lex_state = 0, .external_lex_state = 7},
  [1645] = {.lex_state = 22},
  [1646] = {.lex_state = 14},
  [1647] = {.lex_state = 21},
  [1648] = {.lex_state = 0, .external_lex_state = 7},
  [1649] = {.lex_state = 0, .external_lex_state = 7},
  [1650] = {.lex_state = 21},
  [1651] = {.lex_state = 20},
  [1652] = {.lex_state = 0, .external_lex_state = 7},
  [1653] = {.lex_state = 21},
  [1654] = {.lex_state = 21},
  [1655] = {.lex_state = 21},
  [1656] = {.lex_state = 0, .external_lex_state = 7},
  [1657] = {.lex_state = 0},
  [1658] = {.lex_state = 21},
  [1659] = {.lex_state = 21},
  [1660] = {.lex_state = 21},
  [1661] = {.lex_state = 28},
  [1662] = {.lex_state = 21},
  [1663] = {.lex_state = 15},
  [1664] = {.lex_state = 32, .external_lex_state = 7},
  [1665] = {.lex_state = 0, .external_lex_state = 7},
  [1666] = {.lex_state = 22},
  [1667] = {.lex_state = 0, .external_lex_state = 7},
  [1668] = {.lex_state = 0, .external_lex_state = 7},
  [1669] = {.lex_state = 0, .external_lex_state = 7},
  [1670] = {.lex_state = 22},
  [1671] = {.lex_state = 0},
  [1672] = {.lex_state = 22},
  [1673] = {.lex_state = 32, .external_lex_state = 7},
  [1674] = {.lex_state = 21},
  [1675] = {.lex_state = 21},
  [1676] = {.lex_state = 22},
  [1677] = {.lex_state = 22},
  [1678] = {.lex_state = 30},
  [1679] = {.lex_state = 0},
  [1680] = {.lex_state = 22},
  [1681] = {.lex_state = 0},
  [1682] = {.lex_state = 0, .external_lex_state = 7},
  [1683] = {.lex_state = 0, .external_lex_state = 7},
  [1684] = {.lex_state = 0},
  [1685] = {.lex_state = 21},
  [1686] = {.lex_state = 0, .external_lex_state = 7},
  [1687] = {.lex_state = 0, .external_lex_state = 7},
  [1688] = {.lex_state = 22},
  [1689] = {.lex_state = 0, .external_lex_state = 7},
  [1690] = {.lex_state = 0},
  [1691] = {.lex_state = 0, .external_lex_state = 7},
  [1692] = {.lex_state = 0},
  [1693] = {.lex_state = 22},
  [1694] = {.lex_state = 14},
  [1695] = {.lex_state = 22},
  [1696] = {.lex_state = 28},
  [1697] = {.lex_state = 20},
  [1698] = {.lex_state = 30},
  [1699] = {.lex_state = 0, .external_lex_state = 7},
  [1700] = {.lex_state = 22},
  [1701] = {.lex_state = 22},
  [1702] = {.lex_state = 22},
  [1703] = {.lex_state = 21},
  [1704] = {.lex_state = 0, .external_lex_state = 7},
  [1705] = {.lex_state = 22},
  [1706] = {.lex_state = 22},
  [1707] = {.lex_state = 0, .external_lex_state = 7},
  [1708] = {.lex_state = 22},
  [1709] = {.lex_state = 0, .external_lex_state = 7},
  [1710] = {.lex_state = 0},
  [1711] = {.lex_state = 0, .external_lex_state = 7},
  [1712] = {.lex_state = 22},
  [1713] = {.lex_state = 22},
  [1714] = {.lex_state = 21},
  [1715] = {.lex_state = 22},
  [1716] = {.lex_state = 21},
  [1717] = {.lex_state = 32, .external_lex_state = 7},
  [1718] = {.lex_state = 0, .external_lex_state = 7},
  [1719] = {.lex_state = 21},
  [1720] = {.lex_state = 15},
  [1721] = {.lex_state = 0, .external_lex_state = 7},
  [1722] = {.lex_state = 29},
  [1723] = {.lex_state = 0, .external_lex_state = 7},
  [1724] = {.lex_state = 0},
  [1725] = {.lex_state = 0},
  [1726] = {.lex_state = 0, .external_lex_state = 7},
  [1727] = {.lex_state = 29},
  [1728] = {.lex_state = 0},
  [1729] = {.lex_state = 29},
  [1730] = {.lex_state = 0},
  [1731] = {.lex_state = 29},
  [1732] = {.lex_state = 29},
  [1733] = {.lex_state = 29},
  [1734] = {.lex_state = 29},
  [1735] = {.lex_state = 29},
  [1736] = {.lex_state = 0},
  [1737] = {.lex_state = 29},
  [1738] = {.lex_state = 0},
  [1739] = {.lex_state = 29},
  [1740] = {.lex_state = 29},
  [1741] = {.lex_state = 29},
  [1742] = {.lex_state = 0},
  [1743] = {.lex_state = 29},
  [1744] = {.lex_state = 0, .external_lex_state = 7},
  [1745] = {.lex_state = 0, .external_lex_state = 7},
  [1746] = {.lex_state = 29},
  [1747] = {.lex_state = 29},
  [1748] = {.lex_state = 29},
  [1749] = {.lex_state = 0, .external_lex_state = 7},
  [1750] = {.lex_state = 0},
  [1751] = {.lex_state = 0},
  [1752] = {.lex_state = 29},
  [1753] = {.lex_state = 29},
  [1754] = {.lex_state = 29},
  [1755] = {.lex_state = 29},
  [1756] = {.lex_state = 29},
  [1757] = {.lex_state = 0},
  [1758] = {.lex_state = 0},
  [1759] = {.lex_state = 0},
  [1760] = {.lex_state = 29},
  [1761] = {.lex_state = 29},
  [1762] = {.lex_state = 29},
  [1763] = {.lex_state = 29},
  [1764] = {.lex_state = 0},
  [1765] = {.lex_state = 0},
  [1766] = {.lex_state = 0, .external_lex_state = 7},
  [1767] = {.lex_state = 29},
  [1768] = {.lex_state = 0},
  [1769] = {.lex_state = 0},
  [1770] = {.lex_state = 0},
  [1771] = {.lex_state = 0},
  [1772] = {.lex_state = 0},
  [1773] = {.lex_state = 0, .external_lex_state = 7},
  [1774] = {.lex_state = 0},
  [1775] = {.lex_state = 0},
  [1776] = {.lex_state = 0},
  [1777] = {.lex_state = 0},
  [1778] = {.lex_state = 29},
  [1779] = {.lex_state = 0},
  [1780] = {.lex_state = 29},
  [1781] = {.lex_state = 29},
  [1782] = {.lex_state = 29},
  [1783] = {.lex_state = 0, .external_lex_state = 7},
  [1784] = {.lex_state = 29},
  [1785] = {.lex_state = 29},
  [1786] = {.lex_state = 0, .external_lex_state = 7},
  [1787] = {.lex_state = 29},
  [1788] = {.lex_state = 29},
  [1789] = {.lex_state = 29},
  [1790] = {.lex_state = 0},
  [1791] = {.lex_state = 0},
  [1792] = {.lex_state = 29},
  [1793] = {.lex_state = 29},
  [1794] = {.lex_state = 29},
  [1795] = {.lex_state = 29},
  [1796] = {.lex_state = 0, .external_lex_state = 7},
  [1797] = {.lex_state = 0, .external_lex_state = 7},
  [1798] = {.lex_state = 0, .external_lex_state = 7},
  [1799] = {.lex_state = 0, .external_lex_state = 7},
  [1800] = {.lex_state = 0, .external_lex_state = 7},
  [1801] = {.lex_state = 0},
  [1802] = {.lex_state = 0},
  [1803] = {.lex_state = 0, .external_lex_state = 7},
  [1804] = {.lex_state = 0, .external_lex_state = 7},
  [1805] = {.lex_state = 0},
  [1806] = {.lex_state = 0},
  [1807] = {.lex_state = 0, .external_lex_state = 7},
  [1808] = {.lex_state = 0},
  [1809] = {.lex_state = 0},
  [1810] = {.lex_state = 0, .external_lex_state = 7},
  [1811] = {.lex_state = 0, .external_lex_state = 7},
  [1812] = {.lex_state = 0},
  [1813] = {.lex_state = 0, .external_lex_state = 7},
  [1814] = {.lex_state = 0},
  [1815] = {.lex_state = 0, .external_lex_state = 7},
  [1816] = {.lex_state = 0, .external_lex_state = 7},
  [1817] = {.lex_state = 0, .external_lex_state = 7},
  [1818] = {.lex_state = 0, .external_lex_state = 7},
  [1819] = {.lex_state = 0, .external_lex_state = 7},
  [1820] = {.lex_state = 0},
  [1821] = {.lex_state = 0, .external_lex_state = 7},
  [1822] = {.lex_state = 0},
  [1823] = {.lex_state = 0, .external_lex_state = 7},
  [1824] = {.lex_state = 0, .external_lex_state = 7},
  [1825] = {.lex_state = 0, .external_lex_state = 7},
  [1826] = {.lex_state = 0, .external_lex_state = 7},
  [1827] = {.lex_state = 0},
  [1828] = {.lex_state = 0, .external_lex_state = 7},
  [1829] = {.lex_state = 0},
  [1830] = {.lex_state = 0},
  [1831] = {.lex_state = 0, .external_lex_state = 7},
  [1832] = {.lex_state = 0, .external_lex_state = 7},
  [1833] = {.lex_state = 0},
  [1834] = {.lex_state = 0, .external_lex_state = 7},
  [1835] = {.lex_state = 0, .external_lex_state = 7},
  [1836] = {.lex_state = 0, .external_lex_state = 7},
  [1837] = {.lex_state = 0},
  [1838] = {.lex_state = 0, .external_lex_state = 7},
  [1839] = {.lex_state = 0, .external_lex_state = 7},
  [1840] = {.lex_state = 0, .external_lex_state = 7},
  [1841] = {.lex_state = 0, .external_lex_state = 7},
  [1842] = {.lex_state = 0},
  [1843] = {.lex_state = 0, .external_lex_state = 7},
  [1844] = {.lex_state = 0},
  [1845] = {.lex_state = 0, .external_lex_state = 7},
  [1846] = {.lex_state = 0, .external_lex_state = 7},
  [1847] = {.lex_state = 0, .external_lex_state = 7},
  [1848] = {.lex_state = 0, .external_lex_state = 7},
  [1849] = {.lex_state = 0, .external_lex_state = 7},
  [1850] = {.lex_state = 0, .external_lex_state = 7},
  [1851] = {.lex_state = 0},
  [1852] = {.lex_state = 0},
  [1853] = {.lex_state = 0, .external_lex_state = 7},
  [1854] = {.lex_state = 0, .external_lex_state = 7},
  [1855] = {.lex_state = 0},
  [1856] = {.lex_state = 0},
  [1857] = {.lex_state = 0},
  [1858] = {.lex_state = 0, .external_lex_state = 7},
  [1859] = {.lex_state = 0, .external_lex_state = 7},
  [1860] = {.lex_state = 0, .external_lex_state = 7},
  [1861] = {.lex_state = 0, .external_lex_state = 7},
  [1862] = {.lex_state = 0, .external_lex_state = 7},
  [1863] = {.lex_state = 0, .external_lex_state = 7},
  [1864] = {.lex_state = 0, .external_lex_state = 7},
  [1865] = {.lex_state = 0},
  [1866] = {.lex_state = 0, .external_lex_state = 7},
  [1867] = {.lex_state = 0, .external_lex_state = 7},
  [1868] = {.lex_state = 0},
  [1869] = {.lex_state = 0},
  [1870] = {.lex_state = 0},
  [1871] = {.lex_state = 0, .external_lex_state = 7},
  [1872] = {.lex_state = 0, .external_lex_state = 7},
  [1873] = {.lex_state = 0, .external_lex_state = 7},
  [1874] = {.lex_state = 0, .external_lex_state = 7},
  [1875] = {.lex_state = 0, .external_lex_state = 7},
  [1876] = {.lex_state = 0, .external_lex_state = 7},
  [1877] = {.lex_state = 0, .external_lex_state = 7},
  [1878] = {.lex_state = 0},
  [1879] = {.lex_state = 0, .external_lex_state = 7},
  [1880] = {.lex_state = 0},
  [1881] = {.lex_state = 0, .external_lex_state = 7},
  [1882] = {.lex_state = 0, .external_lex_state = 7},
  [1883] = {.lex_state = 0},
  [1884] = {.lex_state = 0, .external_lex_state = 7},
  [1885] = {.lex_state = 0, .external_lex_state = 7},
  [1886] = {.lex_state = 0, .external_lex_state = 7},
  [1887] = {.lex_state = 0},
  [1888] = {.lex_state = 0, .external_lex_state = 7},
  [1889] = {.lex_state = 0, .external_lex_state = 7},
  [1890] = {.lex_state = 0, .external_lex_state = 7},
  [1891] = {.lex_state = 0, .external_lex_state = 7},
  [1892] = {.lex_state = 0, .external_lex_state = 7},
  [1893] = {.lex_state = 0, .external_lex_state = 7},
  [1894] = {.lex_state = 0, .external_lex_state = 7},
  [1895] = {.lex_state = 0},
  [1896] = {.lex_state = 0, .external_lex_state = 7},
  [1897] = {.lex_state = 0, .external_lex_state = 7},
  [1898] = {.lex_state = 0, .external_lex_state = 7},
  [1899] = {.lex_state = 0, .external_lex_state = 7},
  [1900] = {.lex_state = 0, .external_lex_state = 7},
  [1901] = {.lex_state = 0},
  [1902] = {.lex_state = 0, .external_lex_state = 7},
  [1903] = {.lex_state = 0, .external_lex_state = 7},
  [1904] = {.lex_state = 0, .external_lex_state = 7},
  [1905] = {.lex_state = 0, .external_lex_state = 7},
  [1906] = {.lex_state = 0},
  [1907] = {.lex_state = 0, .external_lex_state = 7},
  [1908] = {.lex_state = 0, .external_lex_state = 7},
  [1909] = {.lex_state = 0},
  [1910] = {.lex_state = 0, .external_lex_state = 7},
  [1911] = {.lex_state = 0},
  [1912] = {.lex_state = 0, .external_lex_state = 7},
  [1913] = {.lex_state = 0, .external_lex_state = 7},
  [1914] = {.lex_state = 0},
  [1915] = {.lex_state = 0, .external_lex_state = 7},
  [1916] = {.lex_state = 0, .external_lex_state = 7},
  [1917] = {.lex_state = 0, .external_lex_state = 7},
  [1918] = {.lex_state = 0, .external_lex_state = 9},
  [1919] = {.lex_state = 0, .external_lex_state = 9},
  [1920] = {.lex_state = 0},
  [1921] = {.lex_state = 0},
  [1922] = {.lex_state = 0},
  [1923] = {.lex_state = 0},
  [1924] = {.lex_state = 0, .external_lex_state = 9},
  [1925] = {.lex_state = 0, .external_lex_state = 10},
  [1926] = {.lex_state = 0},
  [1927] = {.lex_state = 0},
  [1928] = {.lex_state = 0},
  [1929] = {.lex_state = 0},
  [1930] = {.lex_state = 0},
  [1931] = {.lex_state = 0},
  [1932] = {.lex_state = 0},
  [1933] = {.lex_state = 0},
  [1934] = {.lex_state = 0},
  [1935] = {.lex_state = 0},
  [1936] = {.lex_state = 0},
  [1937] = {.lex_state = 0, .external_lex_state = 9},
  [1938] = {.lex_state = 0, .external_lex_state = 10},
  [1939] = {.lex_state = 0},
  [1940] = {.lex_state = 0, .external_lex_state = 10},
  [1941] = {.lex_state = 0},
  [1942] = {.lex_state = 0},
  [1943] = {.lex_state = 0, .external_lex_state = 9},
  [1944] = {.lex_state = 0},
  [1945] = {.lex_state = 0},
  [1946] = {.lex_state = 0},
  [1947] = {.lex_state = 0},
  [1948] = {.lex_state = 0},
  [1949] = {.lex_state = 0},
  [1950] = {.lex_state = 0},
  [1951] = {.lex_state = 0},
  [1952] = {.lex_state = 0},
  [1953] = {.lex_state = 0},
  [1954] = {.lex_state = 0},
  [1955] = {.lex_state = 0, .external_lex_state = 9},
  [1956] = {.lex_state = 0, .external_lex_state = 10},
  [1957] = {.lex_state = 0},
  [1958] = {.lex_state = 0},
  [1959] = {.lex_state = 0},
  [1960] = {.lex_state = 0},
  [1961] = {.lex_state = 0, .external_lex_state = 10},
  [1962] = {.lex_state = 0},
  [1963] = {.lex_state = 0},
  [1964] = {.lex_state = 0, .external_lex_state = 9},
  [1965] = {.lex_state = 0},
  [1966] = {.lex_state = 0},
  [1967] = {.lex_state = 0},
  [1968] = {.lex_state = 0, .external_lex_state = 9},
  [1969] = {.lex_state = 0, .external_lex_state = 10},
  [1970] = {.lex_state = 0},
  [1971] = {.lex_state = 0},
  [1972] = {.lex_state = 0},
  [1973] = {.lex_state = 0},
  [1974] = {.lex_state = 0, .external_lex_state = 10},
  [1975] = {.lex_state = 0, .external_lex_state = 9},
  [1976] = {.lex_state = 0},
  [1977] = {.lex_state = 0},
  [1978] = {.lex_state = 0},
  [1979] = {.lex_state = 0, .external_lex_state = 9},
  [1980] = {.lex_state = 0},
  [1981] = {.lex_state = 0},
  [1982] = {.lex_state = 0},
  [1983] = {.lex_state = 0},
  [1984] = {.lex_state = 0},
  [1985] = {.lex_state = 0},
  [1986] = {.lex_state = 0, .external_lex_state = 9},
  [1987] = {.lex_state = 0, .external_lex_state = 10},
  [1988] = {.lex_state = 0},
  [1989] = {.lex_state = 0},
  [1990] = {.lex_state = 0},
  [1991] = {.lex_state = 0},
  [1992] = {.lex_state = 0, .external_lex_state = 10},
  [1993] = {.lex_state = 0},
  [1994] = {.lex_state = 0},
  [1995] = {.lex_state = 0, .external_lex_state = 9},
  [1996] = {.lex_state = 0},
  [1997] = {.lex_state = 0},
  [1998] = {.lex_state = 0},
  [1999] = {.lex_state = 0, .external_lex_state = 9},
  [2000] = {.lex_state = 0, .external_lex_state = 10},
  [2001] = {.lex_state = 0},
  [2002] = {.lex_state = 0},
  [2003] = {.lex_state = 0},
  [2004] = {.lex_state = 0, .external_lex_state = 9},
  [2005] = {.lex_state = 0},
  [2006] = {.lex_state = 0},
  [2007] = {.lex_state = 0},
  [2008] = {.lex_state = 0},
  [2009] = {.lex_state = 0},
  [2010] = {.lex_state = 0},
  [2011] = {.lex_state = 0},
  [2012] = {.lex_state = 0},
  [2013] = {.lex_state = 0},
  [2014] = {.lex_state = 0},
  [2015] = {.lex_state = 0, .external_lex_state = 10},
  [2016] = {.lex_state = 0},
  [2017] = {.lex_state = 0, .external_lex_state = 9},
  [2018] = {.lex_state = 0, .external_lex_state = 10},
  [2019] = {.lex_state = 0},
  [2020] = {.lex_state = 0},
  [2021] = {.lex_state = 0, .external_lex_state = 10},
  [2022] = {.lex_state = 0},
  [2023] = {.lex_state = 0},
  [2024] = {.lex_state = 0},
  [2025] = {.lex_state = 0},
  [2026] = {.lex_state = 0},
  [2027] = {.lex_state = 0},
  [2028] = {.lex_state = 0},
  [2029] = {.lex_state = 0},
  [2030] = {.lex_state = 0, .external_lex_state = 9},
  [2031] = {.lex_state = 0, .external_lex_state = 10},
  [2032] = {.lex_state = 0},
  [2033] = {.lex_state = 0},
  [2034] = {.lex_state = 0},
  [2035] = {.lex_state = 0},
  [2036] = {.lex_state = 0},
  [2037] = {.lex_state = 0},
  [2038] = {.lex_state = 0},
  [2039] = {.lex_state = 0},
  [2040] = {.lex_state = 0},
  [2041] = {.lex_state = 0},
  [2042] = {.lex_state = 0, .external_lex_state = 9},
  [2043] = {.lex_state = 0},
  [2044] = {.lex_state = 0},
  [2045] = {.lex_state = 0, .external_lex_state = 10},
  [2046] = {.lex_state = 0},
  [2047] = {.lex_state = 0},
  [2048] = {.lex_state = 0, .external_lex_state = 10},
  [2049] = {.lex_state = 0},
  [2050] = {.lex_state = 0},
  [2051] = {.lex_state = 0},
  [2052] = {.lex_state = 0},
  [2053] = {.lex_state = 0},
  [2054] = {.lex_state = 0},
  [2055] = {.lex_state = 0},
  [2056] = {.lex_state = 0, .external_lex_state = 10},
  [2057] = {.lex_state = 0},
  [2058] = {.lex_state = 0},
  [2059] = {.lex_state = 0},
  [2060] = {.lex_state = 0, .external_lex_state = 9},
  [2061] = {.lex_state = 0, .external_lex_state = 10},
  [2062] = {.lex_state = 0},
  [2063] = {.lex_state = 0},
  [2064] = {.lex_state = 0},
  [2065] = {.lex_state = 0},
  [2066] = {.lex_state = 0},
  [2067] = {.lex_state = 0},
  [2068] = {.lex_state = 0},
  [2069] = {.lex_state = 0},
  [2070] = {.lex_state = 0},
  [2071] = {.lex_state = 0},
  [2072] = {.lex_state = 0},
  [2073] = {.lex_state = 0},
  [2074] = {.lex_state = 0},
  [2075] = {.lex_state = 0},
  [2076] = {.lex_state = 0, .external_lex_state = 9},
  [2077] = {.lex_state = 0, .external_lex_state = 10},
  [2078] = {.lex_state = 0},
  [2079] = {.lex_state = 0},
  [2080] = {.lex_state = 0},
  [2081] = {.lex_state = 0},
  [2082] = {.lex_state = 0},
  [2083] = {.lex_state = 0},
  [2084] = {.lex_state = 0},
  [2085] = {.lex_state = 0, .external_lex_state = 9},
  [2086] = {.lex_state = 0, .external_lex_state = 10},
  [2087] = {.lex_state = 0},
  [2088] = {.lex_state = 0},
  [2089] = {.lex_state = 0},
  [2090] = {.lex_state = 0},
  [2091] = {.lex_state = 0},
  [2092] = {.lex_state = 0},
  [2093] = {.lex_state = 0},
  [2094] = {.lex_state = 0},
  [2095] = {.lex_state = 0, .external_lex_state = 9},
  [2096] = {.lex_state = 0, .external_lex_state = 10},
  [2097] = {.lex_state = 0},
  [2098] = {.lex_state = 0},
  [2099] = {.lex_state = 0},
  [2100] = {.lex_state = 0},
  [2101] = {.lex_state = 0},
  [2102] = {.lex_state = 0},
  [2103] = {.lex_state = 0},
  [2104] = {.lex_state = 0, .external_lex_state = 9},
  [2105] = {.lex_state = 0, .external_lex_state = 10},
  [2106] = {.lex_state = 0},
  [2107] = {.lex_state = 0},
  [2108] = {.lex_state = 0},
  [2109] = {.lex_state = 0},
  [2110] = {.lex_state = 0},
  [2111] = {.lex_state = 0},
  [2112] = {.lex_state = 0},
  [2113] = {.lex_state = 0},
  [2114] = {.lex_state = 0, .external_lex_state = 9},
  [2115] = {.lex_state = 0, .external_lex_state = 10},
  [2116] = {.lex_state = 0},
  [2117] = {.lex_state = 0},
  [2118] = {.lex_state = 0},
  [2119] = {.lex_state = 0},
  [2120] = {.lex_state = 0},
  [2121] = {.lex_state = 0},
  [2122] = {.lex_state = 0},
  [2123] = {.lex_state = 0, .external_lex_state = 9},
  [2124] = {.lex_state = 0, .external_lex_state = 10},
  [2125] = {.lex_state = 0},
  [2126] = {.lex_state = 0},
  [2127] = {.lex_state = 0},
  [2128] = {.lex_state = 0},
  [2129] = {.lex_state = 0},
  [2130] = {.lex_state = 0},
  [2131] = {.lex_state = 0},
  [2132] = {.lex_state = 0},
  [2133] = {.lex_state = 0},
  [2134] = {.lex_state = 0},
  [2135] = {.lex_state = 0},
  [2136] = {.lex_state = 0},
  [2137] = {.lex_state = 0},
  [2138] = {.lex_state = 0},
  [2139] = {.lex_state = 0},
  [2140] = {.lex_state = 0},
  [2141] = {.lex_state = 0},
  [2142] = {.lex_state = 0},
  [2143] = {.lex_state = 0},
  [2144] = {.lex_state = 0},
  [2145] = {.lex_state = 0, .external_lex_state = 9},
  [2146] = {.lex_state = 0},
  [2147] = {.lex_state = 0, .external_lex_state = 10},
  [2148] = {.lex_state = 0},
  [2149] = {.lex_state = 0},
  [2150] = {.lex_state = 0},
  [2151] = {.lex_state = 0},
  [2152] = {.lex_state = 0},
  [2153] = {.lex_state = 0},
  [2154] = {.lex_state = 0},
  [2155] = {.lex_state = 0, .external_lex_state = 10},
  [2156] = {.lex_state = 0},
  [2157] = {.lex_state = 0, .external_lex_state = 9},
  [2158] = {.lex_state = 0},
  [2159] = {.lex_state = 0},
  [2160] = {.lex_state = 0},
  [2161] = {.lex_state = 0},
  [2162] = {.lex_state = 0, .external_lex_state = 10},
  [2163] = {.lex_state = 0},
  [2164] = {.lex_state = 0},
  [2165] = {.lex_state = 0},
  [2166] = {.lex_state = 0},
  [2167] = {.lex_state = 0, .external_lex_state = 9},
  [2168] = {.lex_state = 0},
  [2169] = {.lex_state = 0},
  [2170] = {.lex_state = 0},
  [2171] = {.lex_state = 0},
  [2172] = {.lex_state = 0},
  [2173] = {.lex_state = 0},
  [2174] = {.lex_state = 0},
  [2175] = {.lex_state = 0},
  [2176] = {.lex_state = 0},
  [2177] = {.lex_state = 0},
  [2178] = {.lex_state = 0},
  [2179] = {.lex_state = 0},
  [2180] = {.lex_state = 0},
  [2181] = {.lex_state = 0},
  [2182] = {.lex_state = 0},
  [2183] = {.lex_state = 0},
  [2184] = {.lex_state = 0},
  [2185] = {.lex_state = 0},
  [2186] = {.lex_state = 0},
  [2187] = {.lex_state = 0},
  [2188] = {.lex_state = 0},
  [2189] = {.lex_state = 0},
  [2190] = {.lex_state = 0},
  [2191] = {.lex_state = 0},
  [2192] = {.lex_state = 0},
  [2193] = {.lex_state = 0},
  [2194] = {.lex_state = 0},
  [2195] = {.lex_state = 0},
  [2196] = {.lex_state = 0},
  [2197] = {.lex_state = 0},
  [2198] = {.lex_state = 0},
  [2199] = {.lex_state = 0},
  [2200] = {.lex_state = 0, .external_lex_state = 10},
  [2201] = {.lex_state = 0, .external_lex_state = 10},
  [2202] = {.lex_state = 0, .external_lex_state = 9},
  [2203] = {.lex_state = 0},
  [2204] = {.lex_state = 0, .external_lex_state = 9},
  [2205] = {.lex_state = 0},
  [2206] = {.lex_state = 0},
  [2207] = {.lex_state = 0},
  [2208] = {.lex_state = 0},
  [2209] = {.lex_state = 0},
  [2210] = {.lex_state = 0, .external_lex_state = 10},
  [2211] = {.lex_state = 0},
  [2212] = {.lex_state = 0},
  [2213] = {.lex_state = 0},
  [2214] = {.lex_state = 0},
  [2215] = {.lex_state = 0},
  [2216] = {.lex_state = 0},
  [2217] = {.lex_state = 0},
  [2218] = {.lex_state = 0},
  [2219] = {.lex_state = 0},
  [2220] = {.lex_state = 0},
  [2221] = {.lex_state = 0},
  [2222] = {.lex_state = 0},
  [2223] = {.lex_state = 0},
  [2224] = {.lex_state = 0},
  [2225] = {.lex_state = 0},
  [2226] = {.lex_state = 0},
  [2227] = {.lex_state = 0},
  [2228] = {.lex_state = 0},
  [2229] = {.lex_state = 0},
  [2230] = {.lex_state = 0},
  [2231] = {.lex_state = 0},
  [2232] = {.lex_state = 0},
  [2233] = {.lex_state = 0},
  [2234] = {.lex_state = 0},
  [2235] = {.lex_state = 0},
  [2236] = {.lex_state = 0},
  [2237] = {.lex_state = 0},
  [2238] = {.lex_state = 0},
  [2239] = {.lex_state = 0, .external_lex_state = 9},
  [2240] = {.lex_state = 0},
  [2241] = {.lex_state = 0},
  [2242] = {.lex_state = 0},
  [2243] = {.lex_state = 0},
  [2244] = {.lex_state = 0},
  [2245] = {.lex_state = 0},
  [2246] = {.lex_state = 0, .external_lex_state = 10},
  [2247] = {.lex_state = 0},
  [2248] = {.lex_state = 0},
  [2249] = {.lex_state = 0},
  [2250] = {.lex_state = 0, .external_lex_state = 9},
  [2251] = {.lex_state = 0},
  [2252] = {.lex_state = 0},
  [2253] = {.lex_state = 0},
  [2254] = {.lex_state = 0},
  [2255] = {.lex_state = 0},
  [2256] = {.lex_state = 0},
  [2257] = {.lex_state = 0},
  [2258] = {.lex_state = 0},
  [2259] = {.lex_state = 0},
  [2260] = {.lex_state = 0},
  [2261] = {.lex_state = 0},
  [2262] = {.lex_state = 0},
  [2263] = {.lex_state = 0},
  [2264] = {.lex_state = 0},
  [2265] = {.lex_state = 0},
  [2266] = {.lex_state = 0},
  [2267] = {.lex_state = 0},
  [2268] = {.lex_state = 0},
  [2269] = {.lex_state = 0},
  [2270] = {.lex_state = 0},
  [2271] = {.lex_state = 0},
  [2272] = {.lex_state = 32},
  [2273] = {.lex_state = 0},
  [2274] = {.lex_state = 0},
  [2275] = {.lex_state = 0},
  [2276] = {.lex_state = 0},
  [2277] = {.lex_state = 0},
  [2278] = {.lex_state = 0},
  [2279] = {.lex_state = 0},
  [2280] = {.lex_state = 0},
  [2281] = {.lex_state = 0},
  [2282] = {.lex_state = 0},
  [2283] = {.lex_state = 0},
  [2284] = {.lex_state = 32},
  [2285] = {.lex_state = 0},
  [2286] = {.lex_state = 0},
  [2287] = {.lex_state = 0},
  [2288] = {.lex_state = 0},
  [2289] = {.lex_state = 0},
  [2290] = {.lex_state = 0},
  [2291] = {.lex_state = 0},
  [2292] = {.lex_state = 0},
  [2293] = {.lex_state = 32},
  [2294] = {.lex_state = 0},
  [2295] = {.lex_state = 0},
  [2296] = {.lex_state = 0},
  [2297] = {.lex_state = 0},
  [2298] = {.lex_state = 0},
  [2299] = {.lex_state = 0},
  [2300] = {.lex_state = 0},
  [2301] = {.lex_state = 0},
  [2302] = {.lex_state = 32},
  [2303] = {.lex_state = 0},
  [2304] = {.lex_state = 0},
  [2305] = {.lex_state = 0},
  [2306] = {.lex_state = 0},
  [2307] = {.lex_state = 0},
  [2308] = {.lex_state = 0},
  [2309] = {.lex_state = 0},
  [2310] = {.lex_state = 0},
  [2311] = {.lex_state = 32},
  [2312] = {.lex_state = 0},
  [2313] = {.lex_state = 0},
  [2314] = {.lex_state = 0},
  [2315] = {.lex_state = 0},
  [2316] = {.lex_state = 0},
  [2317] = {.lex_state = 0},
  [2318] = {.lex_state = 32},
  [2319] = {.lex_state = 0},
  [2320] = {.lex_state = 32},
  [2321] = {.lex_state = 0},
  [2322] = {.lex_state = 0},
  [2323] = {.lex_state = 32},
  [2324] = {.lex_state = 0},
  [2325] = {.lex_state = 0},
  [2326] = {.lex_state = 0},
  [2327] = {.lex_state = 0},
  [2328] = {.lex_state = 0},
  [2329] = {.lex_state = 32},
  [2330] = {.lex_state = 0},
  [2331] = {.lex_state = 0},
  [2332] = {.lex_state = 0},
  [2333] = {.lex_state = 0},
  [2334] = {.lex_state = 0},
  [2335] = {.lex_state = 0},
  [2336] = {.lex_state = 0},
  [2337] = {.lex_state = 0},
  [2338] = {.lex_state = 32},
  [2339] = {.lex_state = 0},
  [2340] = {.lex_state = 0},
  [2341] = {.lex_state = 0},
  [2342] = {.lex_state = 0},
  [2343] = {.lex_state = 0},
  [2344] = {.lex_state = 0},
  [2345] = {.lex_state = 0},
  [2346] = {.lex_state = 32},
  [2347] = {.lex_state = 0},
  [2348] = {.lex_state = 0},
  [2349] = {.lex_state = 0},
  [2350] = {.lex_state = 0},
  [2351] = {.lex_state = 0},
  [2352] = {.lex_state = 0},
  [2353] = {.lex_state = 32},
  [2354] = {.lex_state = 0},
  [2355] = {.lex_state = 0},
  [2356] = {.lex_state = 0},
  [2357] = {.lex_state = 0},
  [2358] = {.lex_state = 0},
  [2359] = {.lex_state = 0},
  [2360] = {.lex_state = 32},
  [2361] = {.lex_state = 0},
  [2362] = {.lex_state = 0},
  [2363] = {.lex_state = 0},
  [2364] = {.lex_state = 0},
  [2365] = {.lex_state = 0},
  [2366] = {.lex_state = 0},
  [2367] = {.lex_state = 32},
  [2368] = {.lex_state = 0},
  [2369] = {.lex_state = 0},
  [2370] = {.lex_state = 0},
  [2371] = {.lex_state = 0},
  [2372] = {.lex_state = 32},
  [2373] = {.lex_state = 0},
  [2374] = {.lex_state = 0},
  [2375] = {.lex_state = 0},
  [2376] = {.lex_state = 0},
  [2377] = {.lex_state = 32},
  [2378] = {.lex_state = 0},
  [2379] = {.lex_state = 0},
  [2380] = {.lex_state = 0},
  [2381] = {.lex_state = 0},
  [2382] = {.lex_state = 0},
  [2383] = {.lex_state = 0},
  [2384] = {.lex_state = 0},
  [2385] = {.lex_state = 0},
  [2386] = {.lex_state = 0},
  [2387] = {.lex_state = 0},
  [2388] = {.lex_state = 0},
  [2389] = {.lex_state = 0},
  [2390] = {.lex_state = 0},
  [2391] = {.lex_state = 0},
  [2392] = {.lex_state = 0},
  [2393] = {.lex_state = 0},
  [2394] = {.lex_state = 0},
  [2395] = {.lex_state = 0},
  [2396] = {.lex_state = 0},
  [2397] = {.lex_state = 0},
  [2398] = {.lex_state = 0},
  [2399] = {.lex_state = 0},
};

enum {
  ts_external_token__automatic_semicolon = 0,
  ts_external_token__simple_string = 1,
  ts_external_token__string_start = 2,
  ts_external_token__string_middle = 3,
  ts_external_token__string_end = 4,
  ts_external_token__multiline_string_start = 5,
  ts_external_token__multiline_string_middle = 6,
  ts_external_token__multiline_string_end = 7,
  ts_external_token_else = 8,
};

static TSSymbol ts_external_scanner_symbol_map[EXTERNAL_TOKEN_COUNT] = {
  [ts_external_token__automatic_semicolon] = sym__automatic_semicolon,
  [ts_external_token__simple_string] = sym__simple_string,
  [ts_external_token__string_start] = sym__string_start,
  [ts_external_token__string_middle] = sym__string_middle,
  [ts_external_token__string_end] = sym__string_end,
  [ts_external_token__multiline_string_start] = sym__multiline_string_start,
  [ts_external_token__multiline_string_middle] = sym__multiline_string_middle,
  [ts_external_token__multiline_string_end] = sym__multiline_string_end,
  [ts_external_token_else] = anon_sym_else,
};

static bool ts_external_scanner_states[11][EXTERNAL_TOKEN_COUNT] = {
  [1] = {
    [ts_external_token__automatic_semicolon] = true,
    [ts_external_token__simple_string] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__string_middle] = true,
    [ts_external_token__string_end] = true,
    [ts_external_token__multiline_string_start] = true,
    [ts_external_token__multiline_string_middle] = true,
    [ts_external_token__multiline_string_end] = true,
    [ts_external_token_else] = true,
  },
  [2] = {
    [ts_external_token__simple_string] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__multiline_string_start] = true,
  },
  [3] = {
    [ts_external_token_else] = true,
  },
  [4] = {
    [ts_external_token__simple_string] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__multiline_string_start] = true,
    [ts_external_token_else] = true,
  },
  [5] = {
    [ts_external_token__automatic_semicolon] = true,
    [ts_external_token_else] = true,
  },
  [6] = {
    [ts_external_token__automatic_semicolon] = true,
    [ts_external_token__simple_string] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__multiline_string_start] = true,
    [ts_external_token_else] = true,
  },
  [7] = {
    [ts_external_token__automatic_semicolon] = true,
  },
  [8] = {
    [ts_external_token__automatic_semicolon] = true,
    [ts_external_token__simple_string] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__multiline_string_start] = true,
  },
  [9] = {
    [ts_external_token__string_middle] = true,
    [ts_external_token__string_end] = true,
  },
  [10] = {
    [ts_external_token__multiline_string_middle] = true,
    [ts_external_token__multiline_string_end] = true,
  },
};

static uint16_t ts_parse_table[LARGE_STATE_COUNT][SYMBOL_COUNT] = {
  [0] = {
    [ts_builtin_sym_end] = ACTIONS(1),
    [sym_identifier] = ACTIONS(1),
    [anon_sym_package] = ACTIONS(1),
    [anon_sym_DOT] = ACTIONS(1),
    [anon_sym_object] = ACTIONS(1),
    [anon_sym_import] = ACTIONS(1),
    [anon_sym_COMMA] = ACTIONS(1),
    [anon_sym_LBRACE] = ACTIONS(1),
    [anon_sym_RBRACE] = ACTIONS(1),
    [anon_sym_EQ_GT] = ACTIONS(1),
    [anon_sym_case] = ACTIONS(1),
    [anon_sym_class] = ACTIONS(1),
    [anon_sym_trait] = ACTIONS(1),
    [anon_sym_LBRACK] = ACTIONS(1),
    [anon_sym_RBRACK] = ACTIONS(1),
    [anon_sym_PLUS] = ACTIONS(1),
    [anon_sym_DASH] = ACTIONS(1),
    [anon_sym_LT_COLON] = ACTIONS(1),
    [anon_sym_GT_COLON] = ACTIONS(1),
    [anon_sym_LT_PERCENT] = ACTIONS(1),
    [anon_sym_COLON] = ACTIONS(1),
    [anon_sym_AT] = ACTIONS(1),
    [anon_sym_val] = ACTIONS(1),
    [anon_sym_EQ] = ACTIONS(1),
    [anon_sym_var] = ACTIONS(1),
    [anon_sym_type] = ACTIONS(1),
    [anon_sym_def] = ACTIONS(1),
    [anon_sym_abstract] = ACTIONS(1),
    [anon_sym_final] = ACTIONS(1),
    [anon_sym_sealed] = ACTIONS(1),
    [anon_sym_implicit] = ACTIONS(1),
    [anon_sym_lazy] = ACTIONS(1),
    [anon_sym_override] = ACTIONS(1),
    [anon_sym_private] = ACTIONS(1),
    [anon_sym_protected] = ACTIONS(1),
    [anon_sym_extends] = ACTIONS(1),
    [anon_sym_LPAREN] = ACTIONS(1),
    [anon_sym_RPAREN] = ACTIONS(1),
    [anon_sym_with] = ACTIONS(1),
    [anon_sym_POUND] = ACTIONS(1),
    [anon_sym_STAR] = ACTIONS(1),
    [anon_sym_PIPE] = ACTIONS(1),
    [anon_sym_if] = ACTIONS(1),
    [anon_sym_else] = ACTIONS(1),
    [anon_sym_match] = ACTIONS(1),
    [anon_sym_try] = ACTIONS(1),
    [anon_sym_catch] = ACTIONS(1),
    [anon_sym_finally] = ACTIONS(1),
    [anon_sym_new] = ACTIONS(1),
    [anon_sym_BANG] = ACTIONS(1),
    [anon_sym_TILDE] = ACTIONS(1),
    [sym_wildcard] = ACTIONS(1),
    [sym_number] = ACTIONS(1),
    [anon_sym_DOLLAR] = ACTIONS(1),
    [anon_sym_SEMI] = ACTIONS(1),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1),
    [sym__simple_string] = ACTIONS(1),
    [sym__string_start] = ACTIONS(1),
    [sym__string_middle] = ACTIONS(1),
    [sym__string_end] = ACTIONS(1),
    [sym__multiline_string_start] = ACTIONS(1),
    [sym__multiline_string_middle] = ACTIONS(1),
    [sym__multiline_string_end] = ACTIONS(1),
  },
  [1] = {
    [sym_compilation_unit] = STATE(2349),
    [sym_package_clause] = STATE(26),
    [sym_package_object] = STATE(26),
    [sym_import_declaration] = STATE(26),
    [sym_object_definition] = STATE(26),
    [sym_class_definition] = STATE(26),
    [sym_trait_definition] = STATE(26),
    [sym_annotation] = STATE(667),
    [sym_val_definition] = STATE(26),
    [sym_val_declaration] = STATE(26),
    [sym_var_declaration] = STATE(26),
    [sym_var_definition] = STATE(26),
    [sym_type_definition] = STATE(26),
    [sym_function_definition] = STATE(26),
    [sym_function_declaration] = STATE(26),
    [sym_modifiers] = STATE(1671),
    [aux_sym_compilation_unit_repeat1] = STATE(26),
    [aux_sym_class_definition_repeat1] = STATE(667),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [ts_builtin_sym_end] = ACTIONS(5),
    [anon_sym_package] = ACTIONS(7),
    [anon_sym_object] = ACTIONS(9),
    [anon_sym_import] = ACTIONS(11),
    [anon_sym_case] = ACTIONS(13),
    [anon_sym_class] = ACTIONS(15),
    [anon_sym_trait] = ACTIONS(17),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(21),
    [anon_sym_var] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_def] = ACTIONS(27),
    [anon_sym_abstract] = ACTIONS(29),
    [anon_sym_final] = ACTIONS(29),
    [anon_sym_sealed] = ACTIONS(29),
    [anon_sym_implicit] = ACTIONS(29),
    [anon_sym_lazy] = ACTIONS(29),
    [anon_sym_override] = ACTIONS(29),
    [anon_sym_private] = ACTIONS(29),
    [anon_sym_protected] = ACTIONS(29),
    [sym_comment] = ACTIONS(3),
  },
  [2] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2273),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_case_clause] = STATE(1844),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [aux_sym_case_block_repeat1] = STATE(1844),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(41),
    [anon_sym_case] = ACTIONS(43),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [3] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2351),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_case_clause] = STATE(1852),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [aux_sym_case_block_repeat1] = STATE(1852),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(77),
    [anon_sym_case] = ACTIONS(43),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [4] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2375),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_case_clause] = STATE(1809),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [aux_sym_case_block_repeat1] = STATE(1809),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(79),
    [anon_sym_case] = ACTIONS(43),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [5] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2327),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_case_clause] = STATE(1909),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [aux_sym_case_block_repeat1] = STATE(1909),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(81),
    [anon_sym_case] = ACTIONS(43),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [6] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2343),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_case_clause] = STATE(1883),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [aux_sym_case_block_repeat1] = STATE(1883),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(83),
    [anon_sym_case] = ACTIONS(43),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [7] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2336),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_case_clause] = STATE(1906),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [aux_sym_case_block_repeat1] = STATE(1906),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(85),
    [anon_sym_case] = ACTIONS(43),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [8] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2361),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_case_clause] = STATE(1829),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [aux_sym_case_block_repeat1] = STATE(1829),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(87),
    [anon_sym_case] = ACTIONS(43),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [9] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2292),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_case_clause] = STATE(1878),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [aux_sym_case_block_repeat1] = STATE(1878),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(89),
    [anon_sym_case] = ACTIONS(43),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [10] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2388),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_case_clause] = STATE(1814),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [aux_sym_case_block_repeat1] = STATE(1814),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(91),
    [anon_sym_case] = ACTIONS(43),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [11] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2325),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_case_clause] = STATE(1851),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [aux_sym_case_block_repeat1] = STATE(1851),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(93),
    [anon_sym_case] = ACTIONS(43),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [12] = {
    [sym_package_clause] = STATE(1723),
    [sym_package_object] = STATE(1723),
    [sym_import_declaration] = STATE(1723),
    [sym_object_definition] = STATE(1723),
    [sym_class_definition] = STATE(1723),
    [sym_trait_definition] = STATE(1723),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1723),
    [sym_val_declaration] = STATE(1723),
    [sym_var_declaration] = STATE(1723),
    [sym_var_definition] = STATE(1723),
    [sym_type_definition] = STATE(1723),
    [sym_function_definition] = STATE(1723),
    [sym_function_declaration] = STATE(1723),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2219),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(865),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(95),
    [anon_sym_case] = ACTIONS(97),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [13] = {
    [sym_package_clause] = STATE(1723),
    [sym_package_object] = STATE(1723),
    [sym_import_declaration] = STATE(1723),
    [sym_object_definition] = STATE(1723),
    [sym_class_definition] = STATE(1723),
    [sym_trait_definition] = STATE(1723),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1723),
    [sym_val_declaration] = STATE(1723),
    [sym_var_declaration] = STATE(1723),
    [sym_var_definition] = STATE(1723),
    [sym_type_definition] = STATE(1723),
    [sym_function_definition] = STATE(1723),
    [sym_function_declaration] = STATE(1723),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2191),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(865),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(99),
    [anon_sym_case] = ACTIONS(101),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [14] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2276),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(103),
    [anon_sym_case] = ACTIONS(105),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [15] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2371),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(107),
    [anon_sym_case] = ACTIONS(105),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [16] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2298),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(109),
    [anon_sym_case] = ACTIONS(105),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [17] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2313),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(111),
    [anon_sym_case] = ACTIONS(105),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [18] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2316),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(113),
    [anon_sym_case] = ACTIONS(105),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [19] = {
    [sym_package_clause] = STATE(1894),
    [sym_package_object] = STATE(1894),
    [sym_import_declaration] = STATE(1894),
    [sym_object_definition] = STATE(1894),
    [sym_class_definition] = STATE(1894),
    [sym_trait_definition] = STATE(1894),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1894),
    [sym_val_declaration] = STATE(1894),
    [sym_var_declaration] = STATE(1894),
    [sym_var_definition] = STATE(1894),
    [sym_type_definition] = STATE(1894),
    [sym_function_definition] = STATE(1894),
    [sym_function_declaration] = STATE(1894),
    [sym_modifiers] = STATE(1692),
    [sym__block] = STATE(2291),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1048),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(115),
    [anon_sym_case] = ACTIONS(105),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [20] = {
    [sym_package_clause] = STATE(1886),
    [sym_package_object] = STATE(1886),
    [sym_import_declaration] = STATE(1886),
    [sym_object_definition] = STATE(1886),
    [sym_class_definition] = STATE(1886),
    [sym_trait_definition] = STATE(1886),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1886),
    [sym_val_declaration] = STATE(1886),
    [sym_var_declaration] = STATE(1886),
    [sym_var_definition] = STATE(1886),
    [sym_type_definition] = STATE(1886),
    [sym_function_definition] = STATE(1886),
    [sym_function_declaration] = STATE(1886),
    [sym_modifiers] = STATE(1692),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1058),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(117),
    [anon_sym_case] = ACTIONS(105),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [21] = {
    [sym_package_clause] = STATE(1886),
    [sym_package_object] = STATE(1886),
    [sym_import_declaration] = STATE(1886),
    [sym_object_definition] = STATE(1886),
    [sym_class_definition] = STATE(1886),
    [sym_trait_definition] = STATE(1886),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1886),
    [sym_val_declaration] = STATE(1886),
    [sym_var_declaration] = STATE(1886),
    [sym_var_definition] = STATE(1886),
    [sym_type_definition] = STATE(1886),
    [sym_function_definition] = STATE(1886),
    [sym_function_declaration] = STATE(1886),
    [sym_modifiers] = STATE(1692),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1058),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(119),
    [anon_sym_case] = ACTIONS(121),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [22] = {
    [sym_package_clause] = STATE(1886),
    [sym_package_object] = STATE(1886),
    [sym_import_declaration] = STATE(1886),
    [sym_object_definition] = STATE(1886),
    [sym_class_definition] = STATE(1886),
    [sym_trait_definition] = STATE(1886),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1886),
    [sym_val_declaration] = STATE(1886),
    [sym_var_declaration] = STATE(1886),
    [sym_var_definition] = STATE(1886),
    [sym_type_definition] = STATE(1886),
    [sym_function_definition] = STATE(1886),
    [sym_function_declaration] = STATE(1886),
    [sym_modifiers] = STATE(1692),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1058),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(117),
    [anon_sym_case] = ACTIONS(123),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [23] = {
    [sym_package_clause] = STATE(1886),
    [sym_package_object] = STATE(1886),
    [sym_import_declaration] = STATE(1886),
    [sym_object_definition] = STATE(1886),
    [sym_class_definition] = STATE(1886),
    [sym_trait_definition] = STATE(1886),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1886),
    [sym_val_declaration] = STATE(1886),
    [sym_var_declaration] = STATE(1886),
    [sym_var_definition] = STATE(1886),
    [sym_type_definition] = STATE(1886),
    [sym_function_definition] = STATE(1886),
    [sym_function_declaration] = STATE(1886),
    [sym_modifiers] = STATE(1692),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1058),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_RBRACE] = ACTIONS(119),
    [anon_sym_case] = ACTIONS(105),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
  [24] = {
    [sym_package_clause] = STATE(1886),
    [sym_package_object] = STATE(1886),
    [sym_import_declaration] = STATE(1886),
    [sym_object_definition] = STATE(1886),
    [sym_class_definition] = STATE(1886),
    [sym_trait_definition] = STATE(1886),
    [sym_annotation] = STATE(668),
    [sym_val_definition] = STATE(1886),
    [sym_val_declaration] = STATE(1886),
    [sym_var_declaration] = STATE(1886),
    [sym_var_definition] = STATE(1886),
    [sym_type_definition] = STATE(1886),
    [sym_function_definition] = STATE(1886),
    [sym_function_declaration] = STATE(1886),
    [sym_modifiers] = STATE(1692),
    [sym_block] = STATE(1063),
    [sym__expression] = STATE(1058),
    [sym_if_expression] = STATE(1063),
    [sym_match_expression] = STATE(1063),
    [sym_try_expression] = STATE(1063),
    [sym_case_block] = STATE(1063),
    [sym_assignment_expression] = STATE(1063),
    [sym_generic_function] = STATE(1063),
    [sym_call_expression] = STATE(1063),
    [sym_field_expression] = STATE(1063),
    [sym_instance_expression] = STATE(1063),
    [sym_infix_expression] = STATE(1063),
    [sym_prefix_expression] = STATE(1063),
    [sym_tuple_expression] = STATE(1063),
    [sym_parenthesized_expression] = STATE(1063),
    [sym_string_transform_expression] = STATE(1063),
    [sym_string] = STATE(1063),
    [aux_sym_class_definition_repeat1] = STATE(668),
    [aux_sym_modifiers_repeat1] = STATE(761),
    [sym_identifier] = ACTIONS(31),
    [anon_sym_package] = ACTIONS(33),
    [anon_sym_object] = ACTIONS(35),
    [anon_sym_import] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(39),
    [anon_sym_case] = ACTIONS(105),
    [anon_sym_class] = ACTIONS(45),
    [anon_sym_trait] = ACTIONS(47),
    [anon_sym_PLUS] = ACTIONS(49),
    [anon_sym_DASH] = ACTIONS(49),
    [anon_sym_AT] = ACTIONS(19),
    [anon_sym_val] = ACTIONS(51),
    [anon_sym_var] = ACTIONS(53),
    [anon_sym_type] = ACTIONS(55),
    [anon_sym_def] = ACTIONS(57),
    [anon_sym_abstract] = ACTIONS(59),
    [anon_sym_final] = ACTIONS(59),
    [anon_sym_sealed] = ACTIONS(59),
    [anon_sym_implicit] = ACTIONS(59),
    [anon_sym_lazy] = ACTIONS(59),
    [anon_sym_override] = ACTIONS(59),
    [anon_sym_private] = ACTIONS(59),
    [anon_sym_protected] = ACTIONS(59),
    [anon_sym_LPAREN] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(63),
    [anon_sym_try] = ACTIONS(65),
    [anon_sym_new] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(49),
    [anon_sym_TILDE] = ACTIONS(49),
    [sym_number] = ACTIONS(69),
    [sym_comment] = ACTIONS(3),
    [sym__simple_string] = ACTIONS(71),
    [sym__string_start] = ACTIONS(73),
    [sym__multiline_string_start] = ACTIONS(75),
  },
};

static uint16_t ts_small_parse_table[] = {
  [0] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(125), 1,
      ts_builtin_sym_end,
    ACTIONS(127), 1,
      anon_sym_package,
    ACTIONS(130), 1,
      anon_sym_object,
    ACTIONS(133), 1,
      anon_sym_import,
    ACTIONS(136), 1,
      anon_sym_case,
    ACTIONS(139), 1,
      anon_sym_class,
    ACTIONS(142), 1,
      anon_sym_trait,
    ACTIONS(145), 1,
      anon_sym_AT,
    ACTIONS(148), 1,
      anon_sym_val,
    ACTIONS(151), 1,
      anon_sym_var,
    ACTIONS(154), 1,
      anon_sym_type,
    ACTIONS(157), 1,
      anon_sym_def,
    STATE(761), 1,
      aux_sym_modifiers_repeat1,
    STATE(1671), 1,
      sym_modifiers,
    STATE(667), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(160), 8,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
    STATE(25), 14,
      sym_package_clause,
      sym_package_object,
      sym_import_declaration,
      sym_object_definition,
      sym_class_definition,
      sym_trait_definition,
      sym_val_definition,
      sym_val_declaration,
      sym_var_declaration,
      sym_var_definition,
      sym_type_definition,
      sym_function_definition,
      sym_function_declaration,
      aux_sym_compilation_unit_repeat1,
  [76] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(7), 1,
      anon_sym_package,
    ACTIONS(9), 1,
      anon_sym_object,
    ACTIONS(11), 1,
      anon_sym_import,
    ACTIONS(13), 1,
      anon_sym_case,
    ACTIONS(15), 1,
      anon_sym_class,
    ACTIONS(17), 1,
      anon_sym_trait,
    ACTIONS(19), 1,
      anon_sym_AT,
    ACTIONS(21), 1,
      anon_sym_val,
    ACTIONS(23), 1,
      anon_sym_var,
    ACTIONS(25), 1,
      anon_sym_type,
    ACTIONS(27), 1,
      anon_sym_def,
    ACTIONS(163), 1,
      ts_builtin_sym_end,
    STATE(761), 1,
      aux_sym_modifiers_repeat1,
    STATE(1671), 1,
      sym_modifiers,
    STATE(667), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(29), 8,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
    STATE(25), 14,
      sym_package_clause,
      sym_package_object,
      sym_import_declaration,
      sym_object_definition,
      sym_class_definition,
      sym_trait_definition,
      sym_val_definition,
      sym_val_declaration,
      sym_var_declaration,
      sym_var_definition,
      sym_type_definition,
      sym_function_definition,
      sym_function_declaration,
      aux_sym_compilation_unit_repeat1,
  [152] = 15,
    ACTIONS(165), 1,
      ts_builtin_sym_end,
    ACTIONS(171), 1,
      anon_sym_DOT,
    ACTIONS(173), 1,
      anon_sym_LBRACK,
    ACTIONS(175), 1,
      anon_sym_EQ,
    ACTIONS(177), 1,
      anon_sym_LPAREN,
    ACTIONS(179), 1,
      anon_sym_match,
    ACTIONS(181), 1,
      anon_sym_catch,
    ACTIONS(183), 1,
      anon_sym_finally,
    ACTIONS(185), 1,
      sym_comment,
    STATE(33), 1,
      sym_arguments,
    STATE(157), 1,
      sym_catch_clause,
    STATE(290), 1,
      sym_type_arguments,
    STATE(306), 1,
      sym_finally_clause,
    ACTIONS(167), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(169), 20,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
  [218] = 15,
    ACTIONS(165), 1,
      ts_builtin_sym_end,
    ACTIONS(171), 1,
      anon_sym_DOT,
    ACTIONS(173), 1,
      anon_sym_LBRACK,
    ACTIONS(175), 1,
      anon_sym_EQ,
    ACTIONS(177), 1,
      anon_sym_LPAREN,
    ACTIONS(179), 1,
      anon_sym_match,
    ACTIONS(181), 1,
      anon_sym_catch,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(187), 1,
      anon_sym_finally,
    STATE(33), 1,
      sym_arguments,
    STATE(275), 1,
      sym_catch_clause,
    STATE(290), 1,
      sym_type_arguments,
    STATE(417), 1,
      sym_finally_clause,
    ACTIONS(167), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(169), 20,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
  [284] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(193), 1,
      sym__simple_string,
    ACTIONS(195), 1,
      sym__string_start,
    ACTIONS(197), 1,
      sym__multiline_string_start,
    STATE(308), 1,
      sym_string,
    ACTIONS(189), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(191), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [334] = 15,
    ACTIONS(165), 1,
      ts_builtin_sym_end,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(201), 1,
      anon_sym_DOT,
    ACTIONS(203), 1,
      anon_sym_LBRACK,
    ACTIONS(205), 1,
      anon_sym_EQ,
    ACTIONS(207), 1,
      anon_sym_LPAREN,
    ACTIONS(209), 1,
      anon_sym_match,
    ACTIONS(211), 1,
      anon_sym_catch,
    ACTIONS(213), 1,
      anon_sym_finally,
    STATE(52), 1,
      sym_arguments,
    STATE(311), 1,
      sym_catch_clause,
    STATE(341), 1,
      sym_type_arguments,
    STATE(389), 1,
      sym_finally_clause,
    ACTIONS(199), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(169), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [399] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(215), 1,
      ts_builtin_sym_end,
    ACTIONS(219), 1,
      anon_sym_LBRACK,
    ACTIONS(221), 1,
      anon_sym_LPAREN,
    ACTIONS(223), 1,
      anon_sym_POUND,
    STATE(309), 1,
      sym_type_arguments,
    STATE(289), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(217), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [450] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(225), 1,
      sym__simple_string,
    ACTIONS(227), 1,
      sym__string_start,
    ACTIONS(229), 1,
      sym__multiline_string_start,
    STATE(346), 1,
      sym_string,
    ACTIONS(189), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(191), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [499] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(235), 1,
      anon_sym_LBRACE,
    STATE(285), 2,
      sym_block,
      sym_case_block,
    ACTIONS(231), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(233), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [544] = 15,
    ACTIONS(165), 1,
      ts_builtin_sym_end,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(201), 1,
      anon_sym_DOT,
    ACTIONS(203), 1,
      anon_sym_LBRACK,
    ACTIONS(205), 1,
      anon_sym_EQ,
    ACTIONS(207), 1,
      anon_sym_LPAREN,
    ACTIONS(209), 1,
      anon_sym_match,
    ACTIONS(211), 1,
      anon_sym_catch,
    ACTIONS(237), 1,
      anon_sym_finally,
    STATE(52), 1,
      sym_arguments,
    STATE(341), 1,
      sym_type_arguments,
    STATE(370), 1,
      sym_catch_clause,
    STATE(443), 1,
      sym_finally_clause,
    ACTIONS(199), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(169), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [609] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(247), 1,
      anon_sym_RPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1182), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [673] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(263), 1,
      anon_sym_RPAREN,
    STATE(1165), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [737] = 8,
    ACTIONS(171), 1,
      anon_sym_DOT,
    ACTIONS(173), 1,
      anon_sym_LBRACK,
    ACTIONS(177), 1,
      anon_sym_LPAREN,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(265), 1,
      ts_builtin_sym_end,
    STATE(33), 1,
      sym_arguments,
    STATE(290), 1,
      sym_type_arguments,
    ACTIONS(267), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [787] = 10,
    ACTIONS(171), 1,
      anon_sym_DOT,
    ACTIONS(173), 1,
      anon_sym_LBRACK,
    ACTIONS(175), 1,
      anon_sym_EQ,
    ACTIONS(177), 1,
      anon_sym_LPAREN,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(269), 1,
      ts_builtin_sym_end,
    STATE(33), 1,
      sym_arguments,
    STATE(290), 1,
      sym_type_arguments,
    ACTIONS(167), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(271), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
  [841] = 8,
    ACTIONS(171), 1,
      anon_sym_DOT,
    ACTIONS(173), 1,
      anon_sym_LBRACK,
    ACTIONS(177), 1,
      anon_sym_LPAREN,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(273), 1,
      ts_builtin_sym_end,
    STATE(33), 1,
      sym_arguments,
    STATE(290), 1,
      sym_type_arguments,
    ACTIONS(275), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [891] = 8,
    ACTIONS(171), 1,
      anon_sym_DOT,
    ACTIONS(173), 1,
      anon_sym_LBRACK,
    ACTIONS(177), 1,
      anon_sym_LPAREN,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(277), 1,
      ts_builtin_sym_end,
    STATE(33), 1,
      sym_arguments,
    STATE(290), 1,
      sym_type_arguments,
    ACTIONS(279), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [941] = 11,
    ACTIONS(171), 1,
      anon_sym_DOT,
    ACTIONS(173), 1,
      anon_sym_LBRACK,
    ACTIONS(175), 1,
      anon_sym_EQ,
    ACTIONS(177), 1,
      anon_sym_LPAREN,
    ACTIONS(179), 1,
      anon_sym_match,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(281), 1,
      ts_builtin_sym_end,
    STATE(33), 1,
      sym_arguments,
    STATE(290), 1,
      sym_type_arguments,
    ACTIONS(167), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(283), 22,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_catch,
      anon_sym_finally,
  [997] = 11,
    ACTIONS(171), 1,
      anon_sym_DOT,
    ACTIONS(173), 1,
      anon_sym_LBRACK,
    ACTIONS(175), 1,
      anon_sym_EQ,
    ACTIONS(177), 1,
      anon_sym_LPAREN,
    ACTIONS(179), 1,
      anon_sym_match,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(285), 1,
      ts_builtin_sym_end,
    STATE(33), 1,
      sym_arguments,
    STATE(290), 1,
      sym_type_arguments,
    ACTIONS(167), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(287), 22,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_catch,
      anon_sym_finally,
  [1053] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(289), 1,
      sym__simple_string,
    ACTIONS(291), 1,
      sym__string_start,
    ACTIONS(293), 1,
      sym__multiline_string_start,
    STATE(432), 1,
      sym_string,
    ACTIONS(189), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(191), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [1101] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(295), 1,
      anon_sym_LBRACK,
    ACTIONS(297), 1,
      anon_sym_LPAREN,
    ACTIONS(299), 1,
      anon_sym_POUND,
    STATE(315), 1,
      sym_type_arguments,
    ACTIONS(215), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    STATE(364), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(217), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [1151] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(301), 1,
      anon_sym_RPAREN,
    STATE(1129), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [1215] = 12,
    ACTIONS(171), 1,
      anon_sym_DOT,
    ACTIONS(173), 1,
      anon_sym_LBRACK,
    ACTIONS(175), 1,
      anon_sym_EQ,
    ACTIONS(177), 1,
      anon_sym_LPAREN,
    ACTIONS(179), 1,
      anon_sym_match,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(303), 1,
      ts_builtin_sym_end,
    ACTIONS(307), 1,
      anon_sym_else,
    STATE(33), 1,
      sym_arguments,
    STATE(290), 1,
      sym_type_arguments,
    ACTIONS(167), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(305), 21,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_catch,
      anon_sym_finally,
  [1273] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(309), 1,
      anon_sym_RPAREN,
    STATE(1158), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [1337] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(311), 1,
      anon_sym_RPAREN,
    STATE(1156), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [1401] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(313), 1,
      anon_sym_RPAREN,
    STATE(1161), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [1465] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(315), 1,
      anon_sym_RPAREN,
    STATE(1142), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [1529] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(317), 1,
      anon_sym_RPAREN,
    STATE(1118), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [1593] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(319), 1,
      anon_sym_LBRACE,
    STATE(368), 2,
      sym_block,
      sym_case_block,
    ACTIONS(231), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(233), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [1637] = 12,
    ACTIONS(171), 1,
      anon_sym_DOT,
    ACTIONS(173), 1,
      anon_sym_LBRACK,
    ACTIONS(175), 1,
      anon_sym_EQ,
    ACTIONS(177), 1,
      anon_sym_LPAREN,
    ACTIONS(179), 1,
      anon_sym_match,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(303), 1,
      ts_builtin_sym_end,
    ACTIONS(321), 1,
      anon_sym_else,
    STATE(33), 1,
      sym_arguments,
    STATE(290), 1,
      sym_type_arguments,
    ACTIONS(167), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(305), 21,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_catch,
      anon_sym_finally,
  [1695] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(323), 1,
      anon_sym_RPAREN,
    STATE(1138), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [1759] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(325), 1,
      anon_sym_RPAREN,
    STATE(1160), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [1823] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(327), 1,
      anon_sym_RPAREN,
    STATE(1212), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [1887] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(329), 1,
      anon_sym_RPAREN,
    STATE(1193), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [1951] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(331), 1,
      anon_sym_RPAREN,
    STATE(1147), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [2015] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(333), 1,
      anon_sym_RPAREN,
    STATE(1159), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [2079] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(335), 1,
      anon_sym_RPAREN,
    STATE(1164), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [2143] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(337), 1,
      anon_sym_RPAREN,
    STATE(1167), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [2207] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(339), 1,
      anon_sym_RPAREN,
    STATE(1169), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [2271] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(341), 1,
      anon_sym_RPAREN,
    STATE(1175), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [2335] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(343), 1,
      anon_sym_RPAREN,
    STATE(1153), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [2399] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(345), 1,
      anon_sym_RPAREN,
    STATE(1179), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [2463] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(347), 1,
      anon_sym_RPAREN,
    STATE(1203), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [2527] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(349), 1,
      anon_sym_RPAREN,
    STATE(1197), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [2591] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(351), 1,
      anon_sym_RPAREN,
    STATE(1184), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [2655] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(353), 1,
      anon_sym_RPAREN,
    STATE(1188), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [2719] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(219), 1,
      anon_sym_LBRACK,
    ACTIONS(223), 1,
      anon_sym_POUND,
    ACTIONS(355), 1,
      ts_builtin_sym_end,
    ACTIONS(359), 1,
      anon_sym_AT,
    STATE(309), 1,
      sym_type_arguments,
    STATE(351), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(357), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [2769] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(361), 1,
      anon_sym_RPAREN,
    STATE(1196), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [2833] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(363), 1,
      anon_sym_RPAREN,
    STATE(1122), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [2897] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(365), 1,
      anon_sym_RPAREN,
    STATE(1127), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [2961] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    ACTIONS(367), 1,
      anon_sym_RPAREN,
    STATE(1205), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [3025] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1346), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [3086] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(314), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [3147] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(993), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [3208] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(994), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [3269] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(391), 1,
      sym_identifier,
    ACTIONS(393), 1,
      anon_sym_LBRACE,
    ACTIONS(397), 1,
      anon_sym_LPAREN,
    ACTIONS(399), 1,
      anon_sym_if,
    ACTIONS(401), 1,
      anon_sym_try,
    ACTIONS(403), 1,
      anon_sym_new,
    ACTIONS(405), 1,
      sym_number,
    ACTIONS(407), 1,
      sym__simple_string,
    ACTIONS(409), 1,
      sym__string_start,
    ACTIONS(411), 1,
      sym__multiline_string_start,
    STATE(736), 1,
      sym__expression,
    ACTIONS(395), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(935), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [3330] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(998), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [3391] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1001), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [3452] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1194), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [3513] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1002), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [3574] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1004), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [3635] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1005), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [3696] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(413), 1,
      anon_sym_LBRACK,
    ACTIONS(415), 1,
      anon_sym_LPAREN,
    ACTIONS(417), 1,
      anon_sym_POUND,
    STATE(427), 1,
      sym_type_arguments,
    ACTIONS(215), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    STATE(424), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(217), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [3745] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(489), 1,
      sym_string,
    ACTIONS(189), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(191), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [3792] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(378), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [3853] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(387), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [3914] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(419), 1,
      sym_identifier,
    ACTIONS(421), 1,
      anon_sym_LBRACE,
    ACTIONS(425), 1,
      anon_sym_LPAREN,
    ACTIONS(427), 1,
      anon_sym_if,
    ACTIONS(429), 1,
      anon_sym_try,
    ACTIONS(431), 1,
      anon_sym_new,
    ACTIONS(433), 1,
      sym_number,
    ACTIONS(435), 1,
      sym__simple_string,
    ACTIONS(437), 1,
      sym__string_start,
    ACTIONS(439), 1,
      sym__multiline_string_start,
    STATE(1132), 1,
      sym__expression,
    ACTIONS(423), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1392), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [3975] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(419), 1,
      sym_identifier,
    ACTIONS(421), 1,
      anon_sym_LBRACE,
    ACTIONS(425), 1,
      anon_sym_LPAREN,
    ACTIONS(427), 1,
      anon_sym_if,
    ACTIONS(429), 1,
      anon_sym_try,
    ACTIONS(431), 1,
      anon_sym_new,
    ACTIONS(433), 1,
      sym_number,
    ACTIONS(435), 1,
      sym__simple_string,
    ACTIONS(437), 1,
      sym__string_start,
    ACTIONS(439), 1,
      sym__multiline_string_start,
    STATE(1130), 1,
      sym__expression,
    ACTIONS(423), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1392), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [4036] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(394), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [4097] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(375), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [4158] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(413), 1,
      anon_sym_LBRACK,
    ACTIONS(417), 1,
      anon_sym_POUND,
    ACTIONS(441), 1,
      anon_sym_AT,
    STATE(427), 1,
      sym_type_arguments,
    STATE(413), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(355), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(357), 22,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [4207] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(201), 1,
      anon_sym_DOT,
    ACTIONS(203), 1,
      anon_sym_LBRACK,
    ACTIONS(207), 1,
      anon_sym_LPAREN,
    ACTIONS(265), 1,
      ts_builtin_sym_end,
    STATE(52), 1,
      sym_arguments,
    STATE(341), 1,
      sym_type_arguments,
    ACTIONS(267), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [4256] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(201), 1,
      anon_sym_DOT,
    ACTIONS(203), 1,
      anon_sym_LBRACK,
    ACTIONS(207), 1,
      anon_sym_LPAREN,
    ACTIONS(277), 1,
      ts_builtin_sym_end,
    STATE(52), 1,
      sym_arguments,
    STATE(341), 1,
      sym_type_arguments,
    ACTIONS(279), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [4305] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(225), 1,
      sym__simple_string,
    ACTIONS(227), 1,
      sym__string_start,
    ACTIONS(229), 1,
      sym__multiline_string_start,
    ACTIONS(319), 1,
      anon_sym_LBRACE,
    ACTIONS(443), 1,
      sym_identifier,
    ACTIONS(447), 1,
      anon_sym_LPAREN,
    ACTIONS(449), 1,
      anon_sym_if,
    ACTIONS(451), 1,
      anon_sym_try,
    ACTIONS(453), 1,
      anon_sym_new,
    ACTIONS(455), 1,
      sym_number,
    STATE(104), 1,
      sym__expression,
    ACTIONS(445), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(353), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [4366] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(225), 1,
      sym__simple_string,
    ACTIONS(227), 1,
      sym__string_start,
    ACTIONS(229), 1,
      sym__multiline_string_start,
    ACTIONS(319), 1,
      anon_sym_LBRACE,
    ACTIONS(443), 1,
      sym_identifier,
    ACTIONS(447), 1,
      anon_sym_LPAREN,
    ACTIONS(449), 1,
      anon_sym_if,
    ACTIONS(451), 1,
      anon_sym_try,
    ACTIONS(453), 1,
      anon_sym_new,
    ACTIONS(455), 1,
      sym_number,
    STATE(111), 1,
      sym__expression,
    ACTIONS(445), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(353), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [4427] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(225), 1,
      sym__simple_string,
    ACTIONS(227), 1,
      sym__string_start,
    ACTIONS(229), 1,
      sym__multiline_string_start,
    ACTIONS(319), 1,
      anon_sym_LBRACE,
    ACTIONS(443), 1,
      sym_identifier,
    ACTIONS(447), 1,
      anon_sym_LPAREN,
    ACTIONS(449), 1,
      anon_sym_if,
    ACTIONS(451), 1,
      anon_sym_try,
    ACTIONS(453), 1,
      anon_sym_new,
    ACTIONS(455), 1,
      sym_number,
    STATE(121), 1,
      sym__expression,
    ACTIONS(445), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(353), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [4488] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1236), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [4549] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(321), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [4610] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1097), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [4671] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1344), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [4732] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(201), 1,
      anon_sym_DOT,
    ACTIONS(203), 1,
      anon_sym_LBRACK,
    ACTIONS(205), 1,
      anon_sym_EQ,
    ACTIONS(207), 1,
      anon_sym_LPAREN,
    ACTIONS(269), 1,
      ts_builtin_sym_end,
    STATE(52), 1,
      sym_arguments,
    STATE(341), 1,
      sym_type_arguments,
    ACTIONS(199), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(271), 22,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
  [4785] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(366), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [4846] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(365), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [4907] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(361), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [4968] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(367), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [5029] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1208), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [5090] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(457), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACE,
    ACTIONS(463), 1,
      anon_sym_LPAREN,
    ACTIONS(465), 1,
      anon_sym_if,
    ACTIONS(467), 1,
      anon_sym_try,
    ACTIONS(469), 1,
      anon_sym_new,
    ACTIONS(471), 1,
      sym_number,
    ACTIONS(473), 1,
      sym__simple_string,
    ACTIONS(475), 1,
      sym__string_start,
    ACTIONS(477), 1,
      sym__multiline_string_start,
    STATE(991), 1,
      sym__expression,
    ACTIONS(461), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1377), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [5151] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(201), 1,
      anon_sym_DOT,
    ACTIONS(203), 1,
      anon_sym_LBRACK,
    ACTIONS(207), 1,
      anon_sym_LPAREN,
    ACTIONS(273), 1,
      ts_builtin_sym_end,
    STATE(52), 1,
      sym_arguments,
    STATE(341), 1,
      sym_type_arguments,
    ACTIONS(275), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [5200] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(457), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACE,
    ACTIONS(463), 1,
      anon_sym_LPAREN,
    ACTIONS(465), 1,
      anon_sym_if,
    ACTIONS(467), 1,
      anon_sym_try,
    ACTIONS(469), 1,
      anon_sym_new,
    ACTIONS(471), 1,
      sym_number,
    ACTIONS(473), 1,
      sym__simple_string,
    ACTIONS(475), 1,
      sym__string_start,
    ACTIONS(477), 1,
      sym__multiline_string_start,
    STATE(775), 1,
      sym__expression,
    ACTIONS(461), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1377), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [5261] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(381), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [5322] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(457), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACE,
    ACTIONS(463), 1,
      anon_sym_LPAREN,
    ACTIONS(465), 1,
      anon_sym_if,
    ACTIONS(467), 1,
      anon_sym_try,
    ACTIONS(469), 1,
      anon_sym_new,
    ACTIONS(471), 1,
      sym_number,
    ACTIONS(473), 1,
      sym__simple_string,
    ACTIONS(475), 1,
      sym__string_start,
    ACTIONS(477), 1,
      sym__multiline_string_start,
    STATE(995), 1,
      sym__expression,
    ACTIONS(461), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1377), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [5383] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(457), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACE,
    ACTIONS(463), 1,
      anon_sym_LPAREN,
    ACTIONS(465), 1,
      anon_sym_if,
    ACTIONS(467), 1,
      anon_sym_try,
    ACTIONS(469), 1,
      anon_sym_new,
    ACTIONS(471), 1,
      sym_number,
    ACTIONS(473), 1,
      sym__simple_string,
    ACTIONS(475), 1,
      sym__string_start,
    ACTIONS(477), 1,
      sym__multiline_string_start,
    STATE(1109), 1,
      sym__expression,
    ACTIONS(461), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1377), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [5444] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(419), 1,
      sym_identifier,
    ACTIONS(421), 1,
      anon_sym_LBRACE,
    ACTIONS(425), 1,
      anon_sym_LPAREN,
    ACTIONS(427), 1,
      anon_sym_if,
    ACTIONS(429), 1,
      anon_sym_try,
    ACTIONS(431), 1,
      anon_sym_new,
    ACTIONS(433), 1,
      sym_number,
    ACTIONS(435), 1,
      sym__simple_string,
    ACTIONS(437), 1,
      sym__string_start,
    ACTIONS(439), 1,
      sym__multiline_string_start,
    STATE(1143), 1,
      sym__expression,
    ACTIONS(423), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1392), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [5505] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(398), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [5566] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(419), 1,
      sym_identifier,
    ACTIONS(421), 1,
      anon_sym_LBRACE,
    ACTIONS(425), 1,
      anon_sym_LPAREN,
    ACTIONS(427), 1,
      anon_sym_if,
    ACTIONS(429), 1,
      anon_sym_try,
    ACTIONS(431), 1,
      anon_sym_new,
    ACTIONS(433), 1,
      sym_number,
    ACTIONS(435), 1,
      sym__simple_string,
    ACTIONS(437), 1,
      sym__string_start,
    ACTIONS(439), 1,
      sym__multiline_string_start,
    STATE(860), 1,
      sym__expression,
    ACTIONS(423), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1392), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [5627] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1347), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [5688] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(225), 1,
      sym__simple_string,
    ACTIONS(227), 1,
      sym__string_start,
    ACTIONS(229), 1,
      sym__multiline_string_start,
    ACTIONS(319), 1,
      anon_sym_LBRACE,
    ACTIONS(443), 1,
      sym_identifier,
    ACTIONS(447), 1,
      anon_sym_LPAREN,
    ACTIONS(449), 1,
      anon_sym_if,
    ACTIONS(451), 1,
      anon_sym_try,
    ACTIONS(453), 1,
      anon_sym_new,
    ACTIONS(455), 1,
      sym_number,
    STATE(128), 1,
      sym__expression,
    ACTIONS(445), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(353), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [5749] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(201), 1,
      anon_sym_DOT,
    ACTIONS(203), 1,
      anon_sym_LBRACK,
    ACTIONS(205), 1,
      anon_sym_EQ,
    ACTIONS(207), 1,
      anon_sym_LPAREN,
    ACTIONS(209), 1,
      anon_sym_match,
    ACTIONS(281), 1,
      ts_builtin_sym_end,
    STATE(52), 1,
      sym_arguments,
    STATE(341), 1,
      sym_type_arguments,
    ACTIONS(199), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(283), 21,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_catch,
      anon_sym_finally,
  [5804] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(419), 1,
      sym_identifier,
    ACTIONS(421), 1,
      anon_sym_LBRACE,
    ACTIONS(425), 1,
      anon_sym_LPAREN,
    ACTIONS(427), 1,
      anon_sym_if,
    ACTIONS(429), 1,
      anon_sym_try,
    ACTIONS(431), 1,
      anon_sym_new,
    ACTIONS(433), 1,
      sym_number,
    ACTIONS(435), 1,
      sym__simple_string,
    ACTIONS(437), 1,
      sym__string_start,
    ACTIONS(439), 1,
      sym__multiline_string_start,
    STATE(1144), 1,
      sym__expression,
    ACTIONS(423), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1392), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [5865] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(457), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACE,
    ACTIONS(463), 1,
      anon_sym_LPAREN,
    ACTIONS(465), 1,
      anon_sym_if,
    ACTIONS(467), 1,
      anon_sym_try,
    ACTIONS(469), 1,
      anon_sym_new,
    ACTIONS(471), 1,
      sym_number,
    ACTIONS(473), 1,
      sym__simple_string,
    ACTIONS(475), 1,
      sym__string_start,
    ACTIONS(477), 1,
      sym__multiline_string_start,
    STATE(1098), 1,
      sym__expression,
    ACTIONS(461), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1377), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [5926] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(193), 1,
      sym__simple_string,
    ACTIONS(195), 1,
      sym__string_start,
    ACTIONS(197), 1,
      sym__multiline_string_start,
    ACTIONS(235), 1,
      anon_sym_LBRACE,
    ACTIONS(479), 1,
      sym_identifier,
    ACTIONS(483), 1,
      anon_sym_LPAREN,
    ACTIONS(485), 1,
      anon_sym_if,
    ACTIONS(487), 1,
      anon_sym_try,
    ACTIONS(489), 1,
      anon_sym_new,
    ACTIONS(491), 1,
      sym_number,
    STATE(40), 1,
      sym__expression,
    ACTIONS(481), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(272), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [5987] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(193), 1,
      sym__simple_string,
    ACTIONS(195), 1,
      sym__string_start,
    ACTIONS(197), 1,
      sym__multiline_string_start,
    ACTIONS(235), 1,
      anon_sym_LBRACE,
    ACTIONS(479), 1,
      sym_identifier,
    ACTIONS(483), 1,
      anon_sym_LPAREN,
    ACTIONS(485), 1,
      anon_sym_if,
    ACTIONS(487), 1,
      anon_sym_try,
    ACTIONS(489), 1,
      anon_sym_new,
    ACTIONS(491), 1,
      sym_number,
    STATE(27), 1,
      sym__expression,
    ACTIONS(481), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(272), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [6048] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(193), 1,
      sym__simple_string,
    ACTIONS(195), 1,
      sym__string_start,
    ACTIONS(197), 1,
      sym__multiline_string_start,
    ACTIONS(235), 1,
      anon_sym_LBRACE,
    ACTIONS(479), 1,
      sym_identifier,
    ACTIONS(483), 1,
      anon_sym_LPAREN,
    ACTIONS(485), 1,
      anon_sym_if,
    ACTIONS(487), 1,
      anon_sym_try,
    ACTIONS(489), 1,
      anon_sym_new,
    ACTIONS(491), 1,
      sym_number,
    STATE(37), 1,
      sym__expression,
    ACTIONS(481), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(272), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [6109] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(390), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [6170] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(201), 1,
      anon_sym_DOT,
    ACTIONS(203), 1,
      anon_sym_LBRACK,
    ACTIONS(205), 1,
      anon_sym_EQ,
    ACTIONS(207), 1,
      anon_sym_LPAREN,
    ACTIONS(209), 1,
      anon_sym_match,
    ACTIONS(285), 1,
      ts_builtin_sym_end,
    STATE(52), 1,
      sym_arguments,
    STATE(341), 1,
      sym_type_arguments,
    ACTIONS(199), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(287), 21,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_catch,
      anon_sym_finally,
  [6225] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(388), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [6286] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(386), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [6347] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(384), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [6408] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(777), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [6469] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1135), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [6530] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(193), 1,
      sym__simple_string,
    ACTIONS(195), 1,
      sym__string_start,
    ACTIONS(197), 1,
      sym__multiline_string_start,
    ACTIONS(235), 1,
      anon_sym_LBRACE,
    ACTIONS(479), 1,
      sym_identifier,
    ACTIONS(483), 1,
      anon_sym_LPAREN,
    ACTIONS(485), 1,
      anon_sym_if,
    ACTIONS(487), 1,
      anon_sym_try,
    ACTIONS(489), 1,
      anon_sym_new,
    ACTIONS(491), 1,
      sym_number,
    STATE(46), 1,
      sym__expression,
    ACTIONS(481), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(272), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [6591] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(703), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [6652] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(774), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [6713] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(383), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [6774] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(289), 1,
      sym__simple_string,
    ACTIONS(291), 1,
      sym__string_start,
    ACTIONS(293), 1,
      sym__multiline_string_start,
    ACTIONS(493), 1,
      sym_identifier,
    ACTIONS(495), 1,
      anon_sym_LBRACE,
    ACTIONS(499), 1,
      anon_sym_LPAREN,
    ACTIONS(501), 1,
      anon_sym_if,
    ACTIONS(503), 1,
      anon_sym_try,
    ACTIONS(505), 1,
      anon_sym_new,
    ACTIONS(507), 1,
      sym_number,
    STATE(278), 1,
      sym__expression,
    ACTIONS(497), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(425), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [6835] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(193), 1,
      sym__simple_string,
    ACTIONS(195), 1,
      sym__string_start,
    ACTIONS(197), 1,
      sym__multiline_string_start,
    ACTIONS(235), 1,
      anon_sym_LBRACE,
    ACTIONS(479), 1,
      sym_identifier,
    ACTIONS(483), 1,
      anon_sym_LPAREN,
    ACTIONS(485), 1,
      anon_sym_if,
    ACTIONS(487), 1,
      anon_sym_try,
    ACTIONS(489), 1,
      anon_sym_new,
    ACTIONS(491), 1,
      sym_number,
    STATE(28), 1,
      sym__expression,
    ACTIONS(481), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(272), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [6896] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(289), 1,
      sym__simple_string,
    ACTIONS(291), 1,
      sym__string_start,
    ACTIONS(293), 1,
      sym__multiline_string_start,
    ACTIONS(493), 1,
      sym_identifier,
    ACTIONS(495), 1,
      anon_sym_LBRACE,
    ACTIONS(499), 1,
      anon_sym_LPAREN,
    ACTIONS(501), 1,
      anon_sym_if,
    ACTIONS(503), 1,
      anon_sym_try,
    ACTIONS(505), 1,
      anon_sym_new,
    ACTIONS(507), 1,
      sym_number,
    STATE(279), 1,
      sym__expression,
    ACTIONS(497), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(425), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [6957] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1104), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [7018] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(380), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [7079] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1069), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [7140] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(371), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [7201] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(513), 1,
      anon_sym_DOT,
    ACTIONS(509), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(511), 27,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [7242] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1107), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [7303] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1360), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [7364] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1362), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [7425] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1374), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [7486] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(989), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [7547] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(193), 1,
      sym__simple_string,
    ACTIONS(195), 1,
      sym__string_start,
    ACTIONS(197), 1,
      sym__multiline_string_start,
    ACTIONS(235), 1,
      anon_sym_LBRACE,
    ACTIONS(479), 1,
      sym_identifier,
    ACTIONS(483), 1,
      anon_sym_LPAREN,
    ACTIONS(485), 1,
      anon_sym_if,
    ACTIONS(487), 1,
      anon_sym_try,
    ACTIONS(489), 1,
      anon_sym_new,
    ACTIONS(491), 1,
      sym_number,
    STATE(41), 1,
      sym__expression,
    ACTIONS(481), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(272), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [7608] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(289), 1,
      sym__simple_string,
    ACTIONS(291), 1,
      sym__string_start,
    ACTIONS(293), 1,
      sym__multiline_string_start,
    ACTIONS(493), 1,
      sym_identifier,
    ACTIONS(495), 1,
      anon_sym_LBRACE,
    ACTIONS(499), 1,
      anon_sym_LPAREN,
    ACTIONS(501), 1,
      anon_sym_if,
    ACTIONS(503), 1,
      anon_sym_try,
    ACTIONS(505), 1,
      anon_sym_new,
    ACTIONS(507), 1,
      sym_number,
    STATE(298), 1,
      sym__expression,
    ACTIONS(497), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(425), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [7669] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(515), 1,
      sym_identifier,
    ACTIONS(517), 1,
      anon_sym_LBRACE,
    ACTIONS(521), 1,
      anon_sym_LPAREN,
    ACTIONS(523), 1,
      anon_sym_if,
    ACTIONS(525), 1,
      anon_sym_try,
    ACTIONS(527), 1,
      anon_sym_new,
    ACTIONS(529), 1,
      sym_number,
    ACTIONS(531), 1,
      sym__simple_string,
    ACTIONS(533), 1,
      sym__string_start,
    ACTIONS(535), 1,
      sym__multiline_string_start,
    STATE(894), 1,
      sym__expression,
    ACTIONS(519), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1191), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [7730] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(515), 1,
      sym_identifier,
    ACTIONS(517), 1,
      anon_sym_LBRACE,
    ACTIONS(521), 1,
      anon_sym_LPAREN,
    ACTIONS(523), 1,
      anon_sym_if,
    ACTIONS(525), 1,
      anon_sym_try,
    ACTIONS(527), 1,
      anon_sym_new,
    ACTIONS(529), 1,
      sym_number,
    ACTIONS(531), 1,
      sym__simple_string,
    ACTIONS(533), 1,
      sym__string_start,
    ACTIONS(535), 1,
      sym__multiline_string_start,
    STATE(724), 1,
      sym__expression,
    ACTIONS(519), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1191), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [7791] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(515), 1,
      sym_identifier,
    ACTIONS(517), 1,
      anon_sym_LBRACE,
    ACTIONS(521), 1,
      anon_sym_LPAREN,
    ACTIONS(523), 1,
      anon_sym_if,
    ACTIONS(525), 1,
      anon_sym_try,
    ACTIONS(527), 1,
      anon_sym_new,
    ACTIONS(529), 1,
      sym_number,
    ACTIONS(531), 1,
      sym__simple_string,
    ACTIONS(533), 1,
      sym__string_start,
    ACTIONS(535), 1,
      sym__multiline_string_start,
    STATE(781), 1,
      sym__expression,
    ACTIONS(519), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1191), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [7852] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1092), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [7913] = 5,
    ACTIONS(183), 1,
      anon_sym_finally,
    ACTIONS(185), 1,
      sym_comment,
    STATE(282), 1,
      sym_finally_clause,
    ACTIONS(537), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(539), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      sym_identifier,
      sym_operator_identifier,
  [7956] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(515), 1,
      sym_identifier,
    ACTIONS(517), 1,
      anon_sym_LBRACE,
    ACTIONS(521), 1,
      anon_sym_LPAREN,
    ACTIONS(523), 1,
      anon_sym_if,
    ACTIONS(525), 1,
      anon_sym_try,
    ACTIONS(527), 1,
      anon_sym_new,
    ACTIONS(529), 1,
      sym_number,
    ACTIONS(531), 1,
      sym__simple_string,
    ACTIONS(533), 1,
      sym__string_start,
    ACTIONS(535), 1,
      sym__multiline_string_start,
    STATE(959), 1,
      sym__expression,
    ACTIONS(519), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1191), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8017] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(457), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACE,
    ACTIONS(463), 1,
      anon_sym_LPAREN,
    ACTIONS(465), 1,
      anon_sym_if,
    ACTIONS(467), 1,
      anon_sym_try,
    ACTIONS(469), 1,
      anon_sym_new,
    ACTIONS(471), 1,
      sym_number,
    ACTIONS(473), 1,
      sym__simple_string,
    ACTIONS(475), 1,
      sym__string_start,
    ACTIONS(477), 1,
      sym__multiline_string_start,
    STATE(1042), 1,
      sym__expression,
    ACTIONS(461), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1377), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8078] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(391), 1,
      sym_identifier,
    ACTIONS(393), 1,
      anon_sym_LBRACE,
    ACTIONS(397), 1,
      anon_sym_LPAREN,
    ACTIONS(399), 1,
      anon_sym_if,
    ACTIONS(401), 1,
      anon_sym_try,
    ACTIONS(403), 1,
      anon_sym_new,
    ACTIONS(405), 1,
      sym_number,
    ACTIONS(407), 1,
      sym__simple_string,
    ACTIONS(409), 1,
      sym__string_start,
    ACTIONS(411), 1,
      sym__multiline_string_start,
    STATE(717), 1,
      sym__expression,
    ACTIONS(395), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(935), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8139] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(391), 1,
      sym_identifier,
    ACTIONS(393), 1,
      anon_sym_LBRACE,
    ACTIONS(397), 1,
      anon_sym_LPAREN,
    ACTIONS(399), 1,
      anon_sym_if,
    ACTIONS(401), 1,
      anon_sym_try,
    ACTIONS(403), 1,
      anon_sym_new,
    ACTIONS(405), 1,
      sym_number,
    ACTIONS(407), 1,
      sym__simple_string,
    ACTIONS(409), 1,
      sym__string_start,
    ACTIONS(411), 1,
      sym__multiline_string_start,
    STATE(674), 1,
      sym__expression,
    ACTIONS(395), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(935), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8200] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(391), 1,
      sym_identifier,
    ACTIONS(393), 1,
      anon_sym_LBRACE,
    ACTIONS(397), 1,
      anon_sym_LPAREN,
    ACTIONS(399), 1,
      anon_sym_if,
    ACTIONS(401), 1,
      anon_sym_try,
    ACTIONS(403), 1,
      anon_sym_new,
    ACTIONS(405), 1,
      sym_number,
    ACTIONS(407), 1,
      sym__simple_string,
    ACTIONS(409), 1,
      sym__string_start,
    ACTIONS(411), 1,
      sym__multiline_string_start,
    STATE(715), 1,
      sym__expression,
    ACTIONS(395), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(935), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8261] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(543), 27,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [8302] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(457), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACE,
    ACTIONS(463), 1,
      anon_sym_LPAREN,
    ACTIONS(465), 1,
      anon_sym_if,
    ACTIONS(467), 1,
      anon_sym_try,
    ACTIONS(469), 1,
      anon_sym_new,
    ACTIONS(471), 1,
      sym_number,
    ACTIONS(473), 1,
      sym__simple_string,
    ACTIONS(475), 1,
      sym__string_start,
    ACTIONS(477), 1,
      sym__multiline_string_start,
    STATE(1045), 1,
      sym__expression,
    ACTIONS(461), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1377), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8363] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1113), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8424] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(391), 1,
      sym_identifier,
    ACTIONS(393), 1,
      anon_sym_LBRACE,
    ACTIONS(397), 1,
      anon_sym_LPAREN,
    ACTIONS(399), 1,
      anon_sym_if,
    ACTIONS(401), 1,
      anon_sym_try,
    ACTIONS(403), 1,
      anon_sym_new,
    ACTIONS(405), 1,
      sym_number,
    ACTIONS(407), 1,
      sym__simple_string,
    ACTIONS(409), 1,
      sym__string_start,
    ACTIONS(411), 1,
      sym__multiline_string_start,
    STATE(748), 1,
      sym__expression,
    ACTIONS(395), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(935), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8485] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1020), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8546] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(391), 1,
      sym_identifier,
    ACTIONS(393), 1,
      anon_sym_LBRACE,
    ACTIONS(397), 1,
      anon_sym_LPAREN,
    ACTIONS(399), 1,
      anon_sym_if,
    ACTIONS(401), 1,
      anon_sym_try,
    ACTIONS(403), 1,
      anon_sym_new,
    ACTIONS(405), 1,
      sym_number,
    ACTIONS(407), 1,
      sym__simple_string,
    ACTIONS(409), 1,
      sym__string_start,
    ACTIONS(411), 1,
      sym__multiline_string_start,
    STATE(746), 1,
      sym__expression,
    ACTIONS(395), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(935), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8607] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(765), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8668] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(457), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACE,
    ACTIONS(463), 1,
      anon_sym_LPAREN,
    ACTIONS(465), 1,
      anon_sym_if,
    ACTIONS(467), 1,
      anon_sym_try,
    ACTIONS(469), 1,
      anon_sym_new,
    ACTIONS(471), 1,
      sym_number,
    ACTIONS(473), 1,
      sym__simple_string,
    ACTIONS(475), 1,
      sym__string_start,
    ACTIONS(477), 1,
      sym__multiline_string_start,
    STATE(1052), 1,
      sym__expression,
    ACTIONS(461), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1377), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8729] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(419), 1,
      sym_identifier,
    ACTIONS(421), 1,
      anon_sym_LBRACE,
    ACTIONS(425), 1,
      anon_sym_LPAREN,
    ACTIONS(427), 1,
      anon_sym_if,
    ACTIONS(429), 1,
      anon_sym_try,
    ACTIONS(431), 1,
      anon_sym_new,
    ACTIONS(433), 1,
      sym_number,
    ACTIONS(435), 1,
      sym__simple_string,
    ACTIONS(437), 1,
      sym__string_start,
    ACTIONS(439), 1,
      sym__multiline_string_start,
    STATE(1401), 1,
      sym__expression,
    ACTIONS(423), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1392), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8790] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1099), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8851] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1157), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8912] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(992), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [8973] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1044), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9034] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(225), 1,
      sym__simple_string,
    ACTIONS(227), 1,
      sym__string_start,
    ACTIONS(229), 1,
      sym__multiline_string_start,
    ACTIONS(319), 1,
      anon_sym_LBRACE,
    ACTIONS(443), 1,
      sym_identifier,
    ACTIONS(447), 1,
      anon_sym_LPAREN,
    ACTIONS(449), 1,
      anon_sym_if,
    ACTIONS(451), 1,
      anon_sym_try,
    ACTIONS(453), 1,
      anon_sym_new,
    ACTIONS(455), 1,
      sym_number,
    STATE(96), 1,
      sym__expression,
    ACTIONS(445), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(353), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9095] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(225), 1,
      sym__simple_string,
    ACTIONS(227), 1,
      sym__string_start,
    ACTIONS(229), 1,
      sym__multiline_string_start,
    ACTIONS(319), 1,
      anon_sym_LBRACE,
    ACTIONS(443), 1,
      sym_identifier,
    ACTIONS(447), 1,
      anon_sym_LPAREN,
    ACTIONS(449), 1,
      anon_sym_if,
    ACTIONS(451), 1,
      anon_sym_try,
    ACTIONS(453), 1,
      anon_sym_new,
    ACTIONS(455), 1,
      sym_number,
    STATE(30), 1,
      sym__expression,
    ACTIONS(445), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(353), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9156] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(355), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9217] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(225), 1,
      sym__simple_string,
    ACTIONS(227), 1,
      sym__string_start,
    ACTIONS(229), 1,
      sym__multiline_string_start,
    ACTIONS(319), 1,
      anon_sym_LBRACE,
    ACTIONS(443), 1,
      sym_identifier,
    ACTIONS(447), 1,
      anon_sym_LPAREN,
    ACTIONS(449), 1,
      anon_sym_if,
    ACTIONS(451), 1,
      anon_sym_try,
    ACTIONS(453), 1,
      anon_sym_new,
    ACTIONS(455), 1,
      sym_number,
    STATE(95), 1,
      sym__expression,
    ACTIONS(445), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(353), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9278] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1021), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9339] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1211), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9400] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1017), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9461] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(289), 1,
      sym__simple_string,
    ACTIONS(291), 1,
      sym__string_start,
    ACTIONS(293), 1,
      sym__multiline_string_start,
    ACTIONS(493), 1,
      sym_identifier,
    ACTIONS(495), 1,
      anon_sym_LBRACE,
    ACTIONS(499), 1,
      anon_sym_LPAREN,
    ACTIONS(501), 1,
      anon_sym_if,
    ACTIONS(503), 1,
      anon_sym_try,
    ACTIONS(505), 1,
      anon_sym_new,
    ACTIONS(507), 1,
      sym_number,
    STATE(312), 1,
      sym__expression,
    ACTIONS(497), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(425), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9522] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(360), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9583] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(350), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9644] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(225), 1,
      sym__simple_string,
    ACTIONS(227), 1,
      sym__string_start,
    ACTIONS(229), 1,
      sym__multiline_string_start,
    ACTIONS(319), 1,
      anon_sym_LBRACE,
    ACTIONS(443), 1,
      sym_identifier,
    ACTIONS(447), 1,
      anon_sym_LPAREN,
    ACTIONS(449), 1,
      anon_sym_if,
    ACTIONS(451), 1,
      anon_sym_try,
    ACTIONS(453), 1,
      anon_sym_new,
    ACTIONS(455), 1,
      sym_number,
    STATE(34), 1,
      sym__expression,
    ACTIONS(445), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(353), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9705] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(391), 1,
      sym_identifier,
    ACTIONS(393), 1,
      anon_sym_LBRACE,
    ACTIONS(397), 1,
      anon_sym_LPAREN,
    ACTIONS(399), 1,
      anon_sym_if,
    ACTIONS(401), 1,
      anon_sym_try,
    ACTIONS(403), 1,
      anon_sym_new,
    ACTIONS(405), 1,
      sym_number,
    ACTIONS(407), 1,
      sym__simple_string,
    ACTIONS(409), 1,
      sym__string_start,
    ACTIONS(411), 1,
      sym__multiline_string_start,
    STATE(719), 1,
      sym__expression,
    ACTIONS(395), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(935), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9766] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(356), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9827] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(391), 1,
      sym_identifier,
    ACTIONS(393), 1,
      anon_sym_LBRACE,
    ACTIONS(397), 1,
      anon_sym_LPAREN,
    ACTIONS(399), 1,
      anon_sym_if,
    ACTIONS(401), 1,
      anon_sym_try,
    ACTIONS(403), 1,
      anon_sym_new,
    ACTIONS(405), 1,
      sym_number,
    ACTIONS(407), 1,
      sym__simple_string,
    ACTIONS(409), 1,
      sym__string_start,
    ACTIONS(411), 1,
      sym__multiline_string_start,
    STATE(722), 1,
      sym__expression,
    ACTIONS(395), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(935), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9888] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(349), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [9949] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1025), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10010] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1028), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10071] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(333), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10132] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1139), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10193] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1054), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10254] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1093), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10315] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(332), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10376] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1166), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10437] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(457), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACE,
    ACTIONS(463), 1,
      anon_sym_LPAREN,
    ACTIONS(465), 1,
      anon_sym_if,
    ACTIONS(467), 1,
      anon_sym_try,
    ACTIONS(469), 1,
      anon_sym_new,
    ACTIONS(471), 1,
      sym_number,
    ACTIONS(473), 1,
      sym__simple_string,
    ACTIONS(475), 1,
      sym__string_start,
    ACTIONS(477), 1,
      sym__multiline_string_start,
    STATE(1057), 1,
      sym__expression,
    ACTIONS(461), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1377), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10498] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1190), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10559] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(327), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10620] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(324), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10681] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(515), 1,
      sym_identifier,
    ACTIONS(517), 1,
      anon_sym_LBRACE,
    ACTIONS(521), 1,
      anon_sym_LPAREN,
    ACTIONS(523), 1,
      anon_sym_if,
    ACTIONS(525), 1,
      anon_sym_try,
    ACTIONS(527), 1,
      anon_sym_new,
    ACTIONS(529), 1,
      sym_number,
    ACTIONS(531), 1,
      sym__simple_string,
    ACTIONS(533), 1,
      sym__string_start,
    ACTIONS(535), 1,
      sym__multiline_string_start,
    STATE(845), 1,
      sym__expression,
    ACTIONS(519), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1191), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10742] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1378), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10803] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1029), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10864] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(768), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10925] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1030), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [10986] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1088), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [11047] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1091), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [11108] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1085), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [11169] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1111), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [11230] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(295), 1,
      anon_sym_LBRACK,
    ACTIONS(299), 1,
      anon_sym_POUND,
    ACTIONS(547), 1,
      anon_sym_AT,
    STATE(315), 1,
      sym_type_arguments,
    ACTIONS(355), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    STATE(406), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(357), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [11279] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1094), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [11340] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(999), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [11401] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1066), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [11462] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1084), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [11523] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(372), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [11584] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1082), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [11645] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1078), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [11706] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1252), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [11767] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(515), 1,
      sym_identifier,
    ACTIONS(517), 1,
      anon_sym_LBRACE,
    ACTIONS(521), 1,
      anon_sym_LPAREN,
    ACTIONS(523), 1,
      anon_sym_if,
    ACTIONS(525), 1,
      anon_sym_try,
    ACTIONS(527), 1,
      anon_sym_new,
    ACTIONS(529), 1,
      sym_number,
    ACTIONS(531), 1,
      sym__simple_string,
    ACTIONS(533), 1,
      sym__string_start,
    ACTIONS(535), 1,
      sym__multiline_string_start,
    STATE(878), 1,
      sym__expression,
    ACTIONS(519), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1191), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [11828] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(515), 1,
      sym_identifier,
    ACTIONS(517), 1,
      anon_sym_LBRACE,
    ACTIONS(521), 1,
      anon_sym_LPAREN,
    ACTIONS(523), 1,
      anon_sym_if,
    ACTIONS(525), 1,
      anon_sym_try,
    ACTIONS(527), 1,
      anon_sym_new,
    ACTIONS(529), 1,
      sym_number,
    ACTIONS(531), 1,
      sym__simple_string,
    ACTIONS(533), 1,
      sym__string_start,
    ACTIONS(535), 1,
      sym__multiline_string_start,
    STATE(875), 1,
      sym__expression,
    ACTIONS(519), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1191), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [11889] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1075), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [11950] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1370), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [12011] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(515), 1,
      sym_identifier,
    ACTIONS(517), 1,
      anon_sym_LBRACE,
    ACTIONS(521), 1,
      anon_sym_LPAREN,
    ACTIONS(523), 1,
      anon_sym_if,
    ACTIONS(525), 1,
      anon_sym_try,
    ACTIONS(527), 1,
      anon_sym_new,
    ACTIONS(529), 1,
      sym_number,
    ACTIONS(531), 1,
      sym__simple_string,
    ACTIONS(533), 1,
      sym__string_start,
    ACTIONS(535), 1,
      sym__multiline_string_start,
    STATE(871), 1,
      sym__expression,
    ACTIONS(519), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1191), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [12072] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1278), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [12133] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(193), 1,
      sym__simple_string,
    ACTIONS(195), 1,
      sym__string_start,
    ACTIONS(197), 1,
      sym__multiline_string_start,
    ACTIONS(235), 1,
      anon_sym_LBRACE,
    ACTIONS(479), 1,
      sym_identifier,
    ACTIONS(483), 1,
      anon_sym_LPAREN,
    ACTIONS(485), 1,
      anon_sym_if,
    ACTIONS(487), 1,
      anon_sym_try,
    ACTIONS(489), 1,
      anon_sym_new,
    ACTIONS(491), 1,
      sym_number,
    STATE(42), 1,
      sym__expression,
    ACTIONS(481), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(272), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [12194] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(515), 1,
      sym_identifier,
    ACTIONS(517), 1,
      anon_sym_LBRACE,
    ACTIONS(521), 1,
      anon_sym_LPAREN,
    ACTIONS(523), 1,
      anon_sym_if,
    ACTIONS(525), 1,
      anon_sym_try,
    ACTIONS(527), 1,
      anon_sym_new,
    ACTIONS(529), 1,
      sym_number,
    ACTIONS(531), 1,
      sym__simple_string,
    ACTIONS(533), 1,
      sym__string_start,
    ACTIONS(535), 1,
      sym__multiline_string_start,
    STATE(870), 1,
      sym__expression,
    ACTIONS(519), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1191), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [12255] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1199), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [12316] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 5,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(551), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [12355] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 5,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(555), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [12394] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 5,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(559), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [12433] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(374), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [12494] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(373), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [12555] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1068), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [12616] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1116), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [12677] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(752), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [12738] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(376), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [12799] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(193), 1,
      sym__simple_string,
    ACTIONS(195), 1,
      sym__string_start,
    ACTIONS(197), 1,
      sym__multiline_string_start,
    ACTIONS(235), 1,
      anon_sym_LBRACE,
    ACTIONS(479), 1,
      sym_identifier,
    ACTIONS(483), 1,
      anon_sym_LPAREN,
    ACTIONS(485), 1,
      anon_sym_if,
    ACTIONS(487), 1,
      anon_sym_try,
    ACTIONS(489), 1,
      anon_sym_new,
    ACTIONS(491), 1,
      sym_number,
    STATE(38), 1,
      sym__expression,
    ACTIONS(481), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(272), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [12860] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(751), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [12921] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(289), 1,
      sym__simple_string,
    ACTIONS(291), 1,
      sym__string_start,
    ACTIONS(293), 1,
      sym__multiline_string_start,
    ACTIONS(493), 1,
      sym_identifier,
    ACTIONS(495), 1,
      anon_sym_LBRACE,
    ACTIONS(499), 1,
      anon_sym_LPAREN,
    ACTIONS(501), 1,
      anon_sym_if,
    ACTIONS(503), 1,
      anon_sym_try,
    ACTIONS(505), 1,
      anon_sym_new,
    ACTIONS(507), 1,
      sym_number,
    STATE(274), 1,
      sym__expression,
    ACTIONS(497), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(425), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [12982] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(289), 1,
      sym__simple_string,
    ACTIONS(291), 1,
      sym__string_start,
    ACTIONS(293), 1,
      sym__multiline_string_start,
    ACTIONS(493), 1,
      sym_identifier,
    ACTIONS(495), 1,
      anon_sym_LBRACE,
    ACTIONS(499), 1,
      anon_sym_LPAREN,
    ACTIONS(501), 1,
      anon_sym_if,
    ACTIONS(503), 1,
      anon_sym_try,
    ACTIONS(505), 1,
      anon_sym_new,
    ACTIONS(507), 1,
      sym_number,
    STATE(273), 1,
      sym__expression,
    ACTIONS(497), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(425), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [13043] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(382), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [13104] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1072), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [13165] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(495), 1,
      anon_sym_LBRACE,
    STATE(408), 2,
      sym_block,
      sym_case_block,
    ACTIONS(231), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(233), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [13208] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1071), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [13269] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(385), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [13330] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(778), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [13391] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(289), 1,
      sym__simple_string,
    ACTIONS(291), 1,
      sym__string_start,
    ACTIONS(293), 1,
      sym__multiline_string_start,
    ACTIONS(493), 1,
      sym_identifier,
    ACTIONS(495), 1,
      anon_sym_LBRACE,
    ACTIONS(499), 1,
      anon_sym_LPAREN,
    ACTIONS(501), 1,
      anon_sym_if,
    ACTIONS(503), 1,
      anon_sym_try,
    ACTIONS(505), 1,
      anon_sym_new,
    ACTIONS(507), 1,
      sym_number,
    STATE(270), 1,
      sym__expression,
    ACTIONS(497), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(425), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [13452] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(193), 1,
      sym__simple_string,
    ACTIONS(195), 1,
      sym__string_start,
    ACTIONS(197), 1,
      sym__multiline_string_start,
    ACTIONS(235), 1,
      anon_sym_LBRACE,
    ACTIONS(479), 1,
      sym_identifier,
    ACTIONS(483), 1,
      anon_sym_LPAREN,
    ACTIONS(485), 1,
      anon_sym_if,
    ACTIONS(487), 1,
      anon_sym_try,
    ACTIONS(489), 1,
      anon_sym_new,
    ACTIONS(491), 1,
      sym_number,
    STATE(39), 1,
      sym__expression,
    ACTIONS(481), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(272), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [13513] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      sym_identifier,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(61), 1,
      anon_sym_LPAREN,
    ACTIONS(63), 1,
      anon_sym_if,
    ACTIONS(65), 1,
      anon_sym_try,
    ACTIONS(67), 1,
      anon_sym_new,
    ACTIONS(69), 1,
      sym_number,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    STATE(1067), 1,
      sym__expression,
    ACTIONS(49), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1063), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [13574] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(318), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [13635] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(391), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [13696] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(419), 1,
      sym_identifier,
    ACTIONS(421), 1,
      anon_sym_LBRACE,
    ACTIONS(425), 1,
      anon_sym_LPAREN,
    ACTIONS(427), 1,
      anon_sym_if,
    ACTIONS(429), 1,
      anon_sym_try,
    ACTIONS(431), 1,
      anon_sym_new,
    ACTIONS(433), 1,
      sym_number,
    ACTIONS(435), 1,
      sym__simple_string,
    ACTIONS(437), 1,
      sym__string_start,
    ACTIONS(439), 1,
      sym__multiline_string_start,
    STATE(1137), 1,
      sym__expression,
    ACTIONS(423), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1392), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [13757] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(419), 1,
      sym_identifier,
    ACTIONS(421), 1,
      anon_sym_LBRACE,
    ACTIONS(425), 1,
      anon_sym_LPAREN,
    ACTIONS(427), 1,
      anon_sym_if,
    ACTIONS(429), 1,
      anon_sym_try,
    ACTIONS(431), 1,
      anon_sym_new,
    ACTIONS(433), 1,
      sym_number,
    ACTIONS(435), 1,
      sym__simple_string,
    ACTIONS(437), 1,
      sym__string_start,
    ACTIONS(439), 1,
      sym__multiline_string_start,
    STATE(1136), 1,
      sym__expression,
    ACTIONS(423), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1392), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [13818] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(319), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [13879] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(289), 1,
      sym__simple_string,
    ACTIONS(291), 1,
      sym__string_start,
    ACTIONS(293), 1,
      sym__multiline_string_start,
    ACTIONS(493), 1,
      sym_identifier,
    ACTIONS(495), 1,
      anon_sym_LBRACE,
    ACTIONS(499), 1,
      anon_sym_LPAREN,
    ACTIONS(501), 1,
      anon_sym_if,
    ACTIONS(503), 1,
      anon_sym_try,
    ACTIONS(505), 1,
      anon_sym_new,
    ACTIONS(507), 1,
      sym_number,
    STATE(269), 1,
      sym__expression,
    ACTIONS(497), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(425), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [13940] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(391), 1,
      sym_identifier,
    ACTIONS(393), 1,
      anon_sym_LBRACE,
    ACTIONS(397), 1,
      anon_sym_LPAREN,
    ACTIONS(399), 1,
      anon_sym_if,
    ACTIONS(401), 1,
      anon_sym_try,
    ACTIONS(403), 1,
      anon_sym_new,
    ACTIONS(405), 1,
      sym_number,
    ACTIONS(407), 1,
      sym__simple_string,
    ACTIONS(409), 1,
      sym__string_start,
    ACTIONS(411), 1,
      sym__multiline_string_start,
    STATE(728), 1,
      sym__expression,
    ACTIONS(395), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(935), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [14001] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(320), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [14062] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(331), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [14123] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(345), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [14184] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1458), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [14245] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(193), 1,
      sym__simple_string,
    ACTIONS(195), 1,
      sym__string_start,
    ACTIONS(197), 1,
      sym__multiline_string_start,
    ACTIONS(235), 1,
      anon_sym_LBRACE,
    ACTIONS(479), 1,
      sym_identifier,
    ACTIONS(483), 1,
      anon_sym_LPAREN,
    ACTIONS(485), 1,
      anon_sym_if,
    ACTIONS(487), 1,
      anon_sym_try,
    ACTIONS(489), 1,
      anon_sym_new,
    ACTIONS(491), 1,
      sym_number,
    STATE(53), 1,
      sym__expression,
    ACTIONS(481), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(272), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [14306] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(363), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [14367] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    ACTIONS(245), 1,
      anon_sym_LPAREN,
    ACTIONS(249), 1,
      anon_sym_if,
    ACTIONS(251), 1,
      anon_sym_try,
    ACTIONS(253), 1,
      anon_sym_new,
    ACTIONS(255), 1,
      sym_number,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1055), 1,
      sym__expression,
    ACTIONS(243), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(1284), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [14428] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(369), 1,
      sym_identifier,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(375), 1,
      anon_sym_LPAREN,
    ACTIONS(377), 1,
      anon_sym_if,
    ACTIONS(379), 1,
      anon_sym_try,
    ACTIONS(381), 1,
      anon_sym_new,
    ACTIONS(383), 1,
      sym_number,
    ACTIONS(385), 1,
      sym__simple_string,
    ACTIONS(387), 1,
      sym__string_start,
    ACTIONS(389), 1,
      sym__multiline_string_start,
    STATE(377), 1,
      sym__expression,
    ACTIONS(373), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_TILDE,
    STATE(477), 16,
      sym_block,
      sym_if_expression,
      sym_match_expression,
      sym_try_expression,
      sym_case_block,
      sym_assignment_expression,
      sym_generic_function,
      sym_call_expression,
      sym_field_expression,
      sym_instance_expression,
      sym_infix_expression,
      sym_prefix_expression,
      sym_tuple_expression,
      sym_parenthesized_expression,
      sym_string_transform_expression,
      sym_string,
  [14489] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(561), 30,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_RBRACK,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_STAR,
  [14525] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(543), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [14565] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(285), 1,
      ts_builtin_sym_end,
    ACTIONS(565), 1,
      anon_sym_DOT,
    ACTIONS(567), 1,
      anon_sym_LBRACK,
    ACTIONS(569), 1,
      anon_sym_EQ,
    ACTIONS(571), 1,
      anon_sym_LPAREN,
    ACTIONS(573), 1,
      anon_sym_match,
    STATE(245), 1,
      sym_arguments,
    STATE(429), 1,
      sym_type_arguments,
    ACTIONS(563), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(287), 20,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
  [14619] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(281), 1,
      ts_builtin_sym_end,
    ACTIONS(565), 1,
      anon_sym_DOT,
    ACTIONS(567), 1,
      anon_sym_LBRACK,
    ACTIONS(569), 1,
      anon_sym_EQ,
    ACTIONS(571), 1,
      anon_sym_LPAREN,
    ACTIONS(573), 1,
      anon_sym_match,
    STATE(245), 1,
      sym_arguments,
    STATE(429), 1,
      sym_type_arguments,
    ACTIONS(563), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(283), 20,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
  [14673] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(577), 27,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [14711] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(189), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(191), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [14749] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(273), 1,
      ts_builtin_sym_end,
    ACTIONS(565), 1,
      anon_sym_DOT,
    ACTIONS(567), 1,
      anon_sym_LBRACK,
    ACTIONS(571), 1,
      anon_sym_LPAREN,
    STATE(245), 1,
      sym_arguments,
    STATE(429), 1,
      sym_type_arguments,
    ACTIONS(275), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [14797] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(269), 1,
      ts_builtin_sym_end,
    ACTIONS(565), 1,
      anon_sym_DOT,
    ACTIONS(567), 1,
      anon_sym_LBRACK,
    ACTIONS(569), 1,
      anon_sym_EQ,
    ACTIONS(571), 1,
      anon_sym_LPAREN,
    STATE(245), 1,
      sym_arguments,
    STATE(429), 1,
      sym_type_arguments,
    ACTIONS(563), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(271), 21,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
  [14849] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(187), 1,
      anon_sym_finally,
    STATE(403), 1,
      sym_finally_clause,
    ACTIONS(537), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(539), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [14891] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(581), 27,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [14929] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(583), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(585), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [14967] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(277), 1,
      ts_builtin_sym_end,
    ACTIONS(565), 1,
      anon_sym_DOT,
    ACTIONS(567), 1,
      anon_sym_LBRACK,
    ACTIONS(571), 1,
      anon_sym_LPAREN,
    STATE(245), 1,
      sym_arguments,
    STATE(429), 1,
      sym_type_arguments,
    ACTIONS(279), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [15015] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(265), 1,
      ts_builtin_sym_end,
    ACTIONS(565), 1,
      anon_sym_DOT,
    ACTIONS(567), 1,
      anon_sym_LBRACK,
    ACTIONS(571), 1,
      anon_sym_LPAREN,
    STATE(245), 1,
      sym_arguments,
    STATE(429), 1,
      sym_type_arguments,
    ACTIONS(267), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [15063] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(587), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(589), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15101] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(591), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(593), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15139] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(595), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(597), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15177] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(599), 1,
      anon_sym_DOT,
    ACTIONS(509), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(511), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [15217] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(601), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(603), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15255] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(605), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(607), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15293] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(609), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(611), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15331] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(613), 30,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_RBRACK,
      anon_sym_LT_COLON,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_extends,
      anon_sym_LPAREN,
  [15367] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 5,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(555), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15405] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(221), 1,
      anon_sym_LPAREN,
    ACTIONS(615), 1,
      ts_builtin_sym_end,
    STATE(301), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(617), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [15447] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(619), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(621), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15485] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 5,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(551), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15523] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(581), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15561] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(623), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(625), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15599] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(577), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15637] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(627), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(629), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15675] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 5,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(559), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15713] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(631), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(633), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15751] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(303), 1,
      ts_builtin_sym_end,
    ACTIONS(565), 1,
      anon_sym_DOT,
    ACTIONS(567), 1,
      anon_sym_LBRACK,
    ACTIONS(569), 1,
      anon_sym_EQ,
    ACTIONS(571), 1,
      anon_sym_LPAREN,
    ACTIONS(573), 1,
      anon_sym_match,
    ACTIONS(635), 1,
      anon_sym_else,
    STATE(245), 1,
      sym_arguments,
    STATE(429), 1,
      sym_type_arguments,
    ACTIONS(563), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(305), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [15807] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(637), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(639), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15845] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(641), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(643), 27,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [15883] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(645), 1,
      ts_builtin_sym_end,
    ACTIONS(649), 1,
      anon_sym_LPAREN,
    STATE(301), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(647), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [15925] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(652), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(654), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [15963] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(656), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(658), 27,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [16001] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(660), 30,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_RBRACK,
      anon_sym_LT_COLON,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_extends,
      anon_sym_LPAREN,
  [16037] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(664), 27,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [16075] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(537), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(539), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [16113] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(666), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(668), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [16151] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(670), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(672), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [16189] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(674), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(676), 27,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [16227] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(678), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(680), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [16265] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(213), 1,
      anon_sym_finally,
    STATE(358), 1,
      sym_finally_clause,
    ACTIONS(537), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(539), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      sym_identifier,
      sym_operator_identifier,
  [16307] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(303), 1,
      ts_builtin_sym_end,
    ACTIONS(565), 1,
      anon_sym_DOT,
    ACTIONS(567), 1,
      anon_sym_LBRACK,
    ACTIONS(569), 1,
      anon_sym_EQ,
    ACTIONS(571), 1,
      anon_sym_LPAREN,
    ACTIONS(573), 1,
      anon_sym_match,
    ACTIONS(682), 1,
      anon_sym_else,
    STATE(245), 1,
      sym_arguments,
    STATE(429), 1,
      sym_type_arguments,
    ACTIONS(563), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(305), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [16363] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    STATE(475), 2,
      sym_block,
      sym_case_block,
    ACTIONS(231), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(233), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [16405] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(684), 1,
      ts_builtin_sym_end,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(688), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [16458] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(674), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(676), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [16495] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(666), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(668), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [16532] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(700), 1,
      ts_builtin_sym_end,
    ACTIONS(704), 1,
      anon_sym_AT,
    STATE(317), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(702), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [16573] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(707), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(709), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [16626] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(711), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(713), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [16679] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(715), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(717), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [16732] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(719), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(721), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [16785] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 5,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(551), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [16822] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(577), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [16859] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(723), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(725), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [16912] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 5,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(555), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [16949] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(581), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [16986] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(727), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(729), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [17039] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(735), 1,
      anon_sym_LBRACK,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(450), 1,
      sym_type_parameters,
    STATE(545), 1,
      sym_extends_clause,
    STATE(660), 1,
      sym_template_body,
    STATE(449), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(731), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [17090] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 5,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(559), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [17127] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(678), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(680), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [17164] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(741), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(743), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [17217] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(745), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(747), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [17270] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(749), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(751), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [17323] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(735), 1,
      anon_sym_LBRACK,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(757), 1,
      anon_sym_COLON,
    ACTIONS(759), 1,
      anon_sym_EQ,
    ACTIONS(761), 1,
      anon_sym_LPAREN,
    STATE(490), 1,
      sym_type_parameters,
    STATE(624), 1,
      sym_block,
    STATE(486), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(753), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [17374] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(763), 1,
      anon_sym_DOT,
    ACTIONS(509), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(511), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [17413] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(656), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(658), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [17450] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(765), 1,
      anon_sym_LPAREN,
    ACTIONS(645), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    STATE(337), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(647), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [17491] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(735), 1,
      anon_sym_LBRACK,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(483), 1,
      sym_type_parameters,
    STATE(562), 1,
      sym_extends_clause,
    STATE(625), 1,
      sym_template_body,
    STATE(480), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(768), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [17542] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACK,
    ACTIONS(664), 27,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [17579] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(577), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [17616] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(619), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(621), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [17653] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(581), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [17690] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(664), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [17727] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(641), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(643), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [17764] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(770), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(772), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [17817] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(670), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(672), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [17854] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(587), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(589), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [17891] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(583), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(585), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [17928] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(774), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(776), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [17981] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(778), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(780), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [18034] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(359), 1,
      anon_sym_AT,
    ACTIONS(782), 1,
      ts_builtin_sym_end,
    STATE(317), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(784), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [18075] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(543), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [18114] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(189), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(191), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [18151] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(735), 1,
      anon_sym_LBRACK,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(464), 1,
      sym_type_parameters,
    STATE(570), 1,
      sym_extends_clause,
    STATE(587), 1,
      sym_template_body,
    STATE(492), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(786), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [18202] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(788), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(790), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [18255] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(265), 1,
      ts_builtin_sym_end,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(267), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [18302] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(591), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(593), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [18339] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(595), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(597), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [18376] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(735), 1,
      anon_sym_LBRACK,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(761), 1,
      anon_sym_LPAREN,
    ACTIONS(794), 1,
      anon_sym_COLON,
    ACTIONS(796), 1,
      anon_sym_EQ,
    STATE(454), 1,
      sym_type_parameters,
    STATE(632), 1,
      sym_block,
    STATE(456), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(792), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [18427] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(277), 1,
      ts_builtin_sym_end,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(279), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [18474] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(798), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(800), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [18527] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(609), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(611), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [18564] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(802), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(804), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [18617] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(297), 1,
      anon_sym_LPAREN,
    ACTIONS(615), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    STATE(337), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(617), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [18658] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(806), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(808), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [18711] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(810), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(812), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [18764] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(814), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(816), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [18817] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(605), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(607), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [18854] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(631), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(633), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [18891] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(237), 1,
      anon_sym_finally,
    STATE(459), 1,
      sym_finally_clause,
    ACTIONS(537), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(539), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [18932] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(818), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(820), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [18985] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(822), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(824), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [19038] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(826), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(828), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [19091] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(830), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(832), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [19144] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(285), 1,
      ts_builtin_sym_end,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(287), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [19197] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(834), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(836), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [19250] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(838), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(840), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [19303] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(269), 1,
      ts_builtin_sym_end,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(271), 20,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
  [19354] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(627), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(629), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [19391] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(842), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(844), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [19444] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(846), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(848), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [19497] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(850), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(852), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [19550] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(854), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(856), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [19603] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(858), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(860), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [19656] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(862), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(864), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [19709] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(866), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(868), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [19762] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(273), 1,
      ts_builtin_sym_end,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(275), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [19809] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(870), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(872), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [19862] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(537), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(539), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [19899] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(874), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(876), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [19952] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(878), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(880), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [20005] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(735), 1,
      anon_sym_LBRACK,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(761), 1,
      anon_sym_LPAREN,
    ACTIONS(884), 1,
      anon_sym_COLON,
    ACTIONS(886), 1,
      anon_sym_EQ,
    STATE(453), 1,
      sym_type_parameters,
    STATE(643), 1,
      sym_block,
    STATE(451), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(882), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [20056] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(601), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(603), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [20093] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(281), 1,
      ts_builtin_sym_end,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(283), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [20146] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(637), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(639), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [20183] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(652), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(654), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [20220] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(623), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(625), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [20257] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_DOT,
    ACTIONS(692), 1,
      anon_sym_LBRACK,
    ACTIONS(694), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_match,
    ACTIONS(888), 1,
      ts_builtin_sym_end,
    STATE(313), 1,
      sym_arguments,
    STATE(468), 1,
      sym_type_arguments,
    ACTIONS(686), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(890), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [20310] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(735), 1,
      anon_sym_LBRACK,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(470), 1,
      sym_type_parameters,
    STATE(577), 1,
      sym_extends_clause,
    STATE(595), 1,
      sym_template_body,
    STATE(444), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(892), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [20361] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(664), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [20397] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(577), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [20433] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(581), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [20469] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(595), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(597), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [20505] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(609), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(611), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [20541] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
    ACTIONS(664), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [20577] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(547), 1,
      anon_sym_AT,
    ACTIONS(782), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    STATE(407), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(784), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [20617] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(894), 1,
      anon_sym_AT,
    ACTIONS(700), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    STATE(407), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(702), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [20657] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(605), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(607), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [20693] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(897), 1,
      ts_builtin_sym_end,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(905), 1,
      anon_sym_with,
    STATE(419), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(899), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(901), 22,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_PIPE,
  [20737] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(637), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(639), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [20773] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(577), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [20809] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(641), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(643), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [20845] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(441), 1,
      anon_sym_AT,
    STATE(437), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(782), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(784), 22,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [20885] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(666), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(668), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [20921] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(907), 1,
      anon_sym_LPAREN,
    ACTIONS(645), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    STATE(415), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(647), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [20961] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(652), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(654), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [20997] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(537), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(539), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [21033] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(656), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(658), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [21069] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(905), 1,
      anon_sym_with,
    ACTIONS(910), 1,
      ts_builtin_sym_end,
    STATE(438), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(912), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [21109] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(591), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(593), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [21145] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(678), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(680), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [21181] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 5,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(551), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [21217] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 5,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(559), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [21253] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(415), 1,
      anon_sym_LPAREN,
    ACTIONS(615), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    STATE(415), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(617), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [21293] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(189), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(191), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [21329] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(631), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(633), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [21365] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(674), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(676), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [21401] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(627), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(629), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [21437] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(619), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(621), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [21473] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(623), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(625), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [21509] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 2,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
    ACTIONS(559), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [21545] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(670), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(672), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [21581] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 2,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
    ACTIONS(555), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [21617] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(905), 1,
      anon_sym_with,
    ACTIONS(914), 1,
      ts_builtin_sym_end,
    STATE(419), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(916), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [21657] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(587), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(589), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [21693] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(583), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(585), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_else,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [21729] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(918), 1,
      anon_sym_AT,
    STATE(437), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(700), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(702), 22,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [21769] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(921), 1,
      ts_builtin_sym_end,
    ACTIONS(925), 1,
      anon_sym_with,
    STATE(438), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(923), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [21809] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(581), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [21845] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 5,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(555), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [21881] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 2,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
    ACTIONS(551), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [21917] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 4,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(664), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [21953] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(537), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(539), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [21988] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(572), 1,
      sym_extends_clause,
    STATE(606), 1,
      sym_template_body,
    STATE(509), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(928), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [22033] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(930), 1,
      anon_sym_with,
    STATE(445), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(921), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(923), 22,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_operator_identifier,
  [22072] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(933), 1,
      ts_builtin_sym_end,
    ACTIONS(935), 26,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [22107] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(761), 1,
      anon_sym_LPAREN,
    ACTIONS(939), 1,
      anon_sym_COLON,
    ACTIONS(941), 1,
      anon_sym_EQ,
    STATE(616), 1,
      sym_block,
    STATE(498), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(937), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [22152] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(627), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(629), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [22187] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(559), 1,
      sym_extends_clause,
    STATE(588), 1,
      sym_template_body,
    STATE(509), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(943), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [22232] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(567), 1,
      sym_extends_clause,
    STATE(665), 1,
      sym_template_body,
    STATE(461), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(945), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [22277] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(761), 1,
      anon_sym_LPAREN,
    ACTIONS(949), 1,
      anon_sym_COLON,
    ACTIONS(951), 1,
      anon_sym_EQ,
    STATE(642), 1,
      sym_block,
    STATE(498), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(947), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [22322] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(953), 1,
      anon_sym_with,
    STATE(473), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(914), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(916), 22,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_operator_identifier,
  [22361] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(761), 1,
      anon_sym_LPAREN,
    ACTIONS(957), 1,
      anon_sym_COLON,
    ACTIONS(959), 1,
      anon_sym_EQ,
    STATE(640), 1,
      sym_block,
    STATE(447), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(955), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [22406] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(761), 1,
      anon_sym_LPAREN,
    ACTIONS(963), 1,
      anon_sym_COLON,
    ACTIONS(965), 1,
      anon_sym_EQ,
    STATE(628), 1,
      sym_block,
    STATE(471), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(961), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [22451] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(761), 1,
      anon_sym_LPAREN,
    ACTIONS(969), 1,
      anon_sym_COLON,
    ACTIONS(971), 1,
      anon_sym_EQ,
    STATE(621), 1,
      sym_block,
    STATE(498), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(967), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [22496] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(761), 1,
      anon_sym_LPAREN,
    ACTIONS(975), 1,
      anon_sym_COLON,
    ACTIONS(977), 1,
      anon_sym_EQ,
    STATE(629), 1,
      sym_block,
    STATE(498), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(973), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [22541] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(591), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(593), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [22576] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(623), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(625), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [22611] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(595), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(597), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [22646] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(551), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [22681] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(556), 1,
      sym_extends_clause,
    STATE(653), 1,
      sym_template_body,
    STATE(509), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(979), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [22726] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(583), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(585), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [22761] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(555), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [22796] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(552), 1,
      sym_extends_clause,
    STATE(647), 1,
      sym_template_body,
    STATE(472), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(981), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [22841] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(953), 1,
      anon_sym_with,
    STATE(473), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(983), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(897), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(901), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [22884] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(631), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(633), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [22919] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(985), 1,
      anon_sym_with,
    STATE(467), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(921), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    ACTIONS(923), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_operator_identifier,
  [22958] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(619), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(621), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [22993] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(988), 1,
      anon_sym_with,
    STATE(487), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(914), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    ACTIONS(916), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_operator_identifier,
  [23032] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(554), 1,
      sym_extends_clause,
    STATE(603), 1,
      sym_template_body,
    STATE(478), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(990), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [23077] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(761), 1,
      anon_sym_LPAREN,
    ACTIONS(994), 1,
      anon_sym_COLON,
    ACTIONS(996), 1,
      anon_sym_EQ,
    STATE(596), 1,
      sym_block,
    STATE(498), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(992), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [23122] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(575), 1,
      sym_extends_clause,
    STATE(663), 1,
      sym_template_body,
    STATE(509), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(998), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [23167] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(953), 1,
      anon_sym_with,
    STATE(445), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(910), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(912), 22,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_operator_identifier,
  [23206] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(609), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(611), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [23241] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(605), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(607), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [23276] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(559), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [23311] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(189), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(191), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [23346] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(571), 1,
      sym_extends_clause,
    STATE(636), 1,
      sym_template_body,
    STATE(509), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(1000), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [23391] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(587), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(589), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [23426] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(569), 1,
      sym_extends_clause,
    STATE(618), 1,
      sym_template_body,
    STATE(509), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(1002), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [23471] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(637), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(639), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [23506] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(577), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [23541] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(547), 1,
      sym_extends_clause,
    STATE(622), 1,
      sym_template_body,
    STATE(484), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(1004), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [23586] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(564), 1,
      sym_extends_clause,
    STATE(593), 1,
      sym_template_body,
    STATE(509), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(1006), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [23631] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(652), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(654), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [23666] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(761), 1,
      anon_sym_LPAREN,
    ACTIONS(1010), 1,
      anon_sym_COLON,
    ACTIONS(1012), 1,
      anon_sym_EQ,
    STATE(657), 1,
      sym_block,
    STATE(498), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(1008), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [23711] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(988), 1,
      anon_sym_with,
    STATE(467), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(910), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    ACTIONS(912), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_operator_identifier,
  [23750] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(581), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [23785] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(670), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(672), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [23820] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(761), 1,
      anon_sym_LPAREN,
    ACTIONS(1016), 1,
      anon_sym_COLON,
    ACTIONS(1018), 1,
      anon_sym_EQ,
    STATE(651), 1,
      sym_block,
    STATE(455), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(1014), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [23865] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(988), 1,
      anon_sym_with,
    STATE(487), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(897), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    ACTIONS(1020), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(901), 20,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [23908] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(737), 1,
      anon_sym_extends,
    ACTIONS(739), 1,
      anon_sym_LPAREN,
    STATE(555), 1,
      sym_extends_clause,
    STATE(666), 1,
      sym_template_body,
    STATE(509), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(1022), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [23953] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(666), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(668), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [23988] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(678), 4,
      ts_builtin_sym_end,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(680), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_match,
      sym_identifier,
      sym_operator_identifier,
  [24023] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(933), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    ACTIONS(935), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [24057] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(897), 1,
      ts_builtin_sym_end,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(899), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(901), 22,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_PIPE,
  [24095] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(914), 1,
      ts_builtin_sym_end,
    ACTIONS(916), 25,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [24129] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1026), 1,
      anon_sym_LPAREN,
    STATE(498), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(1024), 23,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [24165] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(735), 1,
      anon_sym_LBRACK,
    ACTIONS(737), 1,
      anon_sym_extends,
    STATE(517), 1,
      sym_type_parameters,
    STATE(568), 1,
      sym_extends_clause,
    STATE(659), 1,
      sym_template_body,
    ACTIONS(1029), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [24209] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(559), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [24243] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(555), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [24277] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(933), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(935), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [24311] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(551), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [24345] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1031), 1,
      ts_builtin_sym_end,
    ACTIONS(1037), 1,
      anon_sym_EQ,
    ACTIONS(1033), 4,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(1035), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [24382] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(914), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(916), 22,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_operator_identifier,
  [24415] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1039), 1,
      ts_builtin_sym_end,
    ACTIONS(1043), 1,
      anon_sym_EQ,
    ACTIONS(1033), 4,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(1041), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [24452] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1045), 1,
      ts_builtin_sym_end,
    ACTIONS(1049), 1,
      anon_sym_EQ,
    ACTIONS(1033), 4,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(1047), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [24489] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(897), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    ACTIONS(1020), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(901), 20,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [24526] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1053), 1,
      anon_sym_LPAREN,
    STATE(509), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(1051), 22,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_extends,
  [24561] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1056), 1,
      ts_builtin_sym_end,
    ACTIONS(1060), 1,
      anon_sym_EQ,
    ACTIONS(1033), 4,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(1058), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [24598] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1062), 1,
      ts_builtin_sym_end,
    ACTIONS(1066), 1,
      anon_sym_EQ,
    ACTIONS(1033), 4,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(1064), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [24635] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(983), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(897), 3,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_LPAREN,
    ACTIONS(901), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [24672] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(914), 2,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
    ACTIONS(916), 23,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_operator_identifier,
  [24705] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1068), 1,
      ts_builtin_sym_end,
    ACTIONS(1072), 1,
      anon_sym_EQ,
    ACTIONS(1033), 4,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(1070), 19,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [24742] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(561), 1,
      ts_builtin_sym_end,
    ACTIONS(1074), 24,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [24775] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1076), 24,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_LPAREN,
  [24805] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(737), 1,
      anon_sym_extends,
    STATE(544), 1,
      sym_extends_clause,
    STATE(662), 1,
      sym_template_body,
    ACTIONS(1078), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [24843] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1080), 24,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_LPAREN,
  [24873] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1082), 24,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_LPAREN,
  [24903] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1084), 24,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_EQ,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_LPAREN,
  [24933] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    ACTIONS(737), 1,
      anon_sym_extends,
    STATE(561), 1,
      sym_extends_clause,
    STATE(614), 1,
      sym_template_body,
    ACTIONS(1086), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [24971] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(509), 6,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_AT,
      anon_sym_LPAREN,
      anon_sym_POUND,
    ACTIONS(511), 16,
      anon_sym_case,
      anon_sym_class,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_wildcard,
  [25004] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(1092), 1,
      anon_sym_EQ,
    STATE(601), 1,
      sym_block,
    ACTIONS(1090), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25039] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(1096), 1,
      anon_sym_EQ,
    STATE(602), 1,
      sym_block,
    ACTIONS(1094), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25074] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(1100), 1,
      anon_sym_EQ,
    STATE(591), 1,
      sym_block,
    ACTIONS(1098), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25109] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1104), 1,
      anon_sym_DOT,
    STATE(540), 1,
      aux_sym_package_identifier_repeat1,
    ACTIONS(1102), 21,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25142] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(1108), 1,
      anon_sym_EQ,
    STATE(654), 1,
      sym_block,
    ACTIONS(1106), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25177] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1110), 23,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_extends,
      anon_sym_LPAREN,
  [25206] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(1114), 1,
      anon_sym_EQ,
    STATE(634), 1,
      sym_block,
    ACTIONS(1112), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25241] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(1118), 1,
      anon_sym_EQ,
    STATE(598), 1,
      sym_block,
    ACTIONS(1116), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25276] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1120), 23,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_extends,
      anon_sym_LPAREN,
  [25305] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(1124), 1,
      anon_sym_EQ,
    STATE(620), 1,
      sym_block,
    ACTIONS(1122), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25340] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1126), 23,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_extends,
      anon_sym_LPAREN,
  [25369] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1130), 1,
      anon_sym_LPAREN,
    STATE(584), 1,
      sym_arguments,
    ACTIONS(1128), 21,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25402] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(1134), 1,
      anon_sym_EQ,
    STATE(612), 1,
      sym_block,
    ACTIONS(1132), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25437] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1138), 1,
      anon_sym_DOT,
    STATE(536), 1,
      aux_sym_package_identifier_repeat1,
    ACTIONS(1136), 21,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25470] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(1143), 1,
      anon_sym_EQ,
    STATE(641), 1,
      sym_block,
    ACTIONS(1141), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25505] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(1147), 1,
      anon_sym_EQ,
    STATE(645), 1,
      sym_block,
    ACTIONS(1145), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25540] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(1151), 1,
      anon_sym_EQ,
    STATE(623), 1,
      sym_block,
    ACTIONS(1149), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25575] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1104), 1,
      anon_sym_DOT,
    STATE(536), 1,
      aux_sym_package_identifier_repeat1,
    ACTIONS(1153), 21,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25608] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_LBRACE,
    ACTIONS(1157), 1,
      anon_sym_EQ,
    STATE(626), 1,
      sym_block,
    ACTIONS(1155), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25643] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 6,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_AT,
      anon_sym_LPAREN,
      anon_sym_POUND,
    ACTIONS(543), 16,
      anon_sym_case,
      anon_sym_class,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_wildcard,
  [25676] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1159), 23,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_extends,
      anon_sym_LPAREN,
  [25705] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(646), 1,
      sym_template_body,
    ACTIONS(1161), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25737] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(664), 1,
      sym_template_body,
    ACTIONS(1163), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25769] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(604), 1,
      sym_template_body,
    ACTIONS(1165), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25801] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(592), 1,
      sym_template_body,
    ACTIONS(1167), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25833] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1171), 1,
      anon_sym_DOT,
    ACTIONS(1169), 21,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_COMMA,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25863] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(656), 6,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_AT,
      anon_sym_LPAREN,
      anon_sym_POUND,
    ACTIONS(658), 16,
      anon_sym_case,
      anon_sym_class,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_wildcard,
  [25893] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(557), 22,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_LPAREN,
  [25921] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1175), 1,
      anon_sym_COMMA,
    STATE(563), 1,
      aux_sym_import_declaration_repeat1,
    ACTIONS(1173), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25953] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(585), 1,
      sym_template_body,
    ACTIONS(1177), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [25985] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(662), 6,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_AT,
      anon_sym_LPAREN,
      anon_sym_POUND,
    ACTIONS(664), 16,
      anon_sym_case,
      anon_sym_class,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_wildcard,
  [26015] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(635), 1,
      sym_template_body,
    ACTIONS(1179), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26047] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(661), 1,
      sym_template_body,
    ACTIONS(1181), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26079] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(586), 1,
      sym_template_body,
    ACTIONS(1183), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26111] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(641), 6,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_AT,
      anon_sym_LPAREN,
      anon_sym_POUND,
    ACTIONS(643), 16,
      anon_sym_case,
      anon_sym_class,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_wildcard,
  [26141] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(579), 6,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_AT,
      anon_sym_LPAREN,
      anon_sym_POUND,
    ACTIONS(581), 16,
      anon_sym_case,
      anon_sym_class,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_wildcard,
  [26171] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(649), 1,
      sym_template_body,
    ACTIONS(1185), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26203] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(553), 22,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_LPAREN,
  [26231] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(597), 1,
      sym_template_body,
    ACTIONS(1187), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26263] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(619), 1,
      sym_template_body,
    ACTIONS(1189), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26295] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1175), 1,
      anon_sym_COMMA,
    STATE(574), 1,
      aux_sym_import_declaration_repeat1,
    ACTIONS(1191), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26327] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(631), 1,
      sym_template_body,
    ACTIONS(1193), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26359] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(545), 22,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_DOT,
      anon_sym_object,
      anon_sym_import,
      anon_sym_COMMA,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26387] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(575), 6,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_AT,
      anon_sym_LPAREN,
      anon_sym_POUND,
    ACTIONS(577), 16,
      anon_sym_case,
      anon_sym_class,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_wildcard,
  [26417] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(655), 1,
      sym_template_body,
    ACTIONS(1195), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26449] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(650), 1,
      sym_template_body,
    ACTIONS(1197), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26481] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(594), 1,
      sym_template_body,
    ACTIONS(1199), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26513] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(658), 1,
      sym_template_body,
    ACTIONS(1201), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26545] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(652), 1,
      sym_template_body,
    ACTIONS(1203), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26577] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(637), 1,
      sym_template_body,
    ACTIONS(1205), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26609] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(549), 22,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      anon_sym_LPAREN,
  [26637] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1209), 1,
      anon_sym_COMMA,
    STATE(574), 1,
      aux_sym_import_declaration_repeat1,
    ACTIONS(1207), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26669] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(630), 1,
      sym_template_body,
    ACTIONS(1212), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26701] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1136), 22,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_DOT,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26729] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(733), 1,
      anon_sym_LBRACE,
    STATE(605), 1,
      sym_template_body,
    ACTIONS(1214), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26761] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(674), 6,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_AT,
      anon_sym_LPAREN,
      anon_sym_POUND,
    ACTIONS(676), 16,
      anon_sym_case,
      anon_sym_class,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
      sym_identifier,
      sym_wildcard,
  [26791] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1216), 21,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_COMMA,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26818] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1218), 21,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_COMMA,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26845] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1220), 21,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_COMMA,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26872] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1222), 21,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_COMMA,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26899] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1130), 1,
      anon_sym_LPAREN,
    ACTIONS(1224), 1,
      anon_sym_LBRACK,
    ACTIONS(1226), 1,
      anon_sym_POUND,
    STATE(578), 1,
      sym_type_arguments,
    STATE(680), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(215), 15,
      anon_sym_case,
      anon_sym_class,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26936] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1228), 21,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_LBRACE,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26963] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1230), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [26989] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1232), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27015] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1234), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27041] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1236), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27067] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1238), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27093] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1240), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27119] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1242), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27145] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1244), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27171] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1246), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27197] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1248), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27223] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1250), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27249] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1252), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27275] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1254), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27301] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1256), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27327] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1258), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27353] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1260), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27379] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1262), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27405] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1264), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27431] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1266), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27457] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1268), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27483] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1270), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27509] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1272), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27535] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1274), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27561] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1276), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27587] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1278), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27613] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1280), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27639] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1282), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27665] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1284), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27691] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1286), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27717] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1288), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27743] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(583), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27769] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1290), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27795] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1292), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27821] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1294), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27847] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1296), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27873] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1298), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27899] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1300), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27925] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1302), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27951] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1304), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [27977] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1306), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28003] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1308), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28029] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1310), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28055] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1312), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28081] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1314), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28107] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1316), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28133] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1318), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28159] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1320), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28185] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1322), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28211] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1324), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28237] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1326), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28263] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1328), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28289] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1330), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28315] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1332), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28341] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1334), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28367] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1336), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28393] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1338), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28419] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1340), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28445] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1342), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28471] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1344), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28497] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1346), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28523] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1348), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28549] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1350), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28575] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1352), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28601] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(587), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28627] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1354), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28653] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1356), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28679] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1358), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28705] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1360), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28731] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1362), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28757] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1364), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28783] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1366), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28809] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1368), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28835] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1370), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28861] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1372), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28887] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1374), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28913] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1376), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28939] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1378), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28965] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1380), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [28991] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1382), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [29017] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1384), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [29043] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1386), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [29069] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1388), 20,
      ts_builtin_sym_end,
      anon_sym_package,
      anon_sym_object,
      anon_sym_import,
      anon_sym_case,
      anon_sym_class,
      anon_sym_trait,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [29095] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_AT,
    ACTIONS(1390), 1,
      anon_sym_case,
    ACTIONS(1392), 1,
      anon_sym_class,
    ACTIONS(1394), 1,
      anon_sym_val,
    ACTIONS(1396), 1,
      anon_sym_var,
    ACTIONS(1398), 1,
      anon_sym_type,
    ACTIONS(1400), 1,
      anon_sym_def,
    STATE(761), 1,
      aux_sym_modifiers_repeat1,
    STATE(1681), 1,
      sym_modifiers,
    STATE(697), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(29), 8,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [29140] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_AT,
    ACTIONS(1402), 1,
      anon_sym_case,
    ACTIONS(1404), 1,
      anon_sym_class,
    ACTIONS(1406), 1,
      anon_sym_val,
    ACTIONS(1408), 1,
      anon_sym_var,
    ACTIONS(1410), 1,
      anon_sym_type,
    ACTIONS(1412), 1,
      anon_sym_def,
    STATE(761), 1,
      aux_sym_modifiers_repeat1,
    STATE(1684), 1,
      sym_modifiers,
    STATE(697), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(29), 8,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [29185] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    ACTIONS(1420), 1,
      anon_sym_RPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1791), 2,
      sym__type,
      sym_function_type,
    STATE(1973), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [29229] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    ACTIONS(1420), 1,
      anon_sym_RPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1775), 2,
      sym__type,
      sym_function_type,
    STATE(1973), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [29273] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    ACTIONS(1420), 1,
      anon_sym_RPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1725), 2,
      sym__type,
      sym_function_type,
    STATE(1973), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [29317] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1422), 1,
      sym_identifier,
    ACTIONS(1424), 1,
      anon_sym_object,
    ACTIONS(1426), 1,
      anon_sym_class,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1430), 1,
      sym_wildcard,
    ACTIONS(1432), 1,
      sym_number,
    ACTIONS(1434), 1,
      sym__simple_string,
    ACTIONS(1436), 1,
      sym__string_start,
    ACTIONS(1438), 1,
      sym__multiline_string_start,
    STATE(2275), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1638), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [29363] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    ACTIONS(1420), 1,
      anon_sym_RPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1738), 2,
      sym__type,
      sym_function_type,
    STATE(1973), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [29407] = 15,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1442), 1,
      anon_sym_DOT,
    ACTIONS(1444), 1,
      anon_sym_LBRACK,
    ACTIONS(1446), 1,
      anon_sym_EQ,
    ACTIONS(1448), 1,
      anon_sym_LPAREN,
    ACTIONS(1450), 1,
      anon_sym_match,
    ACTIONS(1452), 1,
      anon_sym_catch,
    ACTIONS(1454), 1,
      anon_sym_finally,
    STATE(698), 1,
      sym_arguments,
    STATE(758), 1,
      sym_catch_clause,
    STATE(916), 1,
      sym_finally_clause,
    STATE(930), 1,
      sym_type_arguments,
    ACTIONS(169), 2,
      anon_sym_case,
      anon_sym_else,
    ACTIONS(1440), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(165), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [29457] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    ACTIONS(1420), 1,
      anon_sym_RPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1765), 2,
      sym__type,
      sym_function_type,
    STATE(1973), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [29501] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    ACTIONS(1420), 1,
      anon_sym_RPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1742), 2,
      sym__type,
      sym_function_type,
    STATE(1973), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [29545] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    ACTIONS(1420), 1,
      anon_sym_RPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1758), 2,
      sym__type,
      sym_function_type,
    STATE(1973), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [29589] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    ACTIONS(1420), 1,
      anon_sym_RPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1770), 2,
      sym__type,
      sym_function_type,
    STATE(1973), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [29633] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    ACTIONS(1420), 1,
      anon_sym_RPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1728), 2,
      sym__type,
      sym_function_type,
    STATE(1973), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [29677] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1130), 1,
      anon_sym_LPAREN,
    STATE(686), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(615), 15,
      anon_sym_case,
      anon_sym_class,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [29705] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(407), 1,
      sym__simple_string,
    ACTIONS(409), 1,
      sym__string_start,
    ACTIONS(411), 1,
      sym__multiline_string_start,
    STATE(931), 1,
      sym_string,
    ACTIONS(189), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(191), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [29739] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    ACTIONS(1420), 1,
      anon_sym_RPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1750), 2,
      sym__type,
      sym_function_type,
    STATE(1973), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [29783] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    ACTIONS(1420), 1,
      anon_sym_RPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1771), 2,
      sym__type,
      sym_function_type,
    STATE(1973), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [29827] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    ACTIONS(1420), 1,
      anon_sym_RPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1774), 2,
      sym__type,
      sym_function_type,
    STATE(1973), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [29871] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1456), 1,
      anon_sym_LBRACK,
    ACTIONS(1458), 1,
      anon_sym_LPAREN,
    ACTIONS(1460), 1,
      anon_sym_POUND,
    STATE(773), 1,
      sym_type_arguments,
    STATE(772), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(215), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(217), 9,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [29907] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1462), 1,
      anon_sym_LPAREN,
    STATE(686), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(645), 15,
      anon_sym_case,
      anon_sym_class,
      anon_sym_AT,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [29935] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    ACTIONS(1420), 1,
      anon_sym_RPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1790), 2,
      sym__type,
      sym_function_type,
    STATE(1973), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [29979] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    ACTIONS(1420), 1,
      anon_sym_RPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1736), 2,
      sym__type,
      sym_function_type,
    STATE(1973), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [30023] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    ACTIONS(1420), 1,
      anon_sym_RPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1779), 2,
      sym__type,
      sym_function_type,
    STATE(1973), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [30067] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1465), 1,
      sym_identifier,
    ACTIONS(1467), 1,
      anon_sym_LPAREN,
    ACTIONS(1469), 1,
      anon_sym_RPAREN,
    ACTIONS(1471), 1,
      sym_wildcard,
    ACTIONS(1473), 1,
      sym_number,
    ACTIONS(1475), 1,
      sym__simple_string,
    ACTIONS(1477), 1,
      sym__string_start,
    ACTIONS(1479), 1,
      sym__multiline_string_start,
    STATE(2312), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1604), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [30110] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1456), 1,
      anon_sym_LBRACK,
    ACTIONS(1460), 1,
      anon_sym_POUND,
    ACTIONS(1481), 1,
      anon_sym_AT,
    STATE(773), 1,
      sym_type_arguments,
    STATE(956), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(355), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(357), 8,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [30145] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_LPAREN,
    ACTIONS(1475), 1,
      sym__simple_string,
    ACTIONS(1477), 1,
      sym__string_start,
    ACTIONS(1479), 1,
      sym__multiline_string_start,
    ACTIONS(1483), 1,
      sym_identifier,
    ACTIONS(1485), 1,
      anon_sym_RPAREN,
    ACTIONS(1487), 1,
      sym_wildcard,
    ACTIONS(1489), 1,
      sym_number,
    STATE(2312), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1615), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [30188] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2052), 2,
      sym__type,
      sym_function_type,
    STATE(2216), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [30229] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_LPAREN,
    ACTIONS(1475), 1,
      sym__simple_string,
    ACTIONS(1477), 1,
      sym__string_start,
    ACTIONS(1479), 1,
      sym__multiline_string_start,
    ACTIONS(1491), 1,
      sym_identifier,
    ACTIONS(1493), 1,
      anon_sym_RPAREN,
    ACTIONS(1495), 1,
      sym_wildcard,
    ACTIONS(1497), 1,
      sym_number,
    STATE(2312), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1620), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [30272] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1499), 1,
      sym_identifier,
    ACTIONS(1501), 1,
      anon_sym_EQ_GT,
    ACTIONS(1503), 1,
      anon_sym_LPAREN,
    STATE(1498), 1,
      sym__annotated_type,
    STATE(2357), 1,
      sym_parameter_types,
    STATE(2360), 1,
      sym_stable_identifier,
    STATE(1605), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1808), 2,
      sym__type,
      sym_function_type,
    STATE(2071), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(896), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [30313] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1499), 1,
      sym_identifier,
    ACTIONS(1501), 1,
      anon_sym_EQ_GT,
    ACTIONS(1503), 1,
      anon_sym_LPAREN,
    STATE(1498), 1,
      sym__annotated_type,
    STATE(2357), 1,
      sym_parameter_types,
    STATE(2360), 1,
      sym_stable_identifier,
    STATE(1605), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1833), 2,
      sym__type,
      sym_function_type,
    STATE(2120), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(896), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [30354] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1505), 1,
      anon_sym_AT,
    STATE(697), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(700), 14,
      anon_sym_case,
      anon_sym_class,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [30381] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(393), 1,
      anon_sym_LBRACE,
    STATE(909), 2,
      sym_block,
      sym_case_block,
    ACTIONS(231), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(233), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [30410] = 7,
    ACTIONS(71), 1,
      sym__simple_string,
    ACTIONS(73), 1,
      sym__string_start,
    ACTIONS(75), 1,
      sym__multiline_string_start,
    ACTIONS(185), 1,
      sym_comment,
    STATE(1112), 1,
      sym_string,
    ACTIONS(189), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(191), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [30443] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_LPAREN,
    ACTIONS(1475), 1,
      sym__simple_string,
    ACTIONS(1477), 1,
      sym__string_start,
    ACTIONS(1479), 1,
      sym__multiline_string_start,
    ACTIONS(1508), 1,
      sym_identifier,
    ACTIONS(1510), 1,
      anon_sym_RPAREN,
    ACTIONS(1512), 1,
      sym_wildcard,
    ACTIONS(1514), 1,
      sym_number,
    STATE(2312), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1616), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [30486] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1516), 1,
      anon_sym_LBRACK,
    ACTIONS(1518), 1,
      anon_sym_LPAREN,
    ACTIONS(1520), 1,
      anon_sym_POUND,
    STATE(975), 1,
      sym_type_arguments,
    STATE(968), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(215), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(217), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [30521] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_LPAREN,
    ACTIONS(1475), 1,
      sym__simple_string,
    ACTIONS(1477), 1,
      sym__string_start,
    ACTIONS(1479), 1,
      sym__multiline_string_start,
    ACTIONS(1522), 1,
      sym_identifier,
    ACTIONS(1524), 1,
      anon_sym_RPAREN,
    ACTIONS(1526), 1,
      sym_wildcard,
    ACTIONS(1528), 1,
      sym_number,
    STATE(2312), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1606), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [30564] = 15,
    ACTIONS(169), 1,
      anon_sym_case,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    ACTIONS(1542), 1,
      anon_sym_catch,
    ACTIONS(1544), 1,
      anon_sym_finally,
    STATE(743), 1,
      sym_arguments,
    STATE(852), 1,
      sym_catch_clause,
    STATE(1056), 1,
      sym_finally_clause,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(165), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [30613] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_LPAREN,
    ACTIONS(1475), 1,
      sym__simple_string,
    ACTIONS(1477), 1,
      sym__string_start,
    ACTIONS(1479), 1,
      sym__multiline_string_start,
    ACTIONS(1546), 1,
      sym_identifier,
    ACTIONS(1548), 1,
      anon_sym_RPAREN,
    ACTIONS(1550), 1,
      sym_wildcard,
    ACTIONS(1552), 1,
      sym_number,
    STATE(2312), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1613), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [30656] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1416), 1,
      anon_sym_EQ_GT,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2035), 2,
      sym__type,
      sym_function_type,
    STATE(2216), 2,
      sym_lazy_parameter_type,
      sym_repeated_parameter_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [30697] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1422), 1,
      sym_identifier,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1430), 1,
      sym_wildcard,
    ACTIONS(1432), 1,
      sym_number,
    ACTIONS(1434), 1,
      sym__simple_string,
    ACTIONS(1436), 1,
      sym__string_start,
    ACTIONS(1438), 1,
      sym__multiline_string_start,
    STATE(2275), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1638), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [30737] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_LPAREN,
    ACTIONS(1475), 1,
      sym__simple_string,
    ACTIONS(1477), 1,
      sym__string_start,
    ACTIONS(1479), 1,
      sym__multiline_string_start,
    ACTIONS(1554), 1,
      sym_identifier,
    ACTIONS(1556), 1,
      sym_wildcard,
    ACTIONS(1558), 1,
      sym_number,
    STATE(2312), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1712), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [30777] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_LPAREN,
    ACTIONS(1475), 1,
      sym__simple_string,
    ACTIONS(1477), 1,
      sym__string_start,
    ACTIONS(1479), 1,
      sym__multiline_string_start,
    ACTIONS(1560), 1,
      sym_identifier,
    ACTIONS(1562), 1,
      sym_wildcard,
    ACTIONS(1564), 1,
      sym_number,
    STATE(2312), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1672), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [30817] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_LPAREN,
    ACTIONS(1475), 1,
      sym__simple_string,
    ACTIONS(1477), 1,
      sym__string_start,
    ACTIONS(1479), 1,
      sym__multiline_string_start,
    ACTIONS(1566), 1,
      sym_identifier,
    ACTIONS(1568), 1,
      sym_wildcard,
    ACTIONS(1570), 1,
      sym_number,
    STATE(2312), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1666), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [30857] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1572), 1,
      anon_sym_DOT,
    ACTIONS(509), 5,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(511), 10,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [30883] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1574), 1,
      anon_sym_LBRACK,
    ACTIONS(1576), 1,
      anon_sym_LPAREN,
    ACTIONS(1578), 1,
      anon_sym_POUND,
    STATE(1039), 1,
      sym_type_arguments,
    ACTIONS(215), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1038), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(217), 8,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [30917] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1516), 1,
      anon_sym_LBRACK,
    ACTIONS(1520), 1,
      anon_sym_POUND,
    ACTIONS(1580), 1,
      anon_sym_AT,
    STATE(975), 1,
      sym_type_arguments,
    STATE(1027), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(355), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(357), 6,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [30951] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1582), 1,
      sym_identifier,
    ACTIONS(1584), 1,
      anon_sym_LPAREN,
    ACTIONS(1586), 1,
      sym_wildcard,
    ACTIONS(1588), 1,
      sym_number,
    ACTIONS(1590), 1,
      sym__simple_string,
    ACTIONS(1592), 1,
      sym__string_start,
    ACTIONS(1594), 1,
      sym__multiline_string_start,
    STATE(2296), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1731), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [30991] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1584), 1,
      anon_sym_LPAREN,
    ACTIONS(1590), 1,
      sym__simple_string,
    ACTIONS(1592), 1,
      sym__string_start,
    ACTIONS(1594), 1,
      sym__multiline_string_start,
    ACTIONS(1596), 1,
      sym_identifier,
    ACTIONS(1598), 1,
      sym_wildcard,
    ACTIONS(1600), 1,
      sym_number,
    STATE(2296), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1753), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [31031] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1442), 1,
      anon_sym_DOT,
    ACTIONS(1444), 1,
      anon_sym_LBRACK,
    ACTIONS(1448), 1,
      anon_sym_LPAREN,
    STATE(698), 1,
      sym_arguments,
    STATE(930), 1,
      sym_type_arguments,
    ACTIONS(265), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(267), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [31065] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_LPAREN,
    ACTIONS(1475), 1,
      sym__simple_string,
    ACTIONS(1477), 1,
      sym__string_start,
    ACTIONS(1479), 1,
      sym__multiline_string_start,
    ACTIONS(1602), 1,
      sym_identifier,
    ACTIONS(1604), 1,
      sym_wildcard,
    ACTIONS(1606), 1,
      sym_number,
    STATE(2312), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1693), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [31105] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1442), 1,
      anon_sym_DOT,
    ACTIONS(1444), 1,
      anon_sym_LBRACK,
    ACTIONS(1448), 1,
      anon_sym_LPAREN,
    STATE(698), 1,
      sym_arguments,
    STATE(930), 1,
      sym_type_arguments,
    ACTIONS(277), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(279), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [31139] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1608), 1,
      anon_sym_LBRACK,
    ACTIONS(1610), 1,
      anon_sym_AT,
    ACTIONS(1612), 1,
      anon_sym_POUND,
    STATE(1086), 1,
      sym_type_arguments,
    STATE(1115), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(355), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(357), 5,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [31173] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1442), 1,
      anon_sym_DOT,
    ACTIONS(1444), 1,
      anon_sym_LBRACK,
    ACTIONS(1446), 1,
      anon_sym_EQ,
    ACTIONS(1448), 1,
      anon_sym_LPAREN,
    ACTIONS(1450), 1,
      anon_sym_match,
    STATE(698), 1,
      sym_arguments,
    STATE(930), 1,
      sym_type_arguments,
    ACTIONS(1440), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(281), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(283), 4,
      anon_sym_case,
      anon_sym_else,
      anon_sym_catch,
      anon_sym_finally,
  [31213] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_LPAREN,
    ACTIONS(1475), 1,
      sym__simple_string,
    ACTIONS(1477), 1,
      sym__string_start,
    ACTIONS(1479), 1,
      sym__multiline_string_start,
    ACTIONS(1614), 1,
      sym_identifier,
    ACTIONS(1616), 1,
      sym_wildcard,
    ACTIONS(1618), 1,
      sym_number,
    STATE(2312), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1680), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [31253] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1620), 1,
      anon_sym_LBRACK,
    ACTIONS(1622), 1,
      anon_sym_LPAREN,
    ACTIONS(1624), 1,
      anon_sym_POUND,
    STATE(1015), 1,
      sym_type_arguments,
    ACTIONS(215), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1013), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(217), 8,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [31287] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1442), 1,
      anon_sym_DOT,
    ACTIONS(1444), 1,
      anon_sym_LBRACK,
    ACTIONS(1446), 1,
      anon_sym_EQ,
    ACTIONS(1448), 1,
      anon_sym_LPAREN,
    ACTIONS(1450), 1,
      anon_sym_match,
    STATE(698), 1,
      sym_arguments,
    STATE(930), 1,
      sym_type_arguments,
    ACTIONS(1440), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(285), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(287), 4,
      anon_sym_case,
      anon_sym_else,
      anon_sym_catch,
      anon_sym_finally,
  [31327] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1584), 1,
      anon_sym_LPAREN,
    ACTIONS(1590), 1,
      sym__simple_string,
    ACTIONS(1592), 1,
      sym__string_start,
    ACTIONS(1594), 1,
      sym__multiline_string_start,
    ACTIONS(1626), 1,
      sym_identifier,
    ACTIONS(1628), 1,
      sym_wildcard,
    ACTIONS(1630), 1,
      sym_number,
    STATE(2296), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1732), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [31367] = 15,
    ACTIONS(169), 1,
      anon_sym_else,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1634), 1,
      anon_sym_DOT,
    ACTIONS(1636), 1,
      anon_sym_LBRACK,
    ACTIONS(1638), 1,
      anon_sym_EQ,
    ACTIONS(1640), 1,
      anon_sym_LPAREN,
    ACTIONS(1642), 1,
      anon_sym_match,
    ACTIONS(1644), 1,
      anon_sym_catch,
    ACTIONS(1646), 1,
      anon_sym_finally,
    STATE(755), 1,
      sym_arguments,
    STATE(1073), 1,
      sym_catch_clause,
    STATE(1207), 1,
      sym_type_arguments,
    STATE(1224), 1,
      sym_finally_clause,
    ACTIONS(165), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(1632), 2,
      sym_identifier,
      sym_operator_identifier,
  [31415] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1584), 1,
      anon_sym_LPAREN,
    ACTIONS(1590), 1,
      sym__simple_string,
    ACTIONS(1592), 1,
      sym__string_start,
    ACTIONS(1594), 1,
      sym__multiline_string_start,
    ACTIONS(1648), 1,
      sym_identifier,
    ACTIONS(1650), 1,
      sym_wildcard,
    ACTIONS(1652), 1,
      sym_number,
    STATE(2296), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1756), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [31455] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1434), 1,
      sym__simple_string,
    ACTIONS(1436), 1,
      sym__string_start,
    ACTIONS(1438), 1,
      sym__multiline_string_start,
    ACTIONS(1654), 1,
      sym_identifier,
    ACTIONS(1656), 1,
      sym_wildcard,
    ACTIONS(1658), 1,
      sym_number,
    STATE(2275), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1662), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [31495] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1584), 1,
      anon_sym_LPAREN,
    ACTIONS(1590), 1,
      sym__simple_string,
    ACTIONS(1592), 1,
      sym__string_start,
    ACTIONS(1594), 1,
      sym__multiline_string_start,
    ACTIONS(1660), 1,
      sym_identifier,
    ACTIONS(1662), 1,
      sym_wildcard,
    ACTIONS(1664), 1,
      sym_number,
    STATE(2296), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1793), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [31535] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1442), 1,
      anon_sym_DOT,
    ACTIONS(1444), 1,
      anon_sym_LBRACK,
    ACTIONS(1446), 1,
      anon_sym_EQ,
    ACTIONS(1448), 1,
      anon_sym_LPAREN,
    ACTIONS(1450), 1,
      anon_sym_match,
    ACTIONS(1666), 1,
      anon_sym_else,
    STATE(698), 1,
      sym_arguments,
    STATE(930), 1,
      sym_type_arguments,
    ACTIONS(1440), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(303), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(305), 3,
      anon_sym_case,
      anon_sym_catch,
      anon_sym_finally,
  [31577] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1434), 1,
      sym__simple_string,
    ACTIONS(1436), 1,
      sym__string_start,
    ACTIONS(1438), 1,
      sym__multiline_string_start,
    ACTIONS(1668), 1,
      sym_identifier,
    ACTIONS(1670), 1,
      sym_wildcard,
    ACTIONS(1672), 1,
      sym_number,
    STATE(2275), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1719), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [31617] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1434), 1,
      sym__simple_string,
    ACTIONS(1436), 1,
      sym__string_start,
    ACTIONS(1438), 1,
      sym__multiline_string_start,
    ACTIONS(1674), 1,
      sym_identifier,
    ACTIONS(1676), 1,
      sym_wildcard,
    ACTIONS(1678), 1,
      sym_number,
    STATE(2275), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1714), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [31657] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1584), 1,
      anon_sym_LPAREN,
    ACTIONS(1590), 1,
      sym__simple_string,
    ACTIONS(1592), 1,
      sym__string_start,
    ACTIONS(1594), 1,
      sym__multiline_string_start,
    ACTIONS(1680), 1,
      sym_identifier,
    ACTIONS(1682), 1,
      sym_wildcard,
    ACTIONS(1684), 1,
      sym_number,
    STATE(2296), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1781), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [31697] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1584), 1,
      anon_sym_LPAREN,
    ACTIONS(1590), 1,
      sym__simple_string,
    ACTIONS(1592), 1,
      sym__string_start,
    ACTIONS(1594), 1,
      sym__multiline_string_start,
    ACTIONS(1686), 1,
      sym_identifier,
    ACTIONS(1688), 1,
      sym_wildcard,
    ACTIONS(1690), 1,
      sym_number,
    STATE(2296), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1722), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [31737] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1584), 1,
      anon_sym_LPAREN,
    ACTIONS(1590), 1,
      sym__simple_string,
    ACTIONS(1592), 1,
      sym__string_start,
    ACTIONS(1594), 1,
      sym__multiline_string_start,
    ACTIONS(1692), 1,
      sym_identifier,
    ACTIONS(1694), 1,
      sym_wildcard,
    ACTIONS(1696), 1,
      sym_number,
    STATE(2296), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1755), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [31777] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1584), 1,
      anon_sym_LPAREN,
    ACTIONS(1590), 1,
      sym__simple_string,
    ACTIONS(1592), 1,
      sym__string_start,
    ACTIONS(1594), 1,
      sym__multiline_string_start,
    ACTIONS(1698), 1,
      sym_identifier,
    ACTIONS(1700), 1,
      sym_wildcard,
    ACTIONS(1702), 1,
      sym_number,
    STATE(2296), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1746), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [31817] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 5,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(543), 10,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [31843] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1442), 1,
      anon_sym_DOT,
    ACTIONS(1444), 1,
      anon_sym_LBRACK,
    ACTIONS(1446), 1,
      anon_sym_EQ,
    ACTIONS(1448), 1,
      anon_sym_LPAREN,
    ACTIONS(1450), 1,
      anon_sym_match,
    ACTIONS(1704), 1,
      anon_sym_else,
    STATE(698), 1,
      sym_arguments,
    STATE(930), 1,
      sym_type_arguments,
    ACTIONS(1440), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(303), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(305), 3,
      anon_sym_case,
      anon_sym_catch,
      anon_sym_finally,
  [31885] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(531), 1,
      sym__simple_string,
    ACTIONS(533), 1,
      sym__string_start,
    ACTIONS(535), 1,
      sym__multiline_string_start,
    STATE(1204), 1,
      sym_string,
    ACTIONS(189), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(191), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [31917] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1584), 1,
      anon_sym_LPAREN,
    ACTIONS(1590), 1,
      sym__simple_string,
    ACTIONS(1592), 1,
      sym__string_start,
    ACTIONS(1594), 1,
      sym__multiline_string_start,
    ACTIONS(1706), 1,
      sym_identifier,
    ACTIONS(1708), 1,
      sym_wildcard,
    ACTIONS(1710), 1,
      sym_number,
    STATE(2296), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1760), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [31957] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1584), 1,
      anon_sym_LPAREN,
    ACTIONS(1590), 1,
      sym__simple_string,
    ACTIONS(1592), 1,
      sym__string_start,
    ACTIONS(1594), 1,
      sym__multiline_string_start,
    ACTIONS(1712), 1,
      sym_identifier,
    ACTIONS(1714), 1,
      sym_wildcard,
    ACTIONS(1716), 1,
      sym_number,
    STATE(2296), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1762), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [31997] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1584), 1,
      anon_sym_LPAREN,
    ACTIONS(1590), 1,
      sym__simple_string,
    ACTIONS(1592), 1,
      sym__string_start,
    ACTIONS(1594), 1,
      sym__multiline_string_start,
    ACTIONS(1718), 1,
      sym_identifier,
    ACTIONS(1720), 1,
      sym_wildcard,
    ACTIONS(1722), 1,
      sym_number,
    STATE(2296), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1761), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [32037] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1584), 1,
      anon_sym_LPAREN,
    ACTIONS(1590), 1,
      sym__simple_string,
    ACTIONS(1592), 1,
      sym__string_start,
    ACTIONS(1594), 1,
      sym__multiline_string_start,
    ACTIONS(1724), 1,
      sym_identifier,
    ACTIONS(1726), 1,
      sym_wildcard,
    ACTIONS(1728), 1,
      sym_number,
    STATE(2296), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1752), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [32077] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1584), 1,
      anon_sym_LPAREN,
    ACTIONS(1590), 1,
      sym__simple_string,
    ACTIONS(1592), 1,
      sym__string_start,
    ACTIONS(1594), 1,
      sym__multiline_string_start,
    ACTIONS(1730), 1,
      sym_identifier,
    ACTIONS(1732), 1,
      sym_wildcard,
    ACTIONS(1734), 1,
      sym_number,
    STATE(2296), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1735), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [32117] = 5,
    ACTIONS(39), 1,
      anon_sym_LBRACE,
    ACTIONS(185), 1,
      sym_comment,
    STATE(1062), 2,
      sym_block,
      sym_case_block,
    ACTIONS(231), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(233), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [32145] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1584), 1,
      anon_sym_LPAREN,
    ACTIONS(1590), 1,
      sym__simple_string,
    ACTIONS(1592), 1,
      sym__string_start,
    ACTIONS(1594), 1,
      sym__multiline_string_start,
    ACTIONS(1736), 1,
      sym_identifier,
    ACTIONS(1738), 1,
      sym_wildcard,
    ACTIONS(1740), 1,
      sym_number,
    STATE(2296), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1727), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [32185] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_LPAREN,
    ACTIONS(1475), 1,
      sym__simple_string,
    ACTIONS(1477), 1,
      sym__string_start,
    ACTIONS(1479), 1,
      sym__multiline_string_start,
    ACTIONS(1742), 1,
      sym_identifier,
    ACTIONS(1744), 1,
      sym_wildcard,
    ACTIONS(1746), 1,
      sym_number,
    STATE(2312), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1713), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [32225] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1442), 1,
      anon_sym_DOT,
    ACTIONS(1444), 1,
      anon_sym_LBRACK,
    ACTIONS(1448), 1,
      anon_sym_LPAREN,
    STATE(698), 1,
      sym_arguments,
    STATE(930), 1,
      sym_type_arguments,
    ACTIONS(273), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(275), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [32259] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1608), 1,
      anon_sym_LBRACK,
    ACTIONS(1612), 1,
      anon_sym_POUND,
    ACTIONS(1748), 1,
      anon_sym_LPAREN,
    STATE(1086), 1,
      sym_type_arguments,
    STATE(1083), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(215), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(217), 6,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [32293] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1442), 1,
      anon_sym_DOT,
    ACTIONS(1444), 1,
      anon_sym_LBRACK,
    ACTIONS(1446), 1,
      anon_sym_EQ,
    ACTIONS(1448), 1,
      anon_sym_LPAREN,
    STATE(698), 1,
      sym_arguments,
    STATE(930), 1,
      sym_type_arguments,
    ACTIONS(1440), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(269), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(271), 5,
      anon_sym_case,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
  [32331] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_LPAREN,
    ACTIONS(1475), 1,
      sym__simple_string,
    ACTIONS(1477), 1,
      sym__string_start,
    ACTIONS(1479), 1,
      sym__multiline_string_start,
    ACTIONS(1750), 1,
      sym_identifier,
    ACTIONS(1752), 1,
      sym_wildcard,
    ACTIONS(1754), 1,
      sym_number,
    STATE(2312), 1,
      sym_stable_type_identifier,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1645), 7,
      sym_case_class_pattern,
      sym_infix_pattern,
      sym_capture_pattern,
      sym_typed_pattern,
      sym_alternative_pattern,
      sym_tuple_pattern,
      sym_string,
  [32371] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1574), 1,
      anon_sym_LBRACK,
    ACTIONS(1578), 1,
      anon_sym_POUND,
    ACTIONS(1756), 1,
      anon_sym_AT,
    STATE(1039), 1,
      sym_type_arguments,
    ACTIONS(355), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1202), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(357), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_with,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [32404] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(269), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(271), 4,
      anon_sym_case,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
  [32441] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(273), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(275), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [32474] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(543), 8,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [32499] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 7,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(559), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [32522] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(517), 1,
      anon_sym_LBRACE,
    STATE(1222), 2,
      sym_block,
      sym_case_block,
    ACTIONS(231), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(233), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [32549] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 5,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(581), 10,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [32572] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 7,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(551), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [32595] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1454), 1,
      anon_sym_finally,
    STATE(906), 1,
      sym_finally_clause,
    ACTIONS(537), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(539), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      sym_identifier,
      sym_operator_identifier,
  [32622] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 5,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(577), 10,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [32645] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1758), 1,
      anon_sym_DOT,
    ACTIONS(509), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(511), 8,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [32670] = 4,
    ACTIONS(3), 1,
      sym_comment,
    STATE(766), 1,
      aux_sym_modifiers_repeat1,
    ACTIONS(1760), 6,
      anon_sym_case,
      anon_sym_class,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
    ACTIONS(1762), 8,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [32695] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(641), 5,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(643), 10,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [32718] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1764), 1,
      anon_sym_LPAREN,
    STATE(763), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(645), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(647), 9,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [32745] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(656), 5,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(658), 10,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [32768] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(285), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(287), 3,
      anon_sym_case,
      anon_sym_catch,
      anon_sym_finally,
  [32807] = 4,
    ACTIONS(3), 1,
      sym_comment,
    STATE(766), 1,
      aux_sym_modifiers_repeat1,
    ACTIONS(1767), 6,
      anon_sym_case,
      anon_sym_class,
      anon_sym_val,
      anon_sym_var,
      anon_sym_type,
      anon_sym_def,
    ACTIONS(1769), 8,
      anon_sym_abstract,
      anon_sym_final,
      anon_sym_sealed,
      anon_sym_implicit,
      anon_sym_lazy,
      anon_sym_override,
      anon_sym_private,
      anon_sym_protected,
  [32832] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 5,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(664), 10,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [32855] = 14,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(1784), 1,
      anon_sym_catch,
    ACTIONS(1786), 1,
      anon_sym_finally,
    STATE(795), 1,
      sym_arguments,
    STATE(1178), 1,
      sym_catch_clause,
    STATE(1293), 1,
      sym_finally_clause,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(165), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [32900] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(473), 1,
      sym__simple_string,
    ACTIONS(475), 1,
      sym__string_start,
    ACTIONS(477), 1,
      sym__multiline_string_start,
    STATE(1354), 1,
      sym_string,
    ACTIONS(189), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(191), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [32931] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1788), 1,
      anon_sym_LBRACK,
    ACTIONS(1790), 1,
      anon_sym_LPAREN,
    ACTIONS(1792), 1,
      anon_sym_POUND,
    STATE(1126), 1,
      sym_type_arguments,
    ACTIONS(215), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1124), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(217), 7,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [32964] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 7,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(555), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [32987] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1458), 1,
      anon_sym_LPAREN,
    STATE(763), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(615), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(617), 9,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [33014] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(674), 5,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(676), 10,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [33037] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(277), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(279), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [33070] = 14,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1796), 1,
      anon_sym_DOT,
    ACTIONS(1798), 1,
      anon_sym_LBRACK,
    ACTIONS(1800), 1,
      anon_sym_EQ,
    ACTIONS(1802), 1,
      anon_sym_LPAREN,
    ACTIONS(1804), 1,
      anon_sym_match,
    ACTIONS(1806), 1,
      anon_sym_catch,
    ACTIONS(1808), 1,
      anon_sym_finally,
    STATE(823), 1,
      sym_arguments,
    STATE(1174), 1,
      sym_catch_clause,
    STATE(1303), 1,
      sym_finally_clause,
    STATE(1330), 1,
      sym_type_arguments,
    ACTIONS(169), 2,
      anon_sym_EQ_GT,
      anon_sym_else,
    ACTIONS(1794), 2,
      sym_identifier,
      sym_operator_identifier,
  [33115] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1810), 1,
      anon_sym_LBRACK,
    ACTIONS(1812), 1,
      anon_sym_LPAREN,
    ACTIONS(1814), 1,
      anon_sym_POUND,
    STATE(1134), 1,
      sym_type_arguments,
    ACTIONS(215), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1133), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(217), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [33148] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(265), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(267), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [33181] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(281), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(283), 3,
      anon_sym_case,
      anon_sym_catch,
      anon_sym_finally,
  [33220] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(257), 1,
      sym__simple_string,
    ACTIONS(259), 1,
      sym__string_start,
    ACTIONS(261), 1,
      sym__multiline_string_start,
    STATE(1363), 1,
      sym_string,
    ACTIONS(189), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(191), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [33251] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1620), 1,
      anon_sym_LBRACK,
    ACTIONS(1624), 1,
      anon_sym_POUND,
    ACTIONS(1816), 1,
      anon_sym_AT,
    STATE(1015), 1,
      sym_type_arguments,
    ACTIONS(355), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1172), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(357), 7,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [33284] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1634), 1,
      anon_sym_DOT,
    ACTIONS(1636), 1,
      anon_sym_LBRACK,
    ACTIONS(1640), 1,
      anon_sym_LPAREN,
    STATE(755), 1,
      sym_arguments,
    STATE(1207), 1,
      sym_type_arguments,
    ACTIONS(265), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(267), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [33316] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(664), 8,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [33338] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1818), 1,
      sym_identifier,
    ACTIONS(1820), 1,
      anon_sym_LPAREN,
    STATE(1586), 1,
      sym__annotated_type,
    STATE(2350), 1,
      sym_parameter_types,
    STATE(2353), 1,
      sym_stable_identifier,
    STATE(1646), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2011), 2,
      sym__type,
      sym_function_type,
    STATE(1064), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [33372] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1818), 1,
      sym_identifier,
    ACTIONS(1820), 1,
      anon_sym_LPAREN,
    STATE(1586), 1,
      sym__annotated_type,
    STATE(2350), 1,
      sym_parameter_types,
    STATE(2353), 1,
      sym_stable_identifier,
    STATE(1646), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2026), 2,
      sym__type,
      sym_function_type,
    STATE(1064), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [33406] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(1824), 1,
      anon_sym_LPAREN,
    STATE(465), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(2295), 1,
      sym_parameter_types,
    STATE(267), 2,
      sym__type,
      sym_function_type,
    STATE(512), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [33440] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(1828), 1,
      anon_sym_LPAREN,
    STATE(1314), 1,
      sym__annotated_type,
    STATE(2374), 1,
      sym_parameter_types,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(1505), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1640), 2,
      sym__type,
      sym_function_type,
    STATE(712), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [33474] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(583), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(585), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [33496] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(1832), 1,
      anon_sym_LPAREN,
    STATE(1367), 1,
      sym__annotated_type,
    STATE(2364), 1,
      sym_parameter_types,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(1521), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1891), 2,
      sym__type,
      sym_function_type,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [33530] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2044), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [33564] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2062), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [33598] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(267), 2,
      sym__type,
      sym_function_type,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [33632] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(543), 9,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [33656] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2075), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [33690] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(435), 1,
      sym__simple_string,
    ACTIONS(437), 1,
      sym__string_start,
    ACTIONS(439), 1,
      sym__multiline_string_start,
    STATE(1417), 1,
      sym_string,
    ACTIONS(189), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(191), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [33720] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(241), 1,
      anon_sym_LBRACE,
    STATE(1269), 2,
      sym_block,
      sym_case_block,
    ACTIONS(231), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(233), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [33746] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2092), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [33780] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2106), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [33814] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1788), 1,
      anon_sym_LBRACK,
    ACTIONS(1792), 1,
      anon_sym_POUND,
    ACTIONS(1838), 1,
      anon_sym_AT,
    STATE(1126), 1,
      sym_type_arguments,
    ACTIONS(355), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1350), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(357), 6,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [33846] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 7,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(555), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [33868] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(1824), 1,
      anon_sym_LPAREN,
    STATE(465), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(2295), 1,
      sym_parameter_types,
    STATE(512), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(599), 2,
      sym__type,
      sym_function_type,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [33902] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2125), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [33936] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1840), 1,
      sym_identifier,
    ACTIONS(1842), 1,
      anon_sym_LPAREN,
    STATE(1180), 1,
      sym__annotated_type,
    STATE(2369), 1,
      sym_parameter_types,
    STATE(2372), 1,
      sym_stable_identifier,
    STATE(1459), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1470), 2,
      sym__type,
      sym_function_type,
    STATE(691), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [33970] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(1846), 1,
      anon_sym_LPAREN,
    STATE(1557), 1,
      sym__annotated_type,
    STATE(2297), 1,
      sym_parameter_types,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1698), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1754), 2,
      sym__type,
      sym_function_type,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34004] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1840), 1,
      sym_identifier,
    ACTIONS(1842), 1,
      anon_sym_LPAREN,
    STATE(1180), 1,
      sym__annotated_type,
    STATE(2369), 1,
      sym_parameter_types,
    STATE(2372), 1,
      sym_stable_identifier,
    STATE(1459), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1485), 2,
      sym__type,
      sym_function_type,
    STATE(691), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34038] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2023), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34072] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(1832), 1,
      anon_sym_LPAREN,
    STATE(1367), 1,
      sym__annotated_type,
    STATE(2364), 1,
      sym_parameter_types,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(1521), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1866), 2,
      sym__type,
      sym_function_type,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34106] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1848), 1,
      anon_sym_DOT,
    ACTIONS(509), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(511), 9,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [34130] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2173), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34164] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2168), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34198] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1990), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34232] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2163), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34266] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2158), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34300] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(1832), 1,
      anon_sym_LPAREN,
    STATE(1367), 1,
      sym__annotated_type,
    STATE(2364), 1,
      sym_parameter_types,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(1521), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1863), 2,
      sym__type,
      sym_function_type,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34334] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(1832), 1,
      anon_sym_LPAREN,
    STATE(1367), 1,
      sym__annotated_type,
    STATE(2364), 1,
      sym_parameter_types,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(1521), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1862), 2,
      sym__type,
      sym_function_type,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34368] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1850), 1,
      sym_identifier,
    ACTIONS(1852), 1,
      anon_sym_LPAREN,
    STATE(409), 1,
      sym__annotated_type,
    STATE(2288), 1,
      sym_parameter_types,
    STATE(2293), 1,
      sym_stable_identifier,
    STATE(496), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(515), 2,
      sym__type,
      sym_function_type,
    STATE(70), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34402] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2011), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34436] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2154), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34470] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(1832), 1,
      anon_sym_LPAREN,
    STATE(1367), 1,
      sym__annotated_type,
    STATE(2364), 1,
      sym_parameter_types,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(1521), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1631), 2,
      sym__type,
      sym_function_type,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34504] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(623), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(625), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [34526] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1854), 1,
      sym_identifier,
    ACTIONS(1856), 1,
      anon_sym_LPAREN,
    STATE(1596), 1,
      sym__annotated_type,
    STATE(2320), 1,
      sym_stable_identifier,
    STATE(2378), 1,
      sym_parameter_types,
    STATE(1653), 2,
      sym__type,
      sym_function_type,
    STATE(1703), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(750), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34560] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2149), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34594] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(1828), 1,
      anon_sym_LPAREN,
    STATE(1314), 1,
      sym__annotated_type,
    STATE(2374), 1,
      sym_parameter_types,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(1505), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1617), 2,
      sym__type,
      sym_function_type,
    STATE(712), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34628] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(459), 1,
      anon_sym_LBRACE,
    STATE(1282), 2,
      sym_block,
      sym_case_block,
    ACTIONS(231), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(233), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [34654] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(1828), 1,
      anon_sym_LPAREN,
    STATE(1314), 1,
      sym__annotated_type,
    STATE(2374), 1,
      sym_parameter_types,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(1505), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1618), 2,
      sym__type,
      sym_function_type,
    STATE(712), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34688] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(1846), 1,
      anon_sym_LPAREN,
    STATE(1557), 1,
      sym__annotated_type,
    STATE(2297), 1,
      sym_parameter_types,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1698), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1778), 2,
      sym__type,
      sym_function_type,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34722] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1840), 1,
      sym_identifier,
    ACTIONS(1842), 1,
      anon_sym_LPAREN,
    STATE(1180), 1,
      sym__annotated_type,
    STATE(2369), 1,
      sym_parameter_types,
    STATE(2372), 1,
      sym_stable_identifier,
    STATE(1459), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1510), 2,
      sym__type,
      sym_function_type,
    STATE(691), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34756] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(1418), 1,
      anon_sym_LPAREN,
    STATE(1592), 1,
      sym__annotated_type,
    STATE(2277), 1,
      sym_parameter_types,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(267), 2,
      sym__type,
      sym_function_type,
    STATE(1697), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34790] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2144), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34824] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1840), 1,
      sym_identifier,
    ACTIONS(1842), 1,
      anon_sym_LPAREN,
    STATE(1180), 1,
      sym__annotated_type,
    STATE(2369), 1,
      sym_parameter_types,
    STATE(2372), 1,
      sym_stable_identifier,
    STATE(1459), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1508), 2,
      sym__type,
      sym_function_type,
    STATE(691), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34858] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(1860), 1,
      anon_sym_LPAREN,
    STATE(491), 1,
      sym__annotated_type,
    STATE(2306), 1,
      sym_parameter_types,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(508), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(525), 2,
      sym__type,
      sym_function_type,
    STATE(212), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34892] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(1824), 1,
      anon_sym_LPAREN,
    STATE(465), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(2295), 1,
      sym_parameter_types,
    STATE(512), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(617), 2,
      sym__type,
      sym_function_type,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34926] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2138), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [34960] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(627), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(629), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [34982] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(735), 1,
      anon_sym_LBRACK,
    ACTIONS(1864), 1,
      anon_sym_LT_COLON,
    ACTIONS(1866), 1,
      anon_sym_GT_COLON,
    ACTIONS(1868), 1,
      anon_sym_LT_PERCENT,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    STATE(1201), 1,
      sym_type_parameters,
    STATE(1387), 1,
      sym_upper_bound,
    STATE(1580), 1,
      sym_lower_bound,
    ACTIONS(1862), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1578), 2,
      sym_view_bound,
      aux_sym__type_parameter_repeat1,
    STATE(1769), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [35022] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(1832), 1,
      anon_sym_LPAREN,
    STATE(1367), 1,
      sym__annotated_type,
    STATE(2364), 1,
      sym_parameter_types,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(1521), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1838), 2,
      sym__type,
      sym_function_type,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35056] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(543), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [35080] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(1824), 1,
      anon_sym_LPAREN,
    STATE(465), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(2295), 1,
      sym_parameter_types,
    STATE(512), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(611), 2,
      sym__type,
      sym_function_type,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35114] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(631), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(633), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [35136] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(1860), 1,
      anon_sym_LPAREN,
    STATE(491), 1,
      sym__annotated_type,
    STATE(2306), 1,
      sym_parameter_types,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(508), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(530), 2,
      sym__type,
      sym_function_type,
    STATE(212), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35170] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1872), 1,
      anon_sym_DOT,
    ACTIONS(509), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(511), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [35194] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2129), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35228] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(1860), 1,
      anon_sym_LPAREN,
    STATE(491), 1,
      sym__annotated_type,
    STATE(2306), 1,
      sym_parameter_types,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(508), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(524), 2,
      sym__type,
      sym_function_type,
    STATE(212), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35262] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1850), 1,
      sym_identifier,
    ACTIONS(1852), 1,
      anon_sym_LPAREN,
    STATE(409), 1,
      sym__annotated_type,
    STATE(2288), 1,
      sym_parameter_types,
    STATE(2293), 1,
      sym_stable_identifier,
    STATE(496), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(514), 2,
      sym__type,
      sym_function_type,
    STATE(70), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35296] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(1860), 1,
      anon_sym_LPAREN,
    STATE(491), 1,
      sym__annotated_type,
    STATE(2306), 1,
      sym_parameter_types,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(267), 2,
      sym__type,
      sym_function_type,
    STATE(508), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(212), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35330] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1634), 1,
      anon_sym_DOT,
    ACTIONS(1636), 1,
      anon_sym_LBRACK,
    ACTIONS(1638), 1,
      anon_sym_EQ,
    ACTIONS(1640), 1,
      anon_sym_LPAREN,
    ACTIONS(1642), 1,
      anon_sym_match,
    ACTIONS(1874), 1,
      anon_sym_else,
    STATE(755), 1,
      sym_arguments,
    STATE(1207), 1,
      sym_type_arguments,
    ACTIONS(303), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(305), 2,
      anon_sym_catch,
      anon_sym_finally,
    ACTIONS(1632), 2,
      sym_identifier,
      sym_operator_identifier,
  [35370] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2121), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35404] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(1846), 1,
      anon_sym_LPAREN,
    STATE(1557), 1,
      sym__annotated_type,
    STATE(2297), 1,
      sym_parameter_types,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1698), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1789), 2,
      sym__type,
      sym_function_type,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35438] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1850), 1,
      sym_identifier,
    ACTIONS(1852), 1,
      anon_sym_LPAREN,
    STATE(409), 1,
      sym__annotated_type,
    STATE(2288), 1,
      sym_parameter_types,
    STATE(2293), 1,
      sym_stable_identifier,
    STATE(496), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(510), 2,
      sym__type,
      sym_function_type,
    STATE(70), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35472] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(1846), 1,
      anon_sym_LPAREN,
    STATE(1557), 1,
      sym__annotated_type,
    STATE(2297), 1,
      sym_parameter_types,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1698), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1767), 2,
      sym__type,
      sym_function_type,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35506] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(1824), 1,
      anon_sym_LPAREN,
    STATE(465), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(2295), 1,
      sym_parameter_types,
    STATE(512), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(633), 2,
      sym__type,
      sym_function_type,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35540] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(1860), 1,
      anon_sym_LPAREN,
    STATE(491), 1,
      sym__annotated_type,
    STATE(2306), 1,
      sym_parameter_types,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(508), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(535), 2,
      sym__type,
      sym_function_type,
    STATE(212), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35574] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1544), 1,
      anon_sym_finally,
    STATE(1105), 1,
      sym_finally_clause,
    ACTIONS(537), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(539), 6,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      sym_identifier,
      sym_operator_identifier,
  [35600] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1876), 1,
      anon_sym_LBRACK,
    ACTIONS(1878), 1,
      anon_sym_LPAREN,
    ACTIONS(1880), 1,
      anon_sym_POUND,
    STATE(1255), 1,
      sym_type_arguments,
    ACTIONS(215), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1251), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(217), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [35632] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1854), 1,
      sym_identifier,
    ACTIONS(1856), 1,
      anon_sym_LPAREN,
    STATE(1479), 1,
      sym__annotated_type,
    STATE(2315), 1,
      sym_parameter_types,
    STATE(2320), 1,
      sym_stable_identifier,
    STATE(1630), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1708), 2,
      sym__type,
      sym_function_type,
    STATE(750), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35666] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2110), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35700] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2102), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35734] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2091), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35768] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(1828), 1,
      anon_sym_LPAREN,
    STATE(1314), 1,
      sym__annotated_type,
    STATE(2374), 1,
      sym_parameter_types,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(1505), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1622), 2,
      sym__type,
      sym_function_type,
    STATE(712), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35802] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2083), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35836] = 14,
    ACTIONS(169), 1,
      anon_sym_EQ_GT,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1884), 1,
      anon_sym_DOT,
    ACTIONS(1886), 1,
      anon_sym_LBRACK,
    ACTIONS(1888), 1,
      anon_sym_EQ,
    ACTIONS(1890), 1,
      anon_sym_LPAREN,
    ACTIONS(1892), 1,
      anon_sym_match,
    ACTIONS(1894), 1,
      anon_sym_catch,
    ACTIONS(1896), 1,
      anon_sym_finally,
    STATE(1009), 1,
      sym_arguments,
    STATE(1365), 1,
      sym_catch_clause,
    STATE(1395), 1,
      sym_finally_clause,
    STATE(1423), 1,
      sym_type_arguments,
    ACTIONS(1882), 2,
      sym_identifier,
      sym_operator_identifier,
  [35880] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(1828), 1,
      anon_sym_LPAREN,
    STATE(1314), 1,
      sym__annotated_type,
    STATE(2374), 1,
      sym_parameter_types,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(1505), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1629), 2,
      sym__type,
      sym_function_type,
    STATE(712), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35914] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2072), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35948] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1898), 1,
      sym_identifier,
    ACTIONS(1900), 1,
      anon_sym_LPAREN,
    STATE(1432), 1,
      sym__annotated_type,
    STATE(2324), 1,
      sym_parameter_types,
    STATE(2329), 1,
      sym_stable_identifier,
    STATE(267), 2,
      sym__type,
      sym_function_type,
    STATE(1579), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(780), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [35982] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 7,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(551), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [36004] = 13,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    ACTIONS(1902), 1,
      anon_sym_RBRACE,
    ACTIONS(1904), 1,
      anon_sym_case,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    STATE(1766), 1,
      aux_sym__block_repeat1,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(1906), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [36046] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(1832), 1,
      anon_sym_LPAREN,
    STATE(1367), 1,
      sym__annotated_type,
    STATE(2364), 1,
      sym_parameter_types,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(1521), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1839), 2,
      sym__type,
      sym_function_type,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36080] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(1824), 1,
      anon_sym_LPAREN,
    STATE(465), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(2295), 1,
      sym_parameter_types,
    STATE(512), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(613), 2,
      sym__type,
      sym_function_type,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36114] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1908), 1,
      sym_identifier,
    ACTIONS(1910), 1,
      anon_sym_LPAREN,
    STATE(1550), 1,
      sym__annotated_type,
    STATE(2342), 1,
      sym_parameter_types,
    STATE(2346), 1,
      sym_stable_identifier,
    STATE(1696), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2003), 2,
      sym__type,
      sym_function_type,
    STATE(1000), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36148] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1912), 1,
      sym_identifier,
    ACTIONS(1914), 1,
      anon_sym_LPAREN,
    STATE(1507), 1,
      sym__annotated_type,
    STATE(2333), 1,
      sym_parameter_types,
    STATE(2338), 1,
      sym_stable_identifier,
    STATE(1621), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1856), 2,
      sym__type,
      sym_function_type,
    STATE(798), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36182] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1634), 1,
      anon_sym_DOT,
    ACTIONS(1636), 1,
      anon_sym_LBRACK,
    ACTIONS(1638), 1,
      anon_sym_EQ,
    ACTIONS(1640), 1,
      anon_sym_LPAREN,
    ACTIONS(1642), 1,
      anon_sym_match,
    STATE(755), 1,
      sym_arguments,
    STATE(1207), 1,
      sym_type_arguments,
    ACTIONS(285), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(1632), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(287), 3,
      anon_sym_else,
      anon_sym_catch,
      anon_sym_finally,
  [36220] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1634), 1,
      anon_sym_DOT,
    ACTIONS(1636), 1,
      anon_sym_LBRACK,
    ACTIONS(1638), 1,
      anon_sym_EQ,
    ACTIONS(1640), 1,
      anon_sym_LPAREN,
    ACTIONS(1642), 1,
      anon_sym_match,
    STATE(755), 1,
      sym_arguments,
    STATE(1207), 1,
      sym_type_arguments,
    ACTIONS(281), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(1632), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(283), 3,
      anon_sym_else,
      anon_sym_catch,
      anon_sym_finally,
  [36258] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2058), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36292] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1912), 1,
      sym_identifier,
    ACTIONS(1914), 1,
      anon_sym_LPAREN,
    STATE(1507), 1,
      sym__annotated_type,
    STATE(2333), 1,
      sym_parameter_types,
    STATE(2338), 1,
      sym_stable_identifier,
    STATE(1621), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1857), 2,
      sym__type,
      sym_function_type,
    STATE(798), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36326] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1898), 1,
      sym_identifier,
    ACTIONS(1900), 1,
      anon_sym_LPAREN,
    STATE(1432), 1,
      sym__annotated_type,
    STATE(2324), 1,
      sym_parameter_types,
    STATE(2329), 1,
      sym_stable_identifier,
    STATE(1579), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1759), 2,
      sym__type,
      sym_function_type,
    STATE(780), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36360] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1634), 1,
      anon_sym_DOT,
    ACTIONS(1636), 1,
      anon_sym_LBRACK,
    ACTIONS(1640), 1,
      anon_sym_LPAREN,
    STATE(755), 1,
      sym_arguments,
    STATE(1207), 1,
      sym_type_arguments,
    ACTIONS(273), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(275), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [36392] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2043), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36426] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2028), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36460] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1634), 1,
      anon_sym_DOT,
    ACTIONS(1636), 1,
      anon_sym_LBRACK,
    ACTIONS(1638), 1,
      anon_sym_EQ,
    ACTIONS(1640), 1,
      anon_sym_LPAREN,
    STATE(755), 1,
      sym_arguments,
    STATE(1207), 1,
      sym_type_arguments,
    ACTIONS(269), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(1632), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(271), 4,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
  [36496] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1818), 1,
      sym_identifier,
    ACTIONS(1820), 1,
      anon_sym_LPAREN,
    STATE(1586), 1,
      sym__annotated_type,
    STATE(2350), 1,
      sym_parameter_types,
    STATE(2353), 1,
      sym_stable_identifier,
    STATE(1646), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2118), 2,
      sym__type,
      sym_function_type,
    STATE(1064), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36530] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2013), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36564] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2032), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36598] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(666), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(668), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [36620] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1997), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36654] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1982), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36688] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1912), 1,
      sym_identifier,
    ACTIONS(1914), 1,
      anon_sym_LPAREN,
    STATE(1507), 1,
      sym__annotated_type,
    STATE(2333), 1,
      sym_parameter_types,
    STATE(2338), 1,
      sym_stable_identifier,
    STATE(267), 2,
      sym__type,
      sym_function_type,
    STATE(1621), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(798), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36722] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1966), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36756] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1951), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36790] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1935), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36824] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(1860), 1,
      anon_sym_LPAREN,
    STATE(491), 1,
      sym__annotated_type,
    STATE(2306), 1,
      sym_parameter_types,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(508), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(532), 2,
      sym__type,
      sym_function_type,
    STATE(212), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36858] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1916), 1,
      anon_sym_LBRACK,
    ACTIONS(1918), 1,
      anon_sym_LPAREN,
    ACTIONS(1920), 1,
      anon_sym_POUND,
    STATE(1279), 1,
      sym_type_arguments,
    ACTIONS(215), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1283), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(217), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [36890] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(1824), 1,
      anon_sym_LPAREN,
    STATE(465), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(2295), 1,
      sym_parameter_types,
    STATE(512), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(639), 2,
      sym__type,
      sym_function_type,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36924] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(1846), 1,
      anon_sym_LPAREN,
    STATE(1557), 1,
      sym__annotated_type,
    STATE(2297), 1,
      sym_parameter_types,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1698), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1785), 2,
      sym__type,
      sym_function_type,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36958] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1920), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [36992] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1634), 1,
      anon_sym_DOT,
    ACTIONS(1636), 1,
      anon_sym_LBRACK,
    ACTIONS(1640), 1,
      anon_sym_LPAREN,
    STATE(755), 1,
      sym_arguments,
    STATE(1207), 1,
      sym_type_arguments,
    ACTIONS(277), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(279), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [37024] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1850), 1,
      sym_identifier,
    ACTIONS(1852), 1,
      anon_sym_LPAREN,
    STATE(409), 1,
      sym__annotated_type,
    STATE(2288), 1,
      sym_parameter_types,
    STATE(2293), 1,
      sym_stable_identifier,
    STATE(496), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(504), 2,
      sym__type,
      sym_function_type,
    STATE(70), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37058] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1810), 1,
      anon_sym_LBRACK,
    ACTIONS(1814), 1,
      anon_sym_POUND,
    ACTIONS(1922), 1,
      anon_sym_AT,
    STATE(1134), 1,
      sym_type_arguments,
    ACTIONS(355), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1300), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(357), 6,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [37090] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1948), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37124] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1908), 1,
      sym_identifier,
    ACTIONS(1910), 1,
      anon_sym_LPAREN,
    STATE(1550), 1,
      sym__annotated_type,
    STATE(2342), 1,
      sym_parameter_types,
    STATE(2346), 1,
      sym_stable_identifier,
    STATE(267), 2,
      sym__type,
      sym_function_type,
    STATE(1696), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1000), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37158] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(1846), 1,
      anon_sym_LPAREN,
    STATE(1557), 1,
      sym__annotated_type,
    STATE(2297), 1,
      sym_parameter_types,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1698), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1740), 2,
      sym__type,
      sym_function_type,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37192] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(1824), 1,
      anon_sym_LPAREN,
    STATE(465), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(2295), 1,
      sym_parameter_types,
    STATE(512), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(600), 2,
      sym__type,
      sym_function_type,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37226] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1924), 1,
      anon_sym_LBRACK,
    ACTIONS(1926), 1,
      anon_sym_LPAREN,
    ACTIONS(1928), 1,
      anon_sym_POUND,
    STATE(1275), 1,
      sym_type_arguments,
    STATE(1306), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(217), 8,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [37256] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(1824), 1,
      anon_sym_LPAREN,
    STATE(465), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(2295), 1,
      sym_parameter_types,
    STATE(512), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(608), 2,
      sym__type,
      sym_function_type,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37290] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(591), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(593), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [37312] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1965), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37346] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(1832), 1,
      anon_sym_LPAREN,
    STATE(1367), 1,
      sym__annotated_type,
    STATE(2364), 1,
      sym_parameter_types,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(1521), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1843), 2,
      sym__type,
      sym_function_type,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37380] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(595), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(597), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [37402] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(1824), 1,
      anon_sym_LPAREN,
    STATE(465), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(2295), 1,
      sym_parameter_types,
    STATE(512), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(609), 2,
      sym__type,
      sym_function_type,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37436] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(609), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(611), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [37458] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(605), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(607), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [37480] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(577), 8,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [37502] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1980), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37536] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(637), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(639), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [37558] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1998), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37592] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(1824), 1,
      anon_sym_LPAREN,
    STATE(465), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(2295), 1,
      sym_parameter_types,
    STATE(512), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(610), 2,
      sym__type,
      sym_function_type,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37626] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(652), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(654), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [37648] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(537), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(539), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [37670] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(1860), 1,
      anon_sym_LPAREN,
    STATE(491), 1,
      sym__annotated_type,
    STATE(2306), 1,
      sym_parameter_types,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(508), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(537), 2,
      sym__type,
      sym_function_type,
    STATE(212), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37704] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(581), 8,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [37726] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(1824), 1,
      anon_sym_LPAREN,
    STATE(465), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(2295), 1,
      sym_parameter_types,
    STATE(512), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(607), 2,
      sym__type,
      sym_function_type,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37760] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(1832), 1,
      anon_sym_LPAREN,
    STATE(1367), 1,
      sym__annotated_type,
    STATE(2364), 1,
      sym_parameter_types,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(1521), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1847), 2,
      sym__type,
      sym_function_type,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37794] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(1860), 1,
      anon_sym_LPAREN,
    STATE(491), 1,
      sym__annotated_type,
    STATE(2306), 1,
      sym_parameter_types,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(508), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(538), 2,
      sym__type,
      sym_function_type,
    STATE(212), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37828] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1818), 1,
      sym_identifier,
    ACTIONS(1820), 1,
      anon_sym_LPAREN,
    STATE(1586), 1,
      sym__annotated_type,
    STATE(2350), 1,
      sym_parameter_types,
    STATE(2353), 1,
      sym_stable_identifier,
    STATE(267), 2,
      sym__type,
      sym_function_type,
    STATE(1646), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1064), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37862] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2033), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37896] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2057), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37930] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(678), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(680), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [37952] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(1846), 1,
      anon_sym_LPAREN,
    STATE(1557), 1,
      sym__annotated_type,
    STATE(2297), 1,
      sym_parameter_types,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1698), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1792), 2,
      sym__type,
      sym_function_type,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [37986] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(1846), 1,
      anon_sym_LPAREN,
    STATE(1557), 1,
      sym__annotated_type,
    STATE(2297), 1,
      sym_parameter_types,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1698), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1788), 2,
      sym__type,
      sym_function_type,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38020] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1854), 1,
      sym_identifier,
    ACTIONS(1856), 1,
      anon_sym_LPAREN,
    STATE(1479), 1,
      sym__annotated_type,
    STATE(2315), 1,
      sym_parameter_types,
    STATE(2320), 1,
      sym_stable_identifier,
    STATE(1630), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1715), 2,
      sym__type,
      sym_function_type,
    STATE(750), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38054] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(1846), 1,
      anon_sym_LPAREN,
    STATE(1557), 1,
      sym__annotated_type,
    STATE(2297), 1,
      sym_parameter_types,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1698), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1784), 2,
      sym__type,
      sym_function_type,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38088] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(619), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(621), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [38110] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(670), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(672), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [38132] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1499), 1,
      sym_identifier,
    ACTIONS(1503), 1,
      anon_sym_LPAREN,
    STATE(1498), 1,
      sym__annotated_type,
    STATE(2357), 1,
      sym_parameter_types,
    STATE(2360), 1,
      sym_stable_identifier,
    STATE(267), 2,
      sym__type,
      sym_function_type,
    STATE(1605), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(896), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38166] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(587), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(589), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [38188] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(1846), 1,
      anon_sym_LPAREN,
    STATE(1557), 1,
      sym__annotated_type,
    STATE(2297), 1,
      sym_parameter_types,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1698), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1782), 2,
      sym__type,
      sym_function_type,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38222] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(189), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(191), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [38244] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1850), 1,
      sym_identifier,
    ACTIONS(1852), 1,
      anon_sym_LPAREN,
    STATE(409), 1,
      sym__annotated_type,
    STATE(2288), 1,
      sym_parameter_types,
    STATE(2293), 1,
      sym_stable_identifier,
    STATE(496), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(506), 2,
      sym__type,
      sym_function_type,
    STATE(70), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38278] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1818), 1,
      sym_identifier,
    ACTIONS(1820), 1,
      anon_sym_LPAREN,
    STATE(1586), 1,
      sym__annotated_type,
    STATE(2350), 1,
      sym_parameter_types,
    STATE(2353), 1,
      sym_stable_identifier,
    STATE(1646), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2136), 2,
      sym__type,
      sym_function_type,
    STATE(1064), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38312] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(1860), 1,
      anon_sym_LPAREN,
    STATE(491), 1,
      sym__annotated_type,
    STATE(2306), 1,
      sym_parameter_types,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(508), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(527), 2,
      sym__type,
      sym_function_type,
    STATE(212), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38346] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(1846), 1,
      anon_sym_LPAREN,
    STATE(1557), 1,
      sym__annotated_type,
    STATE(2297), 1,
      sym_parameter_types,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1698), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1747), 2,
      sym__type,
      sym_function_type,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38380] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(1828), 1,
      anon_sym_LPAREN,
    STATE(1314), 1,
      sym__annotated_type,
    STATE(2374), 1,
      sym_parameter_types,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(1505), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1633), 2,
      sym__type,
      sym_function_type,
    STATE(712), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38414] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(1832), 1,
      anon_sym_LPAREN,
    STATE(1367), 1,
      sym__annotated_type,
    STATE(2364), 1,
      sym_parameter_types,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(1521), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1636), 2,
      sym__type,
      sym_function_type,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38448] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(1846), 1,
      anon_sym_LPAREN,
    STATE(1557), 1,
      sym__annotated_type,
    STATE(2297), 1,
      sym_parameter_types,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1698), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1743), 2,
      sym__type,
      sym_function_type,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38482] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(1860), 1,
      anon_sym_LPAREN,
    STATE(491), 1,
      sym__annotated_type,
    STATE(2306), 1,
      sym_parameter_types,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(508), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(529), 2,
      sym__type,
      sym_function_type,
    STATE(212), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38516] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(1828), 1,
      anon_sym_LPAREN,
    STATE(1314), 1,
      sym__annotated_type,
    STATE(2374), 1,
      sym_parameter_types,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(1505), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1635), 2,
      sym__type,
      sym_function_type,
    STATE(712), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38550] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(601), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(603), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [38572] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(668), 4,
      anon_sym_if,
      anon_sym_try,
      anon_sym_new,
      sym_identifier,
    ACTIONS(666), 10,
      sym__simple_string,
      sym__string_start,
      sym__multiline_string_start,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LPAREN,
      anon_sym_BANG,
      anon_sym_TILDE,
      sym_number,
  [38594] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1840), 1,
      sym_identifier,
    ACTIONS(1842), 1,
      anon_sym_LPAREN,
    STATE(1180), 1,
      sym__annotated_type,
    STATE(2369), 1,
      sym_parameter_types,
    STATE(2372), 1,
      sym_stable_identifier,
    STATE(1459), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1495), 2,
      sym__type,
      sym_function_type,
    STATE(691), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38628] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2159), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38662] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(1824), 1,
      anon_sym_LPAREN,
    STATE(465), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(2295), 1,
      sym_parameter_types,
    STATE(512), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(534), 2,
      sym__type,
      sym_function_type,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38696] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(581), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [38718] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(577), 8,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [38740] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1930), 1,
      anon_sym_DOT,
    ACTIONS(509), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(511), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [38764] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1932), 1,
      anon_sym_LBRACK,
    ACTIONS(1934), 1,
      anon_sym_LPAREN,
    ACTIONS(1936), 1,
      anon_sym_POUND,
    STATE(1266), 1,
      sym_type_arguments,
    STATE(1271), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(215), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
    ACTIONS(217), 5,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [38796] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(1828), 1,
      anon_sym_LPAREN,
    STATE(1314), 1,
      sym__annotated_type,
    STATE(2374), 1,
      sym_parameter_types,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(1505), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1636), 2,
      sym__type,
      sym_function_type,
    STATE(712), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38830] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1840), 1,
      sym_identifier,
    ACTIONS(1842), 1,
      anon_sym_LPAREN,
    STATE(1180), 1,
      sym__annotated_type,
    STATE(2369), 1,
      sym_parameter_types,
    STATE(2372), 1,
      sym_stable_identifier,
    STATE(1459), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1491), 2,
      sym__type,
      sym_function_type,
    STATE(691), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38864] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1481), 1,
      anon_sym_AT,
    STATE(969), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(782), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(784), 8,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [38890] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1854), 1,
      sym_identifier,
    ACTIONS(1856), 1,
      anon_sym_LPAREN,
    STATE(1596), 1,
      sym__annotated_type,
    STATE(2320), 1,
      sym_stable_identifier,
    STATE(2378), 1,
      sym_parameter_types,
    STATE(1650), 2,
      sym__type,
      sym_function_type,
    STATE(1703), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(750), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [38924] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(543), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [38948] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1634), 1,
      anon_sym_DOT,
    ACTIONS(1636), 1,
      anon_sym_LBRACK,
    ACTIONS(1638), 1,
      anon_sym_EQ,
    ACTIONS(1640), 1,
      anon_sym_LPAREN,
    ACTIONS(1642), 1,
      anon_sym_match,
    ACTIONS(1938), 1,
      anon_sym_else,
    STATE(755), 1,
      sym_arguments,
    STATE(1207), 1,
      sym_type_arguments,
    ACTIONS(303), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(305), 2,
      anon_sym_catch,
      anon_sym_finally,
    ACTIONS(1632), 2,
      sym_identifier,
      sym_operator_identifier,
  [38988] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1840), 1,
      sym_identifier,
    ACTIONS(1842), 1,
      anon_sym_LPAREN,
    STATE(1180), 1,
      sym__annotated_type,
    STATE(2369), 1,
      sym_parameter_types,
    STATE(2372), 1,
      sym_stable_identifier,
    STATE(1459), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1489), 2,
      sym__type,
      sym_function_type,
    STATE(691), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39022] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1940), 1,
      anon_sym_LBRACK,
    ACTIONS(1942), 1,
      anon_sym_LPAREN,
    ACTIONS(1944), 1,
      anon_sym_POUND,
    STATE(1253), 1,
      sym_type_arguments,
    ACTIONS(215), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1244), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(217), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [39054] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_SEMI,
    ACTIONS(664), 10,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [39076] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(1860), 1,
      anon_sym_LPAREN,
    STATE(491), 1,
      sym__annotated_type,
    STATE(2306), 1,
      sym_parameter_types,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(508), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(541), 2,
      sym__type,
      sym_function_type,
    STATE(212), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39110] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2041), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39144] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(1828), 1,
      anon_sym_LPAREN,
    STATE(1314), 1,
      sym__annotated_type,
    STATE(2374), 1,
      sym_parameter_types,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(1505), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1619), 2,
      sym__type,
      sym_function_type,
    STATE(712), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39178] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(1836), 1,
      anon_sym_LPAREN,
    STATE(1600), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(2348), 1,
      sym_parameter_types,
    STATE(1663), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(2140), 2,
      sym__type,
      sym_function_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39212] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(1846), 1,
      anon_sym_LPAREN,
    STATE(1557), 1,
      sym__annotated_type,
    STATE(2297), 1,
      sym_parameter_types,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1698), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1739), 2,
      sym__type,
      sym_function_type,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39246] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1518), 1,
      anon_sym_LPAREN,
    STATE(981), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(615), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(617), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [39272] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1946), 1,
      anon_sym_AT,
    STATE(969), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(700), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(702), 8,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [39298] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(1832), 1,
      anon_sym_LPAREN,
    STATE(1367), 1,
      sym__annotated_type,
    STATE(2364), 1,
      sym_parameter_types,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(1521), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1876), 2,
      sym__type,
      sym_function_type,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39332] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(1846), 1,
      anon_sym_LPAREN,
    STATE(1557), 1,
      sym__annotated_type,
    STATE(2297), 1,
      sym_parameter_types,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1698), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1729), 2,
      sym__type,
      sym_function_type,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39366] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 7,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(559), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [39388] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(1860), 1,
      anon_sym_LPAREN,
    STATE(491), 1,
      sym__annotated_type,
    STATE(2306), 1,
      sym_parameter_types,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(508), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(523), 2,
      sym__type,
      sym_function_type,
    STATE(212), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39422] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1850), 1,
      sym_identifier,
    ACTIONS(1852), 1,
      anon_sym_LPAREN,
    STATE(409), 1,
      sym__annotated_type,
    STATE(2288), 1,
      sym_parameter_types,
    STATE(2293), 1,
      sym_stable_identifier,
    STATE(496), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(511), 2,
      sym__type,
      sym_function_type,
    STATE(70), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39456] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(674), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(676), 8,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [39478] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(1832), 1,
      anon_sym_LPAREN,
    STATE(1367), 1,
      sym__annotated_type,
    STATE(2364), 1,
      sym_parameter_types,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(1521), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1889), 2,
      sym__type,
      sym_function_type,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39512] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1850), 1,
      sym_identifier,
    ACTIONS(1852), 1,
      anon_sym_LPAREN,
    STATE(409), 1,
      sym__annotated_type,
    STATE(2288), 1,
      sym_parameter_types,
    STATE(2293), 1,
      sym_stable_identifier,
    STATE(496), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(507), 2,
      sym__type,
      sym_function_type,
    STATE(70), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39546] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(1832), 1,
      anon_sym_LPAREN,
    STATE(1367), 1,
      sym__annotated_type,
    STATE(2364), 1,
      sym_parameter_types,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(1521), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1871), 2,
      sym__type,
      sym_function_type,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39580] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(641), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(643), 8,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [39602] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(1828), 1,
      anon_sym_LPAREN,
    STATE(1314), 1,
      sym__annotated_type,
    STATE(2374), 1,
      sym_parameter_types,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(1505), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1601), 2,
      sym__type,
      sym_function_type,
    STATE(712), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39636] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1949), 1,
      anon_sym_LPAREN,
    STATE(981), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(645), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(647), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [39662] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(1828), 1,
      anon_sym_LPAREN,
    STATE(1314), 1,
      sym__annotated_type,
    STATE(2374), 1,
      sym_parameter_types,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(1505), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1625), 2,
      sym__type,
      sym_function_type,
    STATE(712), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39696] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(1828), 1,
      anon_sym_LPAREN,
    STATE(1314), 1,
      sym__annotated_type,
    STATE(2374), 1,
      sym_parameter_types,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(1505), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1639), 2,
      sym__type,
      sym_function_type,
    STATE(712), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39730] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(1832), 1,
      anon_sym_LPAREN,
    STATE(1367), 1,
      sym__annotated_type,
    STATE(2364), 1,
      sym_parameter_types,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(1521), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1884), 2,
      sym__type,
      sym_function_type,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39764] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(656), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(658), 8,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [39786] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(1824), 1,
      anon_sym_LPAREN,
    STATE(465), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(2295), 1,
      sym_parameter_types,
    STATE(512), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(627), 2,
      sym__type,
      sym_function_type,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39820] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(1828), 1,
      anon_sym_LPAREN,
    STATE(1314), 1,
      sym__annotated_type,
    STATE(2374), 1,
      sym_parameter_types,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(1505), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1641), 2,
      sym__type,
      sym_function_type,
    STATE(712), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39854] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(1860), 1,
      anon_sym_LPAREN,
    STATE(491), 1,
      sym__annotated_type,
    STATE(2306), 1,
      sym_parameter_types,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(508), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(539), 2,
      sym__type,
      sym_function_type,
    STATE(212), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [39888] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(751), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(749), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [39925] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(543), 8,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [39948] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1796), 1,
      anon_sym_DOT,
    ACTIONS(1798), 1,
      anon_sym_LBRACK,
    ACTIONS(1802), 1,
      anon_sym_LPAREN,
    STATE(823), 1,
      sym_arguments,
    STATE(1330), 1,
      sym_type_arguments,
    ACTIONS(279), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [39977] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(860), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(858), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [40014] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(868), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(866), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [40051] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(872), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(870), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [40088] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1796), 1,
      anon_sym_DOT,
    ACTIONS(1798), 1,
      anon_sym_LBRACK,
    ACTIONS(1802), 1,
      anon_sym_LPAREN,
    STATE(823), 1,
      sym_arguments,
    STATE(1330), 1,
      sym_type_arguments,
    ACTIONS(267), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [40117] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 6,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(559), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [40138] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(543), 8,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [40161] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(876), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(874), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [40198] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(808), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(806), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [40235] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1876), 1,
      anon_sym_LBRACK,
    ACTIONS(1880), 1,
      anon_sym_POUND,
    ACTIONS(1952), 1,
      anon_sym_AT,
    STATE(1255), 1,
      sym_type_arguments,
    ACTIONS(355), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1422), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(357), 5,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [40266] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(816), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(814), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [40303] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(800), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(798), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [40340] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1954), 1,
      anon_sym_DOT,
    ACTIONS(509), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(511), 8,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [40363] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(812), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(810), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [40400] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(721), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(719), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [40437] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(641), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(643), 9,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [40458] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(631), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(633), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [40479] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1940), 1,
      anon_sym_LBRACK,
    ACTIONS(1944), 1,
      anon_sym_POUND,
    ACTIONS(1956), 1,
      anon_sym_AT,
    STATE(1253), 1,
      sym_type_arguments,
    ACTIONS(355), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1443), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(357), 5,
      anon_sym_EQ_GT,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [40510] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(421), 1,
      anon_sym_LBRACE,
    STATE(1400), 2,
      sym_block,
      sym_case_block,
    ACTIONS(231), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(233), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [40535] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1958), 1,
      anon_sym_LPAREN,
    ACTIONS(645), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1010), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(647), 8,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [40560] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(656), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(658), 9,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [40581] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(664), 9,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [40602] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1622), 1,
      anon_sym_LPAREN,
    ACTIONS(615), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1010), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(617), 8,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [40627] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1961), 1,
      anon_sym_AT,
    STATE(1014), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(700), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(702), 6,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [40652] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(674), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(676), 9,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [40673] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(583), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(585), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [40694] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(828), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(826), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [40731] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(627), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(629), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [40752] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1966), 1,
      anon_sym_LBRACK,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1333), 1,
      sym_type_parameters,
    STATE(1689), 1,
      sym_extends_clause,
    STATE(1841), 1,
      sym_template_body,
    STATE(1332), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(768), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [40787] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(836), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(834), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [40824] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(852), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(850), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [40861] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_SEMI,
    ACTIONS(664), 8,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [40882] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1932), 1,
      anon_sym_LBRACK,
    ACTIONS(1936), 1,
      anon_sym_POUND,
    ACTIONS(1972), 1,
      anon_sym_AT,
    STATE(1266), 1,
      sym_type_arguments,
    STATE(1409), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(355), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
    ACTIONS(357), 4,
      anon_sym_EQ_GT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [40913] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(641), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(643), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [40934] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(864), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(862), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [40971] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 6,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(555), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [40992] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1580), 1,
      anon_sym_AT,
    STATE(1014), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(782), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(784), 6,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [41017] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(880), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(878), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [41054] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(277), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(279), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [41085] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(265), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(267), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [41116] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1974), 1,
      anon_sym_LPAREN,
    ACTIONS(645), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1031), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(647), 8,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [41141] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(656), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(658), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [41162] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1966), 1,
      anon_sym_LBRACK,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1257), 1,
      sym_type_parameters,
    STATE(1642), 1,
      sym_extends_clause,
    STATE(1860), 1,
      sym_template_body,
    STATE(1265), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(892), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [41197] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(581), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [41218] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(664), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [41239] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(623), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(625), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [41260] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(577), 9,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [41281] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1576), 1,
      anon_sym_LPAREN,
    ACTIONS(615), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1031), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(617), 8,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [41306] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(674), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(676), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [41327] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1966), 1,
      anon_sym_LBRACK,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(1979), 1,
      anon_sym_COLON,
    ACTIONS(1981), 1,
      anon_sym_EQ,
    ACTIONS(1983), 1,
      anon_sym_LPAREN,
    STATE(1298), 1,
      sym_type_parameters,
    STATE(1804), 1,
      sym_block,
    STATE(1297), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(882), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [41362] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1224), 1,
      anon_sym_LBRACK,
    ACTIONS(1226), 1,
      anon_sym_POUND,
    ACTIONS(1985), 1,
      anon_sym_LPAREN,
    STATE(578), 1,
      sym_type_arguments,
    STATE(1414), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(215), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_AT,
    ACTIONS(217), 4,
      anon_sym_val,
      anon_sym_var,
      sym_identifier,
      sym_wildcard,
  [41393] = 9,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1796), 1,
      anon_sym_DOT,
    ACTIONS(1798), 1,
      anon_sym_LBRACK,
    ACTIONS(1800), 1,
      anon_sym_EQ,
    ACTIONS(1802), 1,
      anon_sym_LPAREN,
    STATE(823), 1,
      sym_arguments,
    STATE(1330), 1,
      sym_type_arguments,
    ACTIONS(1794), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(271), 5,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
  [41426] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(581), 9,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [41447] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(269), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(271), 3,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
  [41482] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1796), 1,
      anon_sym_DOT,
    ACTIONS(1798), 1,
      anon_sym_LBRACK,
    ACTIONS(1802), 1,
      anon_sym_LPAREN,
    STATE(823), 1,
      sym_arguments,
    STATE(1330), 1,
      sym_type_arguments,
    ACTIONS(275), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [41511] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(678), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(680), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [41532] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(666), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(668), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [41553] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    ACTIONS(1902), 1,
      anon_sym_RBRACE,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    STATE(1815), 1,
      aux_sym__block_repeat1,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(1987), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [41592] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 6,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(551), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [41613] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1966), 1,
      anon_sym_LBRACK,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(1983), 1,
      anon_sym_LPAREN,
    ACTIONS(1989), 1,
      anon_sym_COLON,
    ACTIONS(1991), 1,
      anon_sym_EQ,
    STATE(1291), 1,
      sym_type_parameters,
    STATE(1811), 1,
      sym_block,
    STATE(1299), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(792), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [41648] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(577), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [41669] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1796), 1,
      anon_sym_DOT,
    ACTIONS(1798), 1,
      anon_sym_LBRACK,
    ACTIONS(1800), 1,
      anon_sym_EQ,
    ACTIONS(1802), 1,
      anon_sym_LPAREN,
    ACTIONS(1804), 1,
      anon_sym_match,
    STATE(823), 1,
      sym_arguments,
    STATE(1330), 1,
      sym_type_arguments,
    ACTIONS(1794), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(283), 4,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_catch,
      anon_sym_finally,
  [41704] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(641), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(643), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [41725] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(856), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(854), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [41762] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(273), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(275), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [41793] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(537), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(539), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [41814] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1796), 1,
      anon_sym_DOT,
    ACTIONS(1798), 1,
      anon_sym_LBRACK,
    ACTIONS(1800), 1,
      anon_sym_EQ,
    ACTIONS(1802), 1,
      anon_sym_LPAREN,
    ACTIONS(1804), 1,
      anon_sym_match,
    STATE(823), 1,
      sym_arguments,
    STATE(1330), 1,
      sym_type_arguments,
    ACTIONS(1794), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(287), 4,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_catch,
      anon_sym_finally,
  [41849] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    ACTIONS(1995), 1,
      anon_sym_case,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(1993), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [41886] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(652), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(654), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [41907] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1997), 1,
      anon_sym_LPAREN,
    STATE(1060), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(645), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(647), 6,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [41932] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(637), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(639), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [41953] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(605), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(607), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [41974] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(189), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(191), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [41995] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1916), 1,
      anon_sym_LBRACK,
    ACTIONS(1920), 1,
      anon_sym_POUND,
    ACTIONS(2000), 1,
      anon_sym_AT,
    STATE(1279), 1,
      sym_type_arguments,
    ACTIONS(355), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1456), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(357), 5,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [42026] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(656), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(658), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [42047] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(844), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(842), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [42084] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(840), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(838), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [42121] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(820), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(818), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [42158] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(824), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(822), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [42195] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(581), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [42216] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(285), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(287), 2,
      anon_sym_catch,
      anon_sym_finally,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [42253] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(804), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(802), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [42290] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1646), 1,
      anon_sym_finally,
    STATE(1217), 1,
      sym_finally_clause,
    ACTIONS(537), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(539), 6,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      sym_identifier,
      sym_operator_identifier,
  [42315] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1966), 1,
      anon_sym_LBRACK,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1315), 1,
      sym_type_parameters,
    STATE(1648), 1,
      sym_extends_clause,
    STATE(1828), 1,
      sym_template_body,
    STATE(1320), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(786), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [42350] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(790), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(788), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [42387] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(577), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [42408] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(551), 9,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [42429] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(772), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(770), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [42466] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(555), 9,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [42487] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(559), 9,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [42508] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(664), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [42529] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(743), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(741), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [42566] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1748), 1,
      anon_sym_LPAREN,
    STATE(1060), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(615), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(617), 6,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [42591] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(717), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(715), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [42628] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(713), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(711), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [42665] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(674), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(676), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [42686] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(587), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(589), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [42707] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(709), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(707), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [42744] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1966), 1,
      anon_sym_LBRACK,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(1983), 1,
      anon_sym_LPAREN,
    ACTIONS(2002), 1,
      anon_sym_COLON,
    ACTIONS(2004), 1,
      anon_sym_EQ,
    STATE(1274), 1,
      sym_type_parameters,
    STATE(1913), 1,
      sym_block,
    STATE(1276), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(753), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [42779] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2006), 1,
      anon_sym_AT,
    STATE(1090), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(700), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(702), 5,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [42804] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(890), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(888), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [42841] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(832), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(830), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [42878] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(281), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(283), 2,
      anon_sym_catch,
      anon_sym_finally,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [42915] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(780), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(778), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [42952] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1966), 1,
      anon_sym_LBRACK,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1239), 1,
      sym_type_parameters,
    STATE(1682), 1,
      sym_extends_clause,
    STATE(1910), 1,
      sym_template_body,
    STATE(1256), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(731), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [42987] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(577), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [43008] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(725), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(723), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [43045] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1796), 1,
      anon_sym_DOT,
    ACTIONS(1798), 1,
      anon_sym_LBRACK,
    ACTIONS(1800), 1,
      anon_sym_EQ,
    ACTIONS(1802), 1,
      anon_sym_LPAREN,
    ACTIONS(1804), 1,
      anon_sym_match,
    ACTIONS(2009), 1,
      anon_sym_else,
    STATE(823), 1,
      sym_arguments,
    STATE(1330), 1,
      sym_type_arguments,
    ACTIONS(1794), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(305), 3,
      anon_sym_EQ_GT,
      anon_sym_catch,
      anon_sym_finally,
  [43082] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(729), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(727), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [43119] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(581), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [43140] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(609), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(611), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [43161] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(619), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(621), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [43182] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(601), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(603), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [43203] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(747), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(745), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [43240] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(595), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(597), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [43261] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(664), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [43282] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(688), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(684), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [43319] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(591), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(593), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [43340] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1796), 1,
      anon_sym_DOT,
    ACTIONS(1798), 1,
      anon_sym_LBRACK,
    ACTIONS(1800), 1,
      anon_sym_EQ,
    ACTIONS(1802), 1,
      anon_sym_LPAREN,
    ACTIONS(1804), 1,
      anon_sym_match,
    ACTIONS(2011), 1,
      anon_sym_else,
    STATE(823), 1,
      sym_arguments,
    STATE(1330), 1,
      sym_type_arguments,
    ACTIONS(1794), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(305), 3,
      anon_sym_EQ_GT,
      anon_sym_catch,
      anon_sym_finally,
  [43377] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2013), 1,
      anon_sym_DOT,
    ACTIONS(509), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(511), 8,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [43400] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(848), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(846), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [43437] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(670), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(672), 7,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [43458] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(776), 1,
      anon_sym_case,
    ACTIONS(1532), 1,
      anon_sym_DOT,
    ACTIONS(1534), 1,
      anon_sym_LBRACK,
    ACTIONS(1536), 1,
      anon_sym_EQ,
    ACTIONS(1538), 1,
      anon_sym_LPAREN,
    ACTIONS(1540), 1,
      anon_sym_match,
    STATE(743), 1,
      sym_arguments,
    STATE(1102), 1,
      sym_type_arguments,
    ACTIONS(1530), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(774), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [43495] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1924), 1,
      anon_sym_LBRACK,
    ACTIONS(1928), 1,
      anon_sym_POUND,
    ACTIONS(2015), 1,
      anon_sym_AT,
    STATE(1275), 1,
      sym_type_arguments,
    STATE(1402), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(357), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [43524] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1610), 1,
      anon_sym_AT,
    STATE(1090), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(782), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(784), 5,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [43549] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2019), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(1947), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [43587] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(583), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(585), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [43607] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2021), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2127), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [43645] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2023), 1,
      anon_sym_LPAREN,
    ACTIONS(645), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1119), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(647), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [43669] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2026), 1,
      anon_sym_LPAREN,
    ACTIONS(645), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1120), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(647), 7,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [43693] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(656), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(658), 8,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [43713] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2029), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2016), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [43751] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(664), 8,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [43771] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1790), 1,
      anon_sym_LPAREN,
    ACTIONS(615), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1120), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(617), 7,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [43795] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(656), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(658), 8,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [43815] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(674), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(676), 8,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [43835] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2031), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2027), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [43873] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(664), 8,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [43893] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2033), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2119), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [43931] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1884), 1,
      anon_sym_DOT,
    ACTIONS(1886), 1,
      anon_sym_LBRACK,
    ACTIONS(1888), 1,
      anon_sym_EQ,
    ACTIONS(1890), 1,
      anon_sym_LPAREN,
    ACTIONS(1892), 1,
      anon_sym_match,
    STATE(1009), 1,
      sym_arguments,
    STATE(1423), 1,
      sym_type_arguments,
    ACTIONS(1882), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(287), 3,
      anon_sym_EQ_GT,
      anon_sym_catch,
      anon_sym_finally,
  [43965] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(581), 8,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [43985] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1884), 1,
      anon_sym_DOT,
    ACTIONS(1886), 1,
      anon_sym_LBRACK,
    ACTIONS(1888), 1,
      anon_sym_EQ,
    ACTIONS(1890), 1,
      anon_sym_LPAREN,
    ACTIONS(1892), 1,
      anon_sym_match,
    STATE(1009), 1,
      sym_arguments,
    STATE(1423), 1,
      sym_type_arguments,
    ACTIONS(1882), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(283), 3,
      anon_sym_EQ_GT,
      anon_sym_catch,
      anon_sym_finally,
  [44019] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1812), 1,
      anon_sym_LPAREN,
    ACTIONS(615), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1119), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(617), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [44043] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(674), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(676), 8,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [44063] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2035), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(1963), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [44101] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1884), 1,
      anon_sym_DOT,
    ACTIONS(1886), 1,
      anon_sym_LBRACK,
    ACTIONS(1890), 1,
      anon_sym_LPAREN,
    STATE(1009), 1,
      sym_arguments,
    STATE(1423), 1,
      sym_type_arguments,
    ACTIONS(275), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [44129] = 9,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1884), 1,
      anon_sym_DOT,
    ACTIONS(1886), 1,
      anon_sym_LBRACK,
    ACTIONS(1888), 1,
      anon_sym_EQ,
    ACTIONS(1890), 1,
      anon_sym_LPAREN,
    STATE(1009), 1,
      sym_arguments,
    STATE(1423), 1,
      sym_type_arguments,
    ACTIONS(1882), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(271), 4,
      anon_sym_EQ_GT,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
  [44161] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2037), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2108), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [44199] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2039), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2002), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [44237] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(623), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(625), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [44257] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(577), 8,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [44277] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2041), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2036), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [44315] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1884), 1,
      anon_sym_DOT,
    ACTIONS(1886), 1,
      anon_sym_LBRACK,
    ACTIONS(1890), 1,
      anon_sym_LPAREN,
    STATE(1009), 1,
      sym_arguments,
    STATE(1423), 1,
      sym_type_arguments,
    ACTIONS(279), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [44343] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1884), 1,
      anon_sym_DOT,
    ACTIONS(1886), 1,
      anon_sym_LBRACK,
    ACTIONS(1890), 1,
      anon_sym_LPAREN,
    STATE(1009), 1,
      sym_arguments,
    STATE(1423), 1,
      sym_type_arguments,
    ACTIONS(267), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [44371] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(627), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(629), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [44391] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 5,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(543), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [44413] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2043), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2046), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [44451] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 6,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(559), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [44471] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2045), 1,
      anon_sym_AT,
    ACTIONS(700), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1149), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(702), 7,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [44495] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(641), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(643), 8,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [44515] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(631), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(633), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [44535] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(577), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [44555] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2048), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(1981), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [44593] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(577), 8,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [44613] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(581), 8,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [44633] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2050), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(1971), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [44671] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2052), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(1970), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [44709] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2054), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2006), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [44747] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2056), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2066), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [44785] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2058), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2089), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [44823] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2060), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2142), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [44861] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(641), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(643), 8,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [44881] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
    ACTIONS(664), 9,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [44901] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2062), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2084), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [44939] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2064), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(1929), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [44977] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2066), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(1926), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [45015] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2068), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2094), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [45053] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(581), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [45073] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2070), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2111), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [45111] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 6,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(555), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [45131] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2072), 1,
      anon_sym_DOT,
    ACTIONS(509), 5,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(511), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [45153] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1816), 1,
      anon_sym_AT,
    ACTIONS(782), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1149), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(784), 7,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [45177] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(543), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [45199] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1808), 1,
      anon_sym_finally,
    STATE(1273), 1,
      sym_finally_clause,
    ACTIONS(537), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(539), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      sym_identifier,
      sym_operator_identifier,
  [45223] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2074), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2131), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [45261] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(601), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(603), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [45281] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2076), 1,
      anon_sym_AT,
    ACTIONS(700), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1177), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(702), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_with,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [45305] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1786), 1,
      anon_sym_finally,
    STATE(1317), 1,
      sym_finally_clause,
    ACTIONS(537), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(539), 5,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      sym_identifier,
      sym_operator_identifier,
  [45329] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2079), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2176), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [45367] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2083), 1,
      anon_sym_with,
    STATE(1218), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(2081), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(897), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(901), 4,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
  [45395] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2085), 1,
      anon_sym_DOT,
    ACTIONS(509), 2,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(511), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [45417] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2087), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(1946), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [45455] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2089), 1,
      anon_sym_with,
    STATE(1183), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(921), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(923), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [45479] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2092), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2171), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [45517] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2083), 1,
      anon_sym_with,
    STATE(1218), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(914), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(916), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [45541] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(666), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(668), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [45561] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 6,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(551), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [45581] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2094), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2166), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [45619] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2096), 1,
      anon_sym_DOT,
    ACTIONS(509), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(511), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [45641] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2098), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2069), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [45679] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(189), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(191), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [45699] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(664), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [45719] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2100), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2068), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [45757] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2102), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2078), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [45795] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 2,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(543), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [45817] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2104), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2161), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [45855] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2106), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(1977), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [45893] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2108), 1,
      anon_sym_DOT,
    ACTIONS(509), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(511), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [45915] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2110), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(1978), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [45953] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(587), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(589), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [45973] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1864), 1,
      anon_sym_LT_COLON,
    ACTIONS(1866), 1,
      anon_sym_GT_COLON,
    ACTIONS(1868), 1,
      anon_sym_LT_PERCENT,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    STATE(1386), 1,
      sym_upper_bound,
    STATE(1587), 1,
      sym_lower_bound,
    ACTIONS(2112), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1590), 2,
      sym_view_bound,
      aux_sym__type_parameter_repeat1,
    STATE(1764), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [46007] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1756), 1,
      anon_sym_AT,
    ACTIONS(782), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1177), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(784), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_with,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [46031] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2114), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2008), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [46069] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(670), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(672), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46089] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2116), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2152), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [46127] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 4,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(551), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46147] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(619), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(621), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46167] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2118), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2009), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [46205] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(543), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [46227] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 4,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(555), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46247] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2120), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2040), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [46285] = 12,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2122), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    STATE(2039), 1,
      aux_sym_tuple_expression_repeat1,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [46323] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(559), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [46343] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 4,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(559), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46363] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2124), 1,
      anon_sym_DOT,
    ACTIONS(509), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(511), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [46385] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(591), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(593), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46405] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(595), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(597), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46425] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2083), 1,
      anon_sym_with,
    STATE(1183), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(910), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(912), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [46449] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(551), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [46469] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(678), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(680), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46489] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(609), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(611), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46509] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(605), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(607), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46529] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(545), 1,
      anon_sym_DOT,
    ACTIONS(541), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(543), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [46551] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(537), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(539), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46571] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(652), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(654), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46591] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(555), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [46611] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(637), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(639), 7,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46631] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(2126), 1,
      anon_sym_LPAREN,
    STATE(469), 1,
      sym__annotated_type,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(513), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(212), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [46658] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(641), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(643), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [46677] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(577), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46696] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(2128), 1,
      anon_sym_LPAREN,
    STATE(1585), 1,
      sym__annotated_type,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1651), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [46723] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(1983), 1,
      anon_sym_LPAREN,
    ACTIONS(2130), 1,
      anon_sym_COLON,
    ACTIONS(2132), 1,
      anon_sym_EQ,
    STATE(1845), 1,
      sym_block,
    STATE(1433), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(967), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [46752] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(577), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46771] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(664), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [46790] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(666), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(668), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46809] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(2134), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [46842] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(581), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [46861] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1912), 1,
      sym_identifier,
    ACTIONS(2136), 1,
      anon_sym_LPAREN,
    STATE(1499), 1,
      sym__annotated_type,
    STATE(2338), 1,
      sym_stable_identifier,
    STATE(1626), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(798), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [46888] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1644), 1,
      sym_extends_clause,
    STATE(1799), 1,
      sym_template_body,
    STATE(1326), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(945), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [46917] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(641), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(643), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [46936] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(641), 2,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(643), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [46955] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2138), 1,
      anon_sym_LPAREN,
    ACTIONS(645), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1242), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(647), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [46978] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(656), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(658), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [46997] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1942), 1,
      anon_sym_LPAREN,
    ACTIONS(615), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1246), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(617), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [47020] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(656), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(658), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [47039] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2141), 1,
      anon_sym_LPAREN,
    ACTIONS(645), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1246), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(647), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [47062] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(664), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [47081] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(2144), 1,
      anon_sym_LPAREN,
    STATE(452), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(505), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [47108] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(627), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(629), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [47127] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(577), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [47146] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1878), 1,
      anon_sym_LPAREN,
    ACTIONS(615), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1242), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(617), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [47169] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(2146), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [47202] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(674), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(676), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [47221] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1898), 1,
      sym_identifier,
    ACTIONS(2148), 1,
      anon_sym_LPAREN,
    STATE(1442), 1,
      sym__annotated_type,
    STATE(2329), 1,
      sym_stable_identifier,
    STATE(1572), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(780), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [47248] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(674), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(676), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [47267] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1711), 1,
      sym_extends_clause,
    STATE(1807), 1,
      sym_template_body,
    STATE(1487), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(943), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [47296] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1665), 1,
      sym_extends_clause,
    STATE(1903), 1,
      sym_template_body,
    STATE(1379), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(990), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [47325] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2152), 1,
      anon_sym_PLUS,
    ACTIONS(2154), 1,
      anon_sym_DASH,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2150), 2,
      sym_identifier,
      sym_wildcard,
    STATE(1446), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    STATE(1928), 4,
      sym__variant_type_parameter,
      sym_covariant_type_parameter,
      sym_contravariant_type_parameter,
      sym__type_parameter,
  [47352] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(583), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(585), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [47371] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 5,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(577), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [47390] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(581), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [47409] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2158), 1,
      anon_sym_LPAREN,
    STATE(1262), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(647), 8,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [47430] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(656), 2,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(658), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [47449] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1908), 1,
      sym_identifier,
    ACTIONS(2161), 1,
      anon_sym_LPAREN,
    STATE(1574), 1,
      sym__annotated_type,
    STATE(2346), 1,
      sym_stable_identifier,
    STATE(1661), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1000), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [47476] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1669), 1,
      sym_extends_clause,
    STATE(1898), 1,
      sym_template_body,
    STATE(1487), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(928), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [47505] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(674), 5,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(676), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [47524] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 2,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(664), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [47543] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1854), 1,
      sym_identifier,
    ACTIONS(2163), 1,
      anon_sym_LPAREN,
    STATE(1465), 1,
      sym__annotated_type,
    STATE(2320), 1,
      sym_stable_identifier,
    STATE(1584), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(750), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [47570] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(605), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(607), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [47589] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1649), 1,
      sym_extends_clause,
    STATE(1834), 1,
      sym_template_body,
    STATE(1487), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(1006), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [47618] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1934), 1,
      anon_sym_LPAREN,
    STATE(1358), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(615), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
    ACTIONS(617), 5,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [47641] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(591), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(593), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [47660] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(595), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(597), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [47679] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(1983), 1,
      anon_sym_LPAREN,
    ACTIONS(2165), 1,
      anon_sym_COLON,
    ACTIONS(2167), 1,
      anon_sym_EQ,
    STATE(1848), 1,
      sym_block,
    STATE(1232), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(1014), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [47708] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(674), 2,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(676), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [47727] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(1983), 1,
      anon_sym_LPAREN,
    ACTIONS(2169), 1,
      anon_sym_COLON,
    ACTIONS(2171), 1,
      anon_sym_EQ,
    STATE(1832), 1,
      sym_block,
    STATE(1433), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(1008), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [47756] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(609), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(611), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [47775] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(2173), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [47808] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(674), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(676), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [47827] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(933), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(935), 8,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [47846] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(2175), 1,
      anon_sym_LPAREN,
    STATE(1334), 1,
      sym__annotated_type,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(1492), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(712), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [47873] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(605), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(607), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [47892] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1918), 1,
      anon_sym_LPAREN,
    ACTIONS(615), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1336), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(617), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [47915] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(189), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(191), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [47934] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(637), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(639), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [47953] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(637), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(639), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [47972] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 3,
      anon_sym_COMMA,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(551), 8,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [47991] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(664), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [48010] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(652), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(654), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48029] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(555), 6,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [48048] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(1983), 1,
      anon_sym_LPAREN,
    ACTIONS(2177), 1,
      anon_sym_COLON,
    ACTIONS(2179), 1,
      anon_sym_EQ,
    STATE(1858), 1,
      sym_block,
    STATE(1316), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(961), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [48077] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 4,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(559), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48096] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(537), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(539), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48115] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(641), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(643), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [48134] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(666), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(668), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48153] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(678), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(680), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48172] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(1983), 1,
      anon_sym_LPAREN,
    ACTIONS(2181), 1,
      anon_sym_COLON,
    ACTIONS(2183), 1,
      anon_sym_EQ,
    STATE(1879), 1,
      sym_block,
    STATE(1433), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(947), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [48201] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(1983), 1,
      anon_sym_LPAREN,
    ACTIONS(2185), 1,
      anon_sym_COLON,
    ACTIONS(2187), 1,
      anon_sym_EQ,
    STATE(1877), 1,
      sym_block,
    STATE(1342), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(955), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [48230] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(1983), 1,
      anon_sym_LPAREN,
    ACTIONS(2189), 1,
      anon_sym_COLON,
    ACTIONS(2191), 1,
      anon_sym_EQ,
    STATE(1800), 1,
      sym_block,
    STATE(1433), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(973), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [48259] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1922), 1,
      anon_sym_AT,
    ACTIONS(782), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1366), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(784), 6,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [48282] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(652), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(654), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48301] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(559), 6,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [48320] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(537), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(539), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48339] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 4,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(551), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48358] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(583), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(585), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48377] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1926), 1,
      anon_sym_LPAREN,
    STATE(1262), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(617), 8,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [48398] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(581), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48417] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(551), 6,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [48436] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(678), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(680), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48455] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(577), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [48474] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(587), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(589), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48493] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1818), 1,
      sym_identifier,
    ACTIONS(2193), 1,
      anon_sym_LPAREN,
    STATE(1593), 1,
      sym__annotated_type,
    STATE(2353), 1,
      sym_stable_identifier,
    STATE(1694), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1064), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [48520] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(601), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(603), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48539] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2197), 1,
      anon_sym_with,
    STATE(1319), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(901), 2,
      anon_sym_case,
      anon_sym_EQ,
    ACTIONS(2195), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(897), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [48566] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1704), 1,
      sym_extends_clause,
    STATE(1867), 1,
      sym_template_body,
    STATE(1345), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(981), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [48595] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(1983), 1,
      anon_sym_LPAREN,
    ACTIONS(2199), 1,
      anon_sym_COLON,
    ACTIONS(2201), 1,
      anon_sym_EQ,
    STATE(1798), 1,
      sym_block,
    STATE(1433), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(992), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [48624] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(595), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(597), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48643] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(609), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(611), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48662] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2197), 1,
      anon_sym_with,
    STATE(1337), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(910), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(912), 5,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_EQ,
      sym_identifier,
      sym_operator_identifier,
  [48685] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1656), 1,
      sym_extends_clause,
    STATE(1810), 1,
      sym_template_body,
    STATE(1487), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(1022), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [48714] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(591), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(593), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48733] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(623), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(625), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48752] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2152), 1,
      anon_sym_PLUS,
    ACTIONS(2154), 1,
      anon_sym_DASH,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2150), 2,
      sym_identifier,
      sym_wildcard,
    STATE(1446), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    STATE(2053), 4,
      sym__variant_type_parameter,
      sym_covariant_type_parameter,
      sym_contravariant_type_parameter,
      sym__type_parameter,
  [48779] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(2203), 1,
      anon_sym_LPAREN,
    STATE(1549), 1,
      sym__annotated_type,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1678), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [48806] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2205), 1,
      anon_sym_with,
    STATE(1325), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(923), 4,
      anon_sym_EQ_GT,
      anon_sym_case,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(921), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [48829] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1707), 1,
      sym_extends_clause,
    STATE(1916), 1,
      sym_template_body,
    STATE(1487), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(979), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [48858] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2152), 1,
      anon_sym_PLUS,
    ACTIONS(2154), 1,
      anon_sym_DASH,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2150), 2,
      sym_identifier,
      sym_wildcard,
    STATE(1446), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    STATE(2223), 4,
      sym__variant_type_parameter,
      sym_covariant_type_parameter,
      sym_contravariant_type_parameter,
      sym__type_parameter,
  [48885] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(623), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(625), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48904] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 4,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(555), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48923] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(619), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(621), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [48942] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(656), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(658), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [48961] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1667), 1,
      sym_extends_clause,
    STATE(1831), 1,
      sym_template_body,
    STATE(1487), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(1002), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [48990] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1668), 1,
      sym_extends_clause,
    STATE(1825), 1,
      sym_template_body,
    STATE(1270), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(1004), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [49019] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2197), 1,
      anon_sym_with,
    STATE(1319), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(914), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(916), 5,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_EQ,
      sym_identifier,
      sym_operator_identifier,
  [49042] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(581), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [49061] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2208), 1,
      anon_sym_LPAREN,
    ACTIONS(645), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1336), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(647), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [49084] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2211), 1,
      anon_sym_with,
    STATE(1337), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(921), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(923), 5,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_EQ,
      sym_identifier,
      sym_operator_identifier,
  [49107] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1850), 1,
      sym_identifier,
    ACTIONS(2214), 1,
      anon_sym_LPAREN,
    STATE(434), 1,
      sym__annotated_type,
    STATE(2293), 1,
      sym_stable_identifier,
    STATE(497), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(70), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [49134] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(664), 8,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [49153] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(601), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(603), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [49172] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(2216), 1,
      anon_sym_LPAREN,
    STATE(1536), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(1720), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [49199] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(1983), 1,
      anon_sym_LPAREN,
    ACTIONS(2218), 1,
      anon_sym_COLON,
    ACTIONS(2220), 1,
      anon_sym_EQ,
    STATE(1874), 1,
      sym_block,
    STATE(1433), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(937), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [49228] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2222), 1,
      anon_sym_with,
    STATE(1375), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(916), 4,
      anon_sym_EQ_GT,
      anon_sym_case,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(914), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [49251] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(2224), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [49284] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1683), 1,
      sym_extends_clause,
    STATE(1816), 1,
      sym_template_body,
    STATE(1487), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(998), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [49313] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(2226), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [49346] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(2228), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [49379] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 5,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(581), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [49398] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 5,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(664), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [49417] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1838), 1,
      anon_sym_AT,
    ACTIONS(782), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1373), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(784), 6,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [49440] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(619), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(621), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [49459] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1840), 1,
      sym_identifier,
    ACTIONS(2230), 1,
      anon_sym_LPAREN,
    STATE(1185), 1,
      sym__annotated_type,
    STATE(2372), 1,
      sym_stable_identifier,
    STATE(1418), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(691), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [49486] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(656), 5,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(658), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [49505] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(670), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(672), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [49524] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(627), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(629), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [49543] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1499), 1,
      sym_identifier,
    ACTIONS(2232), 1,
      anon_sym_LPAREN,
    STATE(1514), 1,
      sym__annotated_type,
    STATE(2360), 1,
      sym_stable_identifier,
    STATE(1624), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(896), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [49570] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(641), 5,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(643), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [49589] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2234), 1,
      anon_sym_LPAREN,
    STATE(1358), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(645), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
    ACTIONS(647), 5,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [49612] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
    ACTIONS(664), 8,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [49631] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(2237), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [49664] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 2,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(581), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [49683] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(2239), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [49716] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(670), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(672), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [49735] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(631), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(633), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [49754] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1896), 1,
      anon_sym_finally,
    STATE(1406), 1,
      sym_finally_clause,
    ACTIONS(537), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(539), 6,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      sym_identifier,
      sym_operator_identifier,
  [49777] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2241), 1,
      anon_sym_AT,
    ACTIONS(700), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1366), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(702), 6,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [49800] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(901), 1,
      anon_sym_case,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2222), 1,
      anon_sym_with,
    STATE(1375), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(2244), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(897), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [49827] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 2,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(577), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [49846] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(631), 5,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(633), 6,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [49865] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(2246), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [49898] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 3,
      anon_sym_COMMA,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(559), 8,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [49917] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(551), 8,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [49936] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2248), 1,
      anon_sym_AT,
    ACTIONS(700), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1373), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(702), 6,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [49959] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(2251), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [49992] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2222), 1,
      anon_sym_with,
    STATE(1325), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(912), 4,
      anon_sym_EQ_GT,
      anon_sym_case,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(910), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [50015] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(2253), 1,
      anon_sym_LPAREN,
    STATE(1343), 1,
      sym__annotated_type,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(1528), 2,
      sym_compound_type,
      sym_infix_type,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [50042] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(189), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(191), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [50061] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(2255), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [50094] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1968), 1,
      anon_sym_extends,
    ACTIONS(1970), 1,
      anon_sym_LPAREN,
    STATE(1686), 1,
      sym_extends_clause,
    STATE(1850), 1,
      sym_template_body,
    STATE(1487), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(1000), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [50123] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(581), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [50142] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(555), 8,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [50161] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(559), 8,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [50180] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(587), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(589), 8,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [50199] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 3,
      anon_sym_COMMA,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(555), 8,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [50218] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(577), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [50237] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1866), 1,
      anon_sym_GT_COLON,
    ACTIONS(1868), 1,
      anon_sym_LT_PERCENT,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    STATE(1539), 1,
      sym_lower_bound,
    ACTIONS(2257), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1540), 2,
      sym_view_bound,
      aux_sym__type_parameter_repeat1,
    STATE(1777), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [50265] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1866), 1,
      anon_sym_GT_COLON,
    ACTIONS(1868), 1,
      anon_sym_LT_PERCENT,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    STATE(1591), 1,
      sym_lower_bound,
    ACTIONS(2259), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1599), 2,
      sym_view_bound,
      aux_sym__type_parameter_repeat1,
    STATE(1730), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [50293] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2261), 1,
      anon_sym_AT,
    STATE(1388), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(700), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
    ACTIONS(702), 4,
      anon_sym_EQ_GT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [50315] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2270), 1,
      anon_sym_AT,
    ACTIONS(2272), 1,
      anon_sym_LPAREN,
    ACTIONS(2274), 1,
      anon_sym_RPAREN,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    STATE(1949), 1,
      aux_sym_case_class_pattern_repeat1,
  [50349] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1966), 1,
      anon_sym_LBRACK,
    ACTIONS(1968), 1,
      anon_sym_extends,
    STATE(1565), 1,
      sym_type_parameters,
    STATE(1643), 1,
      sym_extends_clause,
    STATE(1912), 1,
      sym_template_body,
    ACTIONS(1029), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [50377] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(678), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(680), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [50395] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(189), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(191), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [50413] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(583), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(585), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [50431] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2284), 1,
      anon_sym_COLON,
    ACTIONS(2286), 1,
      anon_sym_AT,
    ACTIONS(2288), 1,
      anon_sym_EQ,
    ACTIONS(2290), 1,
      anon_sym_LPAREN,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    STATE(1939), 1,
      aux_sym_val_declaration_repeat1,
  [50465] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(537), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(539), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [50483] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(652), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(654), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [50501] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(933), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
    ACTIONS(935), 5,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [50519] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(637), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(639), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [50537] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2286), 1,
      anon_sym_AT,
    ACTIONS(2290), 1,
      anon_sym_LPAREN,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2296), 1,
      anon_sym_COLON,
    ACTIONS(2298), 1,
      anon_sym_EQ,
    STATE(1944), 1,
      aux_sym_val_declaration_repeat1,
  [50571] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(605), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(607), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [50589] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1884), 1,
      anon_sym_DOT,
    ACTIONS(1886), 1,
      anon_sym_LBRACK,
    ACTIONS(1888), 1,
      anon_sym_EQ,
    ACTIONS(1890), 1,
      anon_sym_LPAREN,
    ACTIONS(1892), 1,
      anon_sym_match,
    ACTIONS(2300), 1,
      anon_sym_EQ_GT,
    STATE(1009), 1,
      sym_arguments,
    STATE(1423), 1,
      sym_type_arguments,
    ACTIONS(1882), 2,
      sym_identifier,
      sym_operator_identifier,
  [50621] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2015), 1,
      anon_sym_AT,
    STATE(1425), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(784), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [50641] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(609), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(611), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [50659] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2270), 1,
      anon_sym_AT,
    ACTIONS(2272), 1,
      anon_sym_LPAREN,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2302), 1,
      anon_sym_RPAREN,
    STATE(1922), 1,
      aux_sym_case_class_pattern_repeat1,
  [50693] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 1,
      anon_sym_LBRACK,
    ACTIONS(664), 9,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [50711] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(595), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(597), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [50729] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(591), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(593), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [50747] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2304), 1,
      anon_sym_LPAREN,
    STATE(1408), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(645), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_AT,
    ACTIONS(647), 4,
      anon_sym_val,
      anon_sym_var,
      sym_identifier,
      sym_wildcard,
  [50769] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1972), 1,
      anon_sym_AT,
    STATE(1388), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(782), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
    ACTIONS(784), 4,
      anon_sym_EQ_GT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [50791] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2270), 1,
      anon_sym_AT,
    ACTIONS(2272), 1,
      anon_sym_LPAREN,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2307), 1,
      anon_sym_RPAREN,
    STATE(2116), 1,
      aux_sym_case_class_pattern_repeat1,
  [50825] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(623), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(625), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [50843] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(587), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(589), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [50861] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2270), 1,
      anon_sym_AT,
    ACTIONS(2272), 1,
      anon_sym_LPAREN,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2309), 1,
      anon_sym_RPAREN,
    STATE(2112), 1,
      aux_sym_case_class_pattern_repeat1,
  [50895] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1985), 1,
      anon_sym_LPAREN,
    STATE(1408), 2,
      sym_arguments,
      aux_sym_annotation_repeat1,
    ACTIONS(615), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_AT,
    ACTIONS(617), 4,
      anon_sym_val,
      anon_sym_var,
      sym_identifier,
      sym_wildcard,
  [50917] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2270), 1,
      anon_sym_AT,
    ACTIONS(2272), 1,
      anon_sym_LPAREN,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2311), 1,
      anon_sym_RPAREN,
    STATE(1934), 1,
      aux_sym_case_class_pattern_repeat1,
  [50951] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2313), 1,
      anon_sym_AT,
    ACTIONS(700), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1416), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(702), 5,
      anon_sym_EQ_GT,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [50973] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(670), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(672), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [50991] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(914), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(916), 7,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [51009] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(627), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(629), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [51027] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
    ACTIONS(664), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [51045] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2286), 1,
      anon_sym_AT,
    ACTIONS(2290), 1,
      anon_sym_LPAREN,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2316), 1,
      anon_sym_COLON,
    ACTIONS(2318), 1,
      anon_sym_EQ,
    STATE(1941), 1,
      aux_sym_val_declaration_repeat1,
  [51079] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1952), 1,
      anon_sym_AT,
    ACTIONS(782), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1464), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(784), 5,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [51101] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(619), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(621), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [51119] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2286), 1,
      anon_sym_AT,
    ACTIONS(2290), 1,
      anon_sym_LPAREN,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2320), 1,
      anon_sym_COLON,
    ACTIONS(2322), 1,
      anon_sym_EQ,
    STATE(1936), 1,
      aux_sym_val_declaration_repeat1,
  [51153] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2324), 1,
      anon_sym_AT,
    STATE(1425), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(702), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [51173] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(664), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [51191] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2270), 1,
      anon_sym_AT,
    ACTIONS(2272), 1,
      anon_sym_LPAREN,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2327), 1,
      anon_sym_RPAREN,
    STATE(2005), 1,
      aux_sym_case_class_pattern_repeat1,
  [51225] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2286), 1,
      anon_sym_AT,
    ACTIONS(2290), 1,
      anon_sym_LPAREN,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2329), 1,
      anon_sym_COLON,
    ACTIONS(2331), 1,
      anon_sym_EQ,
    STATE(1921), 1,
      aux_sym_val_declaration_repeat1,
  [51259] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(933), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(935), 6,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [51277] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2286), 1,
      anon_sym_AT,
    ACTIONS(2290), 1,
      anon_sym_LPAREN,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2333), 1,
      anon_sym_COLON,
    ACTIONS(2335), 1,
      anon_sym_EQ,
    STATE(1932), 1,
      aux_sym_val_declaration_repeat1,
  [51311] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2337), 1,
      anon_sym_AT,
    ACTIONS(700), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1431), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(702), 5,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [51333] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2342), 1,
      anon_sym_with,
    STATE(1449), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(897), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(2340), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(901), 3,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
  [51359] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2344), 1,
      anon_sym_LPAREN,
    STATE(1433), 2,
      sym_parameters,
      aux_sym_function_definition_repeat1,
    ACTIONS(1024), 7,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_SEMI,
  [51379] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(631), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(633), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [51397] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2347), 1,
      anon_sym_with,
    STATE(1435), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(921), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(923), 6,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      sym_identifier,
      sym_operator_identifier,
  [51419] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
    ACTIONS(664), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [51437] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2286), 1,
      anon_sym_AT,
    ACTIONS(2290), 1,
      anon_sym_LPAREN,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2350), 1,
      anon_sym_COLON,
    ACTIONS(2352), 1,
      anon_sym_EQ,
    STATE(2093), 1,
      aux_sym_val_declaration_repeat1,
  [51471] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(575), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(577), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [51489] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(579), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(581), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [51507] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2286), 1,
      anon_sym_AT,
    ACTIONS(2290), 1,
      anon_sym_LPAREN,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2354), 1,
      anon_sym_COLON,
    ACTIONS(2356), 1,
      anon_sym_EQ,
    STATE(2081), 1,
      aux_sym_val_declaration_repeat1,
  [51541] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2286), 1,
      anon_sym_AT,
    ACTIONS(2290), 1,
      anon_sym_LPAREN,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2358), 1,
      anon_sym_COLON,
    ACTIONS(2360), 1,
      anon_sym_EQ,
    STATE(2064), 1,
      aux_sym_val_declaration_repeat1,
  [51575] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2342), 1,
      anon_sym_with,
    STATE(1449), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(914), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(916), 6,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      sym_identifier,
      sym_operator_identifier,
  [51597] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1956), 1,
      anon_sym_AT,
    ACTIONS(782), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1416), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(784), 5,
      anon_sym_EQ_GT,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [51619] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2286), 1,
      anon_sym_AT,
    ACTIONS(2290), 1,
      anon_sym_LPAREN,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2362), 1,
      anon_sym_COLON,
    ACTIONS(2364), 1,
      anon_sym_EQ,
    STATE(2051), 1,
      aux_sym_val_declaration_repeat1,
  [51653] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 3,
      anon_sym_COMMA,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(551), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [51671] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2152), 1,
      anon_sym_PLUS,
    ACTIONS(2154), 1,
      anon_sym_DASH,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2150), 2,
      sym_identifier,
      sym_wildcard,
    STATE(1526), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    STATE(2222), 3,
      sym_covariant_type_parameter,
      sym_contravariant_type_parameter,
      sym__type_parameter,
  [51697] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(666), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(668), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [51715] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 3,
      anon_sym_COMMA,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(555), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [51733] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2342), 1,
      anon_sym_with,
    STATE(1435), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(910), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(912), 6,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      sym_identifier,
      sym_operator_identifier,
  [51755] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2286), 1,
      anon_sym_AT,
    ACTIONS(2290), 1,
      anon_sym_LPAREN,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2366), 1,
      anon_sym_COLON,
    ACTIONS(2368), 1,
      anon_sym_EQ,
    STATE(1959), 1,
      aux_sym_val_declaration_repeat1,
  [51789] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(662), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(664), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_POUND,
      sym_identifier,
      sym_operator_identifier,
  [51807] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 3,
      anon_sym_COMMA,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(559), 7,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [51825] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(601), 3,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LPAREN,
    ACTIONS(603), 7,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_match,
      anon_sym_catch,
      anon_sym_finally,
      sym_identifier,
      sym_operator_identifier,
  [51843] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2372), 1,
      anon_sym_EQ_GT,
    ACTIONS(2374), 1,
      anon_sym_COLON,
    ACTIONS(2376), 1,
      anon_sym_AT,
    ACTIONS(2378), 1,
      anon_sym_LPAREN,
    ACTIONS(2380), 1,
      anon_sym_PIPE,
    ACTIONS(2382), 1,
      anon_sym_if,
    STATE(2341), 1,
      sym_guard,
    ACTIONS(2370), 2,
      sym_identifier,
      sym_operator_identifier,
  [51875] = 11,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2286), 1,
      anon_sym_AT,
    ACTIONS(2290), 1,
      anon_sym_LPAREN,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2384), 1,
      anon_sym_COLON,
    ACTIONS(2386), 1,
      anon_sym_EQ,
    STATE(1954), 1,
      aux_sym_val_declaration_repeat1,
  [51909] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2000), 1,
      anon_sym_AT,
    ACTIONS(782), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1431), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(784), 5,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [51931] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(559), 7,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [51949] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1774), 1,
      anon_sym_DOT,
    ACTIONS(1776), 1,
      anon_sym_LBRACK,
    ACTIONS(1778), 1,
      anon_sym_EQ,
    ACTIONS(1780), 1,
      anon_sym_LPAREN,
    ACTIONS(1782), 1,
      anon_sym_match,
    ACTIONS(2388), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym_arguments,
    STATE(1351), 1,
      sym_type_arguments,
    ACTIONS(1772), 2,
      sym_identifier,
      sym_operator_identifier,
  [51981] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2081), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(897), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(901), 4,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
  [52003] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2390), 1,
      anon_sym_with,
    STATE(1463), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(910), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(912), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [52025] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(555), 7,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [52043] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(551), 7,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [52061] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2392), 1,
      anon_sym_with,
    STATE(1463), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(921), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(923), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [52083] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2395), 1,
      anon_sym_AT,
    ACTIONS(700), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1464), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(702), 5,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [52105] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2390), 1,
      anon_sym_with,
    STATE(1460), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(914), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(916), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [52127] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(933), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(935), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_with,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [52144] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(555), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [52161] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(559), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [52178] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
    ACTIONS(551), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [52195] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1035), 1,
      anon_sym_case,
    ACTIONS(2398), 1,
      anon_sym_EQ,
    ACTIONS(1031), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(1033), 4,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [52216] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 4,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(551), 5,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [52233] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1898), 1,
      sym_identifier,
    ACTIONS(2148), 1,
      anon_sym_LPAREN,
    STATE(1486), 1,
      sym__annotated_type,
    STATE(2329), 1,
      sym_stable_identifier,
    STATE(780), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [52256] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1854), 1,
      sym_identifier,
    ACTIONS(2163), 1,
      anon_sym_LPAREN,
    STATE(1466), 1,
      sym__annotated_type,
    STATE(2320), 1,
      sym_stable_identifier,
    STATE(750), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [52279] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 3,
      anon_sym_COMMA,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(559), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [52296] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 3,
      anon_sym_COMMA,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(555), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [52313] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2270), 1,
      anon_sym_AT,
    ACTIONS(2272), 1,
      anon_sym_LPAREN,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    STATE(2148), 1,
      aux_sym_case_class_pattern_repeat1,
  [52344] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2270), 1,
      anon_sym_AT,
    ACTIONS(2272), 1,
      anon_sym_LPAREN,
    ACTIONS(2400), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
    ACTIONS(2402), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [52367] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 3,
      anon_sym_COMMA,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(551), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [52384] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2390), 1,
      anon_sym_with,
    STATE(1460), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(897), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(901), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    ACTIONS(2404), 2,
      sym_identifier,
      sym_operator_identifier,
  [52409] = 9,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2270), 1,
      anon_sym_AT,
    ACTIONS(2272), 1,
      anon_sym_LPAREN,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2408), 1,
      anon_sym_PIPE,
    ACTIONS(2406), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [52438] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2270), 1,
      anon_sym_AT,
    ACTIONS(2272), 1,
      anon_sym_LPAREN,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    STATE(2019), 1,
      aux_sym_case_class_pattern_repeat1,
  [52469] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(2126), 1,
      anon_sym_LPAREN,
    STATE(495), 1,
      sym__annotated_type,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(212), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [52492] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2410), 1,
      sym_identifier,
    ACTIONS(2414), 1,
      anon_sym_implicit,
    ACTIONS(2416), 1,
      anon_sym_RPAREN,
    STATE(1958), 1,
      sym_class_parameter,
    ACTIONS(2412), 2,
      anon_sym_val,
      anon_sym_var,
    STATE(1679), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
  [52519] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2270), 1,
      anon_sym_AT,
    ACTIONS(2272), 1,
      anon_sym_LPAREN,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2418), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(2420), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
  [52546] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1041), 1,
      anon_sym_case,
    ACTIONS(2422), 1,
      anon_sym_EQ,
    ACTIONS(1039), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(1033), 4,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [52567] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(933), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(935), 7,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [52584] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2424), 1,
      anon_sym_LPAREN,
    STATE(1487), 2,
      sym_class_parameters,
      aux_sym_class_definition_repeat2,
    ACTIONS(1051), 6,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_extends,
      anon_sym_SEMI,
  [52603] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2376), 1,
      anon_sym_AT,
    ACTIONS(2378), 1,
      anon_sym_LPAREN,
    ACTIONS(2370), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(2420), 4,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
  [52626] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1058), 1,
      anon_sym_case,
    ACTIONS(2427), 1,
      anon_sym_EQ,
    ACTIONS(1056), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(1033), 4,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [52647] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2374), 1,
      anon_sym_COLON,
    ACTIONS(2376), 1,
      anon_sym_AT,
    ACTIONS(2378), 1,
      anon_sym_LPAREN,
    ACTIONS(2370), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(2408), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [52672] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1070), 1,
      anon_sym_case,
    ACTIONS(2429), 1,
      anon_sym_EQ,
    ACTIONS(1068), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(1033), 4,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [52693] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(914), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(916), 5,
      anon_sym_EQ_GT,
      anon_sym_case,
      anon_sym_EQ,
      sym_identifier,
      sym_operator_identifier,
  [52710] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(2128), 1,
      anon_sym_LPAREN,
    STATE(1637), 1,
      sym__annotated_type,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(1008), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [52733] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1912), 1,
      sym_identifier,
    ACTIONS(2136), 1,
      anon_sym_LPAREN,
    STATE(1564), 1,
      sym__annotated_type,
    STATE(2338), 1,
      sym_stable_identifier,
    STATE(798), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [52756] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(561), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(1074), 6,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [52773] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(2203), 1,
      anon_sym_LPAREN,
    STATE(1610), 1,
      sym__annotated_type,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(1114), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [52796] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2431), 1,
      anon_sym_with,
    STATE(1497), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(921), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(923), 5,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      sym_identifier,
      sym_operator_identifier,
  [52817] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2436), 1,
      anon_sym_with,
    STATE(1504), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(897), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(901), 2,
      anon_sym_EQ,
      anon_sym_STAR,
    ACTIONS(2434), 2,
      sym_identifier,
      sym_operator_identifier,
  [52842] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2438), 1,
      anon_sym_with,
    STATE(1527), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(914), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(916), 5,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      sym_identifier,
      sym_operator_identifier,
  [52863] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2376), 1,
      anon_sym_AT,
    ACTIONS(2378), 1,
      anon_sym_LPAREN,
    ACTIONS(2402), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [52884] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1908), 1,
      sym_identifier,
    ACTIONS(2161), 1,
      anon_sym_LPAREN,
    STATE(1632), 1,
      sym__annotated_type,
    STATE(2346), 1,
      sym_stable_identifier,
    STATE(1000), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [52907] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(613), 9,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_extends,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [52922] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(2216), 1,
      anon_sym_LPAREN,
    STATE(1608), 1,
      sym__annotated_type,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(1023), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [52945] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2436), 1,
      anon_sym_with,
    STATE(1516), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(910), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(912), 5,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [52966] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(901), 2,
      anon_sym_case,
      anon_sym_EQ,
    ACTIONS(2195), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(897), 4,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [52987] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1818), 1,
      sym_identifier,
    ACTIONS(2193), 1,
      anon_sym_LPAREN,
    STATE(1623), 1,
      sym__annotated_type,
    STATE(2353), 1,
      sym_stable_identifier,
    STATE(1064), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [53010] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2438), 1,
      anon_sym_with,
    STATE(1527), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(897), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(901), 2,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
    ACTIONS(2440), 2,
      sym_identifier,
      sym_operator_identifier,
  [53035] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1064), 1,
      anon_sym_case,
    ACTIONS(2442), 1,
      anon_sym_EQ,
    ACTIONS(1062), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(1033), 4,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [53056] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 4,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(555), 5,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [53073] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1047), 1,
      anon_sym_case,
    ACTIONS(2444), 1,
      anon_sym_EQ,
    ACTIONS(1045), 3,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_SEMI,
    ACTIONS(1033), 4,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [53094] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2410), 1,
      sym_identifier,
    ACTIONS(2446), 1,
      anon_sym_implicit,
    ACTIONS(2448), 1,
      anon_sym_RPAREN,
    STATE(2174), 1,
      sym_class_parameter,
    ACTIONS(2412), 2,
      anon_sym_val,
      anon_sym_var,
    STATE(1679), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
  [53121] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1850), 1,
      sym_identifier,
    ACTIONS(2214), 1,
      anon_sym_LPAREN,
    STATE(446), 1,
      sym__annotated_type,
    STATE(2293), 1,
      sym_stable_identifier,
    STATE(70), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [53144] = 10,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2270), 1,
      anon_sym_AT,
    ACTIONS(2272), 1,
      anon_sym_LPAREN,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    STATE(2063), 1,
      aux_sym_case_class_pattern_repeat1,
  [53175] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2436), 1,
      anon_sym_with,
    STATE(1504), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(914), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(916), 5,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [53196] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(2175), 1,
      anon_sym_LPAREN,
    STATE(1429), 1,
      sym__annotated_type,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(712), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [53219] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2450), 1,
      anon_sym_with,
    STATE(1516), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(921), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(923), 5,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [53240] = 9,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2270), 1,
      anon_sym_AT,
    ACTIONS(2272), 1,
      anon_sym_LPAREN,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2453), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [53269] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 3,
      anon_sym_COMMA,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(559), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [53286] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1499), 1,
      sym_identifier,
    ACTIONS(2232), 1,
      anon_sym_LPAREN,
    STATE(1566), 1,
      sym__annotated_type,
    STATE(2360), 1,
      sym_stable_identifier,
    STATE(896), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [53309] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 4,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(559), 5,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [53326] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(901), 1,
      anon_sym_case,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2244), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(897), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [53347] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 3,
      anon_sym_COMMA,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(551), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [53364] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(2253), 1,
      anon_sym_LPAREN,
    STATE(1397), 1,
      sym__annotated_type,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(718), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [53387] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(557), 1,
      anon_sym_LPAREN,
    ACTIONS(559), 8,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [53404] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1840), 1,
      sym_identifier,
    ACTIONS(2230), 1,
      anon_sym_LPAREN,
    STATE(1280), 1,
      sym__annotated_type,
    STATE(2372), 1,
      sym_stable_identifier,
    STATE(691), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [53427] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2455), 1,
      anon_sym_AT,
    ACTIONS(700), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1526), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
    ACTIONS(702), 4,
      anon_sym_val,
      anon_sym_var,
      sym_identifier,
      sym_wildcard,
  [53448] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2438), 1,
      anon_sym_with,
    STATE(1497), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(910), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(912), 5,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      sym_identifier,
      sym_operator_identifier,
  [53469] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(916), 4,
      anon_sym_EQ_GT,
      anon_sym_case,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(914), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [53486] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 1,
      anon_sym_LPAREN,
    ACTIONS(555), 8,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [53503] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(2144), 1,
      anon_sym_LPAREN,
    STATE(502), 1,
      sym__annotated_type,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(94), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [53526] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(549), 1,
      anon_sym_LPAREN,
    ACTIONS(551), 8,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [53543] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(660), 9,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_extends,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [53558] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(553), 3,
      anon_sym_COMMA,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(555), 6,
      anon_sym_EQ_GT,
      anon_sym_AT,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [53575] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1840), 1,
      sym_identifier,
    ACTIONS(2230), 1,
      anon_sym_LPAREN,
    STATE(2372), 1,
      sym_stable_identifier,
    STATE(685), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [53595] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1818), 1,
      sym_identifier,
    ACTIONS(2193), 1,
      anon_sym_LPAREN,
    STATE(2353), 1,
      sym_stable_identifier,
    STATE(890), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [53615] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2458), 1,
      anon_sym_with,
    STATE(1546), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(914), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
    ACTIONS(916), 3,
      anon_sym_EQ_GT,
      sym_identifier,
      sym_operator_identifier,
  [53635] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1868), 1,
      anon_sym_LT_PERCENT,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2460), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1710), 2,
      sym_view_bound,
      aux_sym__type_parameter_repeat1,
    STATE(1751), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [53657] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1830), 1,
      sym_identifier,
    ACTIONS(2253), 1,
      anon_sym_LPAREN,
    STATE(2367), 1,
      sym_stable_identifier,
    STATE(747), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [53677] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1868), 1,
      anon_sym_LT_PERCENT,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2462), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1537), 2,
      sym_view_bound,
      aux_sym__type_parameter_repeat1,
    STATE(1772), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [53699] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1868), 1,
      anon_sym_LT_PERCENT,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2462), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1710), 2,
      sym_view_bound,
      aux_sym__type_parameter_repeat1,
    STATE(1772), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [53721] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2286), 1,
      anon_sym_AT,
    ACTIONS(2290), 1,
      anon_sym_LPAREN,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2464), 1,
      anon_sym_COLON,
    ACTIONS(2408), 2,
      anon_sym_EQ,
      anon_sym_PIPE,
  [53747] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1822), 1,
      sym_identifier,
    ACTIONS(2144), 1,
      anon_sym_LPAREN,
    STATE(2272), 1,
      sym_stable_identifier,
    STATE(86), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [53767] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2410), 1,
      sym_identifier,
    ACTIONS(2466), 1,
      anon_sym_RPAREN,
    STATE(2130), 1,
      sym_class_parameter,
    ACTIONS(2412), 2,
      anon_sym_val,
      anon_sym_var,
    STATE(1679), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
  [53791] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1868), 1,
      anon_sym_LT_PERCENT,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2468), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1710), 2,
      sym_view_bound,
      aux_sym__type_parameter_repeat1,
    STATE(1757), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [53813] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1968), 1,
      anon_sym_extends,
    STATE(1691), 1,
      sym_extends_clause,
    STATE(1899), 1,
      sym_template_body,
    ACTIONS(1086), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [53835] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2458), 1,
      anon_sym_with,
    STATE(1552), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(910), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
    ACTIONS(912), 3,
      anon_sym_EQ_GT,
      sym_identifier,
      sym_operator_identifier,
  [53855] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2286), 1,
      anon_sym_AT,
    ACTIONS(2290), 1,
      anon_sym_LPAREN,
    ACTIONS(2400), 1,
      sym_identifier,
    ACTIONS(2402), 4,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_operator_identifier,
  [53877] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2470), 1,
      sym_identifier,
    ACTIONS(2472), 1,
      anon_sym_LPAREN,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(1041), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [53897] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2474), 1,
      anon_sym_with,
    STATE(1551), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(916), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [53915] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(901), 1,
      anon_sym_COLON,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2478), 1,
      anon_sym_with,
    STATE(1560), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(897), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(2476), 2,
      sym_identifier,
      sym_operator_identifier,
  [53939] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2474), 1,
      anon_sym_with,
    STATE(1571), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(912), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [53957] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2480), 1,
      anon_sym_with,
    STATE(1552), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(921), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
    ACTIONS(923), 3,
      anon_sym_EQ_GT,
      sym_identifier,
      sym_operator_identifier,
  [53977] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2286), 1,
      anon_sym_AT,
    ACTIONS(2290), 1,
      anon_sym_LPAREN,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2420), 3,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
  [54001] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1499), 1,
      sym_identifier,
    ACTIONS(2232), 1,
      anon_sym_LPAREN,
    STATE(2360), 1,
      sym_stable_identifier,
    STATE(776), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [54021] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1084), 8,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [54035] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1414), 1,
      sym_identifier,
    ACTIONS(2128), 1,
      anon_sym_LPAREN,
    STATE(2284), 1,
      sym_stable_identifier,
    STATE(961), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [54055] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2474), 1,
      anon_sym_with,
    STATE(1551), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(2483), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(901), 3,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
  [54077] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1908), 1,
      sym_identifier,
    ACTIONS(2161), 1,
      anon_sym_LPAREN,
    STATE(2346), 1,
      sym_stable_identifier,
    STATE(853), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [54097] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1850), 1,
      sym_identifier,
    ACTIONS(2214), 1,
      anon_sym_LPAREN,
    STATE(2293), 1,
      sym_stable_identifier,
    STATE(31), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [54117] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2478), 1,
      anon_sym_with,
    STATE(1575), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(910), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(912), 4,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      sym_identifier,
      sym_operator_identifier,
  [54137] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1826), 1,
      sym_identifier,
    ACTIONS(2175), 1,
      anon_sym_LPAREN,
    STATE(2377), 1,
      sym_stable_identifier,
    STATE(701), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [54157] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2470), 1,
      sym_identifier,
    ACTIONS(2472), 1,
      anon_sym_LPAREN,
    STATE(2323), 1,
      sym_stable_identifier,
    STATE(583), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [54177] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1834), 1,
      sym_identifier,
    ACTIONS(2216), 1,
      anon_sym_LPAREN,
    STATE(2318), 1,
      sym_stable_identifier,
    STATE(953), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [54197] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(933), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(935), 6,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [54213] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    ACTIONS(1968), 1,
      anon_sym_extends,
    STATE(1699), 1,
      sym_extends_clause,
    STATE(1819), 1,
      sym_template_body,
    ACTIONS(1078), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [54235] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(933), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(935), 6,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [54251] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(549), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_AT,
      anon_sym_LPAREN,
    ACTIONS(551), 4,
      anon_sym_val,
      anon_sym_var,
      sym_identifier,
      sym_wildcard,
  [54267] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(553), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_AT,
      anon_sym_LPAREN,
    ACTIONS(555), 4,
      anon_sym_val,
      anon_sym_var,
      sym_identifier,
      sym_wildcard,
  [54283] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(557), 4,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_AT,
      anon_sym_LPAREN,
    ACTIONS(559), 4,
      anon_sym_val,
      anon_sym_var,
      sym_identifier,
      sym_wildcard,
  [54299] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1076), 8,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [54313] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2485), 1,
      anon_sym_with,
    STATE(1571), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(923), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [54331] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(914), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      sym_identifier,
    ACTIONS(916), 5,
      anon_sym_EQ_GT,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      sym_operator_identifier,
  [54347] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1080), 8,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [54361] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2478), 1,
      anon_sym_with,
    STATE(1560), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(914), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(916), 4,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      sym_identifier,
      sym_operator_identifier,
  [54381] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2488), 1,
      anon_sym_with,
    STATE(1575), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(921), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(923), 4,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      sym_identifier,
      sym_operator_identifier,
  [54401] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1844), 1,
      sym_identifier,
    ACTIONS(2203), 1,
      anon_sym_LPAREN,
    STATE(2302), 1,
      sym_stable_identifier,
    STATE(901), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [54421] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2410), 1,
      sym_identifier,
    ACTIONS(2491), 1,
      anon_sym_RPAREN,
    STATE(2014), 1,
      sym_class_parameter,
    ACTIONS(2412), 2,
      anon_sym_val,
      anon_sym_var,
    STATE(1679), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
  [54445] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1868), 1,
      anon_sym_LT_PERCENT,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2259), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1710), 2,
      sym_view_bound,
      aux_sym__type_parameter_repeat1,
    STATE(1730), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [54467] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2340), 1,
      sym_operator_identifier,
    ACTIONS(2493), 1,
      sym_identifier,
    ACTIONS(897), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(901), 3,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
  [54489] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1868), 1,
      anon_sym_LT_PERCENT,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2259), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1599), 2,
      sym_view_bound,
      aux_sym__type_parameter_repeat1,
    STATE(1730), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [54511] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym_identifier,
    ACTIONS(2126), 1,
      anon_sym_LPAREN,
    STATE(2311), 1,
      sym_stable_identifier,
    STATE(44), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [54531] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2495), 1,
      anon_sym_with,
    STATE(1582), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(921), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(923), 4,
      anon_sym_EQ_GT,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [54551] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1082), 8,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [54565] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(914), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(916), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [54581] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2498), 1,
      anon_sym_with,
    STATE(1595), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(914), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(916), 4,
      anon_sym_EQ_GT,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [54601] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(901), 1,
      anon_sym_EQ,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2502), 1,
      anon_sym_with,
    STATE(1589), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(897), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(2500), 2,
      sym_identifier,
      sym_operator_identifier,
  [54625] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1868), 1,
      anon_sym_LT_PERCENT,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2257), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1540), 2,
      sym_view_bound,
      aux_sym__type_parameter_repeat1,
    STATE(1777), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [54647] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1854), 1,
      sym_identifier,
    ACTIONS(2163), 1,
      anon_sym_LPAREN,
    STATE(2320), 1,
      sym_stable_identifier,
    STATE(711), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [54667] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2502), 1,
      anon_sym_with,
    STATE(1598), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(910), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(912), 4,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      sym_identifier,
      sym_operator_identifier,
  [54687] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1868), 1,
      anon_sym_LT_PERCENT,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2257), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1710), 2,
      sym_view_bound,
      aux_sym__type_parameter_repeat1,
    STATE(1777), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [54709] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1868), 1,
      anon_sym_LT_PERCENT,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2504), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1544), 2,
      sym_view_bound,
      aux_sym__type_parameter_repeat1,
    STATE(1776), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [54731] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(901), 1,
      anon_sym_STAR,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2498), 1,
      anon_sym_with,
    STATE(1595), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(897), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(2506), 2,
      sym_identifier,
      sym_operator_identifier,
  [54755] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2502), 1,
      anon_sym_with,
    STATE(1589), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(914), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(916), 4,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      sym_identifier,
      sym_operator_identifier,
  [54775] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1898), 1,
      sym_identifier,
    ACTIONS(2148), 1,
      anon_sym_LPAREN,
    STATE(2329), 1,
      sym_stable_identifier,
    STATE(721), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [54795] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2498), 1,
      anon_sym_with,
    STATE(1582), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(910), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(912), 4,
      anon_sym_EQ_GT,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [54815] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2390), 1,
      anon_sym_with,
    STATE(1460), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(2404), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(901), 4,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
  [54835] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1912), 1,
      sym_identifier,
    ACTIONS(2136), 1,
      anon_sym_LPAREN,
    STATE(2338), 1,
      sym_stable_identifier,
    STATE(770), 5,
      sym__simple_type,
      sym_tuple_type,
      sym_stable_type_identifier,
      sym_generic_type,
      sym_projected_type,
  [54855] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2508), 1,
      anon_sym_with,
    STATE(1598), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(921), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(923), 4,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      sym_identifier,
      sym_operator_identifier,
  [54875] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1868), 1,
      anon_sym_LT_PERCENT,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2504), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1710), 2,
      sym_view_bound,
      aux_sym__type_parameter_repeat1,
    STATE(1776), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [54897] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2458), 1,
      anon_sym_with,
    STATE(1546), 1,
      aux_sym_compound_type_repeat1,
    ACTIONS(2511), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(897), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
  [54919] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(2513), 1,
      anon_sym_EQ,
    STATE(1846), 1,
      sym_block,
    ACTIONS(1141), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [54938] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1126), 7,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_extends,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [54951] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2515), 1,
      anon_sym_DOT,
    STATE(1614), 1,
      aux_sym_package_identifier_repeat1,
    ACTIONS(1102), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [54968] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2274), 1,
      anon_sym_RPAREN,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    STATE(1949), 1,
      aux_sym_case_class_pattern_repeat1,
  [54993] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2434), 1,
      sym_operator_identifier,
    ACTIONS(2517), 1,
      sym_identifier,
    ACTIONS(897), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(901), 2,
      anon_sym_EQ,
      anon_sym_STAR,
  [55014] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2302), 1,
      anon_sym_RPAREN,
    STATE(1922), 1,
      aux_sym_case_class_pattern_repeat1,
  [55039] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2519), 1,
      sym_identifier,
    ACTIONS(2521), 1,
      anon_sym_implicit,
    ACTIONS(2523), 1,
      anon_sym_RPAREN,
    STATE(2082), 1,
      sym_parameter,
    STATE(1802), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
  [55062] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(933), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
    ACTIONS(935), 4,
      anon_sym_EQ_GT,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [55077] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1110), 7,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_extends,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [55090] = 2,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(935), 7,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_with,
      anon_sym_PIPE,
      sym_identifier,
      sym_operator_identifier,
  [55103] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2519), 1,
      sym_identifier,
    ACTIONS(2525), 1,
      anon_sym_implicit,
    ACTIONS(2527), 1,
      anon_sym_RPAREN,
    STATE(2139), 1,
      sym_parameter,
    STATE(1802), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
  [55126] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2410), 1,
      sym_identifier,
    STATE(2221), 1,
      sym_class_parameter,
    ACTIONS(2412), 2,
      anon_sym_val,
      anon_sym_var,
    STATE(1679), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
  [55147] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2307), 1,
      anon_sym_RPAREN,
    STATE(2116), 1,
      aux_sym_case_class_pattern_repeat1,
  [55172] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2515), 1,
      anon_sym_DOT,
    STATE(1628), 1,
      aux_sym_package_identifier_repeat1,
    ACTIONS(1153), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55189] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2309), 1,
      anon_sym_RPAREN,
    STATE(2112), 1,
      aux_sym_case_class_pattern_repeat1,
  [55214] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2327), 1,
      anon_sym_RPAREN,
    STATE(2005), 1,
      aux_sym_case_class_pattern_repeat1,
  [55239] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(2529), 1,
      anon_sym_EQ,
    STATE(1821), 1,
      sym_block,
    ACTIONS(1155), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55258] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(2531), 1,
      anon_sym_EQ,
    STATE(1824), 1,
      sym_block,
    ACTIONS(1149), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55277] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(2533), 1,
      anon_sym_EQ,
    STATE(1835), 1,
      sym_block,
    ACTIONS(1106), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55296] = 8,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2311), 1,
      anon_sym_RPAREN,
    STATE(1934), 1,
      aux_sym_case_class_pattern_repeat1,
  [55321] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2440), 1,
      sym_operator_identifier,
    ACTIONS(2535), 1,
      sym_identifier,
    ACTIONS(897), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(901), 2,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
  [55342] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(2537), 1,
      anon_sym_EQ,
    STATE(1854), 1,
      sym_block,
    ACTIONS(1090), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55361] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(933), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(935), 5,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [55376] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(914), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
    ACTIONS(916), 4,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      anon_sym_STAR,
      sym_operator_identifier,
  [55391] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(2539), 1,
      anon_sym_EQ,
    STATE(1840), 1,
      sym_block,
    ACTIONS(1145), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55410] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(914), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      sym_identifier,
    ACTIONS(916), 4,
      anon_sym_EQ_GT,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
      sym_operator_identifier,
  [55425] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1120), 7,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_extends,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [55438] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2541), 1,
      anon_sym_DOT,
    STATE(1628), 1,
      aux_sym_package_identifier_repeat1,
    ACTIONS(1136), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55455] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(2544), 1,
      anon_sym_EQ,
    STATE(1892), 1,
      sym_block,
    ACTIONS(1098), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55474] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2404), 1,
      sym_operator_identifier,
    ACTIONS(2546), 1,
      sym_identifier,
    ACTIONS(897), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(901), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
  [55495] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2548), 1,
      anon_sym_LPAREN,
    STATE(1749), 1,
      sym_arguments,
    ACTIONS(1128), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55512] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(933), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(935), 5,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_with,
      sym_identifier,
      sym_operator_identifier,
  [55527] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(2550), 1,
      anon_sym_EQ,
    STATE(1915), 1,
      sym_block,
    ACTIONS(1116), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55546] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1159), 7,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_extends,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [55559] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(2552), 1,
      anon_sym_EQ,
    STATE(1904), 1,
      sym_block,
    ACTIONS(1094), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55578] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(561), 7,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_SEMI,
  [55591] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(933), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(935), 5,
      anon_sym_EQ_GT,
      anon_sym_with,
      anon_sym_STAR,
      sym_identifier,
      sym_operator_identifier,
  [55606] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2372), 1,
      anon_sym_EQ_GT,
    ACTIONS(2374), 1,
      anon_sym_COLON,
    ACTIONS(2380), 1,
      anon_sym_PIPE,
    ACTIONS(2382), 1,
      anon_sym_if,
    STATE(2341), 1,
      sym_guard,
    ACTIONS(2370), 2,
      sym_identifier,
      sym_operator_identifier,
  [55629] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(2554), 1,
      anon_sym_EQ,
    STATE(1875), 1,
      sym_block,
    ACTIONS(1132), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55648] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(2556), 1,
      anon_sym_EQ,
    STATE(1859), 1,
      sym_block,
    ACTIONS(1122), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55667] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1977), 1,
      anon_sym_LBRACE,
    ACTIONS(2558), 1,
      anon_sym_EQ,
    STATE(1864), 1,
      sym_block,
    ACTIONS(1112), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55686] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1900), 1,
      sym_template_body,
    ACTIONS(1214), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55702] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1797), 1,
      sym_template_body,
    ACTIONS(1197), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55718] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1882), 1,
      sym_template_body,
    ACTIONS(1195), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55734] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    STATE(2019), 1,
      aux_sym_case_class_pattern_repeat1,
  [55756] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(901), 1,
      anon_sym_EQ,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2500), 1,
      sym_operator_identifier,
    ACTIONS(2560), 1,
      sym_identifier,
    ACTIONS(897), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [55776] = 2,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(629), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [55788] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1861), 1,
      sym_template_body,
    ACTIONS(1201), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55804] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1873), 1,
      sym_template_body,
    ACTIONS(1193), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55820] = 2,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1074), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [55832] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(914), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
    ACTIONS(916), 3,
      anon_sym_EQ_GT,
      anon_sym_STAR,
      sym_operator_identifier,
  [55846] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2562), 1,
      anon_sym_COMMA,
    STATE(1652), 1,
      aux_sym_import_declaration_repeat1,
    ACTIONS(1207), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55862] = 2,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1033), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [55874] = 2,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(625), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [55886] = 2,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2565), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [55898] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1817), 1,
      sym_template_body,
    ACTIONS(1181), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [55914] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2567), 1,
      sym_identifier,
    ACTIONS(2569), 1,
      anon_sym_RPAREN,
    STATE(2047), 1,
      sym_parameter,
    STATE(1802), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
  [55934] = 2,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2571), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [55946] = 2,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2573), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [55958] = 2,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2575), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [55970] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(914), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      sym_identifier,
    ACTIONS(916), 3,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      sym_operator_identifier,
  [55984] = 2,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2402), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [55996] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2511), 1,
      sym_operator_identifier,
    ACTIONS(2577), 1,
      sym_identifier,
    ACTIONS(897), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
  [56014] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(545), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56026] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1853), 1,
      sym_template_body,
    ACTIONS(1179), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56042] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    STATE(2148), 1,
      aux_sym_case_class_pattern_repeat1,
  [56064] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1917), 1,
      sym_template_body,
    ACTIONS(1199), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56080] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1908), 1,
      sym_template_body,
    ACTIONS(1167), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56096] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1849), 1,
      sym_template_body,
    ACTIONS(1205), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56112] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2575), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
    ACTIONS(2579), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
  [56126] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1390), 1,
      anon_sym_case,
    ACTIONS(1392), 1,
      anon_sym_class,
    ACTIONS(1394), 1,
      anon_sym_val,
    ACTIONS(1396), 1,
      anon_sym_var,
    ACTIONS(1398), 1,
      anon_sym_type,
    ACTIONS(1400), 1,
      anon_sym_def,
  [56148] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2400), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
    ACTIONS(2402), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [56162] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1136), 6,
      sym__automatic_semicolon,
      anon_sym_DOT,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56174] = 2,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2581), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [56186] = 2,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2583), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [56198] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2583), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
    ACTIONS(2585), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
  [56212] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2581), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
    ACTIONS(2587), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
  [56226] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(914), 1,
      sym_identifier,
    ACTIONS(916), 5,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_operator_identifier,
  [56240] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2589), 1,
      sym_identifier,
    ACTIONS(2591), 2,
      anon_sym_val,
      anon_sym_var,
    STATE(1526), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
  [56258] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2408), 1,
      anon_sym_PIPE,
    ACTIONS(2406), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [56278] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2593), 1,
      anon_sym_case,
    ACTIONS(2595), 1,
      anon_sym_class,
    ACTIONS(2597), 1,
      anon_sym_val,
    ACTIONS(2599), 1,
      anon_sym_var,
    ACTIONS(2601), 1,
      anon_sym_type,
    ACTIONS(2603), 1,
      anon_sym_def,
  [56300] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1803), 1,
      sym_template_body,
    ACTIONS(1163), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56316] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1890), 1,
      sym_template_body,
    ACTIONS(1212), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56332] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2605), 1,
      anon_sym_case,
    ACTIONS(2607), 1,
      anon_sym_class,
    ACTIONS(2609), 1,
      anon_sym_val,
    ACTIONS(2611), 1,
      anon_sym_var,
    ACTIONS(2613), 1,
      anon_sym_type,
    ACTIONS(2615), 1,
      anon_sym_def,
  [56354] = 2,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(633), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [56366] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1836), 1,
      sym_template_body,
    ACTIONS(1203), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56382] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2617), 1,
      anon_sym_COMMA,
    STATE(1652), 1,
      aux_sym_import_declaration_repeat1,
    ACTIONS(1191), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56398] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2619), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
    ACTIONS(2621), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [56412] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1826), 1,
      sym_template_body,
    ACTIONS(1189), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56428] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2567), 1,
      sym_identifier,
    ACTIONS(2623), 1,
      anon_sym_RPAREN,
    STATE(2109), 1,
      sym_parameter,
    STATE(1802), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
  [56448] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1888), 1,
      sym_template_body,
    ACTIONS(1187), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56464] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1402), 1,
      anon_sym_case,
    ACTIONS(1404), 1,
      anon_sym_class,
    ACTIONS(1406), 1,
      anon_sym_val,
    ACTIONS(1408), 1,
      anon_sym_var,
    ACTIONS(1410), 1,
      anon_sym_type,
    ACTIONS(1412), 1,
      anon_sym_def,
  [56486] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2418), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(2420), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
  [56504] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(914), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
    ACTIONS(916), 3,
      anon_sym_EQ_GT,
      anon_sym_EQ,
      sym_operator_identifier,
  [56518] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(627), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
    ACTIONS(629), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [56532] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(901), 1,
      anon_sym_COLON,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2476), 1,
      sym_operator_identifier,
    ACTIONS(2625), 1,
      sym_identifier,
    ACTIONS(897), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [56552] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(901), 1,
      anon_sym_STAR,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2506), 1,
      sym_operator_identifier,
    ACTIONS(2627), 1,
      sym_identifier,
    ACTIONS(897), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [56572] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(903), 1,
      anon_sym_EQ_GT,
    ACTIONS(2483), 1,
      sym_operator_identifier,
    ACTIONS(2629), 1,
      sym_identifier,
    ACTIONS(901), 3,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
  [56590] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1885), 1,
      sym_template_body,
    ACTIONS(1161), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56606] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2573), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
    ACTIONS(2631), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
  [56620] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(631), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
    ACTIONS(633), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [56634] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2571), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
    ACTIONS(2633), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
  [56648] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2404), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(901), 4,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
  [56662] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1813), 1,
      sym_template_body,
    ACTIONS(1177), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56678] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2565), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
    ACTIONS(2635), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
  [56692] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(623), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
    ACTIONS(625), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [56706] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1818), 1,
      sym_template_body,
    ACTIONS(1183), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56722] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(561), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
    ACTIONS(1074), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [56736] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2617), 1,
      anon_sym_COMMA,
    STATE(1687), 1,
      aux_sym_import_declaration_repeat1,
    ACTIONS(1173), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56752] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2639), 1,
      anon_sym_LT_PERCENT,
    STATE(1710), 2,
      sym_view_bound,
      aux_sym__type_parameter_repeat1,
    ACTIONS(2637), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
  [56768] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1905), 1,
      sym_template_body,
    ACTIONS(1185), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56784] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    ACTIONS(2453), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [56804] = 7,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2264), 1,
      sym_identifier,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2268), 1,
      anon_sym_COLON,
    ACTIONS(2276), 1,
      anon_sym_PIPE,
    ACTIONS(2278), 1,
      sym_operator_identifier,
    STATE(2063), 1,
      aux_sym_case_class_pattern_repeat1,
  [56826] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2370), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(2420), 4,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
  [56840] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1033), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
    ACTIONS(2642), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      sym_identifier,
  [56854] = 2,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2621), 6,
      anon_sym_EQ_GT,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_if,
      sym_identifier,
      sym_operator_identifier,
  [56866] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2644), 1,
      anon_sym_DOT,
    ACTIONS(1169), 5,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56880] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1964), 1,
      anon_sym_LBRACE,
    STATE(1896), 1,
      sym_template_body,
    ACTIONS(1165), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56896] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2374), 1,
      anon_sym_COLON,
    ACTIONS(2370), 2,
      sym_identifier,
      sym_operator_identifier,
    ACTIONS(2408), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [56912] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(916), 2,
      anon_sym_EQ_GT,
      sym_operator_identifier,
    ACTIONS(914), 4,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
      sym_identifier,
  [56926] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(557), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [56937] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2322), 1,
      anon_sym_EQ,
    ACTIONS(2646), 1,
      anon_sym_COLON,
  [56956] = 4,
    ACTIONS(3), 1,
      sym_comment,
    STATE(1766), 1,
      aux_sym__block_repeat1,
    ACTIONS(1902), 2,
      anon_sym_RBRACE,
      anon_sym_case,
    ACTIONS(1906), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [56971] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2567), 1,
      sym_identifier,
    STATE(2206), 1,
      sym_parameter,
    STATE(1802), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
  [56988] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    ACTIONS(2650), 1,
      anon_sym_RPAREN,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    STATE(1962), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
  [57007] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1222), 5,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [57018] = 5,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2464), 1,
      anon_sym_COLON,
    ACTIONS(2408), 2,
      anon_sym_EQ,
      anon_sym_PIPE,
  [57035] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2654), 1,
      anon_sym_RPAREN,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
    STATE(2034), 1,
      aux_sym_tuple_type_repeat1,
  [57054] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1049), 1,
      anon_sym_EQ,
    ACTIONS(2642), 1,
      sym_identifier,
    ACTIONS(1033), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57069] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2504), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1768), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [57084] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2288), 1,
      anon_sym_EQ,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2656), 1,
      anon_sym_COLON,
  [57103] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2298), 1,
      anon_sym_EQ,
    ACTIONS(2658), 1,
      anon_sym_COLON,
  [57122] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2631), 1,
      sym_identifier,
    ACTIONS(2573), 4,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57135] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2633), 1,
      sym_identifier,
    ACTIONS(2571), 4,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57148] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2400), 1,
      sym_identifier,
    ACTIONS(2402), 4,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57161] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2660), 1,
      anon_sym_RPAREN,
    STATE(1931), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
  [57180] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2635), 1,
      sym_identifier,
    ACTIONS(2565), 4,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57193] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2662), 1,
      anon_sym_RPAREN,
    STATE(1993), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
  [57212] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1066), 1,
      anon_sym_EQ,
    ACTIONS(2642), 1,
      sym_identifier,
    ACTIONS(1033), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57227] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1043), 1,
      anon_sym_EQ,
    ACTIONS(2642), 1,
      sym_identifier,
    ACTIONS(1033), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57242] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2579), 1,
      sym_identifier,
    ACTIONS(2575), 4,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57255] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2664), 1,
      anon_sym_RPAREN,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
    STATE(2156), 1,
      aux_sym_tuple_type_repeat1,
  [57274] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2422), 1,
      anon_sym_EQ,
    ACTIONS(2642), 1,
      sym_identifier,
    ACTIONS(1033), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57289] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1216), 5,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [57300] = 4,
    ACTIONS(3), 1,
      sym_comment,
    STATE(1745), 1,
      aux_sym__block_repeat1,
    ACTIONS(1993), 2,
      anon_sym_RBRACE,
      anon_sym_case,
    ACTIONS(2666), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [57315] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2364), 1,
      anon_sym_EQ,
    ACTIONS(2669), 1,
      anon_sym_COLON,
  [57334] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2398), 1,
      anon_sym_EQ,
    ACTIONS(2642), 1,
      sym_identifier,
    ACTIONS(1033), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57349] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(623), 1,
      sym_identifier,
    ACTIONS(625), 4,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57362] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1228), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [57373] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2671), 1,
      anon_sym_RPAREN,
    STATE(1957), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
  [57392] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2673), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1768), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [57407] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2356), 1,
      anon_sym_EQ,
    ACTIONS(2675), 1,
      anon_sym_COLON,
  [57426] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2386), 1,
      anon_sym_EQ,
    ACTIONS(2677), 1,
      anon_sym_COLON,
  [57445] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2642), 1,
      sym_identifier,
    ACTIONS(1033), 4,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57458] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2352), 1,
      anon_sym_EQ,
    ACTIONS(2679), 1,
      anon_sym_COLON,
  [57477] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2368), 1,
      anon_sym_EQ,
    ACTIONS(2681), 1,
      anon_sym_COLON,
  [57496] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2683), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1768), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [57511] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2685), 1,
      anon_sym_RPAREN,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
    STATE(2024), 1,
      aux_sym_tuple_type_repeat1,
  [57530] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2687), 5,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_GT_COLON,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
  [57541] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2331), 1,
      anon_sym_EQ,
    ACTIONS(2689), 1,
      anon_sym_COLON,
  [57560] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2360), 1,
      anon_sym_EQ,
    ACTIONS(2691), 1,
      anon_sym_COLON,
  [57579] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2335), 1,
      anon_sym_EQ,
    ACTIONS(2693), 1,
      anon_sym_COLON,
  [57598] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(631), 1,
      sym_identifier,
    ACTIONS(633), 4,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57611] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2257), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1768), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [57626] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2695), 1,
      anon_sym_RPAREN,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
    STATE(2169), 1,
      aux_sym_tuple_type_repeat1,
  [57645] = 4,
    ACTIONS(3), 1,
      sym_comment,
    STATE(1745), 1,
      aux_sym__block_repeat1,
    ACTIONS(117), 2,
      anon_sym_RBRACE,
      anon_sym_case,
    ACTIONS(2697), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [57660] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1060), 1,
      anon_sym_EQ,
    ACTIONS(2642), 1,
      sym_identifier,
    ACTIONS(1033), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57675] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2701), 1,
      anon_sym_COLON,
    ACTIONS(2699), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1768), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [57690] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2259), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1768), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [57705] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2704), 1,
      anon_sym_RPAREN,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
    STATE(2146), 1,
      aux_sym_tuple_type_repeat1,
  [57724] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2706), 1,
      anon_sym_RPAREN,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
    STATE(2054), 1,
      aux_sym_tuple_type_repeat1,
  [57743] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2460), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1768), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [57758] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1218), 5,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [57769] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2708), 1,
      anon_sym_RPAREN,
    STATE(1984), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
  [57788] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2710), 1,
      anon_sym_RPAREN,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
    STATE(2134), 1,
      aux_sym_tuple_type_repeat1,
  [57807] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2468), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1768), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [57822] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1870), 1,
      anon_sym_COLON,
    ACTIONS(2462), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(1768), 2,
      sym_context_bound,
      aux_sym__type_parameter_repeat2,
  [57837] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(561), 1,
      sym_identifier,
    ACTIONS(1074), 4,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57850] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2712), 1,
      anon_sym_RPAREN,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
    STATE(2117), 1,
      aux_sym_tuple_type_repeat1,
  [57869] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(627), 1,
      sym_identifier,
    ACTIONS(629), 4,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57882] = 6,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2292), 1,
      anon_sym_PIPE,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2318), 1,
      anon_sym_EQ,
    ACTIONS(2714), 1,
      anon_sym_COLON,
  [57901] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2444), 1,
      anon_sym_EQ,
    ACTIONS(2642), 1,
      sym_identifier,
    ACTIONS(1033), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57916] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(549), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [57927] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2442), 1,
      anon_sym_EQ,
    ACTIONS(2642), 1,
      sym_identifier,
    ACTIONS(1033), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57942] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1037), 1,
      anon_sym_EQ,
    ACTIONS(2642), 1,
      sym_identifier,
    ACTIONS(1033), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57957] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(553), 5,
      sym__automatic_semicolon,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [57968] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2619), 1,
      sym_identifier,
    ACTIONS(2621), 4,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57981] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2429), 1,
      anon_sym_EQ,
    ACTIONS(2642), 1,
      sym_identifier,
    ACTIONS(1033), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [57996] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(1072), 1,
      anon_sym_EQ,
    ACTIONS(2642), 1,
      sym_identifier,
    ACTIONS(1033), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [58011] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2716), 1,
      anon_sym_RPAREN,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
    STATE(2079), 1,
      aux_sym_tuple_type_repeat1,
  [58030] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2718), 1,
      anon_sym_RPAREN,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
    STATE(2098), 1,
      aux_sym_tuple_type_repeat1,
  [58049] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2427), 1,
      anon_sym_EQ,
    ACTIONS(2642), 1,
      sym_identifier,
    ACTIONS(1033), 3,
      anon_sym_COLON,
      anon_sym_PIPE,
      sym_operator_identifier,
  [58064] = 4,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2280), 1,
      sym_identifier,
    ACTIONS(2294), 1,
      sym_operator_identifier,
    ACTIONS(2420), 3,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
  [58079] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2587), 1,
      sym_identifier,
    ACTIONS(2581), 4,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_operator_identifier,
  [58092] = 3,
    ACTIONS(185), 1,
      sym_comment,
    ACTIONS(2585), 1,
      sym_identifier,
    ACTIONS(2583), 4,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PIPE,
      sym_operator_identifier,
  [58105] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1220), 5,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58116] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1356), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58126] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1252), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58136] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1386), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58146] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1316), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58156] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2722), 1,
      anon_sym_COLON,
    ACTIONS(2724), 1,
      anon_sym_EQ,
    ACTIONS(2720), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [58170] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2156), 1,
      anon_sym_AT,
    ACTIONS(2726), 1,
      sym_identifier,
    STATE(1526), 2,
      sym_annotation,
      aux_sym_class_definition_repeat1,
  [58184] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1384), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58194] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1344), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58204] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2728), 1,
      anon_sym_RBRACE,
    ACTIONS(2730), 1,
      anon_sym_case,
    STATE(1851), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [58218] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2732), 1,
      anon_sym_RBRACE,
    STATE(1809), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [58232] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1236), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58242] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2736), 1,
      anon_sym_EQ,
    ACTIONS(2734), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [58256] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2738), 1,
      anon_sym_RBRACE,
    STATE(1895), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [58270] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1388), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58280] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1322), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58290] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2742), 1,
      anon_sym_COLON,
    ACTIONS(2744), 1,
      anon_sym_EQ,
    ACTIONS(2740), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [58304] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1230), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58314] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2746), 1,
      anon_sym_RBRACE,
    STATE(1895), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [58328] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(117), 1,
      anon_sym_RBRACE,
    STATE(1745), 1,
      aux_sym__block_repeat1,
    ACTIONS(2748), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [58342] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1382), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58352] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1378), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58362] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1232), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58372] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1380), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58382] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2750), 1,
      anon_sym_RBRACE,
    STATE(1909), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [58396] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1310), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58406] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2752), 1,
      anon_sym_RBRACE,
    STATE(1829), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [58420] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1238), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58430] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1304), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58440] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1302), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58450] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1296), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58460] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2756), 1,
      anon_sym_COLON,
    ACTIONS(2758), 1,
      anon_sym_EQ,
    ACTIONS(2754), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [58474] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1234), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58484] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2760), 1,
      anon_sym_RBRACE,
    STATE(1895), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [58498] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2762), 1,
      anon_sym_COMMA,
    ACTIONS(2764), 1,
      anon_sym_RBRACE,
    ACTIONS(2766), 1,
      anon_sym_EQ_GT,
    STATE(1994), 1,
      aux_sym_import_selectors_repeat1,
  [58514] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1294), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58524] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1370), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58534] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(2770), 1,
      anon_sym_EQ,
    ACTIONS(2768), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [58548] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1246), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58558] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1364), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58568] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1360), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58578] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2762), 1,
      anon_sym_COMMA,
    ACTIONS(2766), 1,
      anon_sym_EQ_GT,
    ACTIONS(2772), 1,
      anon_sym_RBRACE,
    STATE(2074), 1,
      aux_sym_import_selectors_repeat1,
  [58594] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1312), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58604] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1292), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58614] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1348), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58624] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1308), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58634] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2774), 1,
      anon_sym_RBRACE,
    STATE(1852), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [58648] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1286), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58658] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2776), 1,
      anon_sym_RBRACE,
    STATE(1895), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [58672] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1300), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58682] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1340), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58692] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1282), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58702] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1358), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58712] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1332), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58722] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1330), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58732] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2778), 1,
      anon_sym_RBRACE,
    STATE(1895), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [58746] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2780), 1,
      anon_sym_RBRACE,
    STATE(1895), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [58760] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1328), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58770] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1262), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58780] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2782), 1,
      anon_sym_RBRACE,
    STATE(1844), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [58794] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2784), 4,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
  [58804] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2786), 4,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_LT_PERCENT,
      anon_sym_COLON,
  [58814] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1314), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58824] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1298), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58834] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1250), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58844] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1372), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58854] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1274), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58864] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1260), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58874] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1326), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58884] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2788), 1,
      anon_sym_COMMA,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
    ACTIONS(2791), 2,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
  [58898] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1258), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58908] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1352), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58918] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2793), 1,
      sym_identifier,
    ACTIONS(2795), 1,
      anon_sym_LBRACE,
    ACTIONS(2797), 1,
      sym_wildcard,
    STATE(1744), 1,
      sym_import_selectors,
  [58934] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2799), 1,
      sym_identifier,
    ACTIONS(2801), 1,
      anon_sym_LBRACE,
    ACTIONS(2803), 1,
      sym_wildcard,
    STATE(579), 1,
      sym_import_selectors,
  [58950] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2805), 1,
      anon_sym_RBRACE,
    STATE(1883), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [58964] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1324), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58974] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1334), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58984] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1320), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [58994] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1290), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59004] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1284), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59014] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1280), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59024] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1338), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59034] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2807), 1,
      anon_sym_RBRACE,
    STATE(1895), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [59048] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1342), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59058] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2809), 1,
      anon_sym_RBRACE,
    STATE(1814), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [59072] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1240), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59082] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1366), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59092] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2811), 1,
      anon_sym_RBRACE,
    STATE(1895), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [59106] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1278), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59116] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1350), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59126] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1993), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59136] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2813), 1,
      anon_sym_RBRACE,
    STATE(1878), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [59150] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1254), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59160] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1276), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59170] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1318), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59180] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1336), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59190] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1242), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59200] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1368), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59210] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1902), 1,
      anon_sym_RBRACE,
    STATE(1815), 1,
      aux_sym__block_repeat1,
    ACTIONS(1987), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [59224] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2815), 1,
      anon_sym_RBRACE,
    ACTIONS(2817), 1,
      anon_sym_case,
    STATE(1895), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [59238] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1268), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59248] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(583), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59258] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1272), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59268] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1288), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59278] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1270), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59288] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2820), 1,
      anon_sym_RBRACE,
    STATE(1906), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [59302] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(587), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59312] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1266), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59322] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1264), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59332] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1354), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59342] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2822), 1,
      anon_sym_RBRACE,
    STATE(1895), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [59356] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1346), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59366] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1244), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59376] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2730), 1,
      anon_sym_case,
    ACTIONS(2824), 1,
      anon_sym_RBRACE,
    STATE(1895), 2,
      sym_case_clause,
      aux_sym_case_block_repeat1,
  [59390] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1376), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59400] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2828), 1,
      anon_sym_COLON,
    ACTIONS(2830), 1,
      anon_sym_EQ,
    ACTIONS(2826), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [59414] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1374), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59424] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1306), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59434] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2834), 1,
      anon_sym_COLON,
    ACTIONS(2836), 1,
      anon_sym_EQ,
    ACTIONS(2832), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [59448] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1256), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59458] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1362), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59468] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1248), 4,
      sym__automatic_semicolon,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_SEMI,
  [59478] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(2840), 1,
      sym__string_end,
    STATE(1979), 1,
      aux_sym_string_repeat1,
  [59491] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(2842), 1,
      sym__string_end,
    STATE(2060), 1,
      aux_sym_string_repeat1,
  [59504] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2846), 1,
      anon_sym_RPAREN,
    STATE(1931), 1,
      aux_sym_tuple_type_repeat1,
  [59517] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2848), 1,
      anon_sym_COLON,
    STATE(2150), 1,
      aux_sym_val_declaration_repeat1,
  [59530] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2850), 1,
      anon_sym_RPAREN,
    STATE(1952), 1,
      aux_sym_case_class_pattern_repeat1,
  [59543] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(735), 1,
      anon_sym_LBRACK,
    ACTIONS(2852), 1,
      anon_sym_EQ,
    STATE(2270), 1,
      sym_type_parameters,
  [59556] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(2854), 1,
      sym__string_end,
    STATE(1937), 1,
      aux_sym_string_repeat1,
  [59569] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2854), 1,
      sym__multiline_string_end,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    STATE(1938), 1,
      aux_sym_string_repeat2,
  [59582] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2858), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [59595] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2860), 1,
      anon_sym_COMMA,
    ACTIONS(2862), 1,
      anon_sym_RPAREN,
    STATE(2080), 1,
      aux_sym_parameters_repeat1,
  [59608] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2864), 1,
      anon_sym_COMMA,
    ACTIONS(2866), 1,
      anon_sym_RBRACK,
    STATE(2020), 1,
      aux_sym_type_parameters_repeat1,
  [59621] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2868), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [59634] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2870), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [59647] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2872), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [59660] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2874), 1,
      anon_sym_COLON,
    STATE(2150), 1,
      aux_sym_val_declaration_repeat1,
  [59673] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2876), 3,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_RPAREN,
  [59682] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2878), 1,
      anon_sym_RPAREN,
    STATE(1952), 1,
      aux_sym_case_class_pattern_repeat1,
  [59695] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2880), 1,
      anon_sym_RBRACK,
    STATE(1945), 1,
      aux_sym_tuple_type_repeat1,
  [59708] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2882), 1,
      anon_sym_COLON,
    STATE(2150), 1,
      aux_sym_val_declaration_repeat1,
  [59721] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(2884), 1,
      sym__string_end,
    STATE(1979), 1,
      aux_sym_string_repeat1,
  [59734] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(2884), 1,
      sym__multiline_string_end,
    STATE(2015), 1,
      aux_sym_string_repeat2,
  [59747] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2886), 1,
      anon_sym_COLON,
    STATE(2150), 1,
      aux_sym_val_declaration_repeat1,
  [59760] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(2888), 1,
      sym__multiline_string_end,
    STATE(2015), 1,
      aux_sym_string_repeat2,
  [59773] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2890), 1,
      anon_sym_COLON,
    STATE(2150), 1,
      aux_sym_val_declaration_repeat1,
  [59786] = 3,
    ACTIONS(3), 1,
      sym_comment,
    STATE(2218), 1,
      sym__type_parameter,
    ACTIONS(2150), 2,
      sym_identifier,
      sym_wildcard,
  [59797] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(2888), 1,
      sym__string_end,
    STATE(1979), 1,
      aux_sym_string_repeat1,
  [59810] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2892), 1,
      anon_sym_COLON,
    STATE(2150), 1,
      aux_sym_val_declaration_repeat1,
  [59823] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2894), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [59836] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2896), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [59849] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2898), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [59862] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2900), 1,
      anon_sym_RBRACK,
    STATE(1930), 1,
      aux_sym_tuple_type_repeat1,
  [59875] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(2902), 1,
      anon_sym_RPAREN,
    STATE(1952), 1,
      aux_sym_case_class_pattern_repeat1,
  [59888] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(735), 1,
      anon_sym_LBRACK,
    ACTIONS(2904), 1,
      anon_sym_EQ,
    STATE(2269), 1,
      sym_type_parameters,
  [59901] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2906), 1,
      anon_sym_RPAREN,
    STATE(1962), 1,
      aux_sym_tuple_type_repeat1,
  [59914] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2453), 1,
      anon_sym_RPAREN,
    ACTIONS(2908), 1,
      anon_sym_COMMA,
    STATE(1952), 1,
      aux_sym_case_class_pattern_repeat1,
  [59927] = 3,
    ACTIONS(3), 1,
      sym_comment,
    STATE(2213), 1,
      sym__type_parameter,
    ACTIONS(2150), 2,
      sym_identifier,
      sym_wildcard,
  [59938] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2911), 1,
      anon_sym_COLON,
    STATE(2150), 1,
      aux_sym_val_declaration_repeat1,
  [59951] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(2913), 1,
      sym__string_end,
    STATE(1968), 1,
      aux_sym_string_repeat1,
  [59964] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(2913), 1,
      sym__multiline_string_end,
    STATE(1969), 1,
      aux_sym_string_repeat2,
  [59977] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2915), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [59990] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2491), 1,
      anon_sym_RPAREN,
    ACTIONS(2917), 1,
      anon_sym_COMMA,
    STATE(2012), 1,
      aux_sym_class_parameters_repeat1,
  [60003] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(2919), 1,
      anon_sym_COLON,
    STATE(2150), 1,
      aux_sym_val_declaration_repeat1,
  [60016] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2921), 1,
      sym_identifier,
    ACTIONS(2923), 1,
      anon_sym_object,
    STATE(546), 1,
      sym_package_identifier,
  [60029] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(2925), 1,
      sym__multiline_string_end,
    STATE(1940), 1,
      aux_sym_string_repeat2,
  [60042] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2927), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [60055] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2929), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [60068] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(2925), 1,
      sym__string_end,
    STATE(1943), 1,
      aux_sym_string_repeat1,
  [60081] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2931), 1,
      anon_sym_RPAREN,
    STATE(1957), 1,
      aux_sym_tuple_type_repeat1,
  [60094] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2933), 1,
      anon_sym_RBRACK,
    STATE(1976), 1,
      aux_sym_tuple_type_repeat1,
  [60107] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(735), 1,
      anon_sym_LBRACK,
    ACTIONS(2935), 1,
      anon_sym_EQ,
    STATE(2331), 1,
      sym_type_parameters,
  [60120] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(2937), 1,
      sym__string_end,
    STATE(1979), 1,
      aux_sym_string_repeat1,
  [60133] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(2937), 1,
      sym__multiline_string_end,
    STATE(2015), 1,
      aux_sym_string_repeat2,
  [60146] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2939), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [60159] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2941), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [60172] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2943), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [60185] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2945), 1,
      anon_sym_COMMA,
    ACTIONS(2947), 1,
      anon_sym_RPAREN,
    STATE(1996), 1,
      aux_sym_parameter_types_repeat1,
  [60198] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(2949), 1,
      sym__multiline_string_end,
    STATE(2015), 1,
      aux_sym_string_repeat2,
  [60211] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(2949), 1,
      sym__string_end,
    STATE(1979), 1,
      aux_sym_string_repeat1,
  [60224] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2951), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [60237] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2953), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [60250] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2955), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [60263] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2957), 1,
      sym__string_middle,
    ACTIONS(2960), 1,
      sym__string_end,
    STATE(1979), 1,
      aux_sym_string_repeat1,
  [60276] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2962), 1,
      anon_sym_RBRACK,
    STATE(1972), 1,
      aux_sym_tuple_type_repeat1,
  [60289] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2964), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [60302] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2966), 1,
      anon_sym_RPAREN,
    STATE(1993), 1,
      aux_sym_tuple_type_repeat1,
  [60315] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2968), 1,
      sym_identifier,
    STATE(548), 1,
      sym_stable_identifier,
    STATE(580), 1,
      sym__import_expression,
  [60328] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2970), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [60341] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(735), 1,
      anon_sym_LBRACK,
    ACTIONS(2972), 1,
      anon_sym_EQ,
    STATE(2287), 1,
      sym_type_parameters,
  [60354] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(2974), 1,
      sym__string_end,
    STATE(1999), 1,
      aux_sym_string_repeat1,
  [60367] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(2974), 1,
      sym__multiline_string_end,
    STATE(2000), 1,
      aux_sym_string_repeat2,
  [60380] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2976), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [60393] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2978), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [60406] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2791), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
  [60415] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(735), 1,
      anon_sym_LBRACK,
    ACTIONS(2980), 1,
      anon_sym_EQ,
    STATE(2368), 1,
      sym_type_parameters,
  [60428] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(2982), 1,
      sym__multiline_string_end,
    STATE(1974), 1,
      aux_sym_string_repeat2,
  [60441] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2984), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [60454] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2762), 1,
      anon_sym_COMMA,
    ACTIONS(2986), 1,
      anon_sym_RBRACE,
    STATE(2103), 1,
      aux_sym_import_selectors_repeat1,
  [60467] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(2982), 1,
      sym__string_end,
    STATE(1975), 1,
      aux_sym_string_repeat1,
  [60480] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2945), 1,
      anon_sym_COMMA,
    ACTIONS(2988), 1,
      anon_sym_RPAREN,
    STATE(2050), 1,
      aux_sym_parameter_types_repeat1,
  [60493] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2990), 1,
      anon_sym_RBRACK,
    STATE(2007), 1,
      aux_sym_tuple_type_repeat1,
  [60506] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(2992), 1,
      anon_sym_RPAREN,
    STATE(1984), 1,
      aux_sym_tuple_type_repeat1,
  [60519] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(2994), 1,
      sym__string_end,
    STATE(1979), 1,
      aux_sym_string_repeat1,
  [60532] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(2994), 1,
      sym__multiline_string_end,
    STATE(2015), 1,
      aux_sym_string_repeat2,
  [60545] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2762), 1,
      anon_sym_COMMA,
    ACTIONS(2764), 1,
      anon_sym_RBRACE,
    STATE(1994), 1,
      aux_sym_import_selectors_repeat1,
  [60558] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(2996), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [60571] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2998), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
  [60580] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(3000), 1,
      sym__string_end,
    STATE(1918), 1,
      aux_sym_string_repeat1,
  [60593] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(3002), 1,
      anon_sym_RPAREN,
    STATE(1952), 1,
      aux_sym_case_class_pattern_repeat1,
  [60606] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3004), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [60619] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3006), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [60632] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3008), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [60645] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3010), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [60658] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3012), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [60671] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3014), 3,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_RPAREN,
  [60680] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2917), 1,
      anon_sym_COMMA,
    ACTIONS(3016), 1,
      anon_sym_RPAREN,
    STATE(2029), 1,
      aux_sym_class_parameters_repeat1,
  [60693] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3018), 1,
      anon_sym_RPAREN,
    STATE(2024), 1,
      aux_sym_tuple_type_repeat1,
  [60706] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2917), 1,
      anon_sym_COMMA,
    ACTIONS(3016), 1,
      anon_sym_RPAREN,
    STATE(2025), 1,
      aux_sym_class_parameters_repeat1,
  [60719] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3020), 1,
      sym__multiline_string_middle,
    ACTIONS(3023), 1,
      sym__multiline_string_end,
    STATE(2015), 1,
      aux_sym_string_repeat2,
  [60732] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3025), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [60745] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(3027), 1,
      sym__string_end,
    STATE(2030), 1,
      aux_sym_string_repeat1,
  [60758] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(3027), 1,
      sym__multiline_string_end,
    STATE(2031), 1,
      aux_sym_string_repeat2,
  [60771] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(3029), 1,
      anon_sym_RPAREN,
    STATE(1952), 1,
      aux_sym_case_class_pattern_repeat1,
  [60784] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2864), 1,
      anon_sym_COMMA,
    ACTIONS(3031), 1,
      anon_sym_RBRACK,
    STATE(2022), 1,
      aux_sym_type_parameters_repeat1,
  [60797] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2840), 1,
      sym__multiline_string_end,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    STATE(2015), 1,
      aux_sym_string_repeat2,
  [60810] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3033), 1,
      anon_sym_COMMA,
    ACTIONS(3036), 1,
      anon_sym_RBRACK,
    STATE(2022), 1,
      aux_sym_type_parameters_repeat1,
  [60823] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3038), 1,
      anon_sym_RBRACK,
    STATE(2132), 1,
      aux_sym_tuple_type_repeat1,
  [60836] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3040), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [60849] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2917), 1,
      anon_sym_COMMA,
    ACTIONS(3042), 1,
      anon_sym_RPAREN,
    STATE(2029), 1,
      aux_sym_class_parameters_repeat1,
  [60862] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3046), 1,
      anon_sym_EQ,
    ACTIONS(3044), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [60873] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3048), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [60886] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3050), 1,
      anon_sym_RBRACK,
    STATE(2038), 1,
      aux_sym_tuple_type_repeat1,
  [60899] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3052), 1,
      anon_sym_COMMA,
    ACTIONS(3055), 1,
      anon_sym_RPAREN,
    STATE(2029), 1,
      aux_sym_class_parameters_repeat1,
  [60912] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(3057), 1,
      sym__string_end,
    STATE(1979), 1,
      aux_sym_string_repeat1,
  [60925] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(3057), 1,
      sym__multiline_string_end,
    STATE(2015), 1,
      aux_sym_string_repeat2,
  [60938] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3059), 1,
      anon_sym_RPAREN,
    STATE(1989), 1,
      aux_sym_tuple_type_repeat1,
  [60951] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3061), 1,
      anon_sym_RBRACK,
    STATE(2010), 1,
      aux_sym_tuple_type_repeat1,
  [60964] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3063), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [60977] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(3065), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [60988] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3068), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [61001] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3070), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [61014] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3072), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [61027] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3074), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [61040] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3076), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [61053] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3078), 1,
      anon_sym_RBRACK,
    STATE(1988), 1,
      aux_sym_tuple_type_repeat1,
  [61066] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(3080), 1,
      sym__string_end,
    STATE(1979), 1,
      aux_sym_string_repeat1,
  [61079] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3082), 1,
      anon_sym_RPAREN,
    STATE(2054), 1,
      aux_sym_tuple_type_repeat1,
  [61092] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3084), 1,
      anon_sym_RBRACK,
    STATE(2037), 1,
      aux_sym_tuple_type_repeat1,
  [61105] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(3000), 1,
      sym__multiline_string_end,
    STATE(2021), 1,
      aux_sym_string_repeat2,
  [61118] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3086), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [61131] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2860), 1,
      anon_sym_COMMA,
    ACTIONS(2862), 1,
      anon_sym_RPAREN,
    STATE(2059), 1,
      aux_sym_parameters_repeat1,
  [61144] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2842), 1,
      sym__multiline_string_end,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    STATE(2061), 1,
      aux_sym_string_repeat2,
  [61157] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3088), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [61170] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3090), 1,
      anon_sym_COMMA,
    ACTIONS(3093), 1,
      anon_sym_RPAREN,
    STATE(2050), 1,
      aux_sym_parameter_types_repeat1,
  [61183] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(3095), 1,
      anon_sym_COLON,
    STATE(2150), 1,
      aux_sym_val_declaration_repeat1,
  [61196] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2652), 1,
      anon_sym_STAR,
    ACTIONS(3093), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [61207] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2864), 1,
      anon_sym_COMMA,
    ACTIONS(3097), 1,
      anon_sym_RBRACK,
    STATE(2133), 1,
      aux_sym_type_parameters_repeat1,
  [61220] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3099), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [61233] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2146), 1,
      anon_sym_RPAREN,
    ACTIONS(3101), 1,
      anon_sym_COMMA,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [61246] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(3080), 1,
      sym__multiline_string_end,
    STATE(2015), 1,
      aux_sym_string_repeat2,
  [61259] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3104), 1,
      anon_sym_RPAREN,
    STATE(2034), 1,
      aux_sym_tuple_type_repeat1,
  [61272] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3106), 1,
      anon_sym_RBRACK,
    STATE(2067), 1,
      aux_sym_tuple_type_repeat1,
  [61285] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2860), 1,
      anon_sym_COMMA,
    ACTIONS(3108), 1,
      anon_sym_RPAREN,
    STATE(2080), 1,
      aux_sym_parameters_repeat1,
  [61298] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(3110), 1,
      sym__string_end,
    STATE(1979), 1,
      aux_sym_string_repeat1,
  [61311] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(3110), 1,
      sym__multiline_string_end,
    STATE(2015), 1,
      aux_sym_string_repeat2,
  [61324] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3112), 1,
      anon_sym_RBRACK,
    STATE(2049), 1,
      aux_sym_tuple_type_repeat1,
  [61337] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(3114), 1,
      anon_sym_RPAREN,
    STATE(1952), 1,
      aux_sym_case_class_pattern_repeat1,
  [61350] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(3116), 1,
      anon_sym_COLON,
    STATE(2150), 1,
      aux_sym_val_declaration_repeat1,
  [61363] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2860), 1,
      anon_sym_COMMA,
    ACTIONS(3118), 1,
      anon_sym_RPAREN,
    STATE(2080), 1,
      aux_sym_parameters_repeat1,
  [61376] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3120), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [61389] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3122), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [61402] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3124), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [61415] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3126), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [61428] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2917), 1,
      anon_sym_COMMA,
    ACTIONS(3128), 1,
      anon_sym_RPAREN,
    STATE(2029), 1,
      aux_sym_class_parameters_repeat1,
  [61441] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2736), 1,
      anon_sym_EQ,
    ACTIONS(2734), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [61452] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3130), 1,
      anon_sym_RPAREN,
    STATE(2079), 1,
      aux_sym_tuple_type_repeat1,
  [61465] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3132), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [61478] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2762), 1,
      anon_sym_COMMA,
    ACTIONS(3134), 1,
      anon_sym_RBRACE,
    STATE(2103), 1,
      aux_sym_import_selectors_repeat1,
  [61491] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3136), 1,
      anon_sym_RBRACK,
    STATE(2073), 1,
      aux_sym_tuple_type_repeat1,
  [61504] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(3138), 1,
      sym__string_end,
    STATE(2085), 1,
      aux_sym_string_repeat1,
  [61517] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(3138), 1,
      sym__multiline_string_end,
    STATE(2086), 1,
      aux_sym_string_repeat2,
  [61530] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3140), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [61543] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [61556] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3144), 1,
      anon_sym_COMMA,
    ACTIONS(3147), 1,
      anon_sym_RPAREN,
    STATE(2080), 1,
      aux_sym_parameters_repeat1,
  [61569] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(3149), 1,
      anon_sym_COLON,
    STATE(2150), 1,
      aux_sym_val_declaration_repeat1,
  [61582] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2569), 1,
      anon_sym_RPAREN,
    ACTIONS(2860), 1,
      anon_sym_COMMA,
    STATE(1927), 1,
      aux_sym_parameters_repeat1,
  [61595] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3151), 1,
      anon_sym_RBRACK,
    STATE(2088), 1,
      aux_sym_tuple_type_repeat1,
  [61608] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3153), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [61621] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(3155), 1,
      sym__string_end,
    STATE(1979), 1,
      aux_sym_string_repeat1,
  [61634] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(3155), 1,
      sym__multiline_string_end,
    STATE(2015), 1,
      aux_sym_string_repeat2,
  [61647] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3157), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [61660] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3159), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [61673] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3161), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [61686] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3165), 1,
      anon_sym_object,
    STATE(1718), 1,
      sym_package_identifier,
  [61699] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3167), 1,
      anon_sym_RPAREN,
    STATE(2098), 1,
      aux_sym_tuple_type_repeat1,
  [61712] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3169), 1,
      anon_sym_RBRACK,
    STATE(2087), 1,
      aux_sym_tuple_type_repeat1,
  [61725] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2282), 1,
      anon_sym_COMMA,
    ACTIONS(3171), 1,
      anon_sym_COLON,
    STATE(2150), 1,
      aux_sym_val_declaration_repeat1,
  [61738] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3173), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [61751] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(3175), 1,
      sym__string_end,
    STATE(2104), 1,
      aux_sym_string_repeat1,
  [61764] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(3175), 1,
      sym__multiline_string_end,
    STATE(2105), 1,
      aux_sym_string_repeat2,
  [61777] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3177), 1,
      sym_identifier,
    STATE(1709), 1,
      sym__import_expression,
    STATE(1717), 1,
      sym_stable_identifier,
  [61790] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3179), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [61803] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2766), 1,
      anon_sym_EQ_GT,
    ACTIONS(3181), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [61814] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3183), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [61827] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2860), 1,
      anon_sym_COMMA,
    ACTIONS(3185), 1,
      anon_sym_RPAREN,
    STATE(2080), 1,
      aux_sym_parameters_repeat1,
  [61840] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3187), 1,
      anon_sym_RBRACK,
    STATE(2107), 1,
      aux_sym_tuple_type_repeat1,
  [61853] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3181), 1,
      anon_sym_RBRACE,
    ACTIONS(3189), 1,
      anon_sym_COMMA,
    STATE(2103), 1,
      aux_sym_import_selectors_repeat1,
  [61866] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(3192), 1,
      sym__string_end,
    STATE(1979), 1,
      aux_sym_string_repeat1,
  [61879] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(3192), 1,
      sym__multiline_string_end,
    STATE(2015), 1,
      aux_sym_string_repeat2,
  [61892] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3194), 1,
      anon_sym_RBRACK,
    STATE(2100), 1,
      aux_sym_tuple_type_repeat1,
  [61905] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3196), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [61918] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3198), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [61931] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2860), 1,
      anon_sym_COMMA,
    ACTIONS(3185), 1,
      anon_sym_RPAREN,
    STATE(2065), 1,
      aux_sym_parameters_repeat1,
  [61944] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3200), 1,
      anon_sym_RPAREN,
    STATE(2117), 1,
      aux_sym_tuple_type_repeat1,
  [61957] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3202), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [61970] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(3204), 1,
      anon_sym_RPAREN,
    STATE(1952), 1,
      aux_sym_case_class_pattern_repeat1,
  [61983] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3206), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [61996] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(3208), 1,
      sym__string_end,
    STATE(2123), 1,
      aux_sym_string_repeat1,
  [62009] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(3208), 1,
      sym__multiline_string_end,
    STATE(2124), 1,
      aux_sym_string_repeat2,
  [62022] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(3210), 1,
      anon_sym_RPAREN,
    STATE(1952), 1,
      aux_sym_case_class_pattern_repeat1,
  [62035] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3212), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [62048] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3216), 1,
      anon_sym_EQ,
    ACTIONS(3214), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [62059] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3218), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [62072] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2770), 1,
      anon_sym_EQ,
    ACTIONS(2768), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [62083] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3220), 1,
      anon_sym_RBRACK,
    STATE(2126), 1,
      aux_sym_tuple_type_repeat1,
  [62096] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3222), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [62109] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(3224), 1,
      sym__string_end,
    STATE(1979), 1,
      aux_sym_string_repeat1,
  [62122] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(3224), 1,
      sym__multiline_string_end,
    STATE(2015), 1,
      aux_sym_string_repeat2,
  [62135] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3226), 1,
      anon_sym_RBRACK,
    STATE(2113), 1,
      aux_sym_tuple_type_repeat1,
  [62148] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3228), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [62161] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3230), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [62174] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2917), 1,
      anon_sym_COMMA,
    ACTIONS(3232), 1,
      anon_sym_RPAREN,
    STATE(2029), 1,
      aux_sym_class_parameters_repeat1,
  [62187] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3234), 1,
      anon_sym_RPAREN,
    STATE(2134), 1,
      aux_sym_tuple_type_repeat1,
  [62200] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2917), 1,
      anon_sym_COMMA,
    ACTIONS(3232), 1,
      anon_sym_RPAREN,
    STATE(2070), 1,
      aux_sym_class_parameters_repeat1,
  [62213] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3236), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [62226] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3238), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [62239] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2864), 1,
      anon_sym_COMMA,
    ACTIONS(3240), 1,
      anon_sym_RBRACK,
    STATE(2022), 1,
      aux_sym_type_parameters_repeat1,
  [62252] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3242), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [62265] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2762), 1,
      anon_sym_COMMA,
    ACTIONS(2772), 1,
      anon_sym_RBRACE,
    STATE(2074), 1,
      aux_sym_import_selectors_repeat1,
  [62278] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3246), 1,
      anon_sym_EQ,
    ACTIONS(3244), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [62289] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(735), 1,
      anon_sym_LBRACK,
    ACTIONS(3248), 1,
      anon_sym_EQ,
    STATE(2282), 1,
      sym_type_parameters,
  [62302] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3250), 1,
      anon_sym_RBRACK,
    STATE(2141), 1,
      aux_sym_tuple_type_repeat1,
  [62315] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2623), 1,
      anon_sym_RPAREN,
    ACTIONS(2860), 1,
      anon_sym_COMMA,
    STATE(2101), 1,
      aux_sym_parameters_repeat1,
  [62328] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3252), 1,
      anon_sym_RPAREN,
    STATE(2169), 1,
      aux_sym_tuple_type_repeat1,
  [62341] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3254), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [62354] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3256), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [62367] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3258), 1,
      sym_identifier,
    ACTIONS(3260), 1,
      anon_sym_LBRACE,
    STATE(2200), 1,
      sym_block,
  [62380] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3262), 1,
      anon_sym_RPAREN,
    STATE(2146), 1,
      aux_sym_tuple_type_repeat1,
  [62393] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(3264), 1,
      sym__string_end,
    STATE(2157), 1,
      aux_sym_string_repeat1,
  [62406] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3266), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [62419] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(3264), 1,
      sym__multiline_string_end,
    STATE(2155), 1,
      aux_sym_string_repeat2,
  [62432] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2266), 1,
      anon_sym_COMMA,
    ACTIONS(3268), 1,
      anon_sym_RPAREN,
    STATE(1952), 1,
      aux_sym_case_class_pattern_repeat1,
  [62445] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3270), 1,
      anon_sym_RBRACK,
    STATE(2151), 1,
      aux_sym_tuple_type_repeat1,
  [62458] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3272), 1,
      anon_sym_COMMA,
    ACTIONS(3275), 1,
      anon_sym_COLON,
    STATE(2150), 1,
      aux_sym_val_declaration_repeat1,
  [62471] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3277), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [62484] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3279), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [62497] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3177), 1,
      sym_identifier,
    STATE(1717), 1,
      sym_stable_identifier,
    STATE(1773), 1,
      sym__import_expression,
  [62510] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3281), 1,
      anon_sym_RPAREN,
    STATE(2156), 1,
      aux_sym_tuple_type_repeat1,
  [62523] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(3283), 1,
      sym__multiline_string_end,
    STATE(2015), 1,
      aux_sym_string_repeat2,
  [62536] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3285), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [62549] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(3283), 1,
      sym__string_end,
    STATE(1979), 1,
      aux_sym_string_repeat1,
  [62562] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3287), 1,
      anon_sym_RBRACK,
    STATE(2160), 1,
      aux_sym_tuple_type_repeat1,
  [62575] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3289), 1,
      anon_sym_RBRACK,
    STATE(2122), 1,
      aux_sym_tuple_type_repeat1,
  [62588] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3291), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [62601] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3293), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [62614] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      sym__multiline_string_middle,
    ACTIONS(3295), 1,
      sym__multiline_string_end,
    STATE(2056), 1,
      aux_sym_string_repeat2,
  [62627] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3297), 1,
      anon_sym_RBRACK,
    STATE(2165), 1,
      aux_sym_tuple_type_repeat1,
  [62640] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2968), 1,
      sym_identifier,
    STATE(548), 1,
      sym_stable_identifier,
    STATE(551), 1,
      sym__import_expression,
  [62653] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3299), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [62666] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3301), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [62679] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      sym__string_middle,
    ACTIONS(3295), 1,
      sym__string_end,
    STATE(2042), 1,
      aux_sym_string_repeat1,
  [62692] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3303), 1,
      anon_sym_RBRACK,
    STATE(2170), 1,
      aux_sym_tuple_type_repeat1,
  [62705] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3305), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [62718] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3307), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [62731] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3309), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [62744] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3311), 1,
      sym_identifier,
    ACTIONS(3313), 1,
      anon_sym_LBRACE,
    STATE(2202), 1,
      sym_block,
  [62757] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3315), 1,
      anon_sym_RBRACK,
    STATE(2175), 1,
      aux_sym_tuple_type_repeat1,
  [62770] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2466), 1,
      anon_sym_RPAREN,
    ACTIONS(2917), 1,
      anon_sym_COMMA,
    STATE(2128), 1,
      aux_sym_class_parameters_repeat1,
  [62783] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_COMMA,
    ACTIONS(3317), 1,
      anon_sym_RBRACK,
    STATE(1865), 1,
      aux_sym_tuple_type_repeat1,
  [62796] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2017), 1,
      anon_sym_COMMA,
    ACTIONS(3319), 1,
      anon_sym_RPAREN,
    STATE(2055), 1,
      aux_sym_tuple_expression_repeat1,
  [62809] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOLLAR,
    STATE(2147), 1,
      sym_interpolation,
  [62819] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3323), 1,
      anon_sym_DOLLAR,
    STATE(2239), 1,
      sym_interpolation,
  [62829] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3325), 1,
      sym_identifier,
    STATE(638), 1,
      sym__object_definition,
  [62839] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3327), 1,
      anon_sym_LPAREN,
    STATE(79), 1,
      sym_parenthesized_expression,
  [62849] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1392), 1,
      anon_sym_class,
    ACTIONS(3329), 1,
      anon_sym_object,
  [62859] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3331), 1,
      sym_identifier,
    STATE(1907), 1,
      sym__object_definition,
  [62869] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3331), 1,
      sym_identifier,
    STATE(1893), 1,
      sym__object_definition,
  [62879] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1404), 1,
      anon_sym_class,
    ACTIONS(3333), 1,
      anon_sym_object,
  [62889] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3335), 1,
      anon_sym_LBRACE,
    STATE(1286), 1,
      sym_case_block,
  [62899] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3331), 1,
      sym_identifier,
    STATE(1872), 1,
      sym__object_definition,
  [62909] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3335), 1,
      anon_sym_LBRACE,
    STATE(1313), 1,
      sym_case_block,
  [62919] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3337), 2,
      anon_sym_COMMA,
      anon_sym_COLON,
  [62927] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOLLAR,
    STATE(2115), 1,
      sym_interpolation,
  [62937] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3323), 1,
      anon_sym_DOLLAR,
    STATE(2114), 1,
      sym_interpolation,
  [62947] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3339), 2,
      anon_sym_RBRACE,
      anon_sym_case,
  [62955] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOLLAR,
    STATE(2096), 1,
      sym_interpolation,
  [62965] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3323), 1,
      anon_sym_DOLLAR,
    STATE(2095), 1,
      sym_interpolation,
  [62975] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3341), 2,
      sym_identifier,
      sym_wildcard,
  [62983] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOLLAR,
    STATE(2246), 1,
      sym_interpolation,
  [62993] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3343), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [63001] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3181), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [63009] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOLLAR,
    STATE(2077), 1,
      sym_interpolation,
  [63019] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3323), 1,
      anon_sym_DOLLAR,
    STATE(2076), 1,
      sym_interpolation,
  [63029] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3345), 2,
      sym__multiline_string_middle,
      sym__multiline_string_end,
  [63037] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(583), 2,
      sym__multiline_string_middle,
      sym__multiline_string_end,
  [63045] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3345), 2,
      sym__string_middle,
      sym__string_end,
  [63053] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3325), 1,
      sym_identifier,
    STATE(656), 1,
      sym__object_definition,
  [63063] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(583), 2,
      sym__string_middle,
      sym__string_end,
  [63071] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOLLAR,
    STATE(2048), 1,
      sym_interpolation,
  [63081] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3147), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [63089] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3323), 1,
      anon_sym_DOLLAR,
    STATE(1919), 1,
      sym_interpolation,
  [63099] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3347), 1,
      anon_sym_LBRACE,
    STATE(1285), 1,
      sym_case_block,
  [63109] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3349), 1,
      anon_sym_LBRACE,
    STATE(1061), 1,
      sym_case_block,
  [63119] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(587), 2,
      sym__multiline_string_middle,
      sym__multiline_string_end,
  [63127] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3327), 1,
      anon_sym_LPAREN,
    STATE(115), 1,
      sym_parenthesized_expression,
  [63137] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3325), 1,
      sym_identifier,
    STATE(644), 1,
      sym__object_definition,
  [63147] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3351), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [63155] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOLLAR,
    STATE(2018), 1,
      sym_interpolation,
  [63165] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3323), 1,
      anon_sym_DOLLAR,
    STATE(2017), 1,
      sym_interpolation,
  [63175] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3093), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [63183] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3353), 1,
      anon_sym_LBRACE,
    STATE(1398), 1,
      sym_case_block,
  [63193] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3355), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [63201] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3357), 2,
      anon_sym_RBRACE,
      anon_sym_case,
  [63209] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3327), 1,
      anon_sym_LPAREN,
    STATE(123), 1,
      sym_parenthesized_expression,
  [63219] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3055), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [63227] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3359), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [63235] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3361), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [63243] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOLLAR,
    STATE(1987), 1,
      sym_interpolation,
  [63253] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3323), 1,
      anon_sym_DOLLAR,
    STATE(1986), 1,
      sym_interpolation,
  [63263] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3347), 1,
      anon_sym_LBRACE,
    STATE(1340), 1,
      sym_case_block,
  [63273] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3363), 1,
      anon_sym_LBRACE,
    STATE(299), 1,
      sym_case_block,
  [63283] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3327), 1,
      anon_sym_LPAREN,
    STATE(134), 1,
      sym_parenthesized_expression,
  [63293] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3323), 1,
      anon_sym_DOLLAR,
    STATE(2145), 1,
      sym_interpolation,
  [63303] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3365), 1,
      anon_sym_LBRACE,
    STATE(481), 1,
      sym_case_block,
  [63313] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3323), 1,
      anon_sym_DOLLAR,
    STATE(2167), 1,
      sym_interpolation,
  [63323] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOLLAR,
    STATE(1956), 1,
      sym_interpolation,
  [63333] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3323), 1,
      anon_sym_DOLLAR,
    STATE(1955), 1,
      sym_interpolation,
  [63343] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3327), 1,
      anon_sym_LPAREN,
    STATE(152), 1,
      sym_parenthesized_expression,
  [63353] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOLLAR,
    STATE(2162), 1,
      sym_interpolation,
  [63363] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOLLAR,
    STATE(1925), 1,
      sym_interpolation,
  [63373] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3323), 1,
      anon_sym_DOLLAR,
    STATE(1924), 1,
      sym_interpolation,
  [63383] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3327), 1,
      anon_sym_LPAREN,
    STATE(158), 1,
      sym_parenthesized_expression,
  [63393] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2960), 2,
      sym__string_middle,
      sym__string_end,
  [63401] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3367), 1,
      anon_sym_LBRACE,
    STATE(912), 1,
      sym_case_block,
  [63411] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3353), 1,
      anon_sym_LBRACE,
    STATE(1453), 1,
      sym_case_block,
  [63421] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3369), 1,
      anon_sym_LBRACE,
    STATE(410), 1,
      sym_case_block,
  [63431] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOLLAR,
    STATE(1961), 1,
      sym_interpolation,
  [63441] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3323), 1,
      anon_sym_DOLLAR,
    STATE(1964), 1,
      sym_interpolation,
  [63451] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3371), 1,
      sym_identifier,
    STATE(2197), 1,
      sym_renamed_identifier,
  [63461] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3023), 2,
      sym__multiline_string_middle,
      sym__multiline_string_end,
  [63469] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3349), 1,
      anon_sym_LBRACE,
    STATE(1103), 1,
      sym_case_block,
  [63479] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3327), 1,
      anon_sym_LPAREN,
    STATE(263), 1,
      sym_parenthesized_expression,
  [63489] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3373), 1,
      sym_identifier,
    STATE(2001), 1,
      sym_renamed_identifier,
  [63499] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(587), 2,
      sym__string_middle,
      sym__string_end,
  [63507] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3375), 1,
      anon_sym_LBRACE,
    STATE(395), 1,
      sym_case_block,
  [63517] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOLLAR,
    STATE(1992), 1,
      sym_interpolation,
  [63527] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3323), 1,
      anon_sym_DOLLAR,
    STATE(1995), 1,
      sym_interpolation,
  [63537] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3367), 1,
      anon_sym_LBRACE,
    STATE(945), 1,
      sym_case_block,
  [63547] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3327), 1,
      anon_sym_LPAREN,
    STATE(183), 1,
      sym_parenthesized_expression,
  [63557] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3327), 1,
      anon_sym_LPAREN,
    STATE(258), 1,
      sym_parenthesized_expression,
  [63567] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3327), 1,
      anon_sym_LPAREN,
    STATE(203), 1,
      sym_parenthesized_expression,
  [63577] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3377), 1,
      sym_identifier,
    STATE(2135), 1,
      sym_renamed_identifier,
  [63587] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3363), 1,
      anon_sym_LBRACE,
    STATE(284), 1,
      sym_case_block,
  [63597] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3379), 1,
      anon_sym_LBRACE,
    STATE(1227), 1,
      sym_case_block,
  [63607] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOLLAR,
    STATE(2045), 1,
      sym_interpolation,
  [63617] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3323), 1,
      anon_sym_DOLLAR,
    STATE(2004), 1,
      sym_interpolation,
  [63627] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3375), 1,
      anon_sym_LBRACE,
    STATE(393), 1,
      sym_case_block,
  [63637] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3379), 1,
      anon_sym_LBRACE,
    STATE(1176), 1,
      sym_case_block,
  [63647] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3381), 1,
      sym_identifier,
  [63654] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3383), 1,
      sym_identifier,
  [63661] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3385), 1,
      sym_identifier,
  [63668] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3387), 1,
      sym_identifier,
  [63675] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3389), 1,
      anon_sym_EQ,
  [63682] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3391), 1,
      anon_sym_EQ,
  [63689] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3393), 1,
      sym_identifier,
  [63696] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(763), 1,
      anon_sym_DOT,
  [63703] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3395), 1,
      anon_sym_RBRACE,
  [63710] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3397), 1,
      sym_identifier,
  [63717] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3399), 1,
      anon_sym_LPAREN,
  [63724] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3401), 1,
      anon_sym_RBRACE,
  [63731] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3403), 1,
      anon_sym_EQ_GT,
  [63738] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3405), 1,
      anon_sym_EQ_GT,
  [63745] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3407), 1,
      sym_identifier,
  [63752] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3409), 1,
      sym_identifier,
  [63759] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3411), 1,
      sym_identifier,
  [63766] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3413), 1,
      anon_sym_EQ,
  [63773] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3415), 1,
      sym_identifier,
  [63780] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2108), 1,
      anon_sym_DOT,
  [63787] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3417), 1,
      anon_sym_class,
  [63794] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3419), 1,
      sym_identifier,
  [63801] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3421), 1,
      anon_sym_EQ,
  [63808] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3423), 1,
      anon_sym_EQ_GT,
  [63815] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3425), 1,
      sym_identifier,
  [63822] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3427), 1,
      sym_identifier,
  [63829] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3429), 1,
      anon_sym_RBRACE,
  [63836] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3431), 1,
      anon_sym_RBRACE,
  [63843] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(513), 1,
      anon_sym_DOT,
  [63850] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3433), 1,
      sym_identifier,
  [63857] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3435), 1,
      anon_sym_EQ_GT,
  [63864] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3437), 1,
      anon_sym_LPAREN,
  [63871] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3439), 1,
      anon_sym_EQ_GT,
  [63878] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3441), 1,
      anon_sym_RBRACE,
  [63885] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3443), 1,
      sym_identifier,
  [63892] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3445), 1,
      sym_identifier,
  [63899] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3447), 1,
      anon_sym_EQ_GT,
  [63906] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2085), 1,
      anon_sym_DOT,
  [63913] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2607), 1,
      anon_sym_class,
  [63920] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3449), 1,
      sym_identifier,
  [63927] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3451), 1,
      sym_identifier,
  [63934] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3453), 1,
      anon_sym_EQ_GT,
  [63941] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3455), 1,
      sym_identifier,
  [63948] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3457), 1,
      sym_identifier,
  [63955] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3459), 1,
      anon_sym_class,
  [63962] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3461), 1,
      sym_identifier,
  [63969] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(599), 1,
      anon_sym_DOT,
  [63976] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3463), 1,
      anon_sym_LPAREN,
  [63983] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3465), 1,
      anon_sym_RBRACE,
  [63990] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3467), 1,
      sym_identifier,
  [63997] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3469), 1,
      anon_sym_EQ_GT,
  [64004] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3471), 1,
      anon_sym_RBRACE,
  [64011] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3473), 1,
      sym_identifier,
  [64018] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2072), 1,
      anon_sym_DOT,
  [64025] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3475), 1,
      sym_identifier,
  [64032] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1872), 1,
      anon_sym_DOT,
  [64039] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3477), 1,
      sym_identifier,
  [64046] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3479), 1,
      sym_identifier,
  [64053] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1088), 1,
      anon_sym_DOT,
  [64060] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3481), 1,
      anon_sym_EQ_GT,
  [64067] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3483), 1,
      anon_sym_RBRACE,
  [64074] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3485), 1,
      sym_identifier,
  [64081] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3487), 1,
      anon_sym_RBRACE,
  [64088] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3489), 1,
      sym_identifier,
  [64095] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1848), 1,
      anon_sym_DOT,
  [64102] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3491), 1,
      sym_identifier,
  [64109] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3493), 1,
      anon_sym_EQ,
  [64116] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3495), 1,
      sym_identifier,
  [64123] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3497), 1,
      anon_sym_EQ_GT,
  [64130] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3499), 1,
      sym_identifier,
  [64137] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3501), 1,
      anon_sym_EQ_GT,
  [64144] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3503), 1,
      anon_sym_RBRACE,
  [64151] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3505), 1,
      sym_identifier,
  [64158] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1954), 1,
      anon_sym_DOT,
  [64165] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3507), 1,
      sym_identifier,
  [64172] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3509), 1,
      sym_identifier,
  [64179] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3511), 1,
      anon_sym_EQ_GT,
  [64186] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3513), 1,
      anon_sym_EQ_GT,
  [64193] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3515), 1,
      anon_sym_RBRACE,
  [64200] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3517), 1,
      sym_identifier,
  [64207] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3519), 1,
      sym_identifier,
  [64214] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2096), 1,
      anon_sym_DOT,
  [64221] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3521), 1,
      sym_identifier,
  [64228] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3523), 1,
      anon_sym_EQ_GT,
  [64235] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3525), 1,
      ts_builtin_sym_end,
  [64242] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3527), 1,
      anon_sym_EQ_GT,
  [64249] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3529), 1,
      anon_sym_RBRACE,
  [64256] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3531), 1,
      sym_identifier,
  [64263] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2124), 1,
      anon_sym_DOT,
  [64270] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3533), 1,
      sym_identifier,
  [64277] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3535), 1,
      sym_identifier,
  [64284] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3537), 1,
      sym_identifier,
  [64291] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_EQ_GT,
  [64298] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3541), 1,
      sym_identifier,
  [64305] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3543), 1,
      sym_identifier,
  [64312] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2013), 1,
      anon_sym_DOT,
  [64319] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3545), 1,
      anon_sym_RBRACE,
  [64326] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3547), 1,
      sym_identifier,
  [64333] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3549), 1,
      sym_identifier,
  [64340] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3551), 1,
      anon_sym_EQ_GT,
  [64347] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3553), 1,
      sym_identifier,
  [64354] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3555), 1,
      sym_identifier,
  [64361] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1930), 1,
      anon_sym_DOT,
  [64368] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3557), 1,
      anon_sym_EQ,
  [64375] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3559), 1,
      anon_sym_EQ_GT,
  [64382] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3561), 1,
      sym_identifier,
  [64389] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3563), 1,
      anon_sym_RBRACE,
  [64396] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1572), 1,
      anon_sym_DOT,
  [64403] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3565), 1,
      sym_identifier,
  [64410] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3567), 1,
      anon_sym_EQ_GT,
  [64417] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3569), 1,
      anon_sym_RBRACE,
  [64424] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3571), 1,
      sym_identifier,
  [64431] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1758), 1,
      anon_sym_DOT,
  [64438] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3573), 1,
      anon_sym_EQ_GT,
  [64445] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3575), 1,
      sym_identifier,
  [64452] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3577), 1,
      sym_identifier,
  [64459] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3579), 1,
      sym_identifier,
  [64466] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3581), 1,
      sym_identifier,
  [64473] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3583), 1,
      sym_identifier,
  [64480] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3585), 1,
      sym_identifier,
  [64487] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3587), 1,
      sym_identifier,
  [64494] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3589), 1,
      sym_identifier,
  [64501] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3591), 1,
      sym_identifier,
  [64508] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3593), 1,
      anon_sym_RBRACE,
  [64515] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3595), 1,
      sym_identifier,
  [64522] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3597), 1,
      sym_identifier,
  [64529] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3599), 1,
      sym_identifier,
  [64536] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2595), 1,
      anon_sym_class,
  [64543] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3601), 1,
      sym_identifier,
  [64550] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3603), 1,
      sym_identifier,
  [64557] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3605), 1,
      sym_identifier,
  [64564] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3607), 1,
      sym_identifier,
  [64571] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3609), 1,
      sym_identifier,
  [64578] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3611), 1,
      sym_identifier,
  [64585] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3613), 1,
      sym_identifier,
};

static uint32_t ts_small_parse_table_map[] = {
  [SMALL_STATE(25)] = 0,
  [SMALL_STATE(26)] = 76,
  [SMALL_STATE(27)] = 152,
  [SMALL_STATE(28)] = 218,
  [SMALL_STATE(29)] = 284,
  [SMALL_STATE(30)] = 334,
  [SMALL_STATE(31)] = 399,
  [SMALL_STATE(32)] = 450,
  [SMALL_STATE(33)] = 499,
  [SMALL_STATE(34)] = 544,
  [SMALL_STATE(35)] = 609,
  [SMALL_STATE(36)] = 673,
  [SMALL_STATE(37)] = 737,
  [SMALL_STATE(38)] = 787,
  [SMALL_STATE(39)] = 841,
  [SMALL_STATE(40)] = 891,
  [SMALL_STATE(41)] = 941,
  [SMALL_STATE(42)] = 997,
  [SMALL_STATE(43)] = 1053,
  [SMALL_STATE(44)] = 1101,
  [SMALL_STATE(45)] = 1151,
  [SMALL_STATE(46)] = 1215,
  [SMALL_STATE(47)] = 1273,
  [SMALL_STATE(48)] = 1337,
  [SMALL_STATE(49)] = 1401,
  [SMALL_STATE(50)] = 1465,
  [SMALL_STATE(51)] = 1529,
  [SMALL_STATE(52)] = 1593,
  [SMALL_STATE(53)] = 1637,
  [SMALL_STATE(54)] = 1695,
  [SMALL_STATE(55)] = 1759,
  [SMALL_STATE(56)] = 1823,
  [SMALL_STATE(57)] = 1887,
  [SMALL_STATE(58)] = 1951,
  [SMALL_STATE(59)] = 2015,
  [SMALL_STATE(60)] = 2079,
  [SMALL_STATE(61)] = 2143,
  [SMALL_STATE(62)] = 2207,
  [SMALL_STATE(63)] = 2271,
  [SMALL_STATE(64)] = 2335,
  [SMALL_STATE(65)] = 2399,
  [SMALL_STATE(66)] = 2463,
  [SMALL_STATE(67)] = 2527,
  [SMALL_STATE(68)] = 2591,
  [SMALL_STATE(69)] = 2655,
  [SMALL_STATE(70)] = 2719,
  [SMALL_STATE(71)] = 2769,
  [SMALL_STATE(72)] = 2833,
  [SMALL_STATE(73)] = 2897,
  [SMALL_STATE(74)] = 2961,
  [SMALL_STATE(75)] = 3025,
  [SMALL_STATE(76)] = 3086,
  [SMALL_STATE(77)] = 3147,
  [SMALL_STATE(78)] = 3208,
  [SMALL_STATE(79)] = 3269,
  [SMALL_STATE(80)] = 3330,
  [SMALL_STATE(81)] = 3391,
  [SMALL_STATE(82)] = 3452,
  [SMALL_STATE(83)] = 3513,
  [SMALL_STATE(84)] = 3574,
  [SMALL_STATE(85)] = 3635,
  [SMALL_STATE(86)] = 3696,
  [SMALL_STATE(87)] = 3745,
  [SMALL_STATE(88)] = 3792,
  [SMALL_STATE(89)] = 3853,
  [SMALL_STATE(90)] = 3914,
  [SMALL_STATE(91)] = 3975,
  [SMALL_STATE(92)] = 4036,
  [SMALL_STATE(93)] = 4097,
  [SMALL_STATE(94)] = 4158,
  [SMALL_STATE(95)] = 4207,
  [SMALL_STATE(96)] = 4256,
  [SMALL_STATE(97)] = 4305,
  [SMALL_STATE(98)] = 4366,
  [SMALL_STATE(99)] = 4427,
  [SMALL_STATE(100)] = 4488,
  [SMALL_STATE(101)] = 4549,
  [SMALL_STATE(102)] = 4610,
  [SMALL_STATE(103)] = 4671,
  [SMALL_STATE(104)] = 4732,
  [SMALL_STATE(105)] = 4785,
  [SMALL_STATE(106)] = 4846,
  [SMALL_STATE(107)] = 4907,
  [SMALL_STATE(108)] = 4968,
  [SMALL_STATE(109)] = 5029,
  [SMALL_STATE(110)] = 5090,
  [SMALL_STATE(111)] = 5151,
  [SMALL_STATE(112)] = 5200,
  [SMALL_STATE(113)] = 5261,
  [SMALL_STATE(114)] = 5322,
  [SMALL_STATE(115)] = 5383,
  [SMALL_STATE(116)] = 5444,
  [SMALL_STATE(117)] = 5505,
  [SMALL_STATE(118)] = 5566,
  [SMALL_STATE(119)] = 5627,
  [SMALL_STATE(120)] = 5688,
  [SMALL_STATE(121)] = 5749,
  [SMALL_STATE(122)] = 5804,
  [SMALL_STATE(123)] = 5865,
  [SMALL_STATE(124)] = 5926,
  [SMALL_STATE(125)] = 5987,
  [SMALL_STATE(126)] = 6048,
  [SMALL_STATE(127)] = 6109,
  [SMALL_STATE(128)] = 6170,
  [SMALL_STATE(129)] = 6225,
  [SMALL_STATE(130)] = 6286,
  [SMALL_STATE(131)] = 6347,
  [SMALL_STATE(132)] = 6408,
  [SMALL_STATE(133)] = 6469,
  [SMALL_STATE(134)] = 6530,
  [SMALL_STATE(135)] = 6591,
  [SMALL_STATE(136)] = 6652,
  [SMALL_STATE(137)] = 6713,
  [SMALL_STATE(138)] = 6774,
  [SMALL_STATE(139)] = 6835,
  [SMALL_STATE(140)] = 6896,
  [SMALL_STATE(141)] = 6957,
  [SMALL_STATE(142)] = 7018,
  [SMALL_STATE(143)] = 7079,
  [SMALL_STATE(144)] = 7140,
  [SMALL_STATE(145)] = 7201,
  [SMALL_STATE(146)] = 7242,
  [SMALL_STATE(147)] = 7303,
  [SMALL_STATE(148)] = 7364,
  [SMALL_STATE(149)] = 7425,
  [SMALL_STATE(150)] = 7486,
  [SMALL_STATE(151)] = 7547,
  [SMALL_STATE(152)] = 7608,
  [SMALL_STATE(153)] = 7669,
  [SMALL_STATE(154)] = 7730,
  [SMALL_STATE(155)] = 7791,
  [SMALL_STATE(156)] = 7852,
  [SMALL_STATE(157)] = 7913,
  [SMALL_STATE(158)] = 7956,
  [SMALL_STATE(159)] = 8017,
  [SMALL_STATE(160)] = 8078,
  [SMALL_STATE(161)] = 8139,
  [SMALL_STATE(162)] = 8200,
  [SMALL_STATE(163)] = 8261,
  [SMALL_STATE(164)] = 8302,
  [SMALL_STATE(165)] = 8363,
  [SMALL_STATE(166)] = 8424,
  [SMALL_STATE(167)] = 8485,
  [SMALL_STATE(168)] = 8546,
  [SMALL_STATE(169)] = 8607,
  [SMALL_STATE(170)] = 8668,
  [SMALL_STATE(171)] = 8729,
  [SMALL_STATE(172)] = 8790,
  [SMALL_STATE(173)] = 8851,
  [SMALL_STATE(174)] = 8912,
  [SMALL_STATE(175)] = 8973,
  [SMALL_STATE(176)] = 9034,
  [SMALL_STATE(177)] = 9095,
  [SMALL_STATE(178)] = 9156,
  [SMALL_STATE(179)] = 9217,
  [SMALL_STATE(180)] = 9278,
  [SMALL_STATE(181)] = 9339,
  [SMALL_STATE(182)] = 9400,
  [SMALL_STATE(183)] = 9461,
  [SMALL_STATE(184)] = 9522,
  [SMALL_STATE(185)] = 9583,
  [SMALL_STATE(186)] = 9644,
  [SMALL_STATE(187)] = 9705,
  [SMALL_STATE(188)] = 9766,
  [SMALL_STATE(189)] = 9827,
  [SMALL_STATE(190)] = 9888,
  [SMALL_STATE(191)] = 9949,
  [SMALL_STATE(192)] = 10010,
  [SMALL_STATE(193)] = 10071,
  [SMALL_STATE(194)] = 10132,
  [SMALL_STATE(195)] = 10193,
  [SMALL_STATE(196)] = 10254,
  [SMALL_STATE(197)] = 10315,
  [SMALL_STATE(198)] = 10376,
  [SMALL_STATE(199)] = 10437,
  [SMALL_STATE(200)] = 10498,
  [SMALL_STATE(201)] = 10559,
  [SMALL_STATE(202)] = 10620,
  [SMALL_STATE(203)] = 10681,
  [SMALL_STATE(204)] = 10742,
  [SMALL_STATE(205)] = 10803,
  [SMALL_STATE(206)] = 10864,
  [SMALL_STATE(207)] = 10925,
  [SMALL_STATE(208)] = 10986,
  [SMALL_STATE(209)] = 11047,
  [SMALL_STATE(210)] = 11108,
  [SMALL_STATE(211)] = 11169,
  [SMALL_STATE(212)] = 11230,
  [SMALL_STATE(213)] = 11279,
  [SMALL_STATE(214)] = 11340,
  [SMALL_STATE(215)] = 11401,
  [SMALL_STATE(216)] = 11462,
  [SMALL_STATE(217)] = 11523,
  [SMALL_STATE(218)] = 11584,
  [SMALL_STATE(219)] = 11645,
  [SMALL_STATE(220)] = 11706,
  [SMALL_STATE(221)] = 11767,
  [SMALL_STATE(222)] = 11828,
  [SMALL_STATE(223)] = 11889,
  [SMALL_STATE(224)] = 11950,
  [SMALL_STATE(225)] = 12011,
  [SMALL_STATE(226)] = 12072,
  [SMALL_STATE(227)] = 12133,
  [SMALL_STATE(228)] = 12194,
  [SMALL_STATE(229)] = 12255,
  [SMALL_STATE(230)] = 12316,
  [SMALL_STATE(231)] = 12355,
  [SMALL_STATE(232)] = 12394,
  [SMALL_STATE(233)] = 12433,
  [SMALL_STATE(234)] = 12494,
  [SMALL_STATE(235)] = 12555,
  [SMALL_STATE(236)] = 12616,
  [SMALL_STATE(237)] = 12677,
  [SMALL_STATE(238)] = 12738,
  [SMALL_STATE(239)] = 12799,
  [SMALL_STATE(240)] = 12860,
  [SMALL_STATE(241)] = 12921,
  [SMALL_STATE(242)] = 12982,
  [SMALL_STATE(243)] = 13043,
  [SMALL_STATE(244)] = 13104,
  [SMALL_STATE(245)] = 13165,
  [SMALL_STATE(246)] = 13208,
  [SMALL_STATE(247)] = 13269,
  [SMALL_STATE(248)] = 13330,
  [SMALL_STATE(249)] = 13391,
  [SMALL_STATE(250)] = 13452,
  [SMALL_STATE(251)] = 13513,
  [SMALL_STATE(252)] = 13574,
  [SMALL_STATE(253)] = 13635,
  [SMALL_STATE(254)] = 13696,
  [SMALL_STATE(255)] = 13757,
  [SMALL_STATE(256)] = 13818,
  [SMALL_STATE(257)] = 13879,
  [SMALL_STATE(258)] = 13940,
  [SMALL_STATE(259)] = 14001,
  [SMALL_STATE(260)] = 14062,
  [SMALL_STATE(261)] = 14123,
  [SMALL_STATE(262)] = 14184,
  [SMALL_STATE(263)] = 14245,
  [SMALL_STATE(264)] = 14306,
  [SMALL_STATE(265)] = 14367,
  [SMALL_STATE(266)] = 14428,
  [SMALL_STATE(267)] = 14489,
  [SMALL_STATE(268)] = 14525,
  [SMALL_STATE(269)] = 14565,
  [SMALL_STATE(270)] = 14619,
  [SMALL_STATE(271)] = 14673,
  [SMALL_STATE(272)] = 14711,
  [SMALL_STATE(273)] = 14749,
  [SMALL_STATE(274)] = 14797,
  [SMALL_STATE(275)] = 14849,
  [SMALL_STATE(276)] = 14891,
  [SMALL_STATE(277)] = 14929,
  [SMALL_STATE(278)] = 14967,
  [SMALL_STATE(279)] = 15015,
  [SMALL_STATE(280)] = 15063,
  [SMALL_STATE(281)] = 15101,
  [SMALL_STATE(282)] = 15139,
  [SMALL_STATE(283)] = 15177,
  [SMALL_STATE(284)] = 15217,
  [SMALL_STATE(285)] = 15255,
  [SMALL_STATE(286)] = 15293,
  [SMALL_STATE(287)] = 15331,
  [SMALL_STATE(288)] = 15367,
  [SMALL_STATE(289)] = 15405,
  [SMALL_STATE(290)] = 15447,
  [SMALL_STATE(291)] = 15485,
  [SMALL_STATE(292)] = 15523,
  [SMALL_STATE(293)] = 15561,
  [SMALL_STATE(294)] = 15599,
  [SMALL_STATE(295)] = 15637,
  [SMALL_STATE(296)] = 15675,
  [SMALL_STATE(297)] = 15713,
  [SMALL_STATE(298)] = 15751,
  [SMALL_STATE(299)] = 15807,
  [SMALL_STATE(300)] = 15845,
  [SMALL_STATE(301)] = 15883,
  [SMALL_STATE(302)] = 15925,
  [SMALL_STATE(303)] = 15963,
  [SMALL_STATE(304)] = 16001,
  [SMALL_STATE(305)] = 16037,
  [SMALL_STATE(306)] = 16075,
  [SMALL_STATE(307)] = 16113,
  [SMALL_STATE(308)] = 16151,
  [SMALL_STATE(309)] = 16189,
  [SMALL_STATE(310)] = 16227,
  [SMALL_STATE(311)] = 16265,
  [SMALL_STATE(312)] = 16307,
  [SMALL_STATE(313)] = 16363,
  [SMALL_STATE(314)] = 16405,
  [SMALL_STATE(315)] = 16458,
  [SMALL_STATE(316)] = 16495,
  [SMALL_STATE(317)] = 16532,
  [SMALL_STATE(318)] = 16573,
  [SMALL_STATE(319)] = 16626,
  [SMALL_STATE(320)] = 16679,
  [SMALL_STATE(321)] = 16732,
  [SMALL_STATE(322)] = 16785,
  [SMALL_STATE(323)] = 16822,
  [SMALL_STATE(324)] = 16859,
  [SMALL_STATE(325)] = 16912,
  [SMALL_STATE(326)] = 16949,
  [SMALL_STATE(327)] = 16986,
  [SMALL_STATE(328)] = 17039,
  [SMALL_STATE(329)] = 17090,
  [SMALL_STATE(330)] = 17127,
  [SMALL_STATE(331)] = 17164,
  [SMALL_STATE(332)] = 17217,
  [SMALL_STATE(333)] = 17270,
  [SMALL_STATE(334)] = 17323,
  [SMALL_STATE(335)] = 17374,
  [SMALL_STATE(336)] = 17413,
  [SMALL_STATE(337)] = 17450,
  [SMALL_STATE(338)] = 17491,
  [SMALL_STATE(339)] = 17542,
  [SMALL_STATE(340)] = 17579,
  [SMALL_STATE(341)] = 17616,
  [SMALL_STATE(342)] = 17653,
  [SMALL_STATE(343)] = 17690,
  [SMALL_STATE(344)] = 17727,
  [SMALL_STATE(345)] = 17764,
  [SMALL_STATE(346)] = 17817,
  [SMALL_STATE(347)] = 17854,
  [SMALL_STATE(348)] = 17891,
  [SMALL_STATE(349)] = 17928,
  [SMALL_STATE(350)] = 17981,
  [SMALL_STATE(351)] = 18034,
  [SMALL_STATE(352)] = 18075,
  [SMALL_STATE(353)] = 18114,
  [SMALL_STATE(354)] = 18151,
  [SMALL_STATE(355)] = 18202,
  [SMALL_STATE(356)] = 18255,
  [SMALL_STATE(357)] = 18302,
  [SMALL_STATE(358)] = 18339,
  [SMALL_STATE(359)] = 18376,
  [SMALL_STATE(360)] = 18427,
  [SMALL_STATE(361)] = 18474,
  [SMALL_STATE(362)] = 18527,
  [SMALL_STATE(363)] = 18564,
  [SMALL_STATE(364)] = 18617,
  [SMALL_STATE(365)] = 18658,
  [SMALL_STATE(366)] = 18711,
  [SMALL_STATE(367)] = 18764,
  [SMALL_STATE(368)] = 18817,
  [SMALL_STATE(369)] = 18854,
  [SMALL_STATE(370)] = 18891,
  [SMALL_STATE(371)] = 18932,
  [SMALL_STATE(372)] = 18985,
  [SMALL_STATE(373)] = 19038,
  [SMALL_STATE(374)] = 19091,
  [SMALL_STATE(375)] = 19144,
  [SMALL_STATE(376)] = 19197,
  [SMALL_STATE(377)] = 19250,
  [SMALL_STATE(378)] = 19303,
  [SMALL_STATE(379)] = 19354,
  [SMALL_STATE(380)] = 19391,
  [SMALL_STATE(381)] = 19444,
  [SMALL_STATE(382)] = 19497,
  [SMALL_STATE(383)] = 19550,
  [SMALL_STATE(384)] = 19603,
  [SMALL_STATE(385)] = 19656,
  [SMALL_STATE(386)] = 19709,
  [SMALL_STATE(387)] = 19762,
  [SMALL_STATE(388)] = 19809,
  [SMALL_STATE(389)] = 19862,
  [SMALL_STATE(390)] = 19899,
  [SMALL_STATE(391)] = 19952,
  [SMALL_STATE(392)] = 20005,
  [SMALL_STATE(393)] = 20056,
  [SMALL_STATE(394)] = 20093,
  [SMALL_STATE(395)] = 20146,
  [SMALL_STATE(396)] = 20183,
  [SMALL_STATE(397)] = 20220,
  [SMALL_STATE(398)] = 20257,
  [SMALL_STATE(399)] = 20310,
  [SMALL_STATE(400)] = 20361,
  [SMALL_STATE(401)] = 20397,
  [SMALL_STATE(402)] = 20433,
  [SMALL_STATE(403)] = 20469,
  [SMALL_STATE(404)] = 20505,
  [SMALL_STATE(405)] = 20541,
  [SMALL_STATE(406)] = 20577,
  [SMALL_STATE(407)] = 20617,
  [SMALL_STATE(408)] = 20657,
  [SMALL_STATE(409)] = 20693,
  [SMALL_STATE(410)] = 20737,
  [SMALL_STATE(411)] = 20773,
  [SMALL_STATE(412)] = 20809,
  [SMALL_STATE(413)] = 20845,
  [SMALL_STATE(414)] = 20885,
  [SMALL_STATE(415)] = 20921,
  [SMALL_STATE(416)] = 20961,
  [SMALL_STATE(417)] = 20997,
  [SMALL_STATE(418)] = 21033,
  [SMALL_STATE(419)] = 21069,
  [SMALL_STATE(420)] = 21109,
  [SMALL_STATE(421)] = 21145,
  [SMALL_STATE(422)] = 21181,
  [SMALL_STATE(423)] = 21217,
  [SMALL_STATE(424)] = 21253,
  [SMALL_STATE(425)] = 21293,
  [SMALL_STATE(426)] = 21329,
  [SMALL_STATE(427)] = 21365,
  [SMALL_STATE(428)] = 21401,
  [SMALL_STATE(429)] = 21437,
  [SMALL_STATE(430)] = 21473,
  [SMALL_STATE(431)] = 21509,
  [SMALL_STATE(432)] = 21545,
  [SMALL_STATE(433)] = 21581,
  [SMALL_STATE(434)] = 21617,
  [SMALL_STATE(435)] = 21657,
  [SMALL_STATE(436)] = 21693,
  [SMALL_STATE(437)] = 21729,
  [SMALL_STATE(438)] = 21769,
  [SMALL_STATE(439)] = 21809,
  [SMALL_STATE(440)] = 21845,
  [SMALL_STATE(441)] = 21881,
  [SMALL_STATE(442)] = 21917,
  [SMALL_STATE(443)] = 21953,
  [SMALL_STATE(444)] = 21988,
  [SMALL_STATE(445)] = 22033,
  [SMALL_STATE(446)] = 22072,
  [SMALL_STATE(447)] = 22107,
  [SMALL_STATE(448)] = 22152,
  [SMALL_STATE(449)] = 22187,
  [SMALL_STATE(450)] = 22232,
  [SMALL_STATE(451)] = 22277,
  [SMALL_STATE(452)] = 22322,
  [SMALL_STATE(453)] = 22361,
  [SMALL_STATE(454)] = 22406,
  [SMALL_STATE(455)] = 22451,
  [SMALL_STATE(456)] = 22496,
  [SMALL_STATE(457)] = 22541,
  [SMALL_STATE(458)] = 22576,
  [SMALL_STATE(459)] = 22611,
  [SMALL_STATE(460)] = 22646,
  [SMALL_STATE(461)] = 22681,
  [SMALL_STATE(462)] = 22726,
  [SMALL_STATE(463)] = 22761,
  [SMALL_STATE(464)] = 22796,
  [SMALL_STATE(465)] = 22841,
  [SMALL_STATE(466)] = 22884,
  [SMALL_STATE(467)] = 22919,
  [SMALL_STATE(468)] = 22958,
  [SMALL_STATE(469)] = 22993,
  [SMALL_STATE(470)] = 23032,
  [SMALL_STATE(471)] = 23077,
  [SMALL_STATE(472)] = 23122,
  [SMALL_STATE(473)] = 23167,
  [SMALL_STATE(474)] = 23206,
  [SMALL_STATE(475)] = 23241,
  [SMALL_STATE(476)] = 23276,
  [SMALL_STATE(477)] = 23311,
  [SMALL_STATE(478)] = 23346,
  [SMALL_STATE(479)] = 23391,
  [SMALL_STATE(480)] = 23426,
  [SMALL_STATE(481)] = 23471,
  [SMALL_STATE(482)] = 23506,
  [SMALL_STATE(483)] = 23541,
  [SMALL_STATE(484)] = 23586,
  [SMALL_STATE(485)] = 23631,
  [SMALL_STATE(486)] = 23666,
  [SMALL_STATE(487)] = 23711,
  [SMALL_STATE(488)] = 23750,
  [SMALL_STATE(489)] = 23785,
  [SMALL_STATE(490)] = 23820,
  [SMALL_STATE(491)] = 23865,
  [SMALL_STATE(492)] = 23908,
  [SMALL_STATE(493)] = 23953,
  [SMALL_STATE(494)] = 23988,
  [SMALL_STATE(495)] = 24023,
  [SMALL_STATE(496)] = 24057,
  [SMALL_STATE(497)] = 24095,
  [SMALL_STATE(498)] = 24129,
  [SMALL_STATE(499)] = 24165,
  [SMALL_STATE(500)] = 24209,
  [SMALL_STATE(501)] = 24243,
  [SMALL_STATE(502)] = 24277,
  [SMALL_STATE(503)] = 24311,
  [SMALL_STATE(504)] = 24345,
  [SMALL_STATE(505)] = 24382,
  [SMALL_STATE(506)] = 24415,
  [SMALL_STATE(507)] = 24452,
  [SMALL_STATE(508)] = 24489,
  [SMALL_STATE(509)] = 24526,
  [SMALL_STATE(510)] = 24561,
  [SMALL_STATE(511)] = 24598,
  [SMALL_STATE(512)] = 24635,
  [SMALL_STATE(513)] = 24672,
  [SMALL_STATE(514)] = 24705,
  [SMALL_STATE(515)] = 24742,
  [SMALL_STATE(516)] = 24775,
  [SMALL_STATE(517)] = 24805,
  [SMALL_STATE(518)] = 24843,
  [SMALL_STATE(519)] = 24873,
  [SMALL_STATE(520)] = 24903,
  [SMALL_STATE(521)] = 24933,
  [SMALL_STATE(522)] = 24971,
  [SMALL_STATE(523)] = 25004,
  [SMALL_STATE(524)] = 25039,
  [SMALL_STATE(525)] = 25074,
  [SMALL_STATE(526)] = 25109,
  [SMALL_STATE(527)] = 25142,
  [SMALL_STATE(528)] = 25177,
  [SMALL_STATE(529)] = 25206,
  [SMALL_STATE(530)] = 25241,
  [SMALL_STATE(531)] = 25276,
  [SMALL_STATE(532)] = 25305,
  [SMALL_STATE(533)] = 25340,
  [SMALL_STATE(534)] = 25369,
  [SMALL_STATE(535)] = 25402,
  [SMALL_STATE(536)] = 25437,
  [SMALL_STATE(537)] = 25470,
  [SMALL_STATE(538)] = 25505,
  [SMALL_STATE(539)] = 25540,
  [SMALL_STATE(540)] = 25575,
  [SMALL_STATE(541)] = 25608,
  [SMALL_STATE(542)] = 25643,
  [SMALL_STATE(543)] = 25676,
  [SMALL_STATE(544)] = 25705,
  [SMALL_STATE(545)] = 25737,
  [SMALL_STATE(546)] = 25769,
  [SMALL_STATE(547)] = 25801,
  [SMALL_STATE(548)] = 25833,
  [SMALL_STATE(549)] = 25863,
  [SMALL_STATE(550)] = 25893,
  [SMALL_STATE(551)] = 25921,
  [SMALL_STATE(552)] = 25953,
  [SMALL_STATE(553)] = 25985,
  [SMALL_STATE(554)] = 26015,
  [SMALL_STATE(555)] = 26047,
  [SMALL_STATE(556)] = 26079,
  [SMALL_STATE(557)] = 26111,
  [SMALL_STATE(558)] = 26141,
  [SMALL_STATE(559)] = 26171,
  [SMALL_STATE(560)] = 26203,
  [SMALL_STATE(561)] = 26231,
  [SMALL_STATE(562)] = 26263,
  [SMALL_STATE(563)] = 26295,
  [SMALL_STATE(564)] = 26327,
  [SMALL_STATE(565)] = 26359,
  [SMALL_STATE(566)] = 26387,
  [SMALL_STATE(567)] = 26417,
  [SMALL_STATE(568)] = 26449,
  [SMALL_STATE(569)] = 26481,
  [SMALL_STATE(570)] = 26513,
  [SMALL_STATE(571)] = 26545,
  [SMALL_STATE(572)] = 26577,
  [SMALL_STATE(573)] = 26609,
  [SMALL_STATE(574)] = 26637,
  [SMALL_STATE(575)] = 26669,
  [SMALL_STATE(576)] = 26701,
  [SMALL_STATE(577)] = 26729,
  [SMALL_STATE(578)] = 26761,
  [SMALL_STATE(579)] = 26791,
  [SMALL_STATE(580)] = 26818,
  [SMALL_STATE(581)] = 26845,
  [SMALL_STATE(582)] = 26872,
  [SMALL_STATE(583)] = 26899,
  [SMALL_STATE(584)] = 26936,
  [SMALL_STATE(585)] = 26963,
  [SMALL_STATE(586)] = 26989,
  [SMALL_STATE(587)] = 27015,
  [SMALL_STATE(588)] = 27041,
  [SMALL_STATE(589)] = 27067,
  [SMALL_STATE(590)] = 27093,
  [SMALL_STATE(591)] = 27119,
  [SMALL_STATE(592)] = 27145,
  [SMALL_STATE(593)] = 27171,
  [SMALL_STATE(594)] = 27197,
  [SMALL_STATE(595)] = 27223,
  [SMALL_STATE(596)] = 27249,
  [SMALL_STATE(597)] = 27275,
  [SMALL_STATE(598)] = 27301,
  [SMALL_STATE(599)] = 27327,
  [SMALL_STATE(600)] = 27353,
  [SMALL_STATE(601)] = 27379,
  [SMALL_STATE(602)] = 27405,
  [SMALL_STATE(603)] = 27431,
  [SMALL_STATE(604)] = 27457,
  [SMALL_STATE(605)] = 27483,
  [SMALL_STATE(606)] = 27509,
  [SMALL_STATE(607)] = 27535,
  [SMALL_STATE(608)] = 27561,
  [SMALL_STATE(609)] = 27587,
  [SMALL_STATE(610)] = 27613,
  [SMALL_STATE(611)] = 27639,
  [SMALL_STATE(612)] = 27665,
  [SMALL_STATE(613)] = 27691,
  [SMALL_STATE(614)] = 27717,
  [SMALL_STATE(615)] = 27743,
  [SMALL_STATE(616)] = 27769,
  [SMALL_STATE(617)] = 27795,
  [SMALL_STATE(618)] = 27821,
  [SMALL_STATE(619)] = 27847,
  [SMALL_STATE(620)] = 27873,
  [SMALL_STATE(621)] = 27899,
  [SMALL_STATE(622)] = 27925,
  [SMALL_STATE(623)] = 27951,
  [SMALL_STATE(624)] = 27977,
  [SMALL_STATE(625)] = 28003,
  [SMALL_STATE(626)] = 28029,
  [SMALL_STATE(627)] = 28055,
  [SMALL_STATE(628)] = 28081,
  [SMALL_STATE(629)] = 28107,
  [SMALL_STATE(630)] = 28133,
  [SMALL_STATE(631)] = 28159,
  [SMALL_STATE(632)] = 28185,
  [SMALL_STATE(633)] = 28211,
  [SMALL_STATE(634)] = 28237,
  [SMALL_STATE(635)] = 28263,
  [SMALL_STATE(636)] = 28289,
  [SMALL_STATE(637)] = 28315,
  [SMALL_STATE(638)] = 28341,
  [SMALL_STATE(639)] = 28367,
  [SMALL_STATE(640)] = 28393,
  [SMALL_STATE(641)] = 28419,
  [SMALL_STATE(642)] = 28445,
  [SMALL_STATE(643)] = 28471,
  [SMALL_STATE(644)] = 28497,
  [SMALL_STATE(645)] = 28523,
  [SMALL_STATE(646)] = 28549,
  [SMALL_STATE(647)] = 28575,
  [SMALL_STATE(648)] = 28601,
  [SMALL_STATE(649)] = 28627,
  [SMALL_STATE(650)] = 28653,
  [SMALL_STATE(651)] = 28679,
  [SMALL_STATE(652)] = 28705,
  [SMALL_STATE(653)] = 28731,
  [SMALL_STATE(654)] = 28757,
  [SMALL_STATE(655)] = 28783,
  [SMALL_STATE(656)] = 28809,
  [SMALL_STATE(657)] = 28835,
  [SMALL_STATE(658)] = 28861,
  [SMALL_STATE(659)] = 28887,
  [SMALL_STATE(660)] = 28913,
  [SMALL_STATE(661)] = 28939,
  [SMALL_STATE(662)] = 28965,
  [SMALL_STATE(663)] = 28991,
  [SMALL_STATE(664)] = 29017,
  [SMALL_STATE(665)] = 29043,
  [SMALL_STATE(666)] = 29069,
  [SMALL_STATE(667)] = 29095,
  [SMALL_STATE(668)] = 29140,
  [SMALL_STATE(669)] = 29185,
  [SMALL_STATE(670)] = 29229,
  [SMALL_STATE(671)] = 29273,
  [SMALL_STATE(672)] = 29317,
  [SMALL_STATE(673)] = 29363,
  [SMALL_STATE(674)] = 29407,
  [SMALL_STATE(675)] = 29457,
  [SMALL_STATE(676)] = 29501,
  [SMALL_STATE(677)] = 29545,
  [SMALL_STATE(678)] = 29589,
  [SMALL_STATE(679)] = 29633,
  [SMALL_STATE(680)] = 29677,
  [SMALL_STATE(681)] = 29705,
  [SMALL_STATE(682)] = 29739,
  [SMALL_STATE(683)] = 29783,
  [SMALL_STATE(684)] = 29827,
  [SMALL_STATE(685)] = 29871,
  [SMALL_STATE(686)] = 29907,
  [SMALL_STATE(687)] = 29935,
  [SMALL_STATE(688)] = 29979,
  [SMALL_STATE(689)] = 30023,
  [SMALL_STATE(690)] = 30067,
  [SMALL_STATE(691)] = 30110,
  [SMALL_STATE(692)] = 30145,
  [SMALL_STATE(693)] = 30188,
  [SMALL_STATE(694)] = 30229,
  [SMALL_STATE(695)] = 30272,
  [SMALL_STATE(696)] = 30313,
  [SMALL_STATE(697)] = 30354,
  [SMALL_STATE(698)] = 30381,
  [SMALL_STATE(699)] = 30410,
  [SMALL_STATE(700)] = 30443,
  [SMALL_STATE(701)] = 30486,
  [SMALL_STATE(702)] = 30521,
  [SMALL_STATE(703)] = 30564,
  [SMALL_STATE(704)] = 30613,
  [SMALL_STATE(705)] = 30656,
  [SMALL_STATE(706)] = 30697,
  [SMALL_STATE(707)] = 30737,
  [SMALL_STATE(708)] = 30777,
  [SMALL_STATE(709)] = 30817,
  [SMALL_STATE(710)] = 30857,
  [SMALL_STATE(711)] = 30883,
  [SMALL_STATE(712)] = 30917,
  [SMALL_STATE(713)] = 30951,
  [SMALL_STATE(714)] = 30991,
  [SMALL_STATE(715)] = 31031,
  [SMALL_STATE(716)] = 31065,
  [SMALL_STATE(717)] = 31105,
  [SMALL_STATE(718)] = 31139,
  [SMALL_STATE(719)] = 31173,
  [SMALL_STATE(720)] = 31213,
  [SMALL_STATE(721)] = 31253,
  [SMALL_STATE(722)] = 31287,
  [SMALL_STATE(723)] = 31327,
  [SMALL_STATE(724)] = 31367,
  [SMALL_STATE(725)] = 31415,
  [SMALL_STATE(726)] = 31455,
  [SMALL_STATE(727)] = 31495,
  [SMALL_STATE(728)] = 31535,
  [SMALL_STATE(729)] = 31577,
  [SMALL_STATE(730)] = 31617,
  [SMALL_STATE(731)] = 31657,
  [SMALL_STATE(732)] = 31697,
  [SMALL_STATE(733)] = 31737,
  [SMALL_STATE(734)] = 31777,
  [SMALL_STATE(735)] = 31817,
  [SMALL_STATE(736)] = 31843,
  [SMALL_STATE(737)] = 31885,
  [SMALL_STATE(738)] = 31917,
  [SMALL_STATE(739)] = 31957,
  [SMALL_STATE(740)] = 31997,
  [SMALL_STATE(741)] = 32037,
  [SMALL_STATE(742)] = 32077,
  [SMALL_STATE(743)] = 32117,
  [SMALL_STATE(744)] = 32145,
  [SMALL_STATE(745)] = 32185,
  [SMALL_STATE(746)] = 32225,
  [SMALL_STATE(747)] = 32259,
  [SMALL_STATE(748)] = 32293,
  [SMALL_STATE(749)] = 32331,
  [SMALL_STATE(750)] = 32371,
  [SMALL_STATE(751)] = 32404,
  [SMALL_STATE(752)] = 32441,
  [SMALL_STATE(753)] = 32474,
  [SMALL_STATE(754)] = 32499,
  [SMALL_STATE(755)] = 32522,
  [SMALL_STATE(756)] = 32549,
  [SMALL_STATE(757)] = 32572,
  [SMALL_STATE(758)] = 32595,
  [SMALL_STATE(759)] = 32622,
  [SMALL_STATE(760)] = 32645,
  [SMALL_STATE(761)] = 32670,
  [SMALL_STATE(762)] = 32695,
  [SMALL_STATE(763)] = 32718,
  [SMALL_STATE(764)] = 32745,
  [SMALL_STATE(765)] = 32768,
  [SMALL_STATE(766)] = 32807,
  [SMALL_STATE(767)] = 32832,
  [SMALL_STATE(768)] = 32855,
  [SMALL_STATE(769)] = 32900,
  [SMALL_STATE(770)] = 32931,
  [SMALL_STATE(771)] = 32964,
  [SMALL_STATE(772)] = 32987,
  [SMALL_STATE(773)] = 33014,
  [SMALL_STATE(774)] = 33037,
  [SMALL_STATE(775)] = 33070,
  [SMALL_STATE(776)] = 33115,
  [SMALL_STATE(777)] = 33148,
  [SMALL_STATE(778)] = 33181,
  [SMALL_STATE(779)] = 33220,
  [SMALL_STATE(780)] = 33251,
  [SMALL_STATE(781)] = 33284,
  [SMALL_STATE(782)] = 33316,
  [SMALL_STATE(783)] = 33338,
  [SMALL_STATE(784)] = 33372,
  [SMALL_STATE(785)] = 33406,
  [SMALL_STATE(786)] = 33440,
  [SMALL_STATE(787)] = 33474,
  [SMALL_STATE(788)] = 33496,
  [SMALL_STATE(789)] = 33530,
  [SMALL_STATE(790)] = 33564,
  [SMALL_STATE(791)] = 33598,
  [SMALL_STATE(792)] = 33632,
  [SMALL_STATE(793)] = 33656,
  [SMALL_STATE(794)] = 33690,
  [SMALL_STATE(795)] = 33720,
  [SMALL_STATE(796)] = 33746,
  [SMALL_STATE(797)] = 33780,
  [SMALL_STATE(798)] = 33814,
  [SMALL_STATE(799)] = 33846,
  [SMALL_STATE(800)] = 33868,
  [SMALL_STATE(801)] = 33902,
  [SMALL_STATE(802)] = 33936,
  [SMALL_STATE(803)] = 33970,
  [SMALL_STATE(804)] = 34004,
  [SMALL_STATE(805)] = 34038,
  [SMALL_STATE(806)] = 34072,
  [SMALL_STATE(807)] = 34106,
  [SMALL_STATE(808)] = 34130,
  [SMALL_STATE(809)] = 34164,
  [SMALL_STATE(810)] = 34198,
  [SMALL_STATE(811)] = 34232,
  [SMALL_STATE(812)] = 34266,
  [SMALL_STATE(813)] = 34300,
  [SMALL_STATE(814)] = 34334,
  [SMALL_STATE(815)] = 34368,
  [SMALL_STATE(816)] = 34402,
  [SMALL_STATE(817)] = 34436,
  [SMALL_STATE(818)] = 34470,
  [SMALL_STATE(819)] = 34504,
  [SMALL_STATE(820)] = 34526,
  [SMALL_STATE(821)] = 34560,
  [SMALL_STATE(822)] = 34594,
  [SMALL_STATE(823)] = 34628,
  [SMALL_STATE(824)] = 34654,
  [SMALL_STATE(825)] = 34688,
  [SMALL_STATE(826)] = 34722,
  [SMALL_STATE(827)] = 34756,
  [SMALL_STATE(828)] = 34790,
  [SMALL_STATE(829)] = 34824,
  [SMALL_STATE(830)] = 34858,
  [SMALL_STATE(831)] = 34892,
  [SMALL_STATE(832)] = 34926,
  [SMALL_STATE(833)] = 34960,
  [SMALL_STATE(834)] = 34982,
  [SMALL_STATE(835)] = 35022,
  [SMALL_STATE(836)] = 35056,
  [SMALL_STATE(837)] = 35080,
  [SMALL_STATE(838)] = 35114,
  [SMALL_STATE(839)] = 35136,
  [SMALL_STATE(840)] = 35170,
  [SMALL_STATE(841)] = 35194,
  [SMALL_STATE(842)] = 35228,
  [SMALL_STATE(843)] = 35262,
  [SMALL_STATE(844)] = 35296,
  [SMALL_STATE(845)] = 35330,
  [SMALL_STATE(846)] = 35370,
  [SMALL_STATE(847)] = 35404,
  [SMALL_STATE(848)] = 35438,
  [SMALL_STATE(849)] = 35472,
  [SMALL_STATE(850)] = 35506,
  [SMALL_STATE(851)] = 35540,
  [SMALL_STATE(852)] = 35574,
  [SMALL_STATE(853)] = 35600,
  [SMALL_STATE(854)] = 35632,
  [SMALL_STATE(855)] = 35666,
  [SMALL_STATE(856)] = 35700,
  [SMALL_STATE(857)] = 35734,
  [SMALL_STATE(858)] = 35768,
  [SMALL_STATE(859)] = 35802,
  [SMALL_STATE(860)] = 35836,
  [SMALL_STATE(861)] = 35880,
  [SMALL_STATE(862)] = 35914,
  [SMALL_STATE(863)] = 35948,
  [SMALL_STATE(864)] = 35982,
  [SMALL_STATE(865)] = 36004,
  [SMALL_STATE(866)] = 36046,
  [SMALL_STATE(867)] = 36080,
  [SMALL_STATE(868)] = 36114,
  [SMALL_STATE(869)] = 36148,
  [SMALL_STATE(870)] = 36182,
  [SMALL_STATE(871)] = 36220,
  [SMALL_STATE(872)] = 36258,
  [SMALL_STATE(873)] = 36292,
  [SMALL_STATE(874)] = 36326,
  [SMALL_STATE(875)] = 36360,
  [SMALL_STATE(876)] = 36392,
  [SMALL_STATE(877)] = 36426,
  [SMALL_STATE(878)] = 36460,
  [SMALL_STATE(879)] = 36496,
  [SMALL_STATE(880)] = 36530,
  [SMALL_STATE(881)] = 36564,
  [SMALL_STATE(882)] = 36598,
  [SMALL_STATE(883)] = 36620,
  [SMALL_STATE(884)] = 36654,
  [SMALL_STATE(885)] = 36688,
  [SMALL_STATE(886)] = 36722,
  [SMALL_STATE(887)] = 36756,
  [SMALL_STATE(888)] = 36790,
  [SMALL_STATE(889)] = 36824,
  [SMALL_STATE(890)] = 36858,
  [SMALL_STATE(891)] = 36890,
  [SMALL_STATE(892)] = 36924,
  [SMALL_STATE(893)] = 36958,
  [SMALL_STATE(894)] = 36992,
  [SMALL_STATE(895)] = 37024,
  [SMALL_STATE(896)] = 37058,
  [SMALL_STATE(897)] = 37090,
  [SMALL_STATE(898)] = 37124,
  [SMALL_STATE(899)] = 37158,
  [SMALL_STATE(900)] = 37192,
  [SMALL_STATE(901)] = 37226,
  [SMALL_STATE(902)] = 37256,
  [SMALL_STATE(903)] = 37290,
  [SMALL_STATE(904)] = 37312,
  [SMALL_STATE(905)] = 37346,
  [SMALL_STATE(906)] = 37380,
  [SMALL_STATE(907)] = 37402,
  [SMALL_STATE(908)] = 37436,
  [SMALL_STATE(909)] = 37458,
  [SMALL_STATE(910)] = 37480,
  [SMALL_STATE(911)] = 37502,
  [SMALL_STATE(912)] = 37536,
  [SMALL_STATE(913)] = 37558,
  [SMALL_STATE(914)] = 37592,
  [SMALL_STATE(915)] = 37626,
  [SMALL_STATE(916)] = 37648,
  [SMALL_STATE(917)] = 37670,
  [SMALL_STATE(918)] = 37704,
  [SMALL_STATE(919)] = 37726,
  [SMALL_STATE(920)] = 37760,
  [SMALL_STATE(921)] = 37794,
  [SMALL_STATE(922)] = 37828,
  [SMALL_STATE(923)] = 37862,
  [SMALL_STATE(924)] = 37896,
  [SMALL_STATE(925)] = 37930,
  [SMALL_STATE(926)] = 37952,
  [SMALL_STATE(927)] = 37986,
  [SMALL_STATE(928)] = 38020,
  [SMALL_STATE(929)] = 38054,
  [SMALL_STATE(930)] = 38088,
  [SMALL_STATE(931)] = 38110,
  [SMALL_STATE(932)] = 38132,
  [SMALL_STATE(933)] = 38166,
  [SMALL_STATE(934)] = 38188,
  [SMALL_STATE(935)] = 38222,
  [SMALL_STATE(936)] = 38244,
  [SMALL_STATE(937)] = 38278,
  [SMALL_STATE(938)] = 38312,
  [SMALL_STATE(939)] = 38346,
  [SMALL_STATE(940)] = 38380,
  [SMALL_STATE(941)] = 38414,
  [SMALL_STATE(942)] = 38448,
  [SMALL_STATE(943)] = 38482,
  [SMALL_STATE(944)] = 38516,
  [SMALL_STATE(945)] = 38550,
  [SMALL_STATE(946)] = 38572,
  [SMALL_STATE(947)] = 38594,
  [SMALL_STATE(948)] = 38628,
  [SMALL_STATE(949)] = 38662,
  [SMALL_STATE(950)] = 38696,
  [SMALL_STATE(951)] = 38718,
  [SMALL_STATE(952)] = 38740,
  [SMALL_STATE(953)] = 38764,
  [SMALL_STATE(954)] = 38796,
  [SMALL_STATE(955)] = 38830,
  [SMALL_STATE(956)] = 38864,
  [SMALL_STATE(957)] = 38890,
  [SMALL_STATE(958)] = 38924,
  [SMALL_STATE(959)] = 38948,
  [SMALL_STATE(960)] = 38988,
  [SMALL_STATE(961)] = 39022,
  [SMALL_STATE(962)] = 39054,
  [SMALL_STATE(963)] = 39076,
  [SMALL_STATE(964)] = 39110,
  [SMALL_STATE(965)] = 39144,
  [SMALL_STATE(966)] = 39178,
  [SMALL_STATE(967)] = 39212,
  [SMALL_STATE(968)] = 39246,
  [SMALL_STATE(969)] = 39272,
  [SMALL_STATE(970)] = 39298,
  [SMALL_STATE(971)] = 39332,
  [SMALL_STATE(972)] = 39366,
  [SMALL_STATE(973)] = 39388,
  [SMALL_STATE(974)] = 39422,
  [SMALL_STATE(975)] = 39456,
  [SMALL_STATE(976)] = 39478,
  [SMALL_STATE(977)] = 39512,
  [SMALL_STATE(978)] = 39546,
  [SMALL_STATE(979)] = 39580,
  [SMALL_STATE(980)] = 39602,
  [SMALL_STATE(981)] = 39636,
  [SMALL_STATE(982)] = 39662,
  [SMALL_STATE(983)] = 39696,
  [SMALL_STATE(984)] = 39730,
  [SMALL_STATE(985)] = 39764,
  [SMALL_STATE(986)] = 39786,
  [SMALL_STATE(987)] = 39820,
  [SMALL_STATE(988)] = 39854,
  [SMALL_STATE(989)] = 39888,
  [SMALL_STATE(990)] = 39925,
  [SMALL_STATE(991)] = 39948,
  [SMALL_STATE(992)] = 39977,
  [SMALL_STATE(993)] = 40014,
  [SMALL_STATE(994)] = 40051,
  [SMALL_STATE(995)] = 40088,
  [SMALL_STATE(996)] = 40117,
  [SMALL_STATE(997)] = 40138,
  [SMALL_STATE(998)] = 40161,
  [SMALL_STATE(999)] = 40198,
  [SMALL_STATE(1000)] = 40235,
  [SMALL_STATE(1001)] = 40266,
  [SMALL_STATE(1002)] = 40303,
  [SMALL_STATE(1003)] = 40340,
  [SMALL_STATE(1004)] = 40363,
  [SMALL_STATE(1005)] = 40400,
  [SMALL_STATE(1006)] = 40437,
  [SMALL_STATE(1007)] = 40458,
  [SMALL_STATE(1008)] = 40479,
  [SMALL_STATE(1009)] = 40510,
  [SMALL_STATE(1010)] = 40535,
  [SMALL_STATE(1011)] = 40560,
  [SMALL_STATE(1012)] = 40581,
  [SMALL_STATE(1013)] = 40602,
  [SMALL_STATE(1014)] = 40627,
  [SMALL_STATE(1015)] = 40652,
  [SMALL_STATE(1016)] = 40673,
  [SMALL_STATE(1017)] = 40694,
  [SMALL_STATE(1018)] = 40731,
  [SMALL_STATE(1019)] = 40752,
  [SMALL_STATE(1020)] = 40787,
  [SMALL_STATE(1021)] = 40824,
  [SMALL_STATE(1022)] = 40861,
  [SMALL_STATE(1023)] = 40882,
  [SMALL_STATE(1024)] = 40913,
  [SMALL_STATE(1025)] = 40934,
  [SMALL_STATE(1026)] = 40971,
  [SMALL_STATE(1027)] = 40992,
  [SMALL_STATE(1028)] = 41017,
  [SMALL_STATE(1029)] = 41054,
  [SMALL_STATE(1030)] = 41085,
  [SMALL_STATE(1031)] = 41116,
  [SMALL_STATE(1032)] = 41141,
  [SMALL_STATE(1033)] = 41162,
  [SMALL_STATE(1034)] = 41197,
  [SMALL_STATE(1035)] = 41218,
  [SMALL_STATE(1036)] = 41239,
  [SMALL_STATE(1037)] = 41260,
  [SMALL_STATE(1038)] = 41281,
  [SMALL_STATE(1039)] = 41306,
  [SMALL_STATE(1040)] = 41327,
  [SMALL_STATE(1041)] = 41362,
  [SMALL_STATE(1042)] = 41393,
  [SMALL_STATE(1043)] = 41426,
  [SMALL_STATE(1044)] = 41447,
  [SMALL_STATE(1045)] = 41482,
  [SMALL_STATE(1046)] = 41511,
  [SMALL_STATE(1047)] = 41532,
  [SMALL_STATE(1048)] = 41553,
  [SMALL_STATE(1049)] = 41592,
  [SMALL_STATE(1050)] = 41613,
  [SMALL_STATE(1051)] = 41648,
  [SMALL_STATE(1052)] = 41669,
  [SMALL_STATE(1053)] = 41704,
  [SMALL_STATE(1054)] = 41725,
  [SMALL_STATE(1055)] = 41762,
  [SMALL_STATE(1056)] = 41793,
  [SMALL_STATE(1057)] = 41814,
  [SMALL_STATE(1058)] = 41849,
  [SMALL_STATE(1059)] = 41886,
  [SMALL_STATE(1060)] = 41907,
  [SMALL_STATE(1061)] = 41932,
  [SMALL_STATE(1062)] = 41953,
  [SMALL_STATE(1063)] = 41974,
  [SMALL_STATE(1064)] = 41995,
  [SMALL_STATE(1065)] = 42026,
  [SMALL_STATE(1066)] = 42047,
  [SMALL_STATE(1067)] = 42084,
  [SMALL_STATE(1068)] = 42121,
  [SMALL_STATE(1069)] = 42158,
  [SMALL_STATE(1070)] = 42195,
  [SMALL_STATE(1071)] = 42216,
  [SMALL_STATE(1072)] = 42253,
  [SMALL_STATE(1073)] = 42290,
  [SMALL_STATE(1074)] = 42315,
  [SMALL_STATE(1075)] = 42350,
  [SMALL_STATE(1076)] = 42387,
  [SMALL_STATE(1077)] = 42408,
  [SMALL_STATE(1078)] = 42429,
  [SMALL_STATE(1079)] = 42466,
  [SMALL_STATE(1080)] = 42487,
  [SMALL_STATE(1081)] = 42508,
  [SMALL_STATE(1082)] = 42529,
  [SMALL_STATE(1083)] = 42566,
  [SMALL_STATE(1084)] = 42591,
  [SMALL_STATE(1085)] = 42628,
  [SMALL_STATE(1086)] = 42665,
  [SMALL_STATE(1087)] = 42686,
  [SMALL_STATE(1088)] = 42707,
  [SMALL_STATE(1089)] = 42744,
  [SMALL_STATE(1090)] = 42779,
  [SMALL_STATE(1091)] = 42804,
  [SMALL_STATE(1092)] = 42841,
  [SMALL_STATE(1093)] = 42878,
  [SMALL_STATE(1094)] = 42915,
  [SMALL_STATE(1095)] = 42952,
  [SMALL_STATE(1096)] = 42987,
  [SMALL_STATE(1097)] = 43008,
  [SMALL_STATE(1098)] = 43045,
  [SMALL_STATE(1099)] = 43082,
  [SMALL_STATE(1100)] = 43119,
  [SMALL_STATE(1101)] = 43140,
  [SMALL_STATE(1102)] = 43161,
  [SMALL_STATE(1103)] = 43182,
  [SMALL_STATE(1104)] = 43203,
  [SMALL_STATE(1105)] = 43240,
  [SMALL_STATE(1106)] = 43261,
  [SMALL_STATE(1107)] = 43282,
  [SMALL_STATE(1108)] = 43319,
  [SMALL_STATE(1109)] = 43340,
  [SMALL_STATE(1110)] = 43377,
  [SMALL_STATE(1111)] = 43400,
  [SMALL_STATE(1112)] = 43437,
  [SMALL_STATE(1113)] = 43458,
  [SMALL_STATE(1114)] = 43495,
  [SMALL_STATE(1115)] = 43524,
  [SMALL_STATE(1116)] = 43549,
  [SMALL_STATE(1117)] = 43587,
  [SMALL_STATE(1118)] = 43607,
  [SMALL_STATE(1119)] = 43645,
  [SMALL_STATE(1120)] = 43669,
  [SMALL_STATE(1121)] = 43693,
  [SMALL_STATE(1122)] = 43713,
  [SMALL_STATE(1123)] = 43751,
  [SMALL_STATE(1124)] = 43771,
  [SMALL_STATE(1125)] = 43795,
  [SMALL_STATE(1126)] = 43815,
  [SMALL_STATE(1127)] = 43835,
  [SMALL_STATE(1128)] = 43873,
  [SMALL_STATE(1129)] = 43893,
  [SMALL_STATE(1130)] = 43931,
  [SMALL_STATE(1131)] = 43965,
  [SMALL_STATE(1132)] = 43985,
  [SMALL_STATE(1133)] = 44019,
  [SMALL_STATE(1134)] = 44043,
  [SMALL_STATE(1135)] = 44063,
  [SMALL_STATE(1136)] = 44101,
  [SMALL_STATE(1137)] = 44129,
  [SMALL_STATE(1138)] = 44161,
  [SMALL_STATE(1139)] = 44199,
  [SMALL_STATE(1140)] = 44237,
  [SMALL_STATE(1141)] = 44257,
  [SMALL_STATE(1142)] = 44277,
  [SMALL_STATE(1143)] = 44315,
  [SMALL_STATE(1144)] = 44343,
  [SMALL_STATE(1145)] = 44371,
  [SMALL_STATE(1146)] = 44391,
  [SMALL_STATE(1147)] = 44413,
  [SMALL_STATE(1148)] = 44451,
  [SMALL_STATE(1149)] = 44471,
  [SMALL_STATE(1150)] = 44495,
  [SMALL_STATE(1151)] = 44515,
  [SMALL_STATE(1152)] = 44535,
  [SMALL_STATE(1153)] = 44555,
  [SMALL_STATE(1154)] = 44593,
  [SMALL_STATE(1155)] = 44613,
  [SMALL_STATE(1156)] = 44633,
  [SMALL_STATE(1157)] = 44671,
  [SMALL_STATE(1158)] = 44709,
  [SMALL_STATE(1159)] = 44747,
  [SMALL_STATE(1160)] = 44785,
  [SMALL_STATE(1161)] = 44823,
  [SMALL_STATE(1162)] = 44861,
  [SMALL_STATE(1163)] = 44881,
  [SMALL_STATE(1164)] = 44901,
  [SMALL_STATE(1165)] = 44939,
  [SMALL_STATE(1166)] = 44977,
  [SMALL_STATE(1167)] = 45015,
  [SMALL_STATE(1168)] = 45053,
  [SMALL_STATE(1169)] = 45073,
  [SMALL_STATE(1170)] = 45111,
  [SMALL_STATE(1171)] = 45131,
  [SMALL_STATE(1172)] = 45153,
  [SMALL_STATE(1173)] = 45177,
  [SMALL_STATE(1174)] = 45199,
  [SMALL_STATE(1175)] = 45223,
  [SMALL_STATE(1176)] = 45261,
  [SMALL_STATE(1177)] = 45281,
  [SMALL_STATE(1178)] = 45305,
  [SMALL_STATE(1179)] = 45329,
  [SMALL_STATE(1180)] = 45367,
  [SMALL_STATE(1181)] = 45395,
  [SMALL_STATE(1182)] = 45417,
  [SMALL_STATE(1183)] = 45455,
  [SMALL_STATE(1184)] = 45479,
  [SMALL_STATE(1185)] = 45517,
  [SMALL_STATE(1186)] = 45541,
  [SMALL_STATE(1187)] = 45561,
  [SMALL_STATE(1188)] = 45581,
  [SMALL_STATE(1189)] = 45619,
  [SMALL_STATE(1190)] = 45641,
  [SMALL_STATE(1191)] = 45679,
  [SMALL_STATE(1192)] = 45699,
  [SMALL_STATE(1193)] = 45719,
  [SMALL_STATE(1194)] = 45757,
  [SMALL_STATE(1195)] = 45795,
  [SMALL_STATE(1196)] = 45817,
  [SMALL_STATE(1197)] = 45855,
  [SMALL_STATE(1198)] = 45893,
  [SMALL_STATE(1199)] = 45915,
  [SMALL_STATE(1200)] = 45953,
  [SMALL_STATE(1201)] = 45973,
  [SMALL_STATE(1202)] = 46007,
  [SMALL_STATE(1203)] = 46031,
  [SMALL_STATE(1204)] = 46069,
  [SMALL_STATE(1205)] = 46089,
  [SMALL_STATE(1206)] = 46127,
  [SMALL_STATE(1207)] = 46147,
  [SMALL_STATE(1208)] = 46167,
  [SMALL_STATE(1209)] = 46205,
  [SMALL_STATE(1210)] = 46227,
  [SMALL_STATE(1211)] = 46247,
  [SMALL_STATE(1212)] = 46285,
  [SMALL_STATE(1213)] = 46323,
  [SMALL_STATE(1214)] = 46343,
  [SMALL_STATE(1215)] = 46363,
  [SMALL_STATE(1216)] = 46385,
  [SMALL_STATE(1217)] = 46405,
  [SMALL_STATE(1218)] = 46425,
  [SMALL_STATE(1219)] = 46449,
  [SMALL_STATE(1220)] = 46469,
  [SMALL_STATE(1221)] = 46489,
  [SMALL_STATE(1222)] = 46509,
  [SMALL_STATE(1223)] = 46529,
  [SMALL_STATE(1224)] = 46551,
  [SMALL_STATE(1225)] = 46571,
  [SMALL_STATE(1226)] = 46591,
  [SMALL_STATE(1227)] = 46611,
  [SMALL_STATE(1228)] = 46631,
  [SMALL_STATE(1229)] = 46658,
  [SMALL_STATE(1230)] = 46677,
  [SMALL_STATE(1231)] = 46696,
  [SMALL_STATE(1232)] = 46723,
  [SMALL_STATE(1233)] = 46752,
  [SMALL_STATE(1234)] = 46771,
  [SMALL_STATE(1235)] = 46790,
  [SMALL_STATE(1236)] = 46809,
  [SMALL_STATE(1237)] = 46842,
  [SMALL_STATE(1238)] = 46861,
  [SMALL_STATE(1239)] = 46888,
  [SMALL_STATE(1240)] = 46917,
  [SMALL_STATE(1241)] = 46936,
  [SMALL_STATE(1242)] = 46955,
  [SMALL_STATE(1243)] = 46978,
  [SMALL_STATE(1244)] = 46997,
  [SMALL_STATE(1245)] = 47020,
  [SMALL_STATE(1246)] = 47039,
  [SMALL_STATE(1247)] = 47062,
  [SMALL_STATE(1248)] = 47081,
  [SMALL_STATE(1249)] = 47108,
  [SMALL_STATE(1250)] = 47127,
  [SMALL_STATE(1251)] = 47146,
  [SMALL_STATE(1252)] = 47169,
  [SMALL_STATE(1253)] = 47202,
  [SMALL_STATE(1254)] = 47221,
  [SMALL_STATE(1255)] = 47248,
  [SMALL_STATE(1256)] = 47267,
  [SMALL_STATE(1257)] = 47296,
  [SMALL_STATE(1258)] = 47325,
  [SMALL_STATE(1259)] = 47352,
  [SMALL_STATE(1260)] = 47371,
  [SMALL_STATE(1261)] = 47390,
  [SMALL_STATE(1262)] = 47409,
  [SMALL_STATE(1263)] = 47430,
  [SMALL_STATE(1264)] = 47449,
  [SMALL_STATE(1265)] = 47476,
  [SMALL_STATE(1266)] = 47505,
  [SMALL_STATE(1267)] = 47524,
  [SMALL_STATE(1268)] = 47543,
  [SMALL_STATE(1269)] = 47570,
  [SMALL_STATE(1270)] = 47589,
  [SMALL_STATE(1271)] = 47618,
  [SMALL_STATE(1272)] = 47641,
  [SMALL_STATE(1273)] = 47660,
  [SMALL_STATE(1274)] = 47679,
  [SMALL_STATE(1275)] = 47708,
  [SMALL_STATE(1276)] = 47727,
  [SMALL_STATE(1277)] = 47756,
  [SMALL_STATE(1278)] = 47775,
  [SMALL_STATE(1279)] = 47808,
  [SMALL_STATE(1280)] = 47827,
  [SMALL_STATE(1281)] = 47846,
  [SMALL_STATE(1282)] = 47873,
  [SMALL_STATE(1283)] = 47892,
  [SMALL_STATE(1284)] = 47915,
  [SMALL_STATE(1285)] = 47934,
  [SMALL_STATE(1286)] = 47953,
  [SMALL_STATE(1287)] = 47972,
  [SMALL_STATE(1288)] = 47991,
  [SMALL_STATE(1289)] = 48010,
  [SMALL_STATE(1290)] = 48029,
  [SMALL_STATE(1291)] = 48048,
  [SMALL_STATE(1292)] = 48077,
  [SMALL_STATE(1293)] = 48096,
  [SMALL_STATE(1294)] = 48115,
  [SMALL_STATE(1295)] = 48134,
  [SMALL_STATE(1296)] = 48153,
  [SMALL_STATE(1297)] = 48172,
  [SMALL_STATE(1298)] = 48201,
  [SMALL_STATE(1299)] = 48230,
  [SMALL_STATE(1300)] = 48259,
  [SMALL_STATE(1301)] = 48282,
  [SMALL_STATE(1302)] = 48301,
  [SMALL_STATE(1303)] = 48320,
  [SMALL_STATE(1304)] = 48339,
  [SMALL_STATE(1305)] = 48358,
  [SMALL_STATE(1306)] = 48377,
  [SMALL_STATE(1307)] = 48398,
  [SMALL_STATE(1308)] = 48417,
  [SMALL_STATE(1309)] = 48436,
  [SMALL_STATE(1310)] = 48455,
  [SMALL_STATE(1311)] = 48474,
  [SMALL_STATE(1312)] = 48493,
  [SMALL_STATE(1313)] = 48520,
  [SMALL_STATE(1314)] = 48539,
  [SMALL_STATE(1315)] = 48566,
  [SMALL_STATE(1316)] = 48595,
  [SMALL_STATE(1317)] = 48624,
  [SMALL_STATE(1318)] = 48643,
  [SMALL_STATE(1319)] = 48662,
  [SMALL_STATE(1320)] = 48685,
  [SMALL_STATE(1321)] = 48714,
  [SMALL_STATE(1322)] = 48733,
  [SMALL_STATE(1323)] = 48752,
  [SMALL_STATE(1324)] = 48779,
  [SMALL_STATE(1325)] = 48806,
  [SMALL_STATE(1326)] = 48829,
  [SMALL_STATE(1327)] = 48858,
  [SMALL_STATE(1328)] = 48885,
  [SMALL_STATE(1329)] = 48904,
  [SMALL_STATE(1330)] = 48923,
  [SMALL_STATE(1331)] = 48942,
  [SMALL_STATE(1332)] = 48961,
  [SMALL_STATE(1333)] = 48990,
  [SMALL_STATE(1334)] = 49019,
  [SMALL_STATE(1335)] = 49042,
  [SMALL_STATE(1336)] = 49061,
  [SMALL_STATE(1337)] = 49084,
  [SMALL_STATE(1338)] = 49107,
  [SMALL_STATE(1339)] = 49134,
  [SMALL_STATE(1340)] = 49153,
  [SMALL_STATE(1341)] = 49172,
  [SMALL_STATE(1342)] = 49199,
  [SMALL_STATE(1343)] = 49228,
  [SMALL_STATE(1344)] = 49251,
  [SMALL_STATE(1345)] = 49284,
  [SMALL_STATE(1346)] = 49313,
  [SMALL_STATE(1347)] = 49346,
  [SMALL_STATE(1348)] = 49379,
  [SMALL_STATE(1349)] = 49398,
  [SMALL_STATE(1350)] = 49417,
  [SMALL_STATE(1351)] = 49440,
  [SMALL_STATE(1352)] = 49459,
  [SMALL_STATE(1353)] = 49486,
  [SMALL_STATE(1354)] = 49505,
  [SMALL_STATE(1355)] = 49524,
  [SMALL_STATE(1356)] = 49543,
  [SMALL_STATE(1357)] = 49570,
  [SMALL_STATE(1358)] = 49589,
  [SMALL_STATE(1359)] = 49612,
  [SMALL_STATE(1360)] = 49631,
  [SMALL_STATE(1361)] = 49664,
  [SMALL_STATE(1362)] = 49683,
  [SMALL_STATE(1363)] = 49716,
  [SMALL_STATE(1364)] = 49735,
  [SMALL_STATE(1365)] = 49754,
  [SMALL_STATE(1366)] = 49777,
  [SMALL_STATE(1367)] = 49800,
  [SMALL_STATE(1368)] = 49827,
  [SMALL_STATE(1369)] = 49846,
  [SMALL_STATE(1370)] = 49865,
  [SMALL_STATE(1371)] = 49898,
  [SMALL_STATE(1372)] = 49917,
  [SMALL_STATE(1373)] = 49936,
  [SMALL_STATE(1374)] = 49959,
  [SMALL_STATE(1375)] = 49992,
  [SMALL_STATE(1376)] = 50015,
  [SMALL_STATE(1377)] = 50042,
  [SMALL_STATE(1378)] = 50061,
  [SMALL_STATE(1379)] = 50094,
  [SMALL_STATE(1380)] = 50123,
  [SMALL_STATE(1381)] = 50142,
  [SMALL_STATE(1382)] = 50161,
  [SMALL_STATE(1383)] = 50180,
  [SMALL_STATE(1384)] = 50199,
  [SMALL_STATE(1385)] = 50218,
  [SMALL_STATE(1386)] = 50237,
  [SMALL_STATE(1387)] = 50265,
  [SMALL_STATE(1388)] = 50293,
  [SMALL_STATE(1389)] = 50315,
  [SMALL_STATE(1390)] = 50349,
  [SMALL_STATE(1391)] = 50377,
  [SMALL_STATE(1392)] = 50395,
  [SMALL_STATE(1393)] = 50413,
  [SMALL_STATE(1394)] = 50431,
  [SMALL_STATE(1395)] = 50465,
  [SMALL_STATE(1396)] = 50483,
  [SMALL_STATE(1397)] = 50501,
  [SMALL_STATE(1398)] = 50519,
  [SMALL_STATE(1399)] = 50537,
  [SMALL_STATE(1400)] = 50571,
  [SMALL_STATE(1401)] = 50589,
  [SMALL_STATE(1402)] = 50621,
  [SMALL_STATE(1403)] = 50641,
  [SMALL_STATE(1404)] = 50659,
  [SMALL_STATE(1405)] = 50693,
  [SMALL_STATE(1406)] = 50711,
  [SMALL_STATE(1407)] = 50729,
  [SMALL_STATE(1408)] = 50747,
  [SMALL_STATE(1409)] = 50769,
  [SMALL_STATE(1410)] = 50791,
  [SMALL_STATE(1411)] = 50825,
  [SMALL_STATE(1412)] = 50843,
  [SMALL_STATE(1413)] = 50861,
  [SMALL_STATE(1414)] = 50895,
  [SMALL_STATE(1415)] = 50917,
  [SMALL_STATE(1416)] = 50951,
  [SMALL_STATE(1417)] = 50973,
  [SMALL_STATE(1418)] = 50991,
  [SMALL_STATE(1419)] = 51009,
  [SMALL_STATE(1420)] = 51027,
  [SMALL_STATE(1421)] = 51045,
  [SMALL_STATE(1422)] = 51079,
  [SMALL_STATE(1423)] = 51101,
  [SMALL_STATE(1424)] = 51119,
  [SMALL_STATE(1425)] = 51153,
  [SMALL_STATE(1426)] = 51173,
  [SMALL_STATE(1427)] = 51191,
  [SMALL_STATE(1428)] = 51225,
  [SMALL_STATE(1429)] = 51259,
  [SMALL_STATE(1430)] = 51277,
  [SMALL_STATE(1431)] = 51311,
  [SMALL_STATE(1432)] = 51333,
  [SMALL_STATE(1433)] = 51359,
  [SMALL_STATE(1434)] = 51379,
  [SMALL_STATE(1435)] = 51397,
  [SMALL_STATE(1436)] = 51419,
  [SMALL_STATE(1437)] = 51437,
  [SMALL_STATE(1438)] = 51471,
  [SMALL_STATE(1439)] = 51489,
  [SMALL_STATE(1440)] = 51507,
  [SMALL_STATE(1441)] = 51541,
  [SMALL_STATE(1442)] = 51575,
  [SMALL_STATE(1443)] = 51597,
  [SMALL_STATE(1444)] = 51619,
  [SMALL_STATE(1445)] = 51653,
  [SMALL_STATE(1446)] = 51671,
  [SMALL_STATE(1447)] = 51697,
  [SMALL_STATE(1448)] = 51715,
  [SMALL_STATE(1449)] = 51733,
  [SMALL_STATE(1450)] = 51755,
  [SMALL_STATE(1451)] = 51789,
  [SMALL_STATE(1452)] = 51807,
  [SMALL_STATE(1453)] = 51825,
  [SMALL_STATE(1454)] = 51843,
  [SMALL_STATE(1455)] = 51875,
  [SMALL_STATE(1456)] = 51909,
  [SMALL_STATE(1457)] = 51931,
  [SMALL_STATE(1458)] = 51949,
  [SMALL_STATE(1459)] = 51981,
  [SMALL_STATE(1460)] = 52003,
  [SMALL_STATE(1461)] = 52025,
  [SMALL_STATE(1462)] = 52043,
  [SMALL_STATE(1463)] = 52061,
  [SMALL_STATE(1464)] = 52083,
  [SMALL_STATE(1465)] = 52105,
  [SMALL_STATE(1466)] = 52127,
  [SMALL_STATE(1467)] = 52144,
  [SMALL_STATE(1468)] = 52161,
  [SMALL_STATE(1469)] = 52178,
  [SMALL_STATE(1470)] = 52195,
  [SMALL_STATE(1471)] = 52216,
  [SMALL_STATE(1472)] = 52233,
  [SMALL_STATE(1473)] = 52256,
  [SMALL_STATE(1474)] = 52279,
  [SMALL_STATE(1475)] = 52296,
  [SMALL_STATE(1476)] = 52313,
  [SMALL_STATE(1477)] = 52344,
  [SMALL_STATE(1478)] = 52367,
  [SMALL_STATE(1479)] = 52384,
  [SMALL_STATE(1480)] = 52409,
  [SMALL_STATE(1481)] = 52438,
  [SMALL_STATE(1482)] = 52469,
  [SMALL_STATE(1483)] = 52492,
  [SMALL_STATE(1484)] = 52519,
  [SMALL_STATE(1485)] = 52546,
  [SMALL_STATE(1486)] = 52567,
  [SMALL_STATE(1487)] = 52584,
  [SMALL_STATE(1488)] = 52603,
  [SMALL_STATE(1489)] = 52626,
  [SMALL_STATE(1490)] = 52647,
  [SMALL_STATE(1491)] = 52672,
  [SMALL_STATE(1492)] = 52693,
  [SMALL_STATE(1493)] = 52710,
  [SMALL_STATE(1494)] = 52733,
  [SMALL_STATE(1495)] = 52756,
  [SMALL_STATE(1496)] = 52773,
  [SMALL_STATE(1497)] = 52796,
  [SMALL_STATE(1498)] = 52817,
  [SMALL_STATE(1499)] = 52842,
  [SMALL_STATE(1500)] = 52863,
  [SMALL_STATE(1501)] = 52884,
  [SMALL_STATE(1502)] = 52907,
  [SMALL_STATE(1503)] = 52922,
  [SMALL_STATE(1504)] = 52945,
  [SMALL_STATE(1505)] = 52966,
  [SMALL_STATE(1506)] = 52987,
  [SMALL_STATE(1507)] = 53010,
  [SMALL_STATE(1508)] = 53035,
  [SMALL_STATE(1509)] = 53056,
  [SMALL_STATE(1510)] = 53073,
  [SMALL_STATE(1511)] = 53094,
  [SMALL_STATE(1512)] = 53121,
  [SMALL_STATE(1513)] = 53144,
  [SMALL_STATE(1514)] = 53175,
  [SMALL_STATE(1515)] = 53196,
  [SMALL_STATE(1516)] = 53219,
  [SMALL_STATE(1517)] = 53240,
  [SMALL_STATE(1518)] = 53269,
  [SMALL_STATE(1519)] = 53286,
  [SMALL_STATE(1520)] = 53309,
  [SMALL_STATE(1521)] = 53326,
  [SMALL_STATE(1522)] = 53347,
  [SMALL_STATE(1523)] = 53364,
  [SMALL_STATE(1524)] = 53387,
  [SMALL_STATE(1525)] = 53404,
  [SMALL_STATE(1526)] = 53427,
  [SMALL_STATE(1527)] = 53448,
  [SMALL_STATE(1528)] = 53469,
  [SMALL_STATE(1529)] = 53486,
  [SMALL_STATE(1530)] = 53503,
  [SMALL_STATE(1531)] = 53526,
  [SMALL_STATE(1532)] = 53543,
  [SMALL_STATE(1533)] = 53558,
  [SMALL_STATE(1534)] = 53575,
  [SMALL_STATE(1535)] = 53595,
  [SMALL_STATE(1536)] = 53615,
  [SMALL_STATE(1537)] = 53635,
  [SMALL_STATE(1538)] = 53657,
  [SMALL_STATE(1539)] = 53677,
  [SMALL_STATE(1540)] = 53699,
  [SMALL_STATE(1541)] = 53721,
  [SMALL_STATE(1542)] = 53747,
  [SMALL_STATE(1543)] = 53767,
  [SMALL_STATE(1544)] = 53791,
  [SMALL_STATE(1545)] = 53813,
  [SMALL_STATE(1546)] = 53835,
  [SMALL_STATE(1547)] = 53855,
  [SMALL_STATE(1548)] = 53877,
  [SMALL_STATE(1549)] = 53897,
  [SMALL_STATE(1550)] = 53915,
  [SMALL_STATE(1551)] = 53939,
  [SMALL_STATE(1552)] = 53957,
  [SMALL_STATE(1553)] = 53977,
  [SMALL_STATE(1554)] = 54001,
  [SMALL_STATE(1555)] = 54021,
  [SMALL_STATE(1556)] = 54035,
  [SMALL_STATE(1557)] = 54055,
  [SMALL_STATE(1558)] = 54077,
  [SMALL_STATE(1559)] = 54097,
  [SMALL_STATE(1560)] = 54117,
  [SMALL_STATE(1561)] = 54137,
  [SMALL_STATE(1562)] = 54157,
  [SMALL_STATE(1563)] = 54177,
  [SMALL_STATE(1564)] = 54197,
  [SMALL_STATE(1565)] = 54213,
  [SMALL_STATE(1566)] = 54235,
  [SMALL_STATE(1567)] = 54251,
  [SMALL_STATE(1568)] = 54267,
  [SMALL_STATE(1569)] = 54283,
  [SMALL_STATE(1570)] = 54299,
  [SMALL_STATE(1571)] = 54313,
  [SMALL_STATE(1572)] = 54331,
  [SMALL_STATE(1573)] = 54347,
  [SMALL_STATE(1574)] = 54361,
  [SMALL_STATE(1575)] = 54381,
  [SMALL_STATE(1576)] = 54401,
  [SMALL_STATE(1577)] = 54421,
  [SMALL_STATE(1578)] = 54445,
  [SMALL_STATE(1579)] = 54467,
  [SMALL_STATE(1580)] = 54489,
  [SMALL_STATE(1581)] = 54511,
  [SMALL_STATE(1582)] = 54531,
  [SMALL_STATE(1583)] = 54551,
  [SMALL_STATE(1584)] = 54565,
  [SMALL_STATE(1585)] = 54581,
  [SMALL_STATE(1586)] = 54601,
  [SMALL_STATE(1587)] = 54625,
  [SMALL_STATE(1588)] = 54647,
  [SMALL_STATE(1589)] = 54667,
  [SMALL_STATE(1590)] = 54687,
  [SMALL_STATE(1591)] = 54709,
  [SMALL_STATE(1592)] = 54731,
  [SMALL_STATE(1593)] = 54755,
  [SMALL_STATE(1594)] = 54775,
  [SMALL_STATE(1595)] = 54795,
  [SMALL_STATE(1596)] = 54815,
  [SMALL_STATE(1597)] = 54835,
  [SMALL_STATE(1598)] = 54855,
  [SMALL_STATE(1599)] = 54875,
  [SMALL_STATE(1600)] = 54897,
  [SMALL_STATE(1601)] = 54919,
  [SMALL_STATE(1602)] = 54938,
  [SMALL_STATE(1603)] = 54951,
  [SMALL_STATE(1604)] = 54968,
  [SMALL_STATE(1605)] = 54993,
  [SMALL_STATE(1606)] = 55014,
  [SMALL_STATE(1607)] = 55039,
  [SMALL_STATE(1608)] = 55062,
  [SMALL_STATE(1609)] = 55077,
  [SMALL_STATE(1610)] = 55090,
  [SMALL_STATE(1611)] = 55103,
  [SMALL_STATE(1612)] = 55126,
  [SMALL_STATE(1613)] = 55147,
  [SMALL_STATE(1614)] = 55172,
  [SMALL_STATE(1615)] = 55189,
  [SMALL_STATE(1616)] = 55214,
  [SMALL_STATE(1617)] = 55239,
  [SMALL_STATE(1618)] = 55258,
  [SMALL_STATE(1619)] = 55277,
  [SMALL_STATE(1620)] = 55296,
  [SMALL_STATE(1621)] = 55321,
  [SMALL_STATE(1622)] = 55342,
  [SMALL_STATE(1623)] = 55361,
  [SMALL_STATE(1624)] = 55376,
  [SMALL_STATE(1625)] = 55391,
  [SMALL_STATE(1626)] = 55410,
  [SMALL_STATE(1627)] = 55425,
  [SMALL_STATE(1628)] = 55438,
  [SMALL_STATE(1629)] = 55455,
  [SMALL_STATE(1630)] = 55474,
  [SMALL_STATE(1631)] = 55495,
  [SMALL_STATE(1632)] = 55512,
  [SMALL_STATE(1633)] = 55527,
  [SMALL_STATE(1634)] = 55546,
  [SMALL_STATE(1635)] = 55559,
  [SMALL_STATE(1636)] = 55578,
  [SMALL_STATE(1637)] = 55591,
  [SMALL_STATE(1638)] = 55606,
  [SMALL_STATE(1639)] = 55629,
  [SMALL_STATE(1640)] = 55648,
  [SMALL_STATE(1641)] = 55667,
  [SMALL_STATE(1642)] = 55686,
  [SMALL_STATE(1643)] = 55702,
  [SMALL_STATE(1644)] = 55718,
  [SMALL_STATE(1645)] = 55734,
  [SMALL_STATE(1646)] = 55756,
  [SMALL_STATE(1647)] = 55776,
  [SMALL_STATE(1648)] = 55788,
  [SMALL_STATE(1649)] = 55804,
  [SMALL_STATE(1650)] = 55820,
  [SMALL_STATE(1651)] = 55832,
  [SMALL_STATE(1652)] = 55846,
  [SMALL_STATE(1653)] = 55862,
  [SMALL_STATE(1654)] = 55874,
  [SMALL_STATE(1655)] = 55886,
  [SMALL_STATE(1656)] = 55898,
  [SMALL_STATE(1657)] = 55914,
  [SMALL_STATE(1658)] = 55934,
  [SMALL_STATE(1659)] = 55946,
  [SMALL_STATE(1660)] = 55958,
  [SMALL_STATE(1661)] = 55970,
  [SMALL_STATE(1662)] = 55984,
  [SMALL_STATE(1663)] = 55996,
  [SMALL_STATE(1664)] = 56014,
  [SMALL_STATE(1665)] = 56026,
  [SMALL_STATE(1666)] = 56042,
  [SMALL_STATE(1667)] = 56064,
  [SMALL_STATE(1668)] = 56080,
  [SMALL_STATE(1669)] = 56096,
  [SMALL_STATE(1670)] = 56112,
  [SMALL_STATE(1671)] = 56126,
  [SMALL_STATE(1672)] = 56148,
  [SMALL_STATE(1673)] = 56162,
  [SMALL_STATE(1674)] = 56174,
  [SMALL_STATE(1675)] = 56186,
  [SMALL_STATE(1676)] = 56198,
  [SMALL_STATE(1677)] = 56212,
  [SMALL_STATE(1678)] = 56226,
  [SMALL_STATE(1679)] = 56240,
  [SMALL_STATE(1680)] = 56258,
  [SMALL_STATE(1681)] = 56278,
  [SMALL_STATE(1682)] = 56300,
  [SMALL_STATE(1683)] = 56316,
  [SMALL_STATE(1684)] = 56332,
  [SMALL_STATE(1685)] = 56354,
  [SMALL_STATE(1686)] = 56366,
  [SMALL_STATE(1687)] = 56382,
  [SMALL_STATE(1688)] = 56398,
  [SMALL_STATE(1689)] = 56412,
  [SMALL_STATE(1690)] = 56428,
  [SMALL_STATE(1691)] = 56448,
  [SMALL_STATE(1692)] = 56464,
  [SMALL_STATE(1693)] = 56486,
  [SMALL_STATE(1694)] = 56504,
  [SMALL_STATE(1695)] = 56518,
  [SMALL_STATE(1696)] = 56532,
  [SMALL_STATE(1697)] = 56552,
  [SMALL_STATE(1698)] = 56572,
  [SMALL_STATE(1699)] = 56590,
  [SMALL_STATE(1700)] = 56606,
  [SMALL_STATE(1701)] = 56620,
  [SMALL_STATE(1702)] = 56634,
  [SMALL_STATE(1703)] = 56648,
  [SMALL_STATE(1704)] = 56662,
  [SMALL_STATE(1705)] = 56678,
  [SMALL_STATE(1706)] = 56692,
  [SMALL_STATE(1707)] = 56706,
  [SMALL_STATE(1708)] = 56722,
  [SMALL_STATE(1709)] = 56736,
  [SMALL_STATE(1710)] = 56752,
  [SMALL_STATE(1711)] = 56768,
  [SMALL_STATE(1712)] = 56784,
  [SMALL_STATE(1713)] = 56804,
  [SMALL_STATE(1714)] = 56826,
  [SMALL_STATE(1715)] = 56840,
  [SMALL_STATE(1716)] = 56854,
  [SMALL_STATE(1717)] = 56866,
  [SMALL_STATE(1718)] = 56880,
  [SMALL_STATE(1719)] = 56896,
  [SMALL_STATE(1720)] = 56912,
  [SMALL_STATE(1721)] = 56926,
  [SMALL_STATE(1722)] = 56937,
  [SMALL_STATE(1723)] = 56956,
  [SMALL_STATE(1724)] = 56971,
  [SMALL_STATE(1725)] = 56988,
  [SMALL_STATE(1726)] = 57007,
  [SMALL_STATE(1727)] = 57018,
  [SMALL_STATE(1728)] = 57035,
  [SMALL_STATE(1729)] = 57054,
  [SMALL_STATE(1730)] = 57069,
  [SMALL_STATE(1731)] = 57084,
  [SMALL_STATE(1732)] = 57103,
  [SMALL_STATE(1733)] = 57122,
  [SMALL_STATE(1734)] = 57135,
  [SMALL_STATE(1735)] = 57148,
  [SMALL_STATE(1736)] = 57161,
  [SMALL_STATE(1737)] = 57180,
  [SMALL_STATE(1738)] = 57193,
  [SMALL_STATE(1739)] = 57212,
  [SMALL_STATE(1740)] = 57227,
  [SMALL_STATE(1741)] = 57242,
  [SMALL_STATE(1742)] = 57255,
  [SMALL_STATE(1743)] = 57274,
  [SMALL_STATE(1744)] = 57289,
  [SMALL_STATE(1745)] = 57300,
  [SMALL_STATE(1746)] = 57315,
  [SMALL_STATE(1747)] = 57334,
  [SMALL_STATE(1748)] = 57349,
  [SMALL_STATE(1749)] = 57362,
  [SMALL_STATE(1750)] = 57373,
  [SMALL_STATE(1751)] = 57392,
  [SMALL_STATE(1752)] = 57407,
  [SMALL_STATE(1753)] = 57426,
  [SMALL_STATE(1754)] = 57445,
  [SMALL_STATE(1755)] = 57458,
  [SMALL_STATE(1756)] = 57477,
  [SMALL_STATE(1757)] = 57496,
  [SMALL_STATE(1758)] = 57511,
  [SMALL_STATE(1759)] = 57530,
  [SMALL_STATE(1760)] = 57541,
  [SMALL_STATE(1761)] = 57560,
  [SMALL_STATE(1762)] = 57579,
  [SMALL_STATE(1763)] = 57598,
  [SMALL_STATE(1764)] = 57611,
  [SMALL_STATE(1765)] = 57626,
  [SMALL_STATE(1766)] = 57645,
  [SMALL_STATE(1767)] = 57660,
  [SMALL_STATE(1768)] = 57675,
  [SMALL_STATE(1769)] = 57690,
  [SMALL_STATE(1770)] = 57705,
  [SMALL_STATE(1771)] = 57724,
  [SMALL_STATE(1772)] = 57743,
  [SMALL_STATE(1773)] = 57758,
  [SMALL_STATE(1774)] = 57769,
  [SMALL_STATE(1775)] = 57788,
  [SMALL_STATE(1776)] = 57807,
  [SMALL_STATE(1777)] = 57822,
  [SMALL_STATE(1778)] = 57837,
  [SMALL_STATE(1779)] = 57850,
  [SMALL_STATE(1780)] = 57869,
  [SMALL_STATE(1781)] = 57882,
  [SMALL_STATE(1782)] = 57901,
  [SMALL_STATE(1783)] = 57916,
  [SMALL_STATE(1784)] = 57927,
  [SMALL_STATE(1785)] = 57942,
  [SMALL_STATE(1786)] = 57957,
  [SMALL_STATE(1787)] = 57968,
  [SMALL_STATE(1788)] = 57981,
  [SMALL_STATE(1789)] = 57996,
  [SMALL_STATE(1790)] = 58011,
  [SMALL_STATE(1791)] = 58030,
  [SMALL_STATE(1792)] = 58049,
  [SMALL_STATE(1793)] = 58064,
  [SMALL_STATE(1794)] = 58079,
  [SMALL_STATE(1795)] = 58092,
  [SMALL_STATE(1796)] = 58105,
  [SMALL_STATE(1797)] = 58116,
  [SMALL_STATE(1798)] = 58126,
  [SMALL_STATE(1799)] = 58136,
  [SMALL_STATE(1800)] = 58146,
  [SMALL_STATE(1801)] = 58156,
  [SMALL_STATE(1802)] = 58170,
  [SMALL_STATE(1803)] = 58184,
  [SMALL_STATE(1804)] = 58194,
  [SMALL_STATE(1805)] = 58204,
  [SMALL_STATE(1806)] = 58218,
  [SMALL_STATE(1807)] = 58232,
  [SMALL_STATE(1808)] = 58242,
  [SMALL_STATE(1809)] = 58256,
  [SMALL_STATE(1810)] = 58270,
  [SMALL_STATE(1811)] = 58280,
  [SMALL_STATE(1812)] = 58290,
  [SMALL_STATE(1813)] = 58304,
  [SMALL_STATE(1814)] = 58314,
  [SMALL_STATE(1815)] = 58328,
  [SMALL_STATE(1816)] = 58342,
  [SMALL_STATE(1817)] = 58352,
  [SMALL_STATE(1818)] = 58362,
  [SMALL_STATE(1819)] = 58372,
  [SMALL_STATE(1820)] = 58382,
  [SMALL_STATE(1821)] = 58396,
  [SMALL_STATE(1822)] = 58406,
  [SMALL_STATE(1823)] = 58420,
  [SMALL_STATE(1824)] = 58430,
  [SMALL_STATE(1825)] = 58440,
  [SMALL_STATE(1826)] = 58450,
  [SMALL_STATE(1827)] = 58460,
  [SMALL_STATE(1828)] = 58474,
  [SMALL_STATE(1829)] = 58484,
  [SMALL_STATE(1830)] = 58498,
  [SMALL_STATE(1831)] = 58514,
  [SMALL_STATE(1832)] = 58524,
  [SMALL_STATE(1833)] = 58534,
  [SMALL_STATE(1834)] = 58548,
  [SMALL_STATE(1835)] = 58558,
  [SMALL_STATE(1836)] = 58568,
  [SMALL_STATE(1837)] = 58578,
  [SMALL_STATE(1838)] = 58594,
  [SMALL_STATE(1839)] = 58604,
  [SMALL_STATE(1840)] = 58614,
  [SMALL_STATE(1841)] = 58624,
  [SMALL_STATE(1842)] = 58634,
  [SMALL_STATE(1843)] = 58648,
  [SMALL_STATE(1844)] = 58658,
  [SMALL_STATE(1845)] = 58672,
  [SMALL_STATE(1846)] = 58682,
  [SMALL_STATE(1847)] = 58692,
  [SMALL_STATE(1848)] = 58702,
  [SMALL_STATE(1849)] = 58712,
  [SMALL_STATE(1850)] = 58722,
  [SMALL_STATE(1851)] = 58732,
  [SMALL_STATE(1852)] = 58746,
  [SMALL_STATE(1853)] = 58760,
  [SMALL_STATE(1854)] = 58770,
  [SMALL_STATE(1855)] = 58780,
  [SMALL_STATE(1856)] = 58794,
  [SMALL_STATE(1857)] = 58804,
  [SMALL_STATE(1858)] = 58814,
  [SMALL_STATE(1859)] = 58824,
  [SMALL_STATE(1860)] = 58834,
  [SMALL_STATE(1861)] = 58844,
  [SMALL_STATE(1862)] = 58854,
  [SMALL_STATE(1863)] = 58864,
  [SMALL_STATE(1864)] = 58874,
  [SMALL_STATE(1865)] = 58884,
  [SMALL_STATE(1866)] = 58898,
  [SMALL_STATE(1867)] = 58908,
  [SMALL_STATE(1868)] = 58918,
  [SMALL_STATE(1869)] = 58934,
  [SMALL_STATE(1870)] = 58950,
  [SMALL_STATE(1871)] = 58964,
  [SMALL_STATE(1872)] = 58974,
  [SMALL_STATE(1873)] = 58984,
  [SMALL_STATE(1874)] = 58994,
  [SMALL_STATE(1875)] = 59004,
  [SMALL_STATE(1876)] = 59014,
  [SMALL_STATE(1877)] = 59024,
  [SMALL_STATE(1878)] = 59034,
  [SMALL_STATE(1879)] = 59048,
  [SMALL_STATE(1880)] = 59058,
  [SMALL_STATE(1881)] = 59072,
  [SMALL_STATE(1882)] = 59082,
  [SMALL_STATE(1883)] = 59092,
  [SMALL_STATE(1884)] = 59106,
  [SMALL_STATE(1885)] = 59116,
  [SMALL_STATE(1886)] = 59126,
  [SMALL_STATE(1887)] = 59136,
  [SMALL_STATE(1888)] = 59150,
  [SMALL_STATE(1889)] = 59160,
  [SMALL_STATE(1890)] = 59170,
  [SMALL_STATE(1891)] = 59180,
  [SMALL_STATE(1892)] = 59190,
  [SMALL_STATE(1893)] = 59200,
  [SMALL_STATE(1894)] = 59210,
  [SMALL_STATE(1895)] = 59224,
  [SMALL_STATE(1896)] = 59238,
  [SMALL_STATE(1897)] = 59248,
  [SMALL_STATE(1898)] = 59258,
  [SMALL_STATE(1899)] = 59268,
  [SMALL_STATE(1900)] = 59278,
  [SMALL_STATE(1901)] = 59288,
  [SMALL_STATE(1902)] = 59302,
  [SMALL_STATE(1903)] = 59312,
  [SMALL_STATE(1904)] = 59322,
  [SMALL_STATE(1905)] = 59332,
  [SMALL_STATE(1906)] = 59342,
  [SMALL_STATE(1907)] = 59356,
  [SMALL_STATE(1908)] = 59366,
  [SMALL_STATE(1909)] = 59376,
  [SMALL_STATE(1910)] = 59390,
  [SMALL_STATE(1911)] = 59400,
  [SMALL_STATE(1912)] = 59414,
  [SMALL_STATE(1913)] = 59424,
  [SMALL_STATE(1914)] = 59434,
  [SMALL_STATE(1915)] = 59448,
  [SMALL_STATE(1916)] = 59458,
  [SMALL_STATE(1917)] = 59468,
  [SMALL_STATE(1918)] = 59478,
  [SMALL_STATE(1919)] = 59491,
  [SMALL_STATE(1920)] = 59504,
  [SMALL_STATE(1921)] = 59517,
  [SMALL_STATE(1922)] = 59530,
  [SMALL_STATE(1923)] = 59543,
  [SMALL_STATE(1924)] = 59556,
  [SMALL_STATE(1925)] = 59569,
  [SMALL_STATE(1926)] = 59582,
  [SMALL_STATE(1927)] = 59595,
  [SMALL_STATE(1928)] = 59608,
  [SMALL_STATE(1929)] = 59621,
  [SMALL_STATE(1930)] = 59634,
  [SMALL_STATE(1931)] = 59647,
  [SMALL_STATE(1932)] = 59660,
  [SMALL_STATE(1933)] = 59673,
  [SMALL_STATE(1934)] = 59682,
  [SMALL_STATE(1935)] = 59695,
  [SMALL_STATE(1936)] = 59708,
  [SMALL_STATE(1937)] = 59721,
  [SMALL_STATE(1938)] = 59734,
  [SMALL_STATE(1939)] = 59747,
  [SMALL_STATE(1940)] = 59760,
  [SMALL_STATE(1941)] = 59773,
  [SMALL_STATE(1942)] = 59786,
  [SMALL_STATE(1943)] = 59797,
  [SMALL_STATE(1944)] = 59810,
  [SMALL_STATE(1945)] = 59823,
  [SMALL_STATE(1946)] = 59836,
  [SMALL_STATE(1947)] = 59849,
  [SMALL_STATE(1948)] = 59862,
  [SMALL_STATE(1949)] = 59875,
  [SMALL_STATE(1950)] = 59888,
  [SMALL_STATE(1951)] = 59901,
  [SMALL_STATE(1952)] = 59914,
  [SMALL_STATE(1953)] = 59927,
  [SMALL_STATE(1954)] = 59938,
  [SMALL_STATE(1955)] = 59951,
  [SMALL_STATE(1956)] = 59964,
  [SMALL_STATE(1957)] = 59977,
  [SMALL_STATE(1958)] = 59990,
  [SMALL_STATE(1959)] = 60003,
  [SMALL_STATE(1960)] = 60016,
  [SMALL_STATE(1961)] = 60029,
  [SMALL_STATE(1962)] = 60042,
  [SMALL_STATE(1963)] = 60055,
  [SMALL_STATE(1964)] = 60068,
  [SMALL_STATE(1965)] = 60081,
  [SMALL_STATE(1966)] = 60094,
  [SMALL_STATE(1967)] = 60107,
  [SMALL_STATE(1968)] = 60120,
  [SMALL_STATE(1969)] = 60133,
  [SMALL_STATE(1970)] = 60146,
  [SMALL_STATE(1971)] = 60159,
  [SMALL_STATE(1972)] = 60172,
  [SMALL_STATE(1973)] = 60185,
  [SMALL_STATE(1974)] = 60198,
  [SMALL_STATE(1975)] = 60211,
  [SMALL_STATE(1976)] = 60224,
  [SMALL_STATE(1977)] = 60237,
  [SMALL_STATE(1978)] = 60250,
  [SMALL_STATE(1979)] = 60263,
  [SMALL_STATE(1980)] = 60276,
  [SMALL_STATE(1981)] = 60289,
  [SMALL_STATE(1982)] = 60302,
  [SMALL_STATE(1983)] = 60315,
  [SMALL_STATE(1984)] = 60328,
  [SMALL_STATE(1985)] = 60341,
  [SMALL_STATE(1986)] = 60354,
  [SMALL_STATE(1987)] = 60367,
  [SMALL_STATE(1988)] = 60380,
  [SMALL_STATE(1989)] = 60393,
  [SMALL_STATE(1990)] = 60406,
  [SMALL_STATE(1991)] = 60415,
  [SMALL_STATE(1992)] = 60428,
  [SMALL_STATE(1993)] = 60441,
  [SMALL_STATE(1994)] = 60454,
  [SMALL_STATE(1995)] = 60467,
  [SMALL_STATE(1996)] = 60480,
  [SMALL_STATE(1997)] = 60493,
  [SMALL_STATE(1998)] = 60506,
  [SMALL_STATE(1999)] = 60519,
  [SMALL_STATE(2000)] = 60532,
  [SMALL_STATE(2001)] = 60545,
  [SMALL_STATE(2002)] = 60558,
  [SMALL_STATE(2003)] = 60571,
  [SMALL_STATE(2004)] = 60580,
  [SMALL_STATE(2005)] = 60593,
  [SMALL_STATE(2006)] = 60606,
  [SMALL_STATE(2007)] = 60619,
  [SMALL_STATE(2008)] = 60632,
  [SMALL_STATE(2009)] = 60645,
  [SMALL_STATE(2010)] = 60658,
  [SMALL_STATE(2011)] = 60671,
  [SMALL_STATE(2012)] = 60680,
  [SMALL_STATE(2013)] = 60693,
  [SMALL_STATE(2014)] = 60706,
  [SMALL_STATE(2015)] = 60719,
  [SMALL_STATE(2016)] = 60732,
  [SMALL_STATE(2017)] = 60745,
  [SMALL_STATE(2018)] = 60758,
  [SMALL_STATE(2019)] = 60771,
  [SMALL_STATE(2020)] = 60784,
  [SMALL_STATE(2021)] = 60797,
  [SMALL_STATE(2022)] = 60810,
  [SMALL_STATE(2023)] = 60823,
  [SMALL_STATE(2024)] = 60836,
  [SMALL_STATE(2025)] = 60849,
  [SMALL_STATE(2026)] = 60862,
  [SMALL_STATE(2027)] = 60873,
  [SMALL_STATE(2028)] = 60886,
  [SMALL_STATE(2029)] = 60899,
  [SMALL_STATE(2030)] = 60912,
  [SMALL_STATE(2031)] = 60925,
  [SMALL_STATE(2032)] = 60938,
  [SMALL_STATE(2033)] = 60951,
  [SMALL_STATE(2034)] = 60964,
  [SMALL_STATE(2035)] = 60977,
  [SMALL_STATE(2036)] = 60988,
  [SMALL_STATE(2037)] = 61001,
  [SMALL_STATE(2038)] = 61014,
  [SMALL_STATE(2039)] = 61027,
  [SMALL_STATE(2040)] = 61040,
  [SMALL_STATE(2041)] = 61053,
  [SMALL_STATE(2042)] = 61066,
  [SMALL_STATE(2043)] = 61079,
  [SMALL_STATE(2044)] = 61092,
  [SMALL_STATE(2045)] = 61105,
  [SMALL_STATE(2046)] = 61118,
  [SMALL_STATE(2047)] = 61131,
  [SMALL_STATE(2048)] = 61144,
  [SMALL_STATE(2049)] = 61157,
  [SMALL_STATE(2050)] = 61170,
  [SMALL_STATE(2051)] = 61183,
  [SMALL_STATE(2052)] = 61196,
  [SMALL_STATE(2053)] = 61207,
  [SMALL_STATE(2054)] = 61220,
  [SMALL_STATE(2055)] = 61233,
  [SMALL_STATE(2056)] = 61246,
  [SMALL_STATE(2057)] = 61259,
  [SMALL_STATE(2058)] = 61272,
  [SMALL_STATE(2059)] = 61285,
  [SMALL_STATE(2060)] = 61298,
  [SMALL_STATE(2061)] = 61311,
  [SMALL_STATE(2062)] = 61324,
  [SMALL_STATE(2063)] = 61337,
  [SMALL_STATE(2064)] = 61350,
  [SMALL_STATE(2065)] = 61363,
  [SMALL_STATE(2066)] = 61376,
  [SMALL_STATE(2067)] = 61389,
  [SMALL_STATE(2068)] = 61402,
  [SMALL_STATE(2069)] = 61415,
  [SMALL_STATE(2070)] = 61428,
  [SMALL_STATE(2071)] = 61441,
  [SMALL_STATE(2072)] = 61452,
  [SMALL_STATE(2073)] = 61465,
  [SMALL_STATE(2074)] = 61478,
  [SMALL_STATE(2075)] = 61491,
  [SMALL_STATE(2076)] = 61504,
  [SMALL_STATE(2077)] = 61517,
  [SMALL_STATE(2078)] = 61530,
  [SMALL_STATE(2079)] = 61543,
  [SMALL_STATE(2080)] = 61556,
  [SMALL_STATE(2081)] = 61569,
  [SMALL_STATE(2082)] = 61582,
  [SMALL_STATE(2083)] = 61595,
  [SMALL_STATE(2084)] = 61608,
  [SMALL_STATE(2085)] = 61621,
  [SMALL_STATE(2086)] = 61634,
  [SMALL_STATE(2087)] = 61647,
  [SMALL_STATE(2088)] = 61660,
  [SMALL_STATE(2089)] = 61673,
  [SMALL_STATE(2090)] = 61686,
  [SMALL_STATE(2091)] = 61699,
  [SMALL_STATE(2092)] = 61712,
  [SMALL_STATE(2093)] = 61725,
  [SMALL_STATE(2094)] = 61738,
  [SMALL_STATE(2095)] = 61751,
  [SMALL_STATE(2096)] = 61764,
  [SMALL_STATE(2097)] = 61777,
  [SMALL_STATE(2098)] = 61790,
  [SMALL_STATE(2099)] = 61803,
  [SMALL_STATE(2100)] = 61814,
  [SMALL_STATE(2101)] = 61827,
  [SMALL_STATE(2102)] = 61840,
  [SMALL_STATE(2103)] = 61853,
  [SMALL_STATE(2104)] = 61866,
  [SMALL_STATE(2105)] = 61879,
  [SMALL_STATE(2106)] = 61892,
  [SMALL_STATE(2107)] = 61905,
  [SMALL_STATE(2108)] = 61918,
  [SMALL_STATE(2109)] = 61931,
  [SMALL_STATE(2110)] = 61944,
  [SMALL_STATE(2111)] = 61957,
  [SMALL_STATE(2112)] = 61970,
  [SMALL_STATE(2113)] = 61983,
  [SMALL_STATE(2114)] = 61996,
  [SMALL_STATE(2115)] = 62009,
  [SMALL_STATE(2116)] = 62022,
  [SMALL_STATE(2117)] = 62035,
  [SMALL_STATE(2118)] = 62048,
  [SMALL_STATE(2119)] = 62059,
  [SMALL_STATE(2120)] = 62072,
  [SMALL_STATE(2121)] = 62083,
  [SMALL_STATE(2122)] = 62096,
  [SMALL_STATE(2123)] = 62109,
  [SMALL_STATE(2124)] = 62122,
  [SMALL_STATE(2125)] = 62135,
  [SMALL_STATE(2126)] = 62148,
  [SMALL_STATE(2127)] = 62161,
  [SMALL_STATE(2128)] = 62174,
  [SMALL_STATE(2129)] = 62187,
  [SMALL_STATE(2130)] = 62200,
  [SMALL_STATE(2131)] = 62213,
  [SMALL_STATE(2132)] = 62226,
  [SMALL_STATE(2133)] = 62239,
  [SMALL_STATE(2134)] = 62252,
  [SMALL_STATE(2135)] = 62265,
  [SMALL_STATE(2136)] = 62278,
  [SMALL_STATE(2137)] = 62289,
  [SMALL_STATE(2138)] = 62302,
  [SMALL_STATE(2139)] = 62315,
  [SMALL_STATE(2140)] = 62328,
  [SMALL_STATE(2141)] = 62341,
  [SMALL_STATE(2142)] = 62354,
  [SMALL_STATE(2143)] = 62367,
  [SMALL_STATE(2144)] = 62380,
  [SMALL_STATE(2145)] = 62393,
  [SMALL_STATE(2146)] = 62406,
  [SMALL_STATE(2147)] = 62419,
  [SMALL_STATE(2148)] = 62432,
  [SMALL_STATE(2149)] = 62445,
  [SMALL_STATE(2150)] = 62458,
  [SMALL_STATE(2151)] = 62471,
  [SMALL_STATE(2152)] = 62484,
  [SMALL_STATE(2153)] = 62497,
  [SMALL_STATE(2154)] = 62510,
  [SMALL_STATE(2155)] = 62523,
  [SMALL_STATE(2156)] = 62536,
  [SMALL_STATE(2157)] = 62549,
  [SMALL_STATE(2158)] = 62562,
  [SMALL_STATE(2159)] = 62575,
  [SMALL_STATE(2160)] = 62588,
  [SMALL_STATE(2161)] = 62601,
  [SMALL_STATE(2162)] = 62614,
  [SMALL_STATE(2163)] = 62627,
  [SMALL_STATE(2164)] = 62640,
  [SMALL_STATE(2165)] = 62653,
  [SMALL_STATE(2166)] = 62666,
  [SMALL_STATE(2167)] = 62679,
  [SMALL_STATE(2168)] = 62692,
  [SMALL_STATE(2169)] = 62705,
  [SMALL_STATE(2170)] = 62718,
  [SMALL_STATE(2171)] = 62731,
  [SMALL_STATE(2172)] = 62744,
  [SMALL_STATE(2173)] = 62757,
  [SMALL_STATE(2174)] = 62770,
  [SMALL_STATE(2175)] = 62783,
  [SMALL_STATE(2176)] = 62796,
  [SMALL_STATE(2177)] = 62809,
  [SMALL_STATE(2178)] = 62819,
  [SMALL_STATE(2179)] = 62829,
  [SMALL_STATE(2180)] = 62839,
  [SMALL_STATE(2181)] = 62849,
  [SMALL_STATE(2182)] = 62859,
  [SMALL_STATE(2183)] = 62869,
  [SMALL_STATE(2184)] = 62879,
  [SMALL_STATE(2185)] = 62889,
  [SMALL_STATE(2186)] = 62899,
  [SMALL_STATE(2187)] = 62909,
  [SMALL_STATE(2188)] = 62919,
  [SMALL_STATE(2189)] = 62927,
  [SMALL_STATE(2190)] = 62937,
  [SMALL_STATE(2191)] = 62947,
  [SMALL_STATE(2192)] = 62955,
  [SMALL_STATE(2193)] = 62965,
  [SMALL_STATE(2194)] = 62975,
  [SMALL_STATE(2195)] = 62983,
  [SMALL_STATE(2196)] = 62993,
  [SMALL_STATE(2197)] = 63001,
  [SMALL_STATE(2198)] = 63009,
  [SMALL_STATE(2199)] = 63019,
  [SMALL_STATE(2200)] = 63029,
  [SMALL_STATE(2201)] = 63037,
  [SMALL_STATE(2202)] = 63045,
  [SMALL_STATE(2203)] = 63053,
  [SMALL_STATE(2204)] = 63063,
  [SMALL_STATE(2205)] = 63071,
  [SMALL_STATE(2206)] = 63081,
  [SMALL_STATE(2207)] = 63089,
  [SMALL_STATE(2208)] = 63099,
  [SMALL_STATE(2209)] = 63109,
  [SMALL_STATE(2210)] = 63119,
  [SMALL_STATE(2211)] = 63127,
  [SMALL_STATE(2212)] = 63137,
  [SMALL_STATE(2213)] = 63147,
  [SMALL_STATE(2214)] = 63155,
  [SMALL_STATE(2215)] = 63165,
  [SMALL_STATE(2216)] = 63175,
  [SMALL_STATE(2217)] = 63183,
  [SMALL_STATE(2218)] = 63193,
  [SMALL_STATE(2219)] = 63201,
  [SMALL_STATE(2220)] = 63209,
  [SMALL_STATE(2221)] = 63219,
  [SMALL_STATE(2222)] = 63227,
  [SMALL_STATE(2223)] = 63235,
  [SMALL_STATE(2224)] = 63243,
  [SMALL_STATE(2225)] = 63253,
  [SMALL_STATE(2226)] = 63263,
  [SMALL_STATE(2227)] = 63273,
  [SMALL_STATE(2228)] = 63283,
  [SMALL_STATE(2229)] = 63293,
  [SMALL_STATE(2230)] = 63303,
  [SMALL_STATE(2231)] = 63313,
  [SMALL_STATE(2232)] = 63323,
  [SMALL_STATE(2233)] = 63333,
  [SMALL_STATE(2234)] = 63343,
  [SMALL_STATE(2235)] = 63353,
  [SMALL_STATE(2236)] = 63363,
  [SMALL_STATE(2237)] = 63373,
  [SMALL_STATE(2238)] = 63383,
  [SMALL_STATE(2239)] = 63393,
  [SMALL_STATE(2240)] = 63401,
  [SMALL_STATE(2241)] = 63411,
  [SMALL_STATE(2242)] = 63421,
  [SMALL_STATE(2243)] = 63431,
  [SMALL_STATE(2244)] = 63441,
  [SMALL_STATE(2245)] = 63451,
  [SMALL_STATE(2246)] = 63461,
  [SMALL_STATE(2247)] = 63469,
  [SMALL_STATE(2248)] = 63479,
  [SMALL_STATE(2249)] = 63489,
  [SMALL_STATE(2250)] = 63499,
  [SMALL_STATE(2251)] = 63507,
  [SMALL_STATE(2252)] = 63517,
  [SMALL_STATE(2253)] = 63527,
  [SMALL_STATE(2254)] = 63537,
  [SMALL_STATE(2255)] = 63547,
  [SMALL_STATE(2256)] = 63557,
  [SMALL_STATE(2257)] = 63567,
  [SMALL_STATE(2258)] = 63577,
  [SMALL_STATE(2259)] = 63587,
  [SMALL_STATE(2260)] = 63597,
  [SMALL_STATE(2261)] = 63607,
  [SMALL_STATE(2262)] = 63617,
  [SMALL_STATE(2263)] = 63627,
  [SMALL_STATE(2264)] = 63637,
  [SMALL_STATE(2265)] = 63647,
  [SMALL_STATE(2266)] = 63654,
  [SMALL_STATE(2267)] = 63661,
  [SMALL_STATE(2268)] = 63668,
  [SMALL_STATE(2269)] = 63675,
  [SMALL_STATE(2270)] = 63682,
  [SMALL_STATE(2271)] = 63689,
  [SMALL_STATE(2272)] = 63696,
  [SMALL_STATE(2273)] = 63703,
  [SMALL_STATE(2274)] = 63710,
  [SMALL_STATE(2275)] = 63717,
  [SMALL_STATE(2276)] = 63724,
  [SMALL_STATE(2277)] = 63731,
  [SMALL_STATE(2278)] = 63738,
  [SMALL_STATE(2279)] = 63745,
  [SMALL_STATE(2280)] = 63752,
  [SMALL_STATE(2281)] = 63759,
  [SMALL_STATE(2282)] = 63766,
  [SMALL_STATE(2283)] = 63773,
  [SMALL_STATE(2284)] = 63780,
  [SMALL_STATE(2285)] = 63787,
  [SMALL_STATE(2286)] = 63794,
  [SMALL_STATE(2287)] = 63801,
  [SMALL_STATE(2288)] = 63808,
  [SMALL_STATE(2289)] = 63815,
  [SMALL_STATE(2290)] = 63822,
  [SMALL_STATE(2291)] = 63829,
  [SMALL_STATE(2292)] = 63836,
  [SMALL_STATE(2293)] = 63843,
  [SMALL_STATE(2294)] = 63850,
  [SMALL_STATE(2295)] = 63857,
  [SMALL_STATE(2296)] = 63864,
  [SMALL_STATE(2297)] = 63871,
  [SMALL_STATE(2298)] = 63878,
  [SMALL_STATE(2299)] = 63885,
  [SMALL_STATE(2300)] = 63892,
  [SMALL_STATE(2301)] = 63899,
  [SMALL_STATE(2302)] = 63906,
  [SMALL_STATE(2303)] = 63913,
  [SMALL_STATE(2304)] = 63920,
  [SMALL_STATE(2305)] = 63927,
  [SMALL_STATE(2306)] = 63934,
  [SMALL_STATE(2307)] = 63941,
  [SMALL_STATE(2308)] = 63948,
  [SMALL_STATE(2309)] = 63955,
  [SMALL_STATE(2310)] = 63962,
  [SMALL_STATE(2311)] = 63969,
  [SMALL_STATE(2312)] = 63976,
  [SMALL_STATE(2313)] = 63983,
  [SMALL_STATE(2314)] = 63990,
  [SMALL_STATE(2315)] = 63997,
  [SMALL_STATE(2316)] = 64004,
  [SMALL_STATE(2317)] = 64011,
  [SMALL_STATE(2318)] = 64018,
  [SMALL_STATE(2319)] = 64025,
  [SMALL_STATE(2320)] = 64032,
  [SMALL_STATE(2321)] = 64039,
  [SMALL_STATE(2322)] = 64046,
  [SMALL_STATE(2323)] = 64053,
  [SMALL_STATE(2324)] = 64060,
  [SMALL_STATE(2325)] = 64067,
  [SMALL_STATE(2326)] = 64074,
  [SMALL_STATE(2327)] = 64081,
  [SMALL_STATE(2328)] = 64088,
  [SMALL_STATE(2329)] = 64095,
  [SMALL_STATE(2330)] = 64102,
  [SMALL_STATE(2331)] = 64109,
  [SMALL_STATE(2332)] = 64116,
  [SMALL_STATE(2333)] = 64123,
  [SMALL_STATE(2334)] = 64130,
  [SMALL_STATE(2335)] = 64137,
  [SMALL_STATE(2336)] = 64144,
  [SMALL_STATE(2337)] = 64151,
  [SMALL_STATE(2338)] = 64158,
  [SMALL_STATE(2339)] = 64165,
  [SMALL_STATE(2340)] = 64172,
  [SMALL_STATE(2341)] = 64179,
  [SMALL_STATE(2342)] = 64186,
  [SMALL_STATE(2343)] = 64193,
  [SMALL_STATE(2344)] = 64200,
  [SMALL_STATE(2345)] = 64207,
  [SMALL_STATE(2346)] = 64214,
  [SMALL_STATE(2347)] = 64221,
  [SMALL_STATE(2348)] = 64228,
  [SMALL_STATE(2349)] = 64235,
  [SMALL_STATE(2350)] = 64242,
  [SMALL_STATE(2351)] = 64249,
  [SMALL_STATE(2352)] = 64256,
  [SMALL_STATE(2353)] = 64263,
  [SMALL_STATE(2354)] = 64270,
  [SMALL_STATE(2355)] = 64277,
  [SMALL_STATE(2356)] = 64284,
  [SMALL_STATE(2357)] = 64291,
  [SMALL_STATE(2358)] = 64298,
  [SMALL_STATE(2359)] = 64305,
  [SMALL_STATE(2360)] = 64312,
  [SMALL_STATE(2361)] = 64319,
  [SMALL_STATE(2362)] = 64326,
  [SMALL_STATE(2363)] = 64333,
  [SMALL_STATE(2364)] = 64340,
  [SMALL_STATE(2365)] = 64347,
  [SMALL_STATE(2366)] = 64354,
  [SMALL_STATE(2367)] = 64361,
  [SMALL_STATE(2368)] = 64368,
  [SMALL_STATE(2369)] = 64375,
  [SMALL_STATE(2370)] = 64382,
  [SMALL_STATE(2371)] = 64389,
  [SMALL_STATE(2372)] = 64396,
  [SMALL_STATE(2373)] = 64403,
  [SMALL_STATE(2374)] = 64410,
  [SMALL_STATE(2375)] = 64417,
  [SMALL_STATE(2376)] = 64424,
  [SMALL_STATE(2377)] = 64431,
  [SMALL_STATE(2378)] = 64438,
  [SMALL_STATE(2379)] = 64445,
  [SMALL_STATE(2380)] = 64452,
  [SMALL_STATE(2381)] = 64459,
  [SMALL_STATE(2382)] = 64466,
  [SMALL_STATE(2383)] = 64473,
  [SMALL_STATE(2384)] = 64480,
  [SMALL_STATE(2385)] = 64487,
  [SMALL_STATE(2386)] = 64494,
  [SMALL_STATE(2387)] = 64501,
  [SMALL_STATE(2388)] = 64508,
  [SMALL_STATE(2389)] = 64515,
  [SMALL_STATE(2390)] = 64522,
  [SMALL_STATE(2391)] = 64529,
  [SMALL_STATE(2392)] = 64536,
  [SMALL_STATE(2393)] = 64543,
  [SMALL_STATE(2394)] = 64550,
  [SMALL_STATE(2395)] = 64557,
  [SMALL_STATE(2396)] = 64564,
  [SMALL_STATE(2397)] = 64571,
  [SMALL_STATE(2398)] = 64578,
  [SMALL_STATE(2399)] = 64585,
};

static TSParseActionEntry ts_parse_actions[] = {
  [0] = {.entry = {.count = 0, .reusable = false}},
  [1] = {.entry = {.count = 1, .reusable = false}}, RECOVER(),
  [3] = {.entry = {.count = 1, .reusable = true}}, SHIFT_EXTRA(),
  [5] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_compilation_unit, 0),
  [7] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1960),
  [9] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2179),
  [11] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2164),
  [13] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2181),
  [15] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2391),
  [17] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2373),
  [19] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1562),
  [21] = {.entry = {.count = 1, .reusable = true}}, SHIFT(733),
  [23] = {.entry = {.count = 1, .reusable = true}}, SHIFT(734),
  [25] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2363),
  [27] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2356),
  [29] = {.entry = {.count = 1, .reusable = true}}, SHIFT(761),
  [31] = {.entry = {.count = 1, .reusable = false}}, SHIFT(699),
  [33] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2090),
  [35] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2186),
  [37] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2097),
  [39] = {.entry = {.count = 1, .reusable = true}}, SHIFT(10),
  [41] = {.entry = {.count = 1, .reusable = true}}, SHIFT(479),
  [43] = {.entry = {.count = 1, .reusable = false}}, SHIFT(672),
  [45] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2381),
  [47] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2382),
  [49] = {.entry = {.count = 1, .reusable = true}}, SHIFT(132),
  [51] = {.entry = {.count = 1, .reusable = false}}, SHIFT(725),
  [53] = {.entry = {.count = 1, .reusable = false}}, SHIFT(714),
  [55] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2393),
  [57] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2310),
  [59] = {.entry = {.count = 1, .reusable = false}}, SHIFT(761),
  [61] = {.entry = {.count = 1, .reusable = true}}, SHIFT(133),
  [63] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2256),
  [65] = {.entry = {.count = 1, .reusable = false}}, SHIFT(135),
  [67] = {.entry = {.count = 1, .reusable = false}}, SHIFT(136),
  [69] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1063),
  [71] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1007),
  [73] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2262),
  [75] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2261),
  [77] = {.entry = {.count = 1, .reusable = true}}, SHIFT(280),
  [79] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1383),
  [81] = {.entry = {.count = 1, .reusable = true}}, SHIFT(933),
  [83] = {.entry = {.count = 1, .reusable = true}}, SHIFT(435),
  [85] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1200),
  [87] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1412),
  [89] = {.entry = {.count = 1, .reusable = true}}, SHIFT(347),
  [91] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1087),
  [93] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1311),
  [95] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_clause, 4, .production_id = 141),
  [97] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_clause, 4, .production_id = 141),
  [99] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_clause, 3, .production_id = 141),
  [101] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_clause, 3, .production_id = 141),
  [103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2210),
  [105] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2184),
  [107] = {.entry = {.count = 1, .reusable = true}}, SHIFT(648),
  [109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2250),
  [111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1902),
  [113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1823),
  [115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(589),
  [117] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block, 2),
  [119] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block, 3),
  [121] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__block, 3),
  [123] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__block, 2),
  [125] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_compilation_unit_repeat1, 2),
  [127] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compilation_unit_repeat1, 2), SHIFT_REPEAT(1960),
  [130] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compilation_unit_repeat1, 2), SHIFT_REPEAT(2179),
  [133] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compilation_unit_repeat1, 2), SHIFT_REPEAT(2164),
  [136] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compilation_unit_repeat1, 2), SHIFT_REPEAT(2181),
  [139] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compilation_unit_repeat1, 2), SHIFT_REPEAT(2391),
  [142] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compilation_unit_repeat1, 2), SHIFT_REPEAT(2373),
  [145] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compilation_unit_repeat1, 2), SHIFT_REPEAT(1562),
  [148] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compilation_unit_repeat1, 2), SHIFT_REPEAT(733),
  [151] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compilation_unit_repeat1, 2), SHIFT_REPEAT(734),
  [154] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compilation_unit_repeat1, 2), SHIFT_REPEAT(2363),
  [157] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compilation_unit_repeat1, 2), SHIFT_REPEAT(2356),
  [160] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compilation_unit_repeat1, 2), SHIFT_REPEAT(761),
  [163] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_compilation_unit, 1),
  [165] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_try_expression, 2, .production_id = 50),
  [167] = {.entry = {.count = 1, .reusable = false}}, SHIFT(250),
  [169] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_try_expression, 2, .production_id = 50),
  [171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2314),
  [173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(793),
  [175] = {.entry = {.count = 1, .reusable = false}}, SHIFT(239),
  [177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(68),
  [179] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2227),
  [181] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2259),
  [183] = {.entry = {.count = 1, .reusable = false}}, SHIFT(151),
  [185] = {.entry = {.count = 1, .reusable = false}}, SHIFT_EXTRA(),
  [187] = {.entry = {.count = 1, .reusable = false}}, SHIFT(249),
  [189] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression, 1),
  [191] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__expression, 1),
  [193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(297),
  [195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2199),
  [197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2198),
  [199] = {.entry = {.count = 1, .reusable = false}}, SHIFT(98),
  [201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2290),
  [203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(856),
  [205] = {.entry = {.count = 1, .reusable = false}}, SHIFT(97),
  [207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(67),
  [209] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2251),
  [211] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2263),
  [213] = {.entry = {.count = 1, .reusable = false}}, SHIFT(99),
  [215] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_annotation, 2, .production_id = 1),
  [217] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_annotation, 2, .production_id = 1),
  [219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(886),
  [221] = {.entry = {.count = 1, .reusable = true}}, SHIFT(54),
  [223] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2279),
  [225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(369),
  [227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2237),
  [229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2236),
  [231] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_expression, 2, .production_id = 52),
  [233] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_expression, 2, .production_id = 52),
  [235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3),
  [237] = {.entry = {.count = 1, .reusable = false}}, SHIFT(92),
  [239] = {.entry = {.count = 1, .reusable = false}}, SHIFT(779),
  [241] = {.entry = {.count = 1, .reusable = true}}, SHIFT(11),
  [243] = {.entry = {.count = 1, .reusable = true}}, SHIFT(207),
  [245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(82),
  [247] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1520),
  [249] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2257),
  [251] = {.entry = {.count = 1, .reusable = false}}, SHIFT(206),
  [253] = {.entry = {.count = 1, .reusable = false}}, SHIFT(205),
  [255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1284),
  [257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1369),
  [259] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2253),
  [261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2252),
  [263] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1569),
  [265] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_prefix_expression, 2),
  [267] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_prefix_expression, 2),
  [269] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assignment_expression, 3, .production_id = 94),
  [271] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_assignment_expression, 3, .production_id = 94),
  [273] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_infix_expression, 3, .production_id = 40),
  [275] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_infix_expression, 3, .production_id = 40),
  [277] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_instance_expression, 2),
  [279] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_instance_expression, 2),
  [281] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_finally_clause, 2),
  [283] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_finally_clause, 2),
  [285] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_expression, 5, .production_id = 176),
  [287] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_expression, 5, .production_id = 176),
  [289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(426),
  [291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2207),
  [293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2205),
  [295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(872),
  [297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(74),
  [299] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2289),
  [301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(972),
  [303] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_expression, 3, .production_id = 92),
  [305] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_expression, 3, .production_id = 92),
  [307] = {.entry = {.count = 1, .reusable = false}}, SHIFT(227),
  [309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1148),
  [311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(423),
  [313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1524),
  [315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1080),
  [317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(329),
  [319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(9),
  [321] = {.entry = {.count = 1, .reusable = false}}, SHIFT(120),
  [323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(431),
  [325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(996),
  [327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(500),
  [329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1518),
  [331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1302),
  [333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1452),
  [335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1474),
  [337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1468),
  [339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1457),
  [341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1382),
  [343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(550),
  [345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1292),
  [347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(754),
  [349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(296),
  [351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(232),
  [353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1371),
  [355] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__annotated_type, 1),
  [357] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__annotated_type, 1),
  [359] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1559),
  [361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1721),
  [363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1214),
  [365] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1213),
  [367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(476),
  [369] = {.entry = {.count = 1, .reusable = false}}, SHIFT(87),
  [371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2),
  [373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(188),
  [375] = {.entry = {.count = 1, .reusable = true}}, SHIFT(194),
  [377] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2255),
  [379] = {.entry = {.count = 1, .reusable = false}}, SHIFT(186),
  [381] = {.entry = {.count = 1, .reusable = false}}, SHIFT(184),
  [383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(477),
  [385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(466),
  [387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2244),
  [389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2243),
  [391] = {.entry = {.count = 1, .reusable = false}}, SHIFT(681),
  [393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5),
  [395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(162),
  [397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(198),
  [399] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2180),
  [401] = {.entry = {.count = 1, .reusable = false}}, SHIFT(161),
  [403] = {.entry = {.count = 1, .reusable = false}}, SHIFT(160),
  [405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(935),
  [407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(838),
  [409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2225),
  [411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2224),
  [413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(911),
  [415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(56),
  [417] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2352),
  [419] = {.entry = {.count = 1, .reusable = false}}, SHIFT(794),
  [421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(8),
  [423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(122),
  [425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(181),
  [427] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2220),
  [429] = {.entry = {.count = 1, .reusable = false}}, SHIFT(118),
  [431] = {.entry = {.count = 1, .reusable = false}}, SHIFT(116),
  [433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1392),
  [435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1434),
  [437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2193),
  [439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2192),
  [441] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1542),
  [443] = {.entry = {.count = 1, .reusable = false}}, SHIFT(32),
  [445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(179),
  [447] = {.entry = {.count = 1, .reusable = true}}, SHIFT(173),
  [449] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2248),
  [451] = {.entry = {.count = 1, .reusable = false}}, SHIFT(177),
  [453] = {.entry = {.count = 1, .reusable = false}}, SHIFT(176),
  [455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(353),
  [457] = {.entry = {.count = 1, .reusable = false}}, SHIFT(769),
  [459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4),
  [461] = {.entry = {.count = 1, .reusable = true}}, SHIFT(114),
  [463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(200),
  [465] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2211),
  [467] = {.entry = {.count = 1, .reusable = false}}, SHIFT(112),
  [469] = {.entry = {.count = 1, .reusable = false}}, SHIFT(110),
  [471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1377),
  [473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1364),
  [475] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2190),
  [477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2189),
  [479] = {.entry = {.count = 1, .reusable = false}}, SHIFT(29),
  [481] = {.entry = {.count = 1, .reusable = true}}, SHIFT(126),
  [483] = {.entry = {.count = 1, .reusable = true}}, SHIFT(109),
  [485] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2228),
  [487] = {.entry = {.count = 1, .reusable = false}}, SHIFT(125),
  [489] = {.entry = {.count = 1, .reusable = false}}, SHIFT(124),
  [491] = {.entry = {.count = 1, .reusable = true}}, SHIFT(272),
  [493] = {.entry = {.count = 1, .reusable = false}}, SHIFT(43),
  [495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6),
  [497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(140),
  [499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(229),
  [501] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2234),
  [503] = {.entry = {.count = 1, .reusable = false}}, SHIFT(139),
  [505] = {.entry = {.count = 1, .reusable = false}}, SHIFT(138),
  [507] = {.entry = {.count = 1, .reusable = true}}, SHIFT(425),
  [509] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__simple_type, 1, .production_id = 6),
  [511] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__simple_type, 1, .production_id = 6),
  [513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2281),
  [515] = {.entry = {.count = 1, .reusable = false}}, SHIFT(737),
  [517] = {.entry = {.count = 1, .reusable = true}}, SHIFT(7),
  [519] = {.entry = {.count = 1, .reusable = true}}, SHIFT(155),
  [521] = {.entry = {.count = 1, .reusable = true}}, SHIFT(236),
  [523] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2238),
  [525] = {.entry = {.count = 1, .reusable = false}}, SHIFT(154),
  [527] = {.entry = {.count = 1, .reusable = false}}, SHIFT(153),
  [529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1191),
  [531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1151),
  [533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2215),
  [535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2214),
  [537] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_try_expression, 3, .production_id = 50),
  [539] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_try_expression, 3, .production_id = 50),
  [541] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_stable_type_identifier, 3, .production_id = 33),
  [543] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_stable_type_identifier, 3, .production_id = 33),
  [545] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_stable_identifier, 3),
  [547] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1581),
  [549] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_arguments, 4),
  [551] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_arguments, 4),
  [553] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_arguments, 3),
  [555] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_arguments, 3),
  [557] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_arguments, 2),
  [559] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_arguments, 2),
  [561] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 3, .production_id = 68),
  [563] = {.entry = {.count = 1, .reusable = false}}, SHIFT(242),
  [565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2294),
  [567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(808),
  [569] = {.entry = {.count = 1, .reusable = false}}, SHIFT(241),
  [571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(51),
  [573] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2242),
  [575] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_arguments, 4),
  [577] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_arguments, 4),
  [579] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_arguments, 3),
  [581] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_arguments, 3),
  [583] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 3),
  [585] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 3),
  [587] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 2),
  [589] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 2),
  [591] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_block, 2),
  [593] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_block, 2),
  [595] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_try_expression, 4, .production_id = 50),
  [597] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_try_expression, 4, .production_id = 50),
  [599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2286),
  [601] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_catch_clause, 2),
  [603] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_catch_clause, 2),
  [605] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_expression, 3, .production_id = 96),
  [607] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_expression, 3, .production_id = 96),
  [609] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_expression, 4),
  [611] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_expression, 4),
  [613] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 4, .production_id = 103),
  [615] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_annotation, 3, .production_id = 17),
  [617] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_annotation, 3, .production_id = 17),
  [619] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_function, 2, .production_id = 51),
  [621] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_function, 2, .production_id = 51),
  [623] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string, 4),
  [625] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string, 4),
  [627] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string, 3),
  [629] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string, 3),
  [631] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string, 1),
  [633] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string, 1),
  [635] = {.entry = {.count = 1, .reusable = false}}, SHIFT(257),
  [637] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_expression, 3, .production_id = 95),
  [639] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_expression, 3, .production_id = 95),
  [641] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_type, 4),
  [643] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_type, 4),
  [645] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2),
  [647] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_annotation_repeat1, 2),
  [649] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(54),
  [652] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_expression, 3, .production_id = 93),
  [654] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_expression, 3, .production_id = 93),
  [656] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_projected_type, 3, .production_id = 34),
  [658] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_projected_type, 3, .production_id = 34),
  [660] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 3, .production_id = 59),
  [662] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_type, 3),
  [664] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_type, 3),
  [666] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_expression, 3),
  [668] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_expression, 3),
  [670] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_transform_expression, 2),
  [672] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_transform_expression, 2),
  [674] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_type, 2, .production_id = 16),
  [676] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_type, 2, .production_id = 16),
  [678] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_block, 3),
  [680] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_block, 3),
  [682] = {.entry = {.count = 1, .reusable = false}}, SHIFT(93),
  [684] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_definition, 7, .production_id = 154),
  [686] = {.entry = {.count = 1, .reusable = false}}, SHIFT(89),
  [688] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_definition, 7, .production_id = 154),
  [690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2317),
  [692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(883),
  [694] = {.entry = {.count = 1, .reusable = false}}, SHIFT(88),
  [696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(48),
  [698] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2230),
  [700] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_class_definition_repeat1, 2),
  [702] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2),
  [704] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1559),
  [707] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 6, .production_id = 138),
  [709] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 6, .production_id = 138),
  [711] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_definition, 6, .production_id = 135),
  [713] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_definition, 6, .production_id = 135),
  [715] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_val_definition, 6, .production_id = 135),
  [717] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_val_definition, 6, .production_id = 135),
  [719] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 10, .production_id = 205),
  [721] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 10, .production_id = 205),
  [723] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 7, .production_id = 147),
  [725] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 7, .production_id = 147),
  [727] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 7, .production_id = 149),
  [729] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 7, .production_id = 149),
  [731] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 2, .production_id = 1),
  [733] = {.entry = {.count = 1, .reusable = true}}, SHIFT(19),
  [735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1258),
  [737] = {.entry = {.count = 1, .reusable = true}}, SHIFT(949),
  [739] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1483),
  [741] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 6, .production_id = 129),
  [743] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 6, .production_id = 129),
  [745] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_val_definition, 7, .production_id = 154),
  [747] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_val_definition, 7, .production_id = 154),
  [749] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 7, .production_id = 155),
  [751] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 7, .production_id = 155),
  [753] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 2, .production_id = 1),
  [755] = {.entry = {.count = 1, .reusable = true}}, SHIFT(15),
  [757] = {.entry = {.count = 1, .reusable = true}}, SHIFT(889),
  [759] = {.entry = {.count = 1, .reusable = true}}, SHIFT(117),
  [761] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1607),
  [763] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2358),
  [765] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(74),
  [768] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 4, .production_id = 48),
  [770] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 6, .production_id = 126),
  [772] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 6, .production_id = 126),
  [774] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 7, .production_id = 158),
  [776] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 7, .production_id = 158),
  [778] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 7, .production_id = 170),
  [780] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 7, .production_id = 170),
  [782] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__annotated_type, 2),
  [784] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__annotated_type, 2),
  [786] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 3, .production_id = 12),
  [788] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 7, .production_id = 173),
  [790] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 7, .production_id = 173),
  [792] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 3, .production_id = 12),
  [794] = {.entry = {.count = 1, .reusable = true}}, SHIFT(973),
  [796] = {.entry = {.count = 1, .reusable = true}}, SHIFT(253),
  [798] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 9, .production_id = 201),
  [800] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 9, .production_id = 201),
  [802] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 6, .production_id = 114),
  [804] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 6, .production_id = 114),
  [806] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_val_definition, 4, .production_id = 38),
  [808] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_val_definition, 4, .production_id = 38),
  [810] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 9, .production_id = 203),
  [812] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 9, .production_id = 203),
  [814] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 9, .production_id = 199),
  [816] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 9, .production_id = 199),
  [818] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 8, .production_id = 181),
  [820] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 8, .production_id = 181),
  [822] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 6, .production_id = 111),
  [824] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 6, .production_id = 111),
  [826] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 5, .production_id = 75),
  [828] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 5, .production_id = 75),
  [830] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_definition, 6, .production_id = 108),
  [832] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_definition, 6, .production_id = 108),
  [834] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 5, .production_id = 78),
  [836] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 5, .production_id = 78),
  [838] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_val_definition, 6, .production_id = 108),
  [840] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_val_definition, 6, .production_id = 108),
  [842] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 8, .production_id = 183),
  [844] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 8, .production_id = 183),
  [846] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_definition, 4, .production_id = 38),
  [848] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_definition, 4, .production_id = 38),
  [850] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_val_definition, 5, .production_id = 84),
  [852] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_val_definition, 5, .production_id = 84),
  [854] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 8, .production_id = 185),
  [856] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 8, .production_id = 185),
  [858] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_val_definition, 8, .production_id = 190),
  [860] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_val_definition, 8, .production_id = 190),
  [862] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_definition, 5, .production_id = 84),
  [864] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_definition, 5, .production_id = 84),
  [866] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_definition, 8, .production_id = 190),
  [868] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_definition, 8, .production_id = 190),
  [870] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 8, .production_id = 191),
  [872] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 8, .production_id = 191),
  [874] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 8, .production_id = 194),
  [876] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 8, .production_id = 194),
  [878] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 5, .production_id = 87),
  [880] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 5, .production_id = 87),
  [882] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 4, .production_id = 48),
  [884] = {.entry = {.count = 1, .reusable = true}}, SHIFT(851),
  [886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(252),
  [888] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 4, .production_id = 45),
  [890] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 4, .production_id = 45),
  [892] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 90),
  [894] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1581),
  [897] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type, 1),
  [899] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1338),
  [901] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type, 1),
  [903] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameter_types, 1),
  [905] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1512),
  [907] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(56),
  [910] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_compound_type, 2, .production_id = 32),
  [912] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_compound_type, 2, .production_id = 32),
  [914] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_infix_type, 3, .production_id = 40),
  [916] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_infix_type, 3, .production_id = 40),
  [918] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1542),
  [921] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67),
  [923] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67),
  [925] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67), SHIFT_REPEAT(1512),
  [928] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 133),
  [930] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67), SHIFT_REPEAT(1530),
  [933] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 66),
  [935] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 66),
  [937] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 6, .production_id = 139),
  [939] = {.entry = {.count = 1, .reusable = true}}, SHIFT(938),
  [941] = {.entry = {.count = 1, .reusable = true}}, SHIFT(127),
  [943] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 3, .production_id = 15),
  [945] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 3, .production_id = 13),
  [947] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 5, .production_id = 91),
  [949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(921),
  [951] = {.entry = {.count = 1, .reusable = true}}, SHIFT(178),
  [953] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1530),
  [955] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 5, .production_id = 79),
  [957] = {.entry = {.count = 1, .reusable = true}}, SHIFT(917),
  [959] = {.entry = {.count = 1, .reusable = true}}, SHIFT(185),
  [961] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 4, .production_id = 22),
  [963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(839),
  [965] = {.entry = {.count = 1, .reusable = true}}, SHIFT(261),
  [967] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 4, .production_id = 46),
  [969] = {.entry = {.count = 1, .reusable = true}}, SHIFT(830),
  [971] = {.entry = {.count = 1, .reusable = true}}, SHIFT(264),
  [973] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 4, .production_id = 49),
  [975] = {.entry = {.count = 1, .reusable = true}}, SHIFT(842),
  [977] = {.entry = {.count = 1, .reusable = true}}, SHIFT(260),
  [979] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 4, .production_id = 28),
  [981] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 4, .production_id = 22),
  [983] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1248),
  [985] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67), SHIFT_REPEAT(1482),
  [988] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1482),
  [990] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 130),
  [992] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 5, .production_id = 88),
  [994] = {.entry = {.count = 1, .reusable = true}}, SHIFT(943),
  [996] = {.entry = {.count = 1, .reusable = true}}, SHIFT(190),
  [998] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 55),
  [1000] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 7, .production_id = 162),
  [1002] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 82),
  [1004] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 79),
  [1006] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 118),
  [1008] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 3, .production_id = 18),
  [1010] = {.entry = {.count = 1, .reusable = true}}, SHIFT(988),
  [1012] = {.entry = {.count = 1, .reusable = true}}, SHIFT(238),
  [1014] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 3, .production_id = 13),
  [1016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(963),
  [1018] = {.entry = {.count = 1, .reusable = true}}, SHIFT(234),
  [1020] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1228),
  [1022] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 4, .production_id = 25),
  [1024] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_function_definition_repeat1, 2),
  [1026] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_function_definition_repeat1, 2), SHIFT_REPEAT(1607),
  [1029] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_definition, 2, .production_id = 1),
  [1031] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_declaration, 4, .production_id = 36),
  [1033] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_typed_pattern, 3, .production_id = 35),
  [1035] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_declaration, 4, .production_id = 36),
  [1037] = {.entry = {.count = 1, .reusable = false}}, SHIFT(233),
  [1039] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_val_declaration, 4, .production_id = 36),
  [1041] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_val_declaration, 4, .production_id = 36),
  [1043] = {.entry = {.count = 1, .reusable = false}}, SHIFT(266),
  [1045] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_val_declaration, 5, .production_id = 83),
  [1047] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_val_declaration, 5, .production_id = 83),
  [1049] = {.entry = {.count = 1, .reusable = false}}, SHIFT(197),
  [1051] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_class_definition_repeat2, 2),
  [1053] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_class_definition_repeat2, 2), SHIFT_REPEAT(1483),
  [1056] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_declaration, 6, .production_id = 134),
  [1058] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_declaration, 6, .production_id = 134),
  [1060] = {.entry = {.count = 1, .reusable = false}}, SHIFT(130),
  [1062] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_declaration, 5, .production_id = 83),
  [1064] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_declaration, 5, .production_id = 83),
  [1066] = {.entry = {.count = 1, .reusable = false}}, SHIFT(76),
  [1068] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_val_declaration, 6, .production_id = 134),
  [1070] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_val_declaration, 6, .production_id = 134),
  [1072] = {.entry = {.count = 1, .reusable = false}}, SHIFT(131),
  [1074] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 3, .production_id = 68),
  [1076] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 4),
  [1078] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_definition, 3, .production_id = 13),
  [1080] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 2),
  [1082] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 5),
  [1084] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 3),
  [1086] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__object_definition, 1, .production_id = 2),
  [1088] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2354),
  [1090] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 5, .production_id = 86),
  [1092] = {.entry = {.count = 1, .reusable = true}}, SHIFT(193),
  [1094] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 6, .production_id = 128),
  [1096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(137),
  [1098] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 6, .production_id = 113),
  [1100] = {.entry = {.count = 1, .reusable = true}}, SHIFT(144),
  [1102] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_package_identifier, 1),
  [1104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2283),
  [1106] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 8, .production_id = 193),
  [1108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(101),
  [1110] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameters, 3),
  [1112] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 7, .production_id = 157),
  [1114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(108),
  [1116] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 6, .production_id = 125),
  [1118] = {.entry = {.count = 1, .reusable = true}}, SHIFT(142),
  [1120] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameters, 4),
  [1122] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 4, .production_id = 44),
  [1124] = {.entry = {.count = 1, .reusable = true}}, SHIFT(217),
  [1126] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameters, 5),
  [1128] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extends_clause, 2, .production_id = 19),
  [1130] = {.entry = {.count = 1, .reusable = true}}, SHIFT(64),
  [1132] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 6, .production_id = 137),
  [1134] = {.entry = {.count = 1, .reusable = true}}, SHIFT(129),
  [1136] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_package_identifier_repeat1, 2),
  [1138] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_package_identifier_repeat1, 2), SHIFT_REPEAT(2283),
  [1141] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 7, .production_id = 169),
  [1143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(107),
  [1145] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 7, .production_id = 172),
  [1147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(105),
  [1149] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 5, .production_id = 77),
  [1151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(201),
  [1153] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_package_identifier, 2),
  [1155] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 5, .production_id = 74),
  [1157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(202),
  [1159] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameters, 2),
  [1161] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_definition, 4, .production_id = 27),
  [1163] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 3, .production_id = 14),
  [1165] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_package_clause, 2, .production_id = 1),
  [1167] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 117),
  [1169] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__import_expression, 1, .production_id = 4),
  [1171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1869),
  [1173] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_declaration, 2, .production_id = 5),
  [1175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1983),
  [1177] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 54),
  [1179] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 7, .production_id = 161),
  [1181] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 58),
  [1183] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 64),
  [1185] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 4, .production_id = 31),
  [1187] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__object_definition, 2, .production_id = 10),
  [1189] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 81),
  [1191] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_declaration, 3, .production_id = 11),
  [1193] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 7, .production_id = 152),
  [1195] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 4, .production_id = 27),
  [1197] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_definition, 3, .production_id = 14),
  [1199] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 121),
  [1201] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 4, .production_id = 24),
  [1203] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 8, .production_id = 188),
  [1205] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 7, .production_id = 165),
  [1207] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_import_declaration_repeat1, 2, .production_id = 21),
  [1209] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_import_declaration_repeat1, 2, .production_id = 21), SHIFT_REPEAT(1983),
  [1212] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 99),
  [1214] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 132),
  [1216] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__import_expression, 3, .production_id = 4),
  [1218] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_import_declaration_repeat1, 2, .production_id = 5),
  [1220] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_selectors, 3),
  [1222] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_selectors, 4),
  [1224] = {.entry = {.count = 1, .reusable = true}}, SHIFT(964),
  [1226] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2384),
  [1228] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extends_clause, 3, .production_id = 19),
  [1230] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 97),
  [1232] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 107),
  [1234] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 4, .production_id = 23),
  [1236] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 4, .production_id = 30),
  [1238] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_template_body, 2),
  [1240] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_template_body, 3),
  [1242] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 7, .production_id = 148),
  [1244] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 7, .production_id = 150),
  [1246] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 7, .production_id = 151),
  [1248] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 7, .production_id = 153),
  [1250] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 131),
  [1252] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 6, .production_id = 127),
  [1254] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__object_definition, 3, .production_id = 20),
  [1256] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 7, .production_id = 156),
  [1258] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_val_declaration, 5, .production_id = 70),
  [1260] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_declaration, 5, .production_id = 70),
  [1262] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 6, .production_id = 124),
  [1264] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 7, .production_id = 159),
  [1266] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 7, .production_id = 160),
  [1268] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_package_clause, 3, .production_id = 8),
  [1270] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 7, .production_id = 163),
  [1272] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 7, .production_id = 164),
  [1274] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_definition, 5, .production_id = 72),
  [1276] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_val_declaration, 7, .production_id = 166),
  [1278] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_declaration, 7, .production_id = 166),
  [1280] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_definition, 7, .production_id = 167),
  [1282] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_definition, 6, .production_id = 123),
  [1284] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 7, .production_id = 168),
  [1286] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_declaration, 6, .production_id = 122),
  [1288] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__object_definition, 2, .production_id = 9),
  [1290] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 7, .production_id = 171),
  [1292] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_val_declaration, 6, .production_id = 122),
  [1294] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 120),
  [1296] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 119),
  [1298] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 5, .production_id = 73),
  [1300] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 5, .production_id = 76),
  [1302] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 116),
  [1304] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 6, .production_id = 115),
  [1306] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 3, .production_id = 8),
  [1308] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 80),
  [1310] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 6, .production_id = 112),
  [1312] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_definition, 5, .production_id = 85),
  [1314] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 5, .production_id = 53),
  [1316] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 5, .production_id = 89),
  [1318] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 7, .production_id = 143),
  [1320] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 8, .production_id = 182),
  [1322] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 4, .production_id = 23),
  [1324] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_definition, 6, .production_id = 136),
  [1326] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 8, .production_id = 184),
  [1328] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 8, .production_id = 186),
  [1330] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 8, .production_id = 187),
  [1332] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 8, .production_id = 189),
  [1334] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_object_definition, 2, .production_id = 3),
  [1336] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_definition, 4, .production_id = 43),
  [1338] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 6, .production_id = 116),
  [1340] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 8, .production_id = 192),
  [1342] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 6, .production_id = 140),
  [1344] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 5, .production_id = 80),
  [1346] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_object_definition, 3, .production_id = 7),
  [1348] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 8, .production_id = 195),
  [1350] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_definition, 5, .production_id = 62),
  [1352] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 53),
  [1354] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 65),
  [1356] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_definition, 4, .production_id = 29),
  [1358] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 4, .production_id = 26),
  [1360] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 9, .production_id = 200),
  [1362] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 63),
  [1364] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 9, .production_id = 202),
  [1366] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 62),
  [1368] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_package_object, 3, .production_id = 7),
  [1370] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 4, .production_id = 47),
  [1372] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 56),
  [1374] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_definition, 3, .production_id = 8),
  [1376] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 3, .production_id = 8),
  [1378] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 100),
  [1380] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_definition, 4, .production_id = 26),
  [1382] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 98),
  [1384] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 4, .production_id = 29),
  [1386] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 4, .production_id = 26),
  [1388] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 57),
  [1390] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2392),
  [1392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2332),
  [1394] = {.entry = {.count = 1, .reusable = true}}, SHIFT(741),
  [1396] = {.entry = {.count = 1, .reusable = true}}, SHIFT(740),
  [1398] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2265),
  [1400] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2280),
  [1402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2303),
  [1404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2385),
  [1406] = {.entry = {.count = 1, .reusable = true}}, SHIFT(723),
  [1408] = {.entry = {.count = 1, .reusable = true}}, SHIFT(713),
  [1410] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2395),
  [1412] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2386),
  [1414] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1198),
  [1416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(816),
  [1418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(684),
  [1420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2278),
  [1422] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1454),
  [1424] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2182),
  [1426] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2385),
  [1428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(749),
  [1430] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1638),
  [1432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1638),
  [1434] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1685),
  [1436] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2233),
  [1438] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2232),
  [1440] = {.entry = {.count = 1, .reusable = false}}, SHIFT(168),
  [1442] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2271),
  [1444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(846),
  [1446] = {.entry = {.count = 1, .reusable = false}}, SHIFT(166),
  [1448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(66),
  [1450] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2240),
  [1452] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2254),
  [1454] = {.entry = {.count = 1, .reusable = false}}, SHIFT(187),
  [1456] = {.entry = {.count = 1, .reusable = true}}, SHIFT(797),
  [1458] = {.entry = {.count = 1, .reusable = true}}, SHIFT(50),
  [1460] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2344),
  [1462] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(64),
  [1465] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1389),
  [1467] = {.entry = {.count = 1, .reusable = true}}, SHIFT(709),
  [1469] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1787),
  [1471] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1604),
  [1473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1604),
  [1475] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1701),
  [1477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2229),
  [1479] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2177),
  [1481] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1534),
  [1483] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1413),
  [1485] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1670),
  [1487] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1615),
  [1489] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1615),
  [1491] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1415),
  [1493] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1741),
  [1495] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1620),
  [1497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1620),
  [1499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1110),
  [1501] = {.entry = {.count = 1, .reusable = true}}, SHIFT(783),
  [1503] = {.entry = {.count = 1, .reusable = true}}, SHIFT(689),
  [1505] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1562),
  [1508] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1427),
  [1510] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1716),
  [1512] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1616),
  [1514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1616),
  [1516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(796),
  [1518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(73),
  [1520] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2399),
  [1522] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1404),
  [1524] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1660),
  [1526] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1606),
  [1528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1606),
  [1530] = {.entry = {.count = 1, .reusable = false}}, SHIFT(237),
  [1532] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2359),
  [1534] = {.entry = {.count = 1, .reusable = true}}, SHIFT(923),
  [1536] = {.entry = {.count = 1, .reusable = false}}, SHIFT(240),
  [1538] = {.entry = {.count = 1, .reusable = true}}, SHIFT(45),
  [1540] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2209),
  [1542] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2247),
  [1544] = {.entry = {.count = 1, .reusable = false}}, SHIFT(248),
  [1546] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1410),
  [1548] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1688),
  [1550] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1613),
  [1552] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1613),
  [1554] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1517),
  [1556] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1712),
  [1558] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1712),
  [1560] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1477),
  [1562] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1672),
  [1564] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1672),
  [1566] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1476),
  [1568] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1666),
  [1570] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1666),
  [1572] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2398),
  [1574] = {.entry = {.count = 1, .reusable = true}}, SHIFT(859),
  [1576] = {.entry = {.count = 1, .reusable = true}}, SHIFT(69),
  [1578] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2300),
  [1580] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1561),
  [1582] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1394),
  [1584] = {.entry = {.count = 1, .reusable = true}}, SHIFT(745),
  [1586] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1731),
  [1588] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1731),
  [1590] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1763),
  [1592] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2231),
  [1594] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2235),
  [1596] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1455),
  [1598] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1753),
  [1600] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1753),
  [1602] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1484),
  [1604] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1693),
  [1606] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1693),
  [1608] = {.entry = {.count = 1, .reusable = true}}, SHIFT(801),
  [1610] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1538),
  [1612] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2370),
  [1614] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1480),
  [1616] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1680),
  [1618] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1680),
  [1620] = {.entry = {.count = 1, .reusable = true}}, SHIFT(832),
  [1622] = {.entry = {.count = 1, .reusable = true}}, SHIFT(63),
  [1624] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2322),
  [1626] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1399),
  [1628] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1732),
  [1630] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1732),
  [1632] = {.entry = {.count = 1, .reusable = false}}, SHIFT(222),
  [1634] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2274),
  [1636] = {.entry = {.count = 1, .reusable = true}}, SHIFT(809),
  [1638] = {.entry = {.count = 1, .reusable = false}}, SHIFT(221),
  [1640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(55),
  [1642] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2260),
  [1644] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2264),
  [1646] = {.entry = {.count = 1, .reusable = false}}, SHIFT(225),
  [1648] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1450),
  [1650] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1756),
  [1652] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1756),
  [1654] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1500),
  [1656] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1662),
  [1658] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1662),
  [1660] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1553),
  [1662] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1793),
  [1664] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1793),
  [1666] = {.entry = {.count = 1, .reusable = false}}, SHIFT(169),
  [1668] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1490),
  [1670] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1719),
  [1672] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1719),
  [1674] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1488),
  [1676] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1714),
  [1678] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1714),
  [1680] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1421),
  [1682] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1781),
  [1684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1781),
  [1686] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1424),
  [1688] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1722),
  [1690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1722),
  [1692] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1437),
  [1694] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1755),
  [1696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1755),
  [1698] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1444),
  [1700] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1746),
  [1702] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1746),
  [1704] = {.entry = {.count = 1, .reusable = false}}, SHIFT(189),
  [1706] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1428),
  [1708] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1760),
  [1710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1760),
  [1712] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1430),
  [1714] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1762),
  [1716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1762),
  [1718] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1441),
  [1720] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1761),
  [1722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1761),
  [1724] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1440),
  [1726] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1752),
  [1728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1752),
  [1730] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1547),
  [1732] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1735),
  [1734] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1735),
  [1736] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1541),
  [1738] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1727),
  [1740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1727),
  [1742] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1513),
  [1744] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1713),
  [1746] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1713),
  [1748] = {.entry = {.count = 1, .reusable = true}}, SHIFT(58),
  [1750] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1481),
  [1752] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1645),
  [1754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1645),
  [1756] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1588),
  [1758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2380),
  [1760] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_modifiers, 1),
  [1762] = {.entry = {.count = 1, .reusable = true}}, SHIFT(766),
  [1764] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(50),
  [1767] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_modifiers_repeat1, 2),
  [1769] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_modifiers_repeat1, 2), SHIFT_REPEAT(766),
  [1772] = {.entry = {.count = 1, .reusable = false}}, SHIFT(265),
  [1774] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2379),
  [1776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(888),
  [1778] = {.entry = {.count = 1, .reusable = false}}, SHIFT(175),
  [1780] = {.entry = {.count = 1, .reusable = true}}, SHIFT(47),
  [1782] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2185),
  [1784] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2187),
  [1786] = {.entry = {.count = 1, .reusable = false}}, SHIFT(196),
  [1788] = {.entry = {.count = 1, .reusable = true}}, SHIFT(821),
  [1790] = {.entry = {.count = 1, .reusable = true}}, SHIFT(62),
  [1792] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2334),
  [1794] = {.entry = {.count = 1, .reusable = false}}, SHIFT(164),
  [1796] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2337),
  [1798] = {.entry = {.count = 1, .reusable = true}}, SHIFT(789),
  [1800] = {.entry = {.count = 1, .reusable = false}}, SHIFT(159),
  [1802] = {.entry = {.count = 1, .reusable = true}}, SHIFT(72),
  [1804] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2208),
  [1806] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2226),
  [1808] = {.entry = {.count = 1, .reusable = false}}, SHIFT(170),
  [1810] = {.entry = {.count = 1, .reusable = true}}, SHIFT(805),
  [1812] = {.entry = {.count = 1, .reusable = true}}, SHIFT(59),
  [1814] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2355),
  [1816] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1594),
  [1818] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1215),
  [1820] = {.entry = {.count = 1, .reusable = true}}, SHIFT(669),
  [1822] = {.entry = {.count = 1, .reusable = true}}, SHIFT(335),
  [1824] = {.entry = {.count = 1, .reusable = true}}, SHIFT(679),
  [1826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(760),
  [1828] = {.entry = {.count = 1, .reusable = true}}, SHIFT(676),
  [1830] = {.entry = {.count = 1, .reusable = true}}, SHIFT(952),
  [1832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(670),
  [1834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1171),
  [1836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(675),
  [1838] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1597),
  [1840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(710),
  [1842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(678),
  [1844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1181),
  [1846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(688),
  [1848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2321),
  [1850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(145),
  [1852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(682),
  [1854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(840),
  [1856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(673),
  [1858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(283),
  [1860] = {.entry = {.count = 1, .reusable = true}}, SHIFT(671),
  [1862] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_parameter, 1, .production_id = 2),
  [1864] = {.entry = {.count = 1, .reusable = true}}, SHIFT(874),
  [1866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(873),
  [1868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(869),
  [1870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(868),
  [1872] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2299),
  [1874] = {.entry = {.count = 1, .reusable = false}}, SHIFT(246),
  [1876] = {.entry = {.count = 1, .reusable = true}}, SHIFT(812),
  [1878] = {.entry = {.count = 1, .reusable = true}}, SHIFT(61),
  [1880] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2340),
  [1882] = {.entry = {.count = 1, .reusable = false}}, SHIFT(255),
  [1884] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2328),
  [1886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(790),
  [1888] = {.entry = {.count = 1, .reusable = false}}, SHIFT(254),
  [1890] = {.entry = {.count = 1, .reusable = true}}, SHIFT(65),
  [1892] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2217),
  [1894] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2241),
  [1896] = {.entry = {.count = 1, .reusable = false}}, SHIFT(90),
  [1898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(807),
  [1900] = {.entry = {.count = 1, .reusable = true}}, SHIFT(677),
  [1902] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block, 1),
  [1904] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__block, 1),
  [1906] = {.entry = {.count = 1, .reusable = true}}, SHIFT(22),
  [1908] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1189),
  [1910] = {.entry = {.count = 1, .reusable = true}}, SHIFT(687),
  [1912] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1003),
  [1914] = {.entry = {.count = 1, .reusable = true}}, SHIFT(683),
  [1916] = {.entry = {.count = 1, .reusable = true}}, SHIFT(811),
  [1918] = {.entry = {.count = 1, .reusable = true}}, SHIFT(60),
  [1920] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2347),
  [1922] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1554),
  [1924] = {.entry = {.count = 1, .reusable = true}}, SHIFT(877),
  [1926] = {.entry = {.count = 1, .reusable = true}}, SHIFT(49),
  [1928] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2266),
  [1930] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2362),
  [1932] = {.entry = {.count = 1, .reusable = true}}, SHIFT(948),
  [1934] = {.entry = {.count = 1, .reusable = true}}, SHIFT(35),
  [1936] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2390),
  [1938] = {.entry = {.count = 1, .reusable = false}}, SHIFT(228),
  [1940] = {.entry = {.count = 1, .reusable = true}}, SHIFT(897),
  [1942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(57),
  [1944] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2305),
  [1946] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1534),
  [1949] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(73),
  [1952] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1558),
  [1954] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2330),
  [1956] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1556),
  [1958] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(63),
  [1961] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1561),
  [1964] = {.entry = {.count = 1, .reusable = true}}, SHIFT(18),
  [1966] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1323),
  [1968] = {.entry = {.count = 1, .reusable = true}}, SHIFT(818),
  [1970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1511),
  [1972] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1563),
  [1974] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(69),
  [1977] = {.entry = {.count = 1, .reusable = true}}, SHIFT(17),
  [1979] = {.entry = {.count = 1, .reusable = true}}, SHIFT(983),
  [1981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(208),
  [1983] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1611),
  [1985] = {.entry = {.count = 1, .reusable = true}}, SHIFT(36),
  [1987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(20),
  [1989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(858),
  [1991] = {.entry = {.count = 1, .reusable = true}}, SHIFT(192),
  [1993] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__block_repeat1, 2),
  [1995] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__block_repeat1, 2),
  [1997] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(58),
  [2000] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1535),
  [2002] = {.entry = {.count = 1, .reusable = true}}, SHIFT(786),
  [2004] = {.entry = {.count = 1, .reusable = true}}, SHIFT(209),
  [2006] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1538),
  [2009] = {.entry = {.count = 1, .reusable = false}}, SHIFT(91),
  [2011] = {.entry = {.count = 1, .reusable = false}}, SHIFT(199),
  [2013] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2376),
  [2015] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1576),
  [2017] = {.entry = {.count = 1, .reusable = true}}, SHIFT(220),
  [2019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1186),
  [2021] = {.entry = {.count = 1, .reusable = true}}, SHIFT(325),
  [2023] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(59),
  [2026] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(62),
  [2029] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1210),
  [2031] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1226),
  [2033] = {.entry = {.count = 1, .reusable = true}}, SHIFT(799),
  [2035] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1047),
  [2037] = {.entry = {.count = 1, .reusable = true}}, SHIFT(433),
  [2039] = {.entry = {.count = 1, .reusable = true}}, SHIFT(493),
  [2041] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1079),
  [2043] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1290),
  [2045] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1594),
  [2048] = {.entry = {.count = 1, .reusable = true}}, SHIFT(560),
  [2050] = {.entry = {.count = 1, .reusable = true}}, SHIFT(440),
  [2052] = {.entry = {.count = 1, .reusable = true}}, SHIFT(316),
  [2054] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1170),
  [2056] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1448),
  [2058] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1026),
  [2060] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1529),
  [2062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1475),
  [2064] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1568),
  [2066] = {.entry = {.count = 1, .reusable = true}}, SHIFT(882),
  [2068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1467),
  [2070] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1461),
  [2072] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2389),
  [2074] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1381),
  [2076] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1588),
  [2079] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1329),
  [2081] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1352),
  [2083] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1525),
  [2085] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2267),
  [2087] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1509),
  [2089] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67), SHIFT_REPEAT(1525),
  [2092] = {.entry = {.count = 1, .reusable = true}}, SHIFT(231),
  [2094] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1384),
  [2096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2339),
  [2098] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1235),
  [2100] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1533),
  [2102] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1295),
  [2104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1786),
  [2106] = {.entry = {.count = 1, .reusable = true}}, SHIFT(288),
  [2108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2307),
  [2110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(414),
  [2112] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_parameter, 2, .production_id = 60),
  [2114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(771),
  [2116] = {.entry = {.count = 1, .reusable = true}}, SHIFT(463),
  [2118] = {.entry = {.count = 1, .reusable = true}}, SHIFT(307),
  [2120] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1447),
  [2122] = {.entry = {.count = 1, .reusable = true}}, SHIFT(501),
  [2124] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2345),
  [2126] = {.entry = {.count = 1, .reusable = true}}, SHIFT(887),
  [2128] = {.entry = {.count = 1, .reusable = true}}, SHIFT(913),
  [2130] = {.entry = {.count = 1, .reusable = true}}, SHIFT(861),
  [2132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(244),
  [2134] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameter, 3, .production_id = 106),
  [2136] = {.entry = {.count = 1, .reusable = true}}, SHIFT(876),
  [2138] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(61),
  [2141] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(57),
  [2144] = {.entry = {.count = 1, .reusable = true}}, SHIFT(924),
  [2146] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2),
  [2148] = {.entry = {.count = 1, .reusable = true}}, SHIFT(880),
  [2150] = {.entry = {.count = 1, .reusable = false}}, SHIFT(834),
  [2152] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1953),
  [2154] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1942),
  [2156] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1548),
  [2158] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(49),
  [2161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(862),
  [2163] = {.entry = {.count = 1, .reusable = true}}, SHIFT(884),
  [2165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(822),
  [2167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(182),
  [2169] = {.entry = {.count = 1, .reusable = true}}, SHIFT(824),
  [2171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(167),
  [2173] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 3, .production_id = 106),
  [2175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(817),
  [2177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(940),
  [2179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(219),
  [2181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(982),
  [2183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(223),
  [2185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(980),
  [2187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(213),
  [2189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(944),
  [2191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(218),
  [2193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(857),
  [2195] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1281),
  [2197] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1515),
  [2199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(987),
  [2201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(165),
  [2203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(893),
  [2205] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67), SHIFT_REPEAT(1523),
  [2208] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(60),
  [2211] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67), SHIFT_REPEAT(1515),
  [2214] = {.entry = {.count = 1, .reusable = true}}, SHIFT(904),
  [2216] = {.entry = {.count = 1, .reusable = true}}, SHIFT(966),
  [2218] = {.entry = {.count = 1, .reusable = true}}, SHIFT(965),
  [2220] = {.entry = {.count = 1, .reusable = true}}, SHIFT(80),
  [2222] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1523),
  [2224] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameter, 7, .production_id = 204),
  [2226] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 6, .production_id = 198),
  [2228] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameter, 6, .production_id = 198),
  [2230] = {.entry = {.count = 1, .reusable = true}}, SHIFT(828),
  [2232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(855),
  [2234] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(35),
  [2237] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 5, .production_id = 179),
  [2239] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameter, 5, .production_id = 180),
  [2241] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1554),
  [2244] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1376),
  [2246] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameter, 4, .production_id = 146),
  [2248] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1597),
  [2251] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameter, 5, .production_id = 179),
  [2253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(841),
  [2255] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 4, .production_id = 146),
  [2257] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_parameter, 3, .production_id = 101),
  [2259] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_parameter, 2, .production_id = 61),
  [2261] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1563),
  [2264] = {.entry = {.count = 1, .reusable = true}}, SHIFT(708),
  [2266] = {.entry = {.count = 1, .reusable = true}}, SHIFT(707),
  [2268] = {.entry = {.count = 1, .reusable = false}}, SHIFT(928),
  [2270] = {.entry = {.count = 1, .reusable = false}}, SHIFT(716),
  [2272] = {.entry = {.count = 1, .reusable = true}}, SHIFT(704),
  [2274] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1734),
  [2276] = {.entry = {.count = 1, .reusable = false}}, SHIFT(720),
  [2278] = {.entry = {.count = 1, .reusable = false}}, SHIFT(708),
  [2280] = {.entry = {.count = 1, .reusable = true}}, SHIFT(742),
  [2282] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2365),
  [2284] = {.entry = {.count = 1, .reusable = false}}, SHIFT(829),
  [2286] = {.entry = {.count = 1, .reusable = false}}, SHIFT(727),
  [2288] = {.entry = {.count = 1, .reusable = false}}, SHIFT(191),
  [2290] = {.entry = {.count = 1, .reusable = true}}, SHIFT(690),
  [2292] = {.entry = {.count = 1, .reusable = false}}, SHIFT(744),
  [2294] = {.entry = {.count = 1, .reusable = false}}, SHIFT(742),
  [2296] = {.entry = {.count = 1, .reusable = false}}, SHIFT(826),
  [2298] = {.entry = {.count = 1, .reusable = false}}, SHIFT(180),
  [2300] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_guard, 2, .production_id = 175),
  [2302] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1659),
  [2304] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_annotation_repeat1, 2), SHIFT_REPEAT(36),
  [2307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1702),
  [2309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1700),
  [2311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1733),
  [2313] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1556),
  [2316] = {.entry = {.count = 1, .reusable = false}}, SHIFT(848),
  [2318] = {.entry = {.count = 1, .reusable = false}}, SHIFT(256),
  [2320] = {.entry = {.count = 1, .reusable = false}}, SHIFT(843),
  [2322] = {.entry = {.count = 1, .reusable = false}}, SHIFT(259),
  [2324] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1576),
  [2327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1658),
  [2329] = {.entry = {.count = 1, .reusable = false}}, SHIFT(955),
  [2331] = {.entry = {.count = 1, .reusable = false}}, SHIFT(216),
  [2333] = {.entry = {.count = 1, .reusable = false}}, SHIFT(960),
  [2335] = {.entry = {.count = 1, .reusable = false}}, SHIFT(210),
  [2337] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1535),
  [2340] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1254),
  [2342] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1472),
  [2344] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_function_definition_repeat1, 2), SHIFT_REPEAT(1611),
  [2347] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67), SHIFT_REPEAT(1472),
  [2350] = {.entry = {.count = 1, .reusable = false}}, SHIFT(936),
  [2352] = {.entry = {.count = 1, .reusable = false}}, SHIFT(106),
  [2354] = {.entry = {.count = 1, .reusable = false}}, SHIFT(977),
  [2356] = {.entry = {.count = 1, .reusable = false}}, SHIFT(243),
  [2358] = {.entry = {.count = 1, .reusable = false}}, SHIFT(974),
  [2360] = {.entry = {.count = 1, .reusable = false}}, SHIFT(247),
  [2362] = {.entry = {.count = 1, .reusable = false}}, SHIFT(895),
  [2364] = {.entry = {.count = 1, .reusable = false}}, SHIFT(113),
  [2366] = {.entry = {.count = 1, .reusable = false}}, SHIFT(804),
  [2368] = {.entry = {.count = 1, .reusable = false}}, SHIFT(214),
  [2370] = {.entry = {.count = 1, .reusable = false}}, SHIFT(726),
  [2372] = {.entry = {.count = 1, .reusable = false}}, SHIFT(13),
  [2374] = {.entry = {.count = 1, .reusable = false}}, SHIFT(820),
  [2376] = {.entry = {.count = 1, .reusable = false}}, SHIFT(730),
  [2378] = {.entry = {.count = 1, .reusable = true}}, SHIFT(700),
  [2380] = {.entry = {.count = 1, .reusable = false}}, SHIFT(729),
  [2382] = {.entry = {.count = 1, .reusable = false}}, SHIFT(171),
  [2384] = {.entry = {.count = 1, .reusable = false}}, SHIFT(802),
  [2386] = {.entry = {.count = 1, .reusable = false}}, SHIFT(211),
  [2388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(946),
  [2390] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1473),
  [2392] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67), SHIFT_REPEAT(1473),
  [2395] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1558),
  [2398] = {.entry = {.count = 1, .reusable = false}}, SHIFT(156),
  [2400] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_infix_pattern, 3, .production_id = 40),
  [2402] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_infix_pattern, 3, .production_id = 40),
  [2404] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1268),
  [2406] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_alternative_pattern, 3),
  [2408] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_alternative_pattern, 3),
  [2410] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1911),
  [2412] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2268),
  [2414] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1577),
  [2416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(543),
  [2418] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_capture_pattern, 3, .production_id = 37),
  [2420] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_capture_pattern, 3, .production_id = 37),
  [2422] = {.entry = {.count = 1, .reusable = false}}, SHIFT(251),
  [2424] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_class_definition_repeat2, 2), SHIFT_REPEAT(1511),
  [2427] = {.entry = {.count = 1, .reusable = false}}, SHIFT(77),
  [2429] = {.entry = {.count = 1, .reusable = false}}, SHIFT(174),
  [2431] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67), SHIFT_REPEAT(1494),
  [2434] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1356),
  [2436] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1519),
  [2438] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1494),
  [2440] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1238),
  [2442] = {.entry = {.count = 1, .reusable = false}}, SHIFT(146),
  [2444] = {.entry = {.count = 1, .reusable = false}}, SHIFT(141),
  [2446] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1543),
  [2448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1634),
  [2450] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67), SHIFT_REPEAT(1519),
  [2453] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_case_class_pattern_repeat1, 2),
  [2455] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_class_definition_repeat1, 2), SHIFT_REPEAT(1548),
  [2458] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1503),
  [2460] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_parameter, 5, .production_id = 177),
  [2462] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_parameter, 4, .production_id = 144),
  [2464] = {.entry = {.count = 1, .reusable = false}}, SHIFT(803),
  [2466] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1609),
  [2468] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_parameter, 4, .production_id = 145),
  [2470] = {.entry = {.count = 1, .reusable = true}}, SHIFT(522),
  [2472] = {.entry = {.count = 1, .reusable = true}}, SHIFT(881),
  [2474] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1496),
  [2476] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1264),
  [2478] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1501),
  [2480] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67), SHIFT_REPEAT(1503),
  [2483] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1324),
  [2485] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67), SHIFT_REPEAT(1496),
  [2488] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67), SHIFT_REPEAT(1501),
  [2491] = {.entry = {.count = 1, .reusable = true}}, SHIFT(528),
  [2493] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1254),
  [2495] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67), SHIFT_REPEAT(1493),
  [2498] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1493),
  [2500] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1312),
  [2502] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1506),
  [2504] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_parameter, 3, .production_id = 102),
  [2506] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1231),
  [2508] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_type_repeat1, 2, .production_id = 67), SHIFT_REPEAT(1506),
  [2511] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1341),
  [2513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(83),
  [2515] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2387),
  [2517] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1356),
  [2519] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1801),
  [2521] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1657),
  [2523] = {.entry = {.count = 1, .reusable = true}}, SHIFT(518),
  [2525] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1690),
  [2527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1573),
  [2529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(102),
  [2531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(172),
  [2533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(85),
  [2535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1238),
  [2537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(150),
  [2539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(84),
  [2541] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_package_identifier_repeat1, 2), SHIFT_REPEAT(2387),
  [2544] = {.entry = {.count = 1, .reusable = true}}, SHIFT(235),
  [2546] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1268),
  [2548] = {.entry = {.count = 1, .reusable = true}}, SHIFT(71),
  [2550] = {.entry = {.count = 1, .reusable = true}}, SHIFT(215),
  [2552] = {.entry = {.count = 1, .reusable = true}}, SHIFT(195),
  [2554] = {.entry = {.count = 1, .reusable = true}}, SHIFT(78),
  [2556] = {.entry = {.count = 1, .reusable = true}}, SHIFT(143),
  [2558] = {.entry = {.count = 1, .reusable = true}}, SHIFT(81),
  [2560] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1312),
  [2562] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_import_declaration_repeat1, 2, .production_id = 21), SHIFT_REPEAT(2153),
  [2565] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_pattern, 4),
  [2567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1801),
  [2569] = {.entry = {.count = 1, .reusable = true}}, SHIFT(520),
  [2571] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_class_pattern, 4, .production_id = 69),
  [2573] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_class_pattern, 4, .production_id = 71),
  [2575] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_class_pattern, 3, .production_id = 42),
  [2577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1341),
  [2579] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_class_pattern, 3, .production_id = 42),
  [2581] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_class_pattern, 5, .production_id = 109),
  [2583] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_class_pattern, 5, .production_id = 110),
  [2585] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_class_pattern, 5, .production_id = 110),
  [2587] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_class_pattern, 5, .production_id = 109),
  [2589] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1812),
  [2591] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2319),
  [2593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2309),
  [2595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2326),
  [2597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(732),
  [2599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(731),
  [2601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2308),
  [2603] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2304),
  [2605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2285),
  [2607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2394),
  [2609] = {.entry = {.count = 1, .reusable = true}}, SHIFT(738),
  [2611] = {.entry = {.count = 1, .reusable = true}}, SHIFT(739),
  [2613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2397),
  [2615] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2396),
  [2617] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2153),
  [2619] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_class_pattern, 3, .production_id = 39),
  [2621] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_class_pattern, 3, .production_id = 39),
  [2623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1555),
  [2625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1264),
  [2627] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1231),
  [2629] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1324),
  [2631] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_class_pattern, 4, .production_id = 71),
  [2633] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_class_pattern, 4, .production_id = 69),
  [2635] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_pattern, 4),
  [2637] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__type_parameter_repeat1, 2),
  [2639] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__type_parameter_repeat1, 2), SHIFT_REPEAT(869),
  [2642] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_typed_pattern, 3, .production_id = 35),
  [2644] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1868),
  [2646] = {.entry = {.count = 1, .reusable = false}}, SHIFT(847),
  [2648] = {.entry = {.count = 1, .reusable = true}}, SHIFT(705),
  [2650] = {.entry = {.count = 1, .reusable = true}}, SHIFT(405),
  [2652] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1933),
  [2654] = {.entry = {.count = 1, .reusable = true}}, SHIFT(442),
  [2656] = {.entry = {.count = 1, .reusable = false}}, SHIFT(929),
  [2658] = {.entry = {.count = 1, .reusable = false}}, SHIFT(934),
  [2660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1405),
  [2662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1192),
  [2664] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1022),
  [2666] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__block_repeat1, 2), SHIFT_REPEAT(24),
  [2669] = {.entry = {.count = 1, .reusable = false}}, SHIFT(892),
  [2671] = {.entry = {.count = 1, .reusable = true}}, SHIFT(339),
  [2673] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_parameter, 6, .production_id = 197),
  [2675] = {.entry = {.count = 1, .reusable = false}}, SHIFT(971),
  [2677] = {.entry = {.count = 1, .reusable = false}}, SHIFT(939),
  [2679] = {.entry = {.count = 1, .reusable = false}}, SHIFT(899),
  [2681] = {.entry = {.count = 1, .reusable = false}}, SHIFT(942),
  [2683] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_parameter, 5, .production_id = 178),
  [2685] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1163),
  [2687] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_upper_bound, 2, .production_id = 19),
  [2689] = {.entry = {.count = 1, .reusable = false}}, SHIFT(927),
  [2691] = {.entry = {.count = 1, .reusable = false}}, SHIFT(967),
  [2693] = {.entry = {.count = 1, .reusable = false}}, SHIFT(926),
  [2695] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1420),
  [2697] = {.entry = {.count = 1, .reusable = true}}, SHIFT(21),
  [2699] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__type_parameter_repeat2, 2),
  [2701] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__type_parameter_repeat2, 2), SHIFT_REPEAT(868),
  [2704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(962),
  [2706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1359),
  [2708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1426),
  [2710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1106),
  [2712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1339),
  [2714] = {.entry = {.count = 1, .reusable = false}}, SHIFT(849),
  [2716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1436),
  [2718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1451),
  [2720] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 1, .production_id = 2),
  [2722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(695),
  [2724] = {.entry = {.count = 1, .reusable = true}}, SHIFT(226),
  [2726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1914),
  [2728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1321),
  [2730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(706),
  [2732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1272),
  [2734] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 3, .production_id = 105),
  [2736] = {.entry = {.count = 1, .reusable = true}}, SHIFT(147),
  [2738] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1309),
  [2740] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameter, 2, .production_id = 1),
  [2742] = {.entry = {.count = 1, .reusable = true}}, SHIFT(879),
  [2744] = {.entry = {.count = 1, .reusable = true}}, SHIFT(224),
  [2746] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1046),
  [2748] = {.entry = {.count = 1, .reusable = true}}, SHIFT(23),
  [2750] = {.entry = {.count = 1, .reusable = true}}, SHIFT(903),
  [2752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1407),
  [2754] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameter, 3, .production_id = 12),
  [2756] = {.entry = {.count = 1, .reusable = true}}, SHIFT(937),
  [2758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(148),
  [2760] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1391),
  [2762] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2245),
  [2764] = {.entry = {.count = 1, .reusable = true}}, SHIFT(581),
  [2766] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2194),
  [2768] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 4, .production_id = 36),
  [2770] = {.entry = {.count = 1, .reusable = true}}, SHIFT(75),
  [2772] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1796),
  [2774] = {.entry = {.count = 1, .reusable = true}}, SHIFT(281),
  [2776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(494),
  [2778] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1296),
  [2780] = {.entry = {.count = 1, .reusable = true}}, SHIFT(310),
  [2782] = {.entry = {.count = 1, .reusable = true}}, SHIFT(457),
  [2784] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_view_bound, 2, .production_id = 19),
  [2786] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lower_bound, 2, .production_id = 19),
  [2788] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_type_repeat1, 2), SHIFT_REPEAT(810),
  [2791] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_tuple_type_repeat1, 2),
  [2793] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1664),
  [2795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2258),
  [2797] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1744),
  [2799] = {.entry = {.count = 1, .reusable = false}}, SHIFT(565),
  [2801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2249),
  [2803] = {.entry = {.count = 1, .reusable = false}}, SHIFT(579),
  [2805] = {.entry = {.count = 1, .reusable = true}}, SHIFT(420),
  [2807] = {.entry = {.count = 1, .reusable = true}}, SHIFT(330),
  [2809] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1108),
  [2811] = {.entry = {.count = 1, .reusable = true}}, SHIFT(421),
  [2813] = {.entry = {.count = 1, .reusable = true}}, SHIFT(357),
  [2815] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_case_block_repeat1, 2),
  [2817] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_block_repeat1, 2), SHIFT_REPEAT(706),
  [2820] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1216),
  [2822] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1220),
  [2824] = {.entry = {.count = 1, .reusable = true}}, SHIFT(925),
  [2826] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameter, 1, .production_id = 2),
  [2828] = {.entry = {.count = 1, .reusable = true}}, SHIFT(784),
  [2830] = {.entry = {.count = 1, .reusable = true}}, SHIFT(100),
  [2832] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 2, .production_id = 1),
  [2834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(696),
  [2836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(204),
  [2838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2178),
  [2840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1036),
  [2842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(428),
  [2844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(810),
  [2846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1267),
  [2848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(976),
  [2850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1675),
  [2852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(978),
  [2854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(379),
  [2856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2195),
  [2858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(908),
  [2860] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1724),
  [2862] = {.entry = {.count = 1, .reusable = true}}, SHIFT(516),
  [2864] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1327),
  [2866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(304),
  [2868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1567),
  [2870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1310),
  [2872] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1241),
  [2874] = {.entry = {.count = 1, .reusable = true}}, SHIFT(984),
  [2876] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_repeated_parameter_type, 2, .production_id = 42),
  [2878] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1795),
  [2880] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1237),
  [2882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(902),
  [2884] = {.entry = {.count = 1, .reusable = true}}, SHIFT(397),
  [2886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(905),
  [2888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(458),
  [2890] = {.entry = {.count = 1, .reusable = true}}, SHIFT(907),
  [2892] = {.entry = {.count = 1, .reusable = true}}, SHIFT(866),
  [2894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1230),
  [2896] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1471),
  [2898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1221),
  [2900] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1335),
  [2902] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1794),
  [2904] = {.entry = {.count = 1, .reusable = true}}, SHIFT(835),
  [2906] = {.entry = {.count = 1, .reusable = true}}, SHIFT(343),
  [2908] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_class_pattern_repeat1, 2), SHIFT_REPEAT(707),
  [2911] = {.entry = {.count = 1, .reusable = true}}, SHIFT(813),
  [2913] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1647),
  [2915] = {.entry = {.count = 1, .reusable = true}}, SHIFT(300),
  [2917] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1612),
  [2919] = {.entry = {.count = 1, .reusable = true}}, SHIFT(806),
  [2921] = {.entry = {.count = 1, .reusable = false}}, SHIFT(526),
  [2923] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2203),
  [2925] = {.entry = {.count = 1, .reusable = true}}, SHIFT(448),
  [2927] = {.entry = {.count = 1, .reusable = true}}, SHIFT(344),
  [2929] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1101),
  [2931] = {.entry = {.count = 1, .reusable = true}}, SHIFT(305),
  [2933] = {.entry = {.count = 1, .reusable = true}}, SHIFT(276),
  [2935] = {.entry = {.count = 1, .reusable = true}}, SHIFT(891),
  [2937] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1654),
  [2939] = {.entry = {.count = 1, .reusable = true}}, SHIFT(362),
  [2941] = {.entry = {.count = 1, .reusable = true}}, SHIFT(422),
  [2943] = {.entry = {.count = 1, .reusable = true}}, SHIFT(411),
  [2945] = {.entry = {.count = 1, .reusable = true}}, SHIFT(693),
  [2947] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2301),
  [2949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1328),
  [2951] = {.entry = {.count = 1, .reusable = true}}, SHIFT(271),
  [2953] = {.entry = {.count = 1, .reusable = true}}, SHIFT(291),
  [2955] = {.entry = {.count = 1, .reusable = true}}, SHIFT(404),
  [2957] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_string_repeat1, 2), SHIFT_REPEAT(2178),
  [2960] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_string_repeat1, 2),
  [2962] = {.entry = {.count = 1, .reusable = true}}, SHIFT(439),
  [2964] = {.entry = {.count = 1, .reusable = true}}, SHIFT(573),
  [2966] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1035),
  [2968] = {.entry = {.count = 1, .reusable = true}}, SHIFT(548),
  [2970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1294),
  [2972] = {.entry = {.count = 1, .reusable = true}}, SHIFT(788),
  [2974] = {.entry = {.count = 1, .reusable = true}}, SHIFT(833),
  [2976] = {.entry = {.count = 1, .reusable = true}}, SHIFT(566),
  [2978] = {.entry = {.count = 1, .reusable = true}}, SHIFT(557),
  [2980] = {.entry = {.count = 1, .reusable = true}}, SHIFT(986),
  [2982] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1249),
  [2984] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1024),
  [2986] = {.entry = {.count = 1, .reusable = true}}, SHIFT(582),
  [2988] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2335),
  [2990] = {.entry = {.count = 1, .reusable = true}}, SHIFT(488),
  [2992] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1234),
  [2994] = {.entry = {.count = 1, .reusable = true}}, SHIFT(819),
  [2996] = {.entry = {.count = 1, .reusable = true}}, SHIFT(474),
  [2998] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_context_bound, 2, .production_id = 19),
  [3000] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1018),
  [3002] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1674),
  [3004] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1187),
  [3006] = {.entry = {.count = 1, .reusable = true}}, SHIFT(482),
  [3008] = {.entry = {.count = 1, .reusable = true}}, SHIFT(757),
  [3010] = {.entry = {.count = 1, .reusable = true}}, SHIFT(286),
  [3012] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1051),
  [3014] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lazy_parameter_type, 2, .production_id = 19),
  [3016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(531),
  [3018] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1012),
  [3020] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_string_repeat2, 2), SHIFT_REPEAT(2195),
  [3023] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_string_repeat2, 2),
  [3025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1206),
  [3027] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1145),
  [3029] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1655),
  [3031] = {.entry = {.count = 1, .reusable = true}}, SHIFT(287),
  [3033] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_type_parameters_repeat1, 2, .production_id = 104), SHIFT_REPEAT(1327),
  [3036] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_parameters_repeat1, 2, .production_id = 104),
  [3038] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1131),
  [3040] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1006),
  [3042] = {.entry = {.count = 1, .reusable = true}}, SHIFT(533),
  [3044] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameter, 3, .production_id = 105),
  [3046] = {.entry = {.count = 1, .reusable = true}}, SHIFT(149),
  [3048] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1219),
  [3050] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1361),
  [3052] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_class_parameters_repeat1, 2), SHIFT_REPEAT(1612),
  [3055] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_class_parameters_repeat1, 2),
  [3057] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1140),
  [3059] = {.entry = {.count = 1, .reusable = true}}, SHIFT(553),
  [3061] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1034),
  [3063] = {.entry = {.count = 1, .reusable = true}}, SHIFT(412),
  [3065] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_type_repeat1, 2), REDUCE(aux_sym_parameter_types_repeat1, 2),
  [3068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1077),
  [3070] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1233),
  [3072] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1368),
  [3074] = {.entry = {.count = 1, .reusable = true}}, SHIFT(503),
  [3076] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1403),
  [3078] = {.entry = {.count = 1, .reusable = true}}, SHIFT(558),
  [3080] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1748),
  [3082] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1123),
  [3084] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1307),
  [3086] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1308),
  [3088] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1438),
  [3090] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameter_types_repeat1, 2), SHIFT_REPEAT(693),
  [3093] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_parameter_types_repeat1, 2),
  [3095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(900),
  [3097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1532),
  [3099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1162),
  [3101] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2), SHIFT_REPEAT(220),
  [3104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(400),
  [3106] = {.entry = {.count = 1, .reusable = true}}, SHIFT(342),
  [3108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(519),
  [3110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(430),
  [3112] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1439),
  [3114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1737),
  [3116] = {.entry = {.count = 1, .reusable = true}}, SHIFT(867),
  [3118] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1583),
  [3120] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1445),
  [3122] = {.entry = {.count = 1, .reusable = true}}, SHIFT(340),
  [3124] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1522),
  [3126] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1277),
  [3128] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1602),
  [3130] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1247),
  [3132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(294),
  [3134] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1726),
  [3136] = {.entry = {.count = 1, .reusable = true}}, SHIFT(292),
  [3138] = {.entry = {.count = 1, .reusable = true}}, SHIFT(295),
  [3140] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1318),
  [3142] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1240),
  [3144] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2), SHIFT_REPEAT(1724),
  [3147] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2),
  [3149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(831),
  [3151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1070),
  [3153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1478),
  [3155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(293),
  [3157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(910),
  [3159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1076),
  [3161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1049),
  [3163] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1603),
  [3165] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2183),
  [3167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1288),
  [3169] = {.entry = {.count = 1, .reusable = true}}, SHIFT(918),
  [3171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(800),
  [3173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1469),
  [3175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1419),
  [3177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1717),
  [3179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1229),
  [3181] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_import_selectors_repeat1, 2),
  [3183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(759),
  [3185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1570),
  [3187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(326),
  [3189] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_import_selectors_repeat1, 2), SHIFT_REPEAT(2245),
  [3192] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1411),
  [3194] = {.entry = {.count = 1, .reusable = true}}, SHIFT(756),
  [3196] = {.entry = {.count = 1, .reusable = true}}, SHIFT(323),
  [3198] = {.entry = {.count = 1, .reusable = true}}, SHIFT(441),
  [3200] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1128),
  [3202] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1462),
  [3204] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1676),
  [3206] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1096),
  [3208] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1355),
  [3210] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1677),
  [3212] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1150),
  [3214] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameter, 4, .production_id = 36),
  [3216] = {.entry = {.count = 1, .reusable = true}}, SHIFT(119),
  [3218] = {.entry = {.count = 1, .reusable = true}}, SHIFT(864),
  [3220] = {.entry = {.count = 1, .reusable = true}}, SHIFT(950),
  [3222] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1260),
  [3224] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1322),
  [3226] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1100),
  [3228] = {.entry = {.count = 1, .reusable = true}}, SHIFT(951),
  [3230] = {.entry = {.count = 1, .reusable = true}}, SHIFT(322),
  [3232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1627),
  [3234] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1081),
  [3236] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1372),
  [3238] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1141),
  [3240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1502),
  [3242] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1053),
  [3244] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_parameter, 5, .production_id = 83),
  [3246] = {.entry = {.count = 1, .reusable = true}}, SHIFT(103),
  [3248] = {.entry = {.count = 1, .reusable = true}}, SHIFT(850),
  [3250] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1043),
  [3252] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1349),
  [3254] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1037),
  [3256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1531),
  [3258] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2200),
  [3260] = {.entry = {.count = 1, .reusable = true}}, SHIFT(14),
  [3262] = {.entry = {.count = 1, .reusable = true}}, SHIFT(767),
  [3264] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1695),
  [3266] = {.entry = {.count = 1, .reusable = true}}, SHIFT(762),
  [3268] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1705),
  [3270] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1155),
  [3272] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_val_declaration_repeat1, 2, .production_id = 41), SHIFT_REPEAT(2365),
  [3275] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_val_declaration_repeat1, 2, .production_id = 41),
  [3277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1154),
  [3279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(460),
  [3281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(782),
  [3283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1706),
  [3285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(979),
  [3287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1380),
  [3289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1348),
  [3291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1385),
  [3293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1783),
  [3295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1780),
  [3297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1261),
  [3299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1250),
  [3301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1287),
  [3303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1168),
  [3305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1357),
  [3307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1152),
  [3309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(230),
  [3311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2202),
  [3313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(16),
  [3315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(402),
  [3317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(401),
  [3319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1304),
  [3321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2143),
  [3323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2172),
  [3325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(521),
  [3327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(262),
  [3329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2212),
  [3331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1545),
  [3333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2182),
  [3335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1805),
  [3337] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_val_declaration_repeat1, 2, .production_id = 1),
  [3339] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_clause, 4, .production_id = 174),
  [3341] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2196),
  [3343] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_renamed_identifier, 3, .production_id = 142),
  [3345] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interpolation, 2),
  [3347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1806),
  [3349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1880),
  [3351] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_covariant_type_parameter, 2, .production_id = 59),
  [3353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1822),
  [3355] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_contravariant_type_parameter, 2, .production_id = 59),
  [3357] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_clause, 5, .production_id = 196),
  [3359] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__variant_type_parameter, 2),
  [3361] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_parameters_repeat1, 2, .production_id = 59),
  [3363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1842),
  [3365] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1855),
  [3367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1820),
  [3369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1870),
  [3371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2099),
  [3373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1830),
  [3375] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1887),
  [3377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1837),
  [3379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1901),
  [3381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1991),
  [3383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1263),
  [3385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1195),
  [3387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1812),
  [3389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(920),
  [3391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(970),
  [3393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(915),
  [3395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(462),
  [3397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1225),
  [3399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(702),
  [3401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2201),
  [3403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(827),
  [3405] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_types, 2, .dynamic_precedence = 1),
  [3407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(303),
  [3409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(359),
  [3411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(163),
  [3413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(914),
  [3415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(576),
  [3417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2383),
  [3419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(268),
  [3421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(814),
  [3423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(815),
  [3425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(336),
  [3427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(396),
  [3429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(590),
  [3431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(348),
  [3433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(416),
  [3435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(785),
  [3437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(694),
  [3439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(825),
  [3441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2204),
  [3443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(836),
  [3445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1032),
  [3447] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_types, 3, .dynamic_precedence = 1),
  [3449] = {.entry = {.count = 1, .reusable = true}}, SHIFT(392),
  [3451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1245),
  [3453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(844),
  [3455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1223),
  [3457] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2137),
  [3459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2366),
  [3461] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1089),
  [3463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(692),
  [3465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1897),
  [3467] = {.entry = {.count = 1, .reusable = true}}, SHIFT(302),
  [3469] = {.entry = {.count = 1, .reusable = true}}, SHIFT(854),
  [3471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1881),
  [3473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(485),
  [3475] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1827),
  [3477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(792),
  [3479] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1011),
  [3481] = {.entry = {.count = 1, .reusable = true}}, SHIFT(863),
  [3483] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1305),
  [3485] = {.entry = {.count = 1, .reusable = true}}, SHIFT(338),
  [3487] = {.entry = {.count = 1, .reusable = true}}, SHIFT(787),
  [3489] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1396),
  [3491] = {.entry = {.count = 1, .reusable = true}}, SHIFT(997),
  [3493] = {.entry = {.count = 1, .reusable = true}}, SHIFT(919),
  [3495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(354),
  [3497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(885),
  [3499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1121),
  [3501] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_types, 4, .dynamic_precedence = 1),
  [3503] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1117),
  [3505] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1301),
  [3507] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1209),
  [3509] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1243),
  [3511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(12),
  [3513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(898),
  [3515] = {.entry = {.count = 1, .reusable = true}}, SHIFT(436),
  [3517] = {.entry = {.count = 1, .reusable = true}}, SHIFT(764),
  [3519] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1173),
  [3521] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1331),
  [3523] = {.entry = {.count = 1, .reusable = true}}, SHIFT(791),
  [3525] = {.entry = {.count = 1, .reusable = true}},  ACCEPT_INPUT(),
  [3527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(922),
  [3529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(277),
  [3531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(418),
  [3533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(542),
  [3535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1125),
  [3537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(334),
  [3539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(932),
  [3541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(352),
  [3543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1059),
  [3545] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1393),
  [3547] = {.entry = {.count = 1, .reusable = true}}, SHIFT(958),
  [3549] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1967),
  [3551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(941),
  [3553] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2188),
  [3555] = {.entry = {.count = 1, .reusable = true}}, SHIFT(399),
  [3557] = {.entry = {.count = 1, .reusable = true}}, SHIFT(837),
  [3559] = {.entry = {.count = 1, .reusable = true}}, SHIFT(947),
  [3561] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1065),
  [3563] = {.entry = {.count = 1, .reusable = true}}, SHIFT(615),
  [3565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(499),
  [3567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(954),
  [3569] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1259),
  [3571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(990),
  [3573] = {.entry = {.count = 1, .reusable = true}}, SHIFT(957),
  [3575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1289),
  [3577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(753),
  [3579] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1095),
  [3581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1390),
  [3583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1033),
  [3585] = {.entry = {.count = 1, .reusable = true}}, SHIFT(549),
  [3587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1074),
  [3589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1050),
  [3591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1673),
  [3593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1016),
  [3595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1146),
  [3597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1353),
  [3599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(328),
  [3601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1985),
  [3603] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1019),
  [3605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1950),
  [3607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1040),
  [3609] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1923),
  [3611] = {.entry = {.count = 1, .reusable = true}}, SHIFT(735),
  [3613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(985),
};

#ifdef __cplusplus
extern "C" {
#endif
void *tree_sitter_scala_external_scanner_create(void);
void tree_sitter_scala_external_scanner_destroy(void *);
bool tree_sitter_scala_external_scanner_scan(void *, TSLexer *, const bool *);
unsigned tree_sitter_scala_external_scanner_serialize(void *, char *);
void tree_sitter_scala_external_scanner_deserialize(void *, const char *, unsigned);

#ifdef _WIN32
#define extern __declspec(dllexport)
#endif

extern const TSLanguage *tree_sitter_scala(void) {
  static TSLanguage language = {
    .version = LANGUAGE_VERSION,
    .symbol_count = SYMBOL_COUNT,
    .alias_count = ALIAS_COUNT,
    .token_count = TOKEN_COUNT,
    .external_token_count = EXTERNAL_TOKEN_COUNT,
    .state_count = STATE_COUNT,
    .large_state_count = LARGE_STATE_COUNT,
    .production_id_count = PRODUCTION_ID_COUNT,
    .field_count = FIELD_COUNT,
    .max_alias_sequence_length = MAX_ALIAS_SEQUENCE_LENGTH,
    .parse_table = (const uint16_t *)ts_parse_table,
    .small_parse_table = (const uint16_t *)ts_small_parse_table,
    .small_parse_table_map = (const uint32_t *)ts_small_parse_table_map,
    .parse_actions = ts_parse_actions,
    .symbol_names = ts_symbol_names,
    .field_names = ts_field_names,
    .field_map_slices = (const TSFieldMapSlice *)ts_field_map_slices,
    .field_map_entries = (const TSFieldMapEntry *)ts_field_map_entries,
    .symbol_metadata = ts_symbol_metadata,
    .public_symbol_map = ts_symbol_map,
    .alias_map = ts_non_terminal_alias_map,
    .alias_sequences = (const TSSymbol *)ts_alias_sequences,
    .lex_modes = ts_lex_modes,
    .lex_fn = ts_lex,
    .keyword_lex_fn = ts_lex_keywords,
    .keyword_capture_token = sym_identifier,
    .external_scanner = {
      (const bool *)ts_external_scanner_states,
      ts_external_scanner_symbol_map,
      tree_sitter_scala_external_scanner_create,
      tree_sitter_scala_external_scanner_destroy,
      tree_sitter_scala_external_scanner_scan,
      tree_sitter_scala_external_scanner_serialize,
      tree_sitter_scala_external_scanner_deserialize,
    },
  };
  return &language;
}
#ifdef __cplusplus
}
#endif
